#! /usr/bin/perl
#  Copyright 2001-2015 Leslie Richardson

#  This file is part of Open Admin for Schools.

#  Open Admin for Schools is free software; you can redistribute it 
#  and/or modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2 of 
#  the License, or (at your option) any later version.


my $defaultdownloaddir = '/opt/openadmin/central/admin/download';


my %lex = ('Attendance Report' => 'Attendance Report',
	   'Main' => 'Main',
	   'Attendance' => 'Attendance',
	   'Periods Per Day' => 'Periods Per Day',
	   'View/Download' => 'View/Download',
	   'View Log File' => 'View Log File',
	   'Last Name' => 'Last Name',
	   'First' => 'First',
	   'Middle' => 'Middle',
	   'Birthdate' => 'Birthdate',
	   'Status' => 'Status',
	   'Reserve' => 'Reserve',
	   'Mother' => 'Mother',
	   'School Days' => 'School Days',
	   'Error' => 'Error',
	   'Date' => 'Date',
	   'Paper Size' => 'Paper Size',
	   'Letter' => 'Letter',
	   'Legal' => 'Legal',
	   'A4' => 'A4',
	   'Continue' => 'Continue',
	   'Records per Page' => 'Records per Page',
	   'Not Defined' => 'Not Defined',
	   'Grade' => 'Grade',
	   'Student' => 'Student',
	   'Not Found' => 'Not Found',
	   'Band' => 'Band',
	   'Font Size' => 'Font Size',
	   'Group' => 'Group',
	   'HTML' => 'HTML',
	   'Homeroom' => 'Homeroom',
	   'PDF' => 'PDF',
	   'Separate with Spaces' => 'Separate with Spaces',
	   'Report Type' => 'Report Type',
	   'Missing' => 'Missing',
	   'Days Open' => 'Days Open',
	   'Total' => 'Total',
	   'Month' => 'Month',
	   'Average' => 'Average',
	   'Division' => 'Division',
	   'Periods per Day' => 'Periods per Day',
	   'Select' => 'Select',
	   'School' => 'School',
	   'CSV' => 'CSV',
	   'Show' => 'Show',
	   'Gender' => 'Gender',
	   'Raw' => 'Raw',
	   'Results' => 'Results',
	   'Percentage' => 'Percentage',
	   'Remote' => 'Remote',
	   'Database' => 'Database',
	   'View/Download' => 'View/Download',
	   'View Log File' => 'View Log File',
	   'Blank=All' => 'Blank=All',
	   'Check Next Page' => 'Check Next Page',
	   'Late' => 'Late',
	   'Absent' => 'Absent',

	   'Show Withdrawn' => 'Show Withdrawn',
	   'Type' => 'Type',
	   'Enrollment' => 'Enrollment',

	   );

my $self = 'rptaandc1.pl';

use DBI;
use CGI;
use Number::Format qw(:all);


my %skipgrades = ('K' => '1', 'PK' => '1'); # grades to skip over.



my $defaultmaxrecords = 20; # Maximum records per PDF page.
my $group = 'grade';
my $pdf = '1';
my $html;

# Read config variables
eval require "../admin.conf";
if ( $@ ) {
    print $lex{Error}. ": $@<br>\n";
    die $lex{Error}. ": $@\n";
}

eval require "../../lib/libattend.pl";
if ( $@ ) {
    print $lex{Error}. ": $@<br>\n";
    die $lex{Error}. ": $@\n";
}

eval require "../../lib/liblatex.pl";
if ( $@ ) {
    print $lex{Error}. ": $@<br>\n";
    die $lex{Error}. ": $@\n";
}


my $q = new CGI;
my %arr = $q->Vars;
print $q->header( -charset, $charset );


my @tim = localtime(time);
my $year = $tim[5] + 1900;
my $month = $tim[4] + 1;
my $day = $tim[3];
if (length($month) == 1){ $month = "0".$month;}
if (length($day) == 1){ $day = "0".$day;}
my $currsdate = "$year-$month-$day";
my $currdate = "$month[$month] $day, $year";


# Load Remote database
my $dbh;
if ( $arr{db} ) { # from start page
    my $db = $arr{db};
    my $dsn = "DBI:mysql:database=$db;host=$remotehost";
    $dbh = DBI->connect($dsn,$remoteuser,$remotepassword);
}



# HTML Header
my $title = qq{AANDC $lex{'Attendance Report'} 1};
print "$doctype\n<html><head><title>$title</title>
<link rel=\"stylesheet\" href=\"/division.css\" type=\"text/css\">\n";


print "$chartype\n</head><body>\n";
print "[ <a href=\"$homepage\">$lex{Main}</a> ]\n";

print qq{<h1>$title };
if ( $schoolname ) { print " &ndash; $schoolname"; }
print qq{</h1>\n};

#foreach my $key ( sort keys %arr ) { print "K:$key V:$arr{$key}<br>\n"; }

if ( not $arr{page} ) {
    showStartPage();

} elsif ( $arr{page} == 1 ) {
    delete $arr{page};
    selectStudents();

} elsif ( $arr{page} == 2 ) {
    delete $arr{page};
    reportPDF();
}



#----------------
sub showStartPage {
#----------------

    # Get remote databases and years
    my $remotedbase = 'information_schema';
    my $dsnr = "DBI:mysql:database=$remotedbase;host=$remotehost";
    my $dbhr = DBI->connect($dsnr,$remoteuser,$remotepassword);
    #my $dbh = DBI->connect($dsn,'tsecremote','ts99ec4');

    my $sth = $dbhr->prepare("select distinct table_schema from TABLES order by table_schema");
    $sth->execute;
    if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }
    my %remotedb;

    while ( my $db = $sth->fetchrow ) {
#	print qq{DB:$db<br>\n};
	if ( $db eq 'mysql' or $db eq 'information_schema' ) { next; }
	$remotedb{$db} = 1;
    }


    print "<form action=\"$self\" method=\"post\">\n";
    print "<input type=\"hidden\" name=\"page\" value=\"1\">\n";

    print "<table cellpadding=\"3\" cellspacing=\"0\" border=\"0\" ";
    print "style=\"padding:0.5em;border:1px solid gray;\">\n";

    print qq{<tr><td class="bra">$lex{Select} $lex{Remote} $lex{Database}</td><td></td></tr>\n};
    foreach my $db ( sort keys %remotedb ) { 
	print qq{<tr><td class="la"><input type="radio" name="db" value="$db"> $db</td></tr>\n};
    }

#    print "<tr><td class=\"bra\">$lex{'Report Type'}</td>\n";
#    print "<td><select name=\"reporttype\"><option value=\"html\">$lex{HTML}</option>\n";
#    print "<option value=\"csv\">$lex{CSV}</option></select></td></tr>\n";

    print qq{<tr><td colspan="2"><hr></td></tr>\n};

    # Grade
    print qq{<tr><td class="bra">$lex{Grade}</td>\n};
    print qq{<td class="la"><input type="text" name="grade" size="4"> $lex{'Blank=All'}</td></tr>\n};

    # Check next Page
    print qq{<tr><td class="bra">$lex{'Check Next Page'}</td>\n};
    print qq{<td class="la"><input type="checkbox" name="checknextpage" value="1"></td></tr>\n};

    # Show Withdrawn
    print qq{<tr><td class="bra">$lex{'Show Withdrawn'}</td>\n};
    print qq{<td class="la"><input type="checkbox" name="showwithdrawn" value="1" checked="checked"></td></tr>\n};


    # Students per Page
    print qq{<tr><td class="bra">Maxiumum Students per Page</td>\n};
    print qq{<td class="la"><input type="text" name="maxrecords" value="$defaultmaxrecords" size="4"></td></tr>\n};


=head
    print "<tr><td class=\"bra\">$lex{Show} $lex{Gender} $lex{Results}</td>\n";
    print qq{<td><input type="checkbox" name="showgender" value="1" checked="checked"></td></tr>\n};

    print "<tr><td class=\"bra\">$lex{Show} $lex{Raw} $lex{Results}</td>\n";
    print qq{<td><input type="checkbox" name="showraw" value="1" checked="checked"></td></tr>\n};

    print "<tr><td class=\"bra\">$lex{Show} $lex{Percentage} $lex{Results}</td>\n";
    print qq{<td><input type="checkbox" name="showpercent" value="1" checked="checked"></td></tr>\n};

    print "<tr><td class=\"bra\">$lex{Show} $lex{Division}</td>\n";
    print qq{<td><input type="checkbox" name="showdivision" value="1" checked="checked"></td></tr>\n};
=cut


=head
    print "<tr><td class=\"ra\">". $lex{Group}. "</td>\n<td>";
    print "<select name=\"groupname\"><option>$lex{Band}</option>\n";
    print "<option>$lex{Grade}</option><option>$lex{Homeroom}</option>\n";;
    print "</select>\n <input type=\"text\" name=\"groupvalue\" size=\"12\"> ";
    print $lex{'Separate with Spaces'}. "</td></tr>\n";

    # Paper Size
    $defaultpapersize =~ s/paper//; # strip out word paper so lex works; from admin.conf
    my $defpaper = ucfirst( $defaultpapersize );

    print "<tr><td align=\"right\">". $lex{'Paper Size'}. "</td>\n<td><b>";
    print $lex{Legal}. "</b></td></tr>\n";
    print "<select name=\"papersize\">";
    if ( $lex{$defpaper} ) { print "<option>". $lex{$defpaper}. "</option>\n"; }
    my @sizes = qw(Letter A4 Legal);
    foreach my $size ( @sizes ) {
	if ( $size eq $defpaper ) { next; }
	print "<option>$lex{$size}</option>";
    }
    print "</select></td></tr>\n ";

    print "<tr><td align=\"right\">". $lex{'Font Size'}. "</td>\n<td>";
    print "<select name=\"fontsize\"><option>10pt</option><option>11pt</option>\n";
    print "<option>12pt</option></select></td></tr>\n ";

    print "<tr><td class=\"ra\">". $lex{'Records per Page'}. "</td>\n<td>";
    print "<input type=\"text\" name=\"maxrecords\" size=\"4\" value=\"$defmaxrecords\"></td></tr>\n";

=cut

    print qq{<tr><td class="ra"><input type="submit" value="$lex{Continue}"></td></tr>\n};
    print qq{</table></form>\n};

=head1
    print "<script type=\"text/javascript\">\n";
    print "Calendar.setup({ \n";
    print "inputField  : \"monthatt_date\", \n";
    print "ifFormat    : \"%Y-%m-%d\", \n";
    print "button      : \"monthatt_trigger\", \n";
    print "singleClick : false, \n";
    print "step : 1 \n";
    print "});</script>\n";

=cut

    print "</body></html>\n";

    exit;

}

#----------------                                                                                                    
sub selectStudents {
#----------------                                                                                                    

    # foreach my $key ( sort keys %arr ) { print "K:$key V:$arr{$key}<br>\n"; }

    my $studenttable = 'student';
    if ( $arr{showwithdrawn} ) {
	$studenttable = 'studentall';
    }
    delete $arr{showwithdrawn};


    my $checked;
    if ( $arr{checknextpage} ) {
	$checked = qq{checked="checked"};
	delete $arr{checknextpage};
    }

    my $select;
    if ( $arr{grade} ) { # we're picking a grade
	$select = 'where grade = ?';
	print qq{<div style="font-size:120%;font-weight:bold;">$lex{Grade} $arr{grade}</div>\n};
    }


    my $sth = $dbh->prepare("select lastname, firstname, studnum from $studenttable
      $select order by lastname, firstname");
    if ( $select ) {
        $sth->execute( $arr{grade} );
    } else {
        $sth->execute;
    }
    if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr; }

   # Form Header                                                                                                    
    print "<form action=\"$self\" method=\"post\"> \n";
    print qq{<input type="hidden" name="page" value="2">\n};
    print qq{<input type="hidden" name="db" value="$arr{db}">\n};
    print qq{<input type="hidden" name="maxrecords" value="$arr{maxrecords}">\n};

    print "<table cellpadding=\"3\" cellspacing=\"0\" border=\"0\">\n";

    print "<tr><td class=\"la\"><input type=\"submit\" value=\"$lex{Continue}\">";
    print "</td></tr>\n";


    while ( my ( $lastname, $firstname, $studnum ) = $sth->fetchrow ) {

        print "<tr><td class=\"la\">";
        print "<input type=\"checkbox\" name=\"$studnum\" value=\"1\" $checked> ";
	print "<b>$lastname</b>, $firstname ($studnum)</td></tr>\n";

    }

    print "<tr><td class=\"la\"><input type=\"submit\" value=\"$lex{Continue}\">";
    print "</td></tr>\n";

    print "</table></form></body></html>\n";

    exit;

} # end of selectStudents                                                                                            



#------------
sub reportPDF {
#------------

    # foreach my $key ( sort keys %arr ) { print "K:$key V:$arr{$key}<br>\n"; }


    delete $arr{db}; # not needed
    my $maxrecords = $arr{maxrecords};
    delete $arr{maxrecords};
    # only remaining values in %arr hash will be student numbers.

    # Check Remote database
    if ( not $dbh ) {
	print qq{<h3>$lex{Database} $lex{'Not Found'}</h3>\n};
	print qq{</body></html>\n};
	exit;
    }

    # Get student numbers into name sorted order.
    my %sorted;
    my $sth = $dbh->prepare("select lastname, firstname from studentall where studnum = ?");
    foreach my $studnum ( keys %arr ) {
	$sth->execute($studnum);
	if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
	my ($ln, $fn) = $sth->fetchrow;
	$sorted{"$ln$fn$studnum"} = $studnum;
    }
    # %sorted contains sorting key and student number value.



    # Load Configuration Variables: Admin
    my $sth = $dbh->prepare("select id, datavalue from conf_system where filename = 'admin'");
    $sth->execute;
    if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
    while ( my ($id, $datavalue) = $sth->fetchrow ) {
	eval $datavalue;
	if ( $@ ) {
	    print "$lex{Error}: $@<br>\n";
	    die "$lex{Error}: $@\n";
	}
    }

    print "<h3>School Start:$schoolstart | \n";
    print "School End:$schoolend</h3>\n";


    my $enddate = $schoolend;
    my $startdate = $schoolstart;

    # School Days Open
    my %schooldays = mkSchoolDays( $startdate, $enddate, $dbh );
    # returns hash of schooldays in month. key is yyyy-mm and value is schooldays in month


    my ($logfile, $shortname, $filename);
    if ( $pdf ) { # Open TEX file, and print Header
	$logfile = "pdflog$$.txt";
	$shortname = "aandcrtp1$$";
	$filename = "$shortname.tex";
	open(TEX,">$filename") || die "Can't open tex file";
	print_tex_doc_start(); # Setup the start of the file.
    }


    my $sth = $dbh->prepare("select * from studentall where studnum = ?");
    my $sth5 = $dbh->prepare("select studnum from student where studnum = ?");

    my $oldroom;
    my $curroom = -1; # Watch! Not a value entered for a reason...hopefully.
    my $firstflag = 1;  # Flag to watch for first "real" person (not withdrawn)

    foreach my $key ( sort keys %sorted ) {

	my $studnum = $sorted{$key};
	
	# Get student info
	$sth->execute( $studnum );
	if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }
	my $sref = $sth->fetchrow_hashref;

	# LaTeX filter the student record
	foreach my $key ( keys %$sref ) {
	    ( $sref->{$key} ) = latex_filter( $sref->{$key} );
	}

	my %sr = %$sref;

	my $studentname = "<b>$sr{lastname}</b>, $sr{firstname} $sr{initial}";

#	print qq{<div>$studentname ($studnum) $sr{grade}</div>\n};

	# Now let's find which table student from: student or studentwd
	$sth5->execute( $studnum );
	if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }
	my $active = $sth5->rows;  # returns 1 if in student, 0 if not.
	if ( not $active and $html){ 
	    $studentname = "<span style=\"color:red;\">$studentname</span>";
	}

	$oldroom = $curroom;
	$curroom = $sr{grade};

	if ( $curroom ne $oldroom ) { # New Page Heading (band or group changes)

	    if ( not $firstflag ) {
		print TEX "\\hfil{\\bf Attended/Enrolled}\\hfil\n";
		$linecount = 0; 
		print TEX "\\newpage\n\n";
	    } 
	    $firstflag = 0;

	} # End of Header printing section


	# Print this student's record
	printStudent( $sref, $dbh, \%schooldays );

	$linecount++;
	if ( $linecount >= $maxrecords ){ # New page.
	    print TEX "\\newpage\n\n";
	    $linecount = 0;
	}

    } # End of main loop


    print TEX "% print_tex_end here\n";
    print TEX "\\end{document}\n";
    close TEX;

    system("$pdflatex $filename > $logfile");
    system("mv $shortname.pdf $defaultdownloaddir");
    system("mv $logfile $defaultdownloaddir");
    system("rm -f $shortname.*");
 

    print qq{<div style="font-size:130%;font-weight:bold;">[ <a href="$webdownloaddir/$shortname.pdf">};
    print qq{$lex{'View/Download'} AANDC $lex{'Attendance Report'}</a> ]</div>\n};

    print qq{<p style="font-size:120%;">[ <a href="$homepage">$lex{Main}</a> |\n};
    print qq{<a href=\"$webdownloaddir/$logfile\">$lex{'View Log File'}</a> ]</p>\n};
    print qq{</body></html>\n};

    exit;

} end of reportPDF;


#----------------------
sub print_tex_doc_start {
#----------------------

    print TEX "\\documentclass[10pt,legalpaper,oneside,landscape]{article}
\\usepackage{array,colortbl,rotating,inputenc}
$a_latex_header
\\renewcommand{\\familydefault}{\\sfdefault}
\\pagestyle{empty}
\\setlength{\\textwidth}{13in}
\\setlength{\\textheight}{7.7in}
\\setlength{\\hoffset}{-4.3in}
\\setlength{\\voffset}{-0.8in}
\\setlength{\\headsep}{10pt}
\\setlength{\\headheight}{14pt}
\\setlength{\\topmargin}{0pt}
\\setlength{\\parindent}{0pt}
\\setlength{\\tabcolsep}{5pt}
\\setlength{\\extrarowheight}{3pt}
\\pagestyle{headings}
\\markright{$schoolname - $schoolyear \\hfill 
{\\bf AANDC $lex{'Attendance Report'} 1} \\hfill $currdate -- Pg }
\n\n
\\begin{document}\n";
} # End of Print Tex Doc Start



#-------------------
sub print_tex_header {
#-------------------
    # Up to 18 Cols: (8 + 10 months)
    # Grade, Lastname, Firstname, Middlename, Birthdate
    print TEX "\\begin{tabular}{|p{1cm}|p{3cm}|p{1.6cm}|p{2cm}|p{1.8cm}";
    # Treaty, Band, Parent, Up to 10 Months.
    print TEX "|p{1.7cm}|p{2.2cm}|p{3.2cm}|";

    foreach my $yrmo ( sort keys %schooldays ){
	print TEX "p{9mm}|";
    }
    print TEX "}\\hline\n\n";

    # Now print the column header line.
    print TEX "\\rowcolor[gray]{0.90}{\\bf Grade}";
    print TEX "& {\\bf ". $lex{'Last Name'}. "} & {\\bf ". $lex{First}. "} &";
    print TEX "{\\bf ". $lex{Middle}. "} &{\\bf ". $lex{Birthdate}. "} &";
    print TEX "{\\bf IRS\\#} "; 
    # &{\\bf $lex{Band}} & \n";
    # print TEX "{\\bf ". $lex{Mother}. "}";

    foreach my $yrmo ( sort keys %schooldays ){
	my ($ty,$tm) = split(/-/,$yrmo);
	print TEX "& {\\bf $s_month[$tm]}";
    }
    print TEX "\\\\ \\hline\n\n";


    # Now print the schooldays in month
    print TEX "\\rowcolor[gray]{0.90}";
    print TEX "& & & & &". $lex{'School Days'};

    foreach my $yrmo ( sort keys %schooldays ){
	print TEX "& {\\bf $schooldays{$yrmo}}";
    }
    print TEX "\\\\ \\hline\n\n";

}



#---------------
sub printStudent {  # Calc attendance, then print record(s)
#---------------

    my ($sref, $dbh, $schooldaysref ) = @_;
    my %sr = %$sref;
    my %schooldays = %$schooldaysref;

    my $studnum = $sr{studnum};

    my ( %absent, %enrol );
    my $periodsperday = $g_ppd{ $sr{grade} };

    if ( not $periodsperday ){
	if ( not $sr{grade} ){ $sr{grade} = $lex{'Not Found'}; }
	print "<h1>$lex{'Periods Per Day'} $lex{'Not Defined'}<br>";
	print "$lex{Grade}:$sr{grade} $lex{Student}:$studentname</h1>\n";
	print "</body></html>\n";
	exit;
    }



    # Start the table
    print TEX "\\begin{tabular}{|p{30mm}|p{20mm}|p{20mm}|p{20mm}|p{80mm}|}\n\n";

    print TEX "\\multicolumn{3}{l}{\\Large\\bf $sr{lastname}, $sr{firstname} $sr{initial} }\n";
    print TEX "& \\multicolumn{2}{l}{ DOB:$sr{birthdate}, IRS\\#:$sr{treaty})}\n\n";
    print TEX "\\\\ \\hline\n\n";

    # Now print the column header line.
    print TEX "\\rowcolor[gray]{0.90}{\\bf $lex{Month}}";
    print TEX "& {\\bf Teaching} & {\\bf Enrol} & {\\bf Present} &\\bf Absent (Reasons) ";
    print TEX "\\\\ \\hline\n\n";


    my $sth = $dbh->prepare("select absdate, reason from attend where studentid = ? and
      extract(year_month from absdate) = ? order by absdate,period");


    # calculate attendance, one for each month of this school year.
    my %absent;
    foreach my $yrmo ( sort keys %schooldays ) {
#	print "SN:$studnum YrMo:$yrmo<br>\n";

	# yearmonth will have yyyy-mm format
	my $tempyrmo = $yrmo;
	$tempyrmo =~ s/-//; # remove single dash                                                             
	$sth->execute( $studnum, $tempyrmo );
	if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }

	#$absrec = calcMonthlyAttendance( $studnum, $yrmo, $periodsperday,
	# $enddate, \%lexi, $dbh );

	while ( my ($absdate, $reason) = $sth->fetchrow ) {
	    if ( $reason =~ m/.*$absentString.*/ ) {
		 $absent{$yrmo}{$reason}++;
	    }
	}


	# Print Data Structure for this month.
#	foreach my $ym ( sort keys %absent ) {
#	    foreach my $key ( sort keys %{ $absent{$ym} } ) {
#		print "YM:$ym  RSN:$key  VAL: $absent{$ym}{$key}<br>\n";
#	    }
#	}

    }


    # Convert data structure from periods into days using $periodsperday value
    foreach my $ym ( sort keys %absent ) {

#	print "$ym - $sr{firstname} $sr{lastname} ($studnum) PPD:$periodsperday<br>\n";
	
	foreach my $reason ( sort keys %{ $absent{$ym} } ) {
#	    print "$ym: $reason - Count:$absent{$ym}{$reason} ||\n";
	    $absent{$ym}{$reason} =  format_number( $absent{$ym}{$reason} / $periodsperday, 2 );
#	    print "Days:$absent{$ym}{$reason}<br>\n";
	}
#	print "<br>\n";

    }

    
    # calculate enrollment, one for each month
    my $ymref = calcMonthlyEnrollment( $studnum, $schoolstart, $currsdate, $dbh );
    %enrol = %$ymref;  # yyyy-mm -> start,end,days

    # just used to set zero values.
    foreach my $yrmo ( sort keys %schooldays ) { 
	$enrol{ $yrmo }{'days'} = format_number( $enrol{ $yrmo }{'days'}, 1);
    }
   
    # Total Absences for each month
    my %absenttotal;
    foreach my $ym ( sort keys %absent ) {
	my $total;
	foreach my $key ( sort keys %{ $absent{$ym} } ) {
	    $total += $absent{$ym}{$key};
	}
	$absenttotal{$ym} = $total;
    }


    # Now print the rows, one for each month
    foreach my $yrmo ( sort keys %schooldays ) { 

	my $present = 0;
	if ( $enrol{$yrmo}{'days'} > 0  ) {
	    $present = $schooldays{$yrmo} - $absenttotal{$yrmo};
	}

	my $presentpercent = 0;
	if ( $schooldays{$yrmo} and $enrol{$yrmo}{'days'} > 0 ) {
	    $presentpercent = format_number( $present / $schooldays{$yrmo}, 3) * 100;
	}
	$presentpercent .= '\%';  # escaped percent symbol

	print TEX "\\hfil $yrmo & $schooldays{$yrmo} & $enrol{$yrmo}{'days'} & $present ($presentpercent) &";
	foreach my $key ( sort keys %{ $absent{$yrmo} } ) {
	    print TEX "$key - $absent{$yrmo}{$key}\n";
	}
	print TEX " \\\\ \\hline\n";
    }

    print TEX "\\end{tabular}\\bigskip\n\n";

    # Now print the Enrollment records.
    # Start the table
    print TEX "\\begin{tabular}{|l|l|}\n\n";

    print TEX "\\multicolumn{2}{l}{\\large\\bf $lex{Enrollment}}\n";
    print TEX "\\\\ \\hline\n\n";

    # Now print the column header line.
    print TEX "\\rowcolor[gray]{0.90}{\\bf $lex{Date}}";
    print TEX "& {\\bf $lex{Type}} ";
    print TEX "\\\\ \\hline\n\n";

    # print the enrollment records, reverse order.
    my $sth = $dbh->prepare("select * from transfer where studnum = ? order by date");
    $sth->execute( $studnum );
    if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }

    while ( my $ref = $sth->fetchrow_hashref ) {
	my %t = %$ref;

	print TEX qq{$t{date} & $t{type} };
	print TEX qq{ \\\\ \\hline\n};
    }

    print TEX "\\end{tabular}\\bigskip\n\n";

} # end of printStudent

