#!/usr/bin/perl
#  Copyright 2001-2014 Leslie Richardson

#  This file is part of Open Admin for Schools.

# Read Report - read all reading tests from all schools, calc the
# Equiv Grade and then compare to the actual grade and get a
# delta. Compile into per grade, per school and overall stats.


my %lex = ('Error' => 'Error',
	   'Main' => 'Main',
	   'Start Date' => 'Start Date',
	   'End Date' => 'End Date',
	   'Date' => 'Date',
	   'Continue' => 'Continue',
	   'Schools' => 'Schools',
	   'Database' => 'Database',
	   'Reading' => 'Reading',
	   'Report' => 'Report',
	   'Global' => 'Global',
	   'Select' => 'Select',
	   'Split' => 'Split',
	   'Grade' => 'Grade',
	   'Current' => 'Current',
	   'Output' => 'Output',
	   'Combine failed on input' => 'Combine failed on input',
	   'Download CSV File' => 'Download CSV File',
	   'Test' => 'Test',
	   'Group' => 'Group',
	   'Division' => 'Division',
	   'Paper Size' => 'Paper Size',
	   'Letter' => 'Letter',
	   'Legal' => 'Legal',
	   'A4' => 'A4',

	   'Check' => 'Check',
	   'Duplicates' => 'Duplicates',

	   );


my $self = 'checkIRSDuplicate.pl';


use DBI;
use CGI;
use Number::Format qw(:all);
use Time::JulianDay;

eval require "admin.conf";
if ( $@ ) {
    print $lex{Error}. ": $@<br>\n";
    die $lex{Error}. ": $@\n";
}

eval { require "$globdir/global.conf"; };
if ( $@ ) {
    print $lex{Error}. " $self: $@<br>\n";
    die $lex{Error}. "$self: $@\n";
}

@dbase = sort keys %alldbase; # global records.


my @time = localtime(time);
my $year = $time[5] + 1900;
my $month = $time[4] + 1;
my $currdate = "$year-$month-$time[3]";
my $schyear = $year;
if ( $month < 7 ){ $schyear = $schyear - 1; }
my $prevyear = $schyear - 1;

my $currstartdate = "$schyear-08-01"; # Aug 1 of this school year
my $prevstartdate = "$prevyear-08-01"; # August 1 of prev year.


my $q = CGI->new;
my %arr = $q->Vars;
print $q->header( -charset, $charset );


# Script Heading Descriptor
my $title = "$lex{Check} IRS $lex{Duplicates}";

# Print Page Heading
print "$doctype\n<html><head><title>$title</title>\n";
print "<link rel=\"stylesheet\" href=\"/division.css\" type=\"text/css\">\n";

=head
if ( not $arr{page} ) { # calendar popup.
    print "<link rel=\"stylesheet\" type=\"text/css\" media=\"all\" ";
    print "href=\"/js/calendar-blue.css\" title=\"blue\">\n";
    print "<script type=\"text/javascript\" src=\"/js/calendar.js\"></script>\n";
    print "<script type=\"text/javascript\" src=\"/js/lang/calendar-en.js\"></script>\n";
    print "<script type=\"text/javascript\" src=\"/js/calendar-setup.js\"></script>\n";
}
=cut

print "$chartype\n</head><body style=\"padding:0.4em 2em;\">\n";
print "[ <a class=\"alt\" href=\"$homepage\">$lex{Main}</a> ]\n";
print "<h1>$title</h1>\n";

if ( not $arr{page} ) {
    showStartPage();

} elsif ( $arr{page} == 1 ) {
    delete $arr{page};
    checkDuplicates();

}



#------------------
sub checkDuplicates {
#------------------

    # foreach my $key ( sort keys %arr ) { print "K:$key V:$arr{$key}<br>\n"; }

    my $studtable = 'student';
    if ( $arr{showall} ) {
	$studtable = 'studentall';
    }
    delete $arr{showall};

    my $showmissing = $arr{showmissing};
    delete $arr{showmissing};


    foreach my $db ( sort keys %arr ) {

	my $database = $db;
	my $dsn = "DBI:$dbtype:dbname=$database";
	my $dbh = DBI->connect($dsn,$guser,$gpassword);
	my $schoolname = $alldbase{"$db"};

	print "<h2>$schoolname</h2>\n";

	# Get all students who have tests into a @students array (may not be current)
	my $sth1 = $dbh->prepare("select lastname, firstname from studentall where studnum = ?");

	my $first = 1;
	my %irsnumbers = ();
	my $sth = $dbh->prepare("select treaty, studnum from $studtable
          where treaty is not NULL and treaty != '' order by treaty");
	$sth->execute;
	if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }

	while ( my ($irsnum, $studnum) = $sth->fetchrow ) {
	    if ( $irsnumbers{$irsnum} ) { # duplicate
		print "<p>Duplicate Error: $irsnum<br>";

		$sth1->execute($irsnumbers{$irsnum}) ;
		if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
		my ($lname, $fname) = $sth1->fetchrow;
		print "Original: $fname $lname ($irsnumbers{$irsnum})<br>";

		$sth1->execute($studnum) ;
		if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
		($lname, $fname) = $sth1->fetchrow;

		print "Duplicate: $fname $lname ($studnum)</p>";
		$first = 0;
	    } 
	    $irsnumbers{$irsnum} = $studnum;

	}
	if ( $first ) {
	    print "<p>No Duplicates Found!</p>\n";
	}

	if ( $showmissing ) {
	    print "<h3>Missing IRS Numbers (Current Students)</h3>\n";

	    # Check for students without IRS numbers
	    my $sth = $dbh->prepare("select lastname, firstname, studnum from student
              where treaty is NULL or treaty = '' order by lastname, firstname");
	    $sth->execute;
	    if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
	    while ( my ( $lastname, $firstname, $studnum ) = $sth->fetchrow ) {
		print "<div><b>$lastname</b>, $firstname ($studnum)</div>\n";
	    }
	}


    } # next db.


    exit;
}



#----------------
sub showStartPage {
#----------------

    print "<form action=\"$self\" method=\"post\">\n";
    print "<input type=\"hidden\" name=\"page\" value=\"1\">\n";

    print "<table cellpadding=\"3\" cellspacing=\"0\" border=\"0\" ";
    print "style=\"padding:0.5em;border:1px solid gray;\">\n";

    print "<tr><td class=\"bla\">$lex{Select} $lex{Schools}</td><td></td></tr>\n";
    foreach my $db ( @dbase ) { 
	print "<tr><td class=\"la\">";
	print "<input type=\"checkbox\" name=\"$db\" value=\"1\" checked=\"checked\">\n";
	print "$alldbase{$db} ($db)</td></tr>\n";
    }

=head
    # Start Date
    print "<tr><td class=\"bra\">$lex{'Start Date'}</td><td class=\"la\"><input type=\"text\" ";
    print "name=\"startdate\" id=\"sdate\" size=\"10\" value=\"$currstartdate\">";
    print "<button type=\"reset\" id=\"start_trigger\">...</button>\n";
    print "</td></tr>\n";


    # End Date
    print "<tr><td class=\"bra\">$lex{'End Date'}</td><td class=\"la\"><input type=\"text\" ";
    print "name=\"enddate\" id=\"edate\" size=\"10\" value=\"$currdate\">";
    print "<button type=\"reset\" id=\"end_trigger\">...</button></td></tr>\n";

=cut
    # Check all students
    print "<tr><td class=\"bla\" colspan=\"2\">Check Withdrawn? ";
    print "<input type=\"checkbox\" name=\"showall\" value=\"1\" checked=\"ehecked\"></td></tr>\n";

    print "<tr><td class=\"bla\" colspan=\"2\">Show Missing? ";
    print "<input type=\"checkbox\" name=\"showmissing\" value=\"1\"></td></tr>\n";

=head
    # Output Type
    print "<tr><td class=\"bra\">$lex{Output}</td><td class=\"la\">";
    print "<select name=\"output\"><option value=\"html\">HTML</option>\n";
    print "<option value=\"pdf\">PDF</option>";
    print "<option value=\"csv\">CSV</option>";
    print "</select></td></tr>\n";

    # Paper Size, if doing PDF.
    print "<tr><td class=\"bra\">$lex{'Paper Size'}</td>\n<td>";
    print "<select name=\"papersize\"><option value=\"letter\">$lex{Letter}</option>\n";
    print "<option value=\"a4\">$lex{A4}</option><option value=\"legal\">$lex{Legal}</option>";
    print "</select></td></tr>\n ";
=cut

    print "<tr><td class=\"la\" colspan=\"2\"><input type=\"submit\" value=\"$lex{Continue}\"></td></tr>\n";
    print "</form>\n";

=head
    print "<script type=\"text/javascript\">
     Calendar.setup({
        inputField     :    \"sdate\", 
        ifFormat       :    \"%Y-%m-%d\",
        button         :    \"start_trigger\",
        singleClick    :    false,
        step           :    1
    });

    Calendar.setup({
        inputField     :    \"edate\",
        ifFormat       :    \"%Y-%m-%d\",
        button         :    \"end_trigger\",
        singleClick    :    false,
        step           :    1
    });
    </script>\n";
=cut

    print "</body></html>\n";

    exit;

} # end of showStartPage


