#!/usr/bin/perl
#  Copyright 2001-2014 Leslie Richardson

#  This file is part of Open Admin for Schools.

#  Open Admin for Schools is free software; you can redistribute it 
#  and/or modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2 of 
#  the License, or (at your option) any later version.

use DBI;
use CGI;
use Time::JulianDay;

my %lex = ('Missing' => 'Missing',
	   'Main' => 'Main',
	   'Check' => 'Check',
	   'Provincial Number' => 'Provincial Number',
	   'Duplicate' => 'Duplicate',
	   'Grade' => 'Grade',
	   'Homeroom' => 'Homeroom',
	   'Continue' => 'Continue',
	   'Check All' => 'Check All',
	   'Schools' => 'Schools',

	   );


my $self = 'checkprovnum.pl';


# Read Config
eval require "admin.conf";
if ( $@ ) {
    print $lex{Error}. ": $@<br>\n";
    die $lex{Error}. ": $@\n";
}


my @dbase = sort keys %alldbase; # global records.


my $q = new CGI;
print $q->header( -charset, $charset ); 
my %arr = $q->Vars;


my $title = "$lex{Check} $lex{'Provincial Number'}";
print "$doctype\n<html><head><title>$title</title>
<link rel=\"stylesheet\" href=\"$css\" type=\"text/css\">
$chartype\n</head><body style=\"padding:1em;\">\n";

print "[ <a href=\"$homepage\">$lex{Main}</a> ]\n";

print "<h1>$title</h1>\n";


if ( not $arr{page} ) {
    showStartPage();

} else {
    delete $arr{page};
}



foreach my $dbase ( sort keys %arr ) {

    print qq{<h2>$alldbase{$dbase}</h2>\n};
    print qq{<div>\n}; # school section


    my $dsn = "DBI:$dbtype:dbname=$dbase";
    my $dbh = DBI->connect($dsn,$guser,$gpassword);
    $dbh->{mysql_enable_utf8} = 1;

    my $sth = $dbh->prepare("select studnum, lastname, firstname, provnum from studentall
      where (provnum != '' and provnum is not NULL) order by provnum");
    $sth->execute;
    if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr;}

    my $currstud = '-1';
    my $currname;

    my $first = 1;


    while ( $ref = $sth->fetchrow_hashref ) {
	my %r = %$ref;

	$oldstud = $currstud;
	$currstud = $r{provnum};
	$oldname = $currname;
	$currname = "$r{lastname}, $r{firstname} ($r{studnum})";


	if ($currstud == $oldstud) { # Ack! We have a duplicate!

	    if ( $first ) {
		print qq{<table cellpadding="3" cellspacing="0" border="1" style="margin:1em;">\n};
		print qq{<caption style="font-size:130%;font-weight:bold;">};
		print qq{$lex{Duplicate} $lex{'Provincial Number'}/Parity Error</caption>\n};
#		print qq{<tr><th>$lex{Student}</th></tr>\n};
		$first = 0;
	    }

	    print qq{<tr><td>Duplicate for student number: $currstud</td>\n};
	    print qq{<td>$currname and $oldname are duplicated!</td></tr>\n};

	}

	# Check Current Provnum for parity
	my $temppn = $r{provnum};
	my $tempcd = chop( $temppn );  
	my $checkdigit = &checkprovnum( $temppn );
	if ( $checkdigit != $tempcd ) {

	    if ( $first ) {
		print qq{<table cellpadding="3" cellspacing="0" border="1" style="margin:1em;">\n};
		print qq{<caption style="font-size:130%;font-weight:bold;">};
		print qq{$lex{Duplicate} $lex{'Provincial Number'}/Parity Error</caption>\n};

		$first = 0;
	    }

	    print qq{<tr><td>Parity Error: $currname</td><td>Provnum:$r{provnum}</td></tr>\n};

	    #print "<td><form action=\"$cgiurl/studed.pl\" method=\"post\" style=\"display:inline;\">\n";
	    #print "<input type=\"hidden\" name=\"id\" value=\"$studid\">\n";
	    #print "<input type=\"submit\" name=\"submit\" value=\"Edit\"></form>\n";
	}
    }

    print qq{</table>\n};


    # Now check for missing provincial numbers
    my $sth = $dbh->prepare("select studnum, lastname, firstname, grade, homeroom from student
      where provnum = '' or provnum is NULL order by grade, lastname, firstname");
    $sth->execute;
    if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr;}

    my $first = 1;

    while ( $ref = $sth->fetchrow_hashref ) {
	my %r = %$ref;

	if ( $first ) {
	    print qq{<table cellpadding="3" cellspacing="0" border="1" style="margin:1em;">\n};
	    print qq{<caption style="font-size:130%;font-weight:bold;">};
	    print qq{$lex{Missing} $lex{'Provincial Number'}</caption>\n};
	    $first = 0;
	}

	print qq{<tr><td class="la">$r{lastname}, $r{firstname} ($r{studnum})</td>};
	print qq{<td>$lex{Grade} $r{grade} / $lex{Homeroom} $r{homeroom}</tr>\n};

    }

    print qq{</table>\n};

}

print qq{</body></html>\n};


#----------------
sub showStartPage { # Entry Values for Custom Script
#----------------

    my $checked;
    if ( $arr{checked} ) {
	$checked = qq{checked="checked"};
    }

    print qq{<form action="$self" method="post" style="display:inline;margin:1em;">\n};
    print qq{<input type="hidden" name="checked" value="1">\n};
    print qq{<input type="submit" value="$lex{'Check All'} $lex{Schools}" style="display:inline;"></form>\n};


    print "<form action=\"$self\" method=\"post\">\n";
    print "<input type=\"hidden\" name=\"page\" value=\"1\">\n";

    print qq{<h3>$lex{Select} $lex{Schools}</h3>\n};
    print "<table cellpadding=\"3\" cellspacing=\"0\" border=\"0\">\n";
    
    foreach my $db ( @dbase ) { 

	print qq{<tr><td class="la">};
	print qq{<input type="checkbox" name="$db" value="1" $checked></td>\n};
	print qq{<td class="la">$alldbase{$db} ($db)</td></tr>\n};
    }

    print qq{<tr><td class="la" colspan="2"><input type="submit" value="$lex{Continue}"></td></tr>\n};
    print qq{</table>\n};
    print qq{</form></body></html>\n};

    exit;

}


#----------------
sub checkprovnum {
#----------------

    my @pn = split(//,$_[0]);
    for ( my $i=1; $i<=8; $i += 2) {
	$pn[$i] *= 2;
	if ($pn[$i] > 9) {
	    my @tmp = split(//,$pn[$i]);
	    $pn[$i] = $tmp[0] + $tmp[1];
        }
    }
    
    my $tempnum = 0;
    foreach my $pnum (@pn) {
	$tempnum += $pnum;
    }
    @pn = split(//,$tempnum);   
    my $retval= pop(@pn);
    $retval = 9 - $retval;
    return $retval;

}


