#!/usr/bin/perl
#  Copyright 2001-2014 Leslie Richardson

#  This file is part of Open Admin for Schools.

#  Open Admin for Schools is free software; you can redistribute it 
#  and/or modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2 of 
#  the License, or (at your option) any later version.

use DBI;
use CGI;
use Time::JulianDay;

my %lex = ('View' => 'View',
	   'Main' => 'Main',
	   'Dates' => 'Dates',
	   'School Year' => 'School Year',
	   'School Start' => 'School Start',
	   'School End' => 'School End',
	   'Term' => 'Term',
	   'Date' => 'Date',
	   'Type' => 'Type',
	   'Description' => 'Description',
	   'Track' => 'Track',
	   'Error' => 'Error',
	   'Day' => 'Day',
	   'Grade' => 'Grade',
	   'Default' => 'Default',
	   'Fraction' => 'Fraction',
	   'Cycle' => 'Cycle',
	   'Day' => 'Day',
	   'No' => 'No',
	   'Yes' => 'Yes',
	   'Closed' => 'Closed',
	   'School Year' => 'School Year',
	   'Continue' => 'Continue',
	   'Closed' => 'Closed',
	   'Open' => 'Open',
	   'Start' => 'Start',
	   'End' => 'End',
	   'Select' => 'Select',
	   'Schools' => 'Schools',
	   'Total' => 'Total',
	   'Check All' => 'Check All',

	   );

my %dowclosed = ('0' => 1, '6' => 1); # Day 0 is Sunday, Day 6 is Saturday

# Read Config
eval require "admin.conf";
if ( $@ ) {
    print $lex{Error}. ": $@<br>\n";
    die $lex{Error}. ": $@\n";
}


my @dbase = sort keys %alldbase; # global records.


my $q = new CGI;
print $q->header( -charset, $charset ); 
my %arr = $q->Vars;


my $title = "$lex{View} $lex{'School Year'}";
print "$doctype\n<html><head><title>$title</title>
<link rel=\"stylesheet\" href=\"$css\" type=\"text/css\">
$chartype\n</head><body style=\"padding:1em;\">\n";

print "[ <a href=\"$homepage\">$lex{Main}</a> ]\n";

print "<h1>$title</h1>\n";

if ( not $arr{page} ) {
    showStartPage();

} else {
    delete $arr{page};
}


foreach my $dbase ( sort keys %arr ) {

    print qq{<h2>$alldbase{$dbase}</h2>\n};
    print qq{<div>\n}; # school section


    my $dsn = "DBI:$dbtype:dbname=$dbase";
    my $dbh = DBI->connect($dsn,$guser,$gpassword);
    $dbh->{mysql_enable_utf8} = 1;

    # get school year info from configuration system
    my $sth = $dbh->prepare("select datavalue from conf_system where sectionname = 'schoolyear'");
    $sth->execute;
    while ( my $datavalue = $sth->fetchrow ) {
	eval $datavalue;
	if ( $@ ) {
	    print $lex{Error}. " $@<br>\n";
	    die $lex{Error}. " $@\n";
	}
    }


    # populate jdclosed with dates from dates table; used by calcTermDays;
    my %jdclosed;
    my $sth = $dbh->prepare("select date, dayfraction from dates 
      where date is not NULL and date != ''");
    $sth->execute;
    if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }

    while ( my ( $date, $dayfraction ) = $sth->fetchrow ) {
	my $jd = julian_day( split( /-/, $date ) );
	$jdclosed{$jd} = $dayfraction;
    }



    # Print Term Dates stored in configuration system.
    print qq{<div style="float:left;">\n};  # left side div

    print "<table cellpadding=\"5\" border=\"0\" cellspacing=\"0\">\n";
    print "<tr><td class=\"ra\">$lex{'School Year'}</td><td class=\"bla\">$schoolyear</td></tr>\n";
    print "<tr><td class=\"ra\">$lex{'School Start'}</td><td class=\"bla\">$schoolstart</td></tr>\n";
    print "<tr><td class=\"ra\">$lex{'School End'}</td><td class=\"bla\">$schoolend</td></tr>\n";
    print "</table><p></p>\n";


    # Print Terms 
    if ( $g_MultiTrack ) { # Print Multi Track terms.
	my $deftrk; # variable to store default track marker.(**)

	# Loop through all tracks.
	foreach my $trk ( sort keys %g_MTrackTerm ) {

	    if ( $trk == $g_DefaultTrack ) { 
		$deftrk = '<span style="color:red;">**</span>'; 
	    } else { $deftrk = ''; }

	    print qq{<table cellpadding="3" border="0" cellspacing="0" };
	    print qq{style="border:1px solid gray;padding:0.5em;">\n};
	    
	    print "<tr><td colspan=\"3\" class=\"bcn\">$lex{Track} $trk $deftrk - $g_TrackDisplay{$trk}</td></tr>\n";
	    print "<tr><td colspan=\"3\" class=\"bcn\">$lex{Grade} ";
	    foreach my $grade ( sort {$a <=> $b} keys %g_MTrackTermType ) {
		if ( $g_MTrackTermType{$grade} eq $trk ) { print "$grade "; }
	    }
	    print "</td></tr>\n";


	    # Print Terms in this Track
	    my $totaldays;
	    print qq{<tr><th></th><th>$lex{Start}</th><th>$lex{End}</th><th>$lex{Open}</th><th>$lex{Closed}</th></tr>\n};
	    foreach my $trm ( sort keys %{ $g_MTrackTerm{$trk} } ) {
		print "<tr><td class=\"bla\">$lex{Term} $trm ($g_TermDisplay{$trk}{$trm})</td>";
		print "<td class=\"la\">$g_MTrackTerm{$trk}{$trm}{start}</td>";
		print "<td class=\"la\">$g_MTrackTerm{$trk}{$trm}{end}</td>\n";

		my $val = calcTermDays($g_MTrackTerm{$trk}{$trm}{start}, $g_MTrackTerm{$trk}{$trm}{end}, \%jdclosed );
		my ($open,$closed) = split(':', $val);
		$totaldays += $open;
		print qq{<td class="cn">$open</td><td class="cn">$closed</td></tr>\n};

	    }
	    print qq{<tr><td colspan="3" class="bra">$lex{Total}</td><td class="bcn">$totaldays</td><td></td></tr>\n};

	    if ( $deftrk ) { print qq{<tr><td colspan="4">$deftrk = $lex{Default} $lex{Track}</td></tr>\n}; } 
	    print "</table>\n";
	}


    } else { # Print Normal Terms

	print "<table cellpadding=\"3\" border=\"0\" cellspacing=\"0\" ";
	print "style=\"border:1px solid gray;padding:0.5em;\">\n";

	my $totaldays;
	print qq{<tr><th></th><th>$lex{Start}</th><th>$lex{End}</th><th>$lex{Open}</th><th>$lex{Closed}</th></tr>\n};
	foreach my $trm ( sort keys %g_termstart ) {
	    print qq{<tr><td class="bla">$g_TermDisplay{$g_DefaultTrack}{$trm}</td><td class="la">$g_termstart{$trm}</td>\n};
	    print qq{<td class="la">$g_termend{$trm}</td>\n};

	    my $val = calcTermDays($g_termstart{$trm}, $g_termend{$trm}, \%jdclosed );
	    my ($open,$closed) = split(':', $val);
	    $totaldays += $open;
	    if ( not $closed ) { $closed = '0'; }
	    print qq{<td class="cn">$open</td><td class="cn">$closed</td></tr>\n};


	}
	print qq{<tr><td colspan="3" class="bra">$lex{Total}</td><td class="bcn">$totaldays</td><td></td></tr>\n};

	if ( $deftrk ) { print qq{<tr><td colspan="4">$deftrk = $lex{Default} $lex{Track}</td></tr>\n}; } 
	print "</table>\n";

    }

    # change div's from left to right.
    print qq{</div>\n};
    #print qq{<br clear="left">\n};


    # Print the Stored Dates (in Dates Table)
    print qq{<table cellpadding="3" border="1" cellspacing="0" style="float:left;margin:0 2em;">\n};
    print qq{<caption style="font-size:130%;font-weight:bold;">$lex{Dates} $lex{Closed}</caption>\n};

    print "<tr><th>$lex{Date}</th><th>$lex{Day}</th><th>$lex{Type}</th>\n";
    print "<th>$lex{Description}</th><th>$lex{Cycle}<br>$lex{Day}</th>";
    print "<th>$lex{Day}<br>$lex{Fraction}</tr>\n";


    my $sth = $dbh->prepare("select * from dates order by date");
    $sth->execute;
    if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }

    while ( my ( $id, $date, $type, $desc1, $desc2, $dayincycle, $dayfraction ) = $sth->fetchrow ) {
	my ($year, $month, $day ) = split(/-/, $date);
	my $newdate = $year .'-'. $s_month[$month]. q{-}. $day;
	my $jd = julian_day( split( /-/, $date ) );
	my $dow = day_of_week($jd) + 1;

	if ( not $dayincycle ) { 
	    $dayincycle = $lex{No}; 
	} else { 
	    $dayincycle = $lex{Yes};
	}

	print "<tr><td>$newdate</td><td>$dow[$dow]</td><td>$type</td><td>$desc1</td>\n";
	print "<td class=\"cn\">$dayincycle</td><td>$dayfraction</td></tr>\n";

    }

    print qq{</table><p></p>\n};
    print qq{<br clear="left">\n};
    print qq{</div>\n}; # end of this school div

}

print qq{</body></html>\n};



#----------------
sub showStartPage { # Entry Values for Custom Script
#----------------

    my $checked;
    if ( $arr{checked} ) {
	$checked = qq{checked="checked"};
    }

    print qq{<form action="$self" method="post" style="display:inline;margin:1em;">\n};
    print qq{<input type="hidden" name="checked" value="1">\n};
    print qq{<input type="submit" value="$lex{'Check All'} $lex{Schools}" style="display:inline;"></form>\n};



    print "<form action=\"$self\" method=\"post\">\n";
    print "<input type=\"hidden\" name=\"page\" value=\"1\">\n";

    print qq{<h3>$lex{Select} $lex{Schools}</h3>\n};
    print "<table cellpadding=\"3\" cellspacing=\"0\" border=\"0\">\n";
    
    foreach my $db ( @dbase ) { 
	print qq{<tr><td class="la">};
	print qq{<input type="checkbox" name="$db" value="1" $checked></td>\n};
	print qq{<td class="la">$alldbase{$db} ($db)</td></tr>\n};
    }

    print qq{<tr><td class="la" colspan="2"><input type="submit" value="$lex{Continue}"></td></tr>\n};
    print qq{</table>\n};
    print qq{</form></body></html>\n};

    exit;

}



#---------------
sub calcTermDays {
#---------------

    # passed a start date and an end date for a term, and a ref to a
    # hash storing the closure dates in julian day format, it will
    # return a count of the days open in the month.

    # outline: convert start/end dates to jd. Loop over all days. Use modulus function to ident weekend days.

    my ($startdate, $enddate, $closeref ) = @_;
    my %jdclosed = %$closeref;

=head
    print "JDClosed<br>\n";
    foreach my $key ( sort keys %jdclosed ) {
	print "K:$key V:$jdclosed{$key}<br>\n";
    }
    print "End JDClosed<br>\n";
=cut

    my $startjd = julian_day(split(/-/,$startdate));
    my $endjd = julian_day(split(/-/,$enddate));

#    print "ST:$startdate - $startjd END:$enddate - $endjd<br>\n";

    my %modclosed; # modulus for days closed
    foreach my $inc ( 0..6 ) {
	my $testjd = $startjd + $inc;

	my $dow = day_of_week($testjd);
	if ( $dowclosed{$dow} ) { # $testjd is a closed dow
	    my $mod = $testjd % 7;
	    $modclosed{$mod} = 1;
	}
    }

#    foreach my $d ( sort keys %modclosed ) {
#	print "Closed Mod: $d<br>\n";
#    }

    my ($daysopen, $daysclosed);
    foreach my $jd ( $startjd..$endjd ) {

	my $dow = day_of_week( $jd );
	my $mod = $jd % 7;

	if ( $modclosed{$mod} ) { # closed for weekend
	    next;
	}

	# check if a day closed
	my $cl = $jdclosed{ $jd };
	if ( $cl  ) { # this gives us the fraction closed
	    $daysclosed += $cl;
	    # only add on the fraction of day open
	    $daysopen += (1 - $cl );
#	    print "<br>INSIDE<br>\n";
	} else {
	    $daysopen += 1;
	}
#	print "JD:$jd Count:$daysopen<br>\n";

    }

#    print "ST:$startdate END:$enddate  Count:$daysopen / $daysclosed<br>\n";

    return "$daysopen:$daysclosed";

}

