#!/usr/bin/perl
#  Copyright 2001-2013 Leslie Richardson

#  This file is part of Open Admin for Schools.

#  Open Admin for Schools is free software; you can redistribute it 
#  and/or modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2 of 
#  the License, or (at your option) any later version.

my %lex = ('Main' => 'Main',
	   'Error' => 'Error',
	   'Find' => 'Find',
	   'Dropouts' => 'Dropouts',
	   'School' => 'School',
	   'Database' => 'Database',

	   'Continue' => 'Continue',
	   'Select' => 'Select',
	   'Student' => 'Student',
	   'Homeroom' => 'Homeroom',
	   'Grade' => 'Grade',
	   'Blank=All' => 'Blank=All',
	   'No Students Found' => 'No Students Found',
	   'Start Date' => 'Start Date',
	   'End Date' => 'End Date',
	   'Records' => 'Records',
	   'Delete' => 'Delete',
	   'Add' => 'Add',
	   'Record(s)' => 'Record(s)',
	   'Not Found' => 'Not Found',
	   'Previous' => 'Previous',
	   'Year' => 'Year',
	   'Current' => 'Current',
	   'Enrol' => 'Enrol',
	   'Withdraw' => 'Withdraw',
	   'Check' => 'Check',
	   'Next Page' => 'Next Page',
	   'Remove' => 'Remove',

	   );


use DBI;
use CGI;
use Time::JulianDay;
use Number::Format;

my $self = 'finddropouts.pl';

my %dropoutcodes = qw(18 1 19 1 21 1 24 1 50 1 52 1);


my @time = localtime(time);
my $year = $time[5] + 1900;
my $month = $time[4] + 1;
my $currdate = "$year-$month-$time[3]";




# Read config variables
eval require "admin.conf";
if ( $@ ) {
    print $lex{Error}. ": $@<br>\n";
    die $lex{Error}. ": $@\n";
}

eval require "../lib/libattend.pl";
if ( $@ ) {
    print $lex{Error}. ": $@<br>\n";
    die $lex{Error}. ": $@\n";
}

eval { require "$globdir/global.conf"; };
if ( $@ ) {
    print $lex{Error}. " $self: $@<br>\n";
    die $lex{Error}. "$self: $@\n";
}

@dbase = sort keys %alldbase; # global records.


my $q = new CGI;
print $q->header; 
my %arr = $q->Vars;

my ($dsn, $dbh, $schoolname );
if ( $arr{db} ) {
    my $database = $arr{db};
    $dsn = "DBI:$dbtype:dbname=$database";
    $dbh = DBI->connect($dsn,$guser,$gpassword);
    $schoolname = $alldbase{"$arr{db}"};

    # Get the g_ppd Record from configuration system
#    my $sth = $dbh->prepare("select datavalue from conf_system where dataname = ?");
#    $sth->execute( 'g_ppd' );
#    my $datavalue = $sth->fetchrow;
#    eval $datavalue;
#    if ( $@ ) {
#	print $lex{Error}. " $@<br>\n";
#	die $lex{Error}. " $@\n";
}

#my $dsn = "DBI:$dbtype:dbname=$dbase";
#my $dbh = DBI->connect($dsn,$user,$password);

# Page Header
my $title = "$lex{Find} $lex{Dropouts}";
print "$doctype\n<html><head><title>$title</title>\n";
print "<link rel=\"stylesheet\" href=\"/division.css\" type=\"text/css\">\n";

=head
if ( not $arr{page} ) { # calendar popup.
    print "<link rel=\"stylesheet\" type=\"text/css\" media=\"all\" ";
    print "href=\"/js/calendar-blue.css\" title=\"blue\">\n";
    print "<script type=\"text/javascript\" src=\"/js/calendar.js\"></script>\n";
    print "<script type=\"text/javascript\" src=\"/js/lang/calendar-en.js\"></script>\n";
    print "<script type=\"text/javascript\" src=\"/js/calendar-setup.js\"></script>\n";
}
=cut

print "$chartype\n</head><body>\n";

print "[ <a href=\"$homepage\">$lex{Main}</a> ]\n";
print "<h1>$title</h1>\n";


if ( not $arr{page} ) {
    showStartPage();

} elsif ( $arr{page} == 1 ) {
    delete $arr{page};
    findDropouts();
} 



#----------------
sub showStartPage {
#----------------

    print "<form action=\"$self\" method=\"post\">\n";
    print "<input type=\"hidden\" name=\"page\" value=\"1\">\n";

    print "<table cellpadding=\"3\" cellspacing=\"0\" border=\"0\" ";
    print "style=\"padding:0.5em;border:1px solid gray;\">\n";


    print "<tr><td class=\"bra\">$lex{Select} $lex{School}</td><td class=\"la\">";
    print "<select name=\"db\"><option></option>\n";
    foreach my $db ( @dbase ) { print "<option value=\"$db\">$alldbase{$db}</option>"; }
    print "</select></td></tr>\n";

    print "<tr><td></td><td><input type=\"submit\" value=\"$lex{Continue}\"></td></tr>\n";

    print "</table></form>\n";
    print "</body></html>\n";

    exit;

}



#---------------
sub findDropouts {
#---------------

    # foreach my $key ( sort keys %arr ) { print "K:$key V:$arr{$key}<br>\n"; }
    # exit;

    my $cutoffdate;
    if ( $month > 9 ) { # current year.
	$cutoffdate = "$year-09-30";
    } else {
	$year--;
	$cutoffdate = "$year-09-30";
    }
    my $cutoffjd = julian_day( split(/-/, $cutoffdate));
    # print "Cutoff:$cutoffjd<br>\n";


    # Find all student enrolled last Sept 30th.
    my %students;
    my $sth = $dbh->prepare("select lastname, firstname, studnum from student");
    $sth->execute;
    if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
    while ( my ($lastname, $firstname, $studnum) = $sth->fetchrow ) {
	$students{$studnum} = "$lastname, $firstname";
    }

    my %endstudents = %students; # there at year end.

    my %exitreasons; # track numbers of each type of withdrawal.


    # Get Enrollments/Withdrawals
    my $sth = $dbh->prepare("select * from transfer order by date desc");
    $sth->execute;
    if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }

    my $count;
    while ( my $sref = $sth->fetchrow_hashref ) {
	my %sr = %$sref;
	# print "Date:$sr{date}<br>\n";
	# print "SR:", %sr, "<br><br>\n";

	my $jd = julian_day(split(/-/, $sr{date}));
	if ( $jd < $cutoffjd ) { # we are done with changes to student.
	    last;
	}
        #print "<div>$sr{lastname} $sr{firstname}</div>\n";

	my $tstudnum = $sr{studnum};
	if ( $sr{type} eq 'withdraw' ) {
	    if ( exists $students{$tstudnum} ) {
		print "Error: $tstudnum ($sr{lastname} $sr{firstname}) Withdraw - Adding Record In<br>\n";
	    } else {
		$students{$tstudnum} = "$sr{lastname} $sr{firstname}";
	    }

	} else { # doing enrol, so previously there should be no record.
	    if ( not exists $students{$tstudnum} ) { # don't add something already there.
		print "Error: $tstudnum ( $sr{lastname} $sr{firstname} ) Enrol - Removing record<br>\n";
	    } else {
		delete $students{$tstudnum}; 
	    }
	}
    }

    %startstudents = %students;
    my $startsize = keys %startstudents;

    my %dropouts;
    foreach my $studnum ( keys %startstudents ) {
	if ( not exists $endstudents{$studnum} ) {
	    $dropouts{$studnum} = $startstudents{$studnum};
	}
    }

    my $count;
    $sth = $dbh->prepare("select * from transfer where studnum = ? and type = 'withdraw' order by date desc");

    my $first = 1;

    foreach my $studnum ( keys %dropouts ) {

	# Get Withdrawal Info
	$sth->execute($studnum);
	if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
	my $ref = $sth->fetchrow_hashref;
	my $exittype = $ref->{exittype};

	$exitreasons{$exittype}++;

	if ( $dropoutcodes{$exittype} ) { # we have a dropout.

	    if ( $first ) {
		print "<h3>Dropouts</h3>\n";
		print qq{<table cellspacing="0" cellpadding="3" border="1">\n};
		print qq{<tr><th>Student</th><th>Reason</th><th>Date</th></tr>\n};
		$first = 0;
	    }

	    $count++;
	    print "<tr><td>$count. $dropouts{$studnum} ($studnum)</td><td>$g_wdraw{$exittype}($exittype)</td><td>$ref->{date}</td></tr>\n";

	}
    }

    if ( $first ) {
	print "<h3>No Dropouts Found</h3>\n";
    } else {
	print qq{</table>\n};
    }

    my $droprate = format_number( $count / $startsize * 100, 2);
    print "<h3>Cohort Size: $startsize<br>Dropouts: $count<br>Dropout Rate: $droprate %</h3>\n";

    print "<h3>Withdrawal Reasons</h3>\n";
    print qq{<table cellspacing="0" cellpadding="3" border="1">\n};
    foreach my $exittype ( keys %exitreasons ) {
	print "<tr><td>$g_wdraw{$exittype}($exittype)</td><td>$exitreasons{$exittype}</td></tr>\n";
    }
    print "</table><p></p>\n";


    print "</body></html>\n";

    exit;

}



#---------------
sub findStudents {
#---------------

    my $currcutoff = shift; # current cutoff for this year. (yyyy-mm-dd)
    my $cutoffjd = julian_day( split('-', $currcutoff));    

    my @students = ();

    # Search All Students
    my $sth = $dbh->prepare("select distinct studnum from transfer 
      where studnum != '' and studnum is not NULL");
    $sth->execute;
    if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }


    my $count;
    while ( my $studnum = $sth->fetchrow ) {
	my @blocks = ();
	@blocks = findEnrollmentBlocks( $studnum, $ppyeardate, $currdate, $dbh );
	foreach my $ref ( @blocks ) {
	    my $startjd = julian_day( split('-', $ref->{start} ));
	    my $endjd = julian_day( split('-', $ref->{end} ));
	    if ( $startjd <= $cutoffjd and $endjd >= $cutoffjd ) { # enrollment block has date within
		push @students, $studnum;
		$count++;
#		print "$count. $studnum - $ref->{start}:$ref->{end}<br>\n";
		next;

	    }
	}
    }

    return \@students;

}
