#!/usr/bin/perl
#  Copyright 2001-2015 Leslie Richardson

#  This file is part of Open Admin for Schools.

# Read Report - reference script that will read student data into data
# structure and then report by student, grade, or school. There can
# then be a variety of reports from this.


my %lex = ('Error' => 'Error',
	   'Main' => 'Main',
	   'Grade' => 'Grade',
	   'Blank=All' => 'Blank=All',
	   'Start Date' => 'Start Date',
	   'End Date' => 'End Date',
	   'Split' => 'Split',
	   'Date' => 'Date',
	   'Continue' => 'Continue',
	   'Mean' => 'Mean',
	   'Median' => 'Median',
	   'Range' => 'Range',
	   'StdDev' => 'StdDev',
	   'Previous' => 'Previous',
	   'Current' => 'Current',
	   'School' => 'School',
	   'Database' => 'Database',
	   'Reading' => 'Reading',
	   'Report' => 'Report',
	   'Literacy Intervention' => 'Literacy Intervention',

	   );


my $self = 'lliRpt1.pl';

use DBI;
use CGI;
use Cwd;
use Number::Format qw(:all);
use Time::JulianDay;

eval require "../admin.conf";
if ( $@ ) {
    print $lex{Error}. ": $@<br>\n";
    die $lex{Error}. ": $@\n";
}


my @time = localtime(time);
my $year = $time[5] + 1900;
my $month = $time[4] + 1;
my $currdate = "$year-$month-$time[3]";
my $schyear = $year;
if ( $month < 7 ){ $schyear = $schyear - 1; }
my $prevyear = $schyear - 1;

my $currstartdate = "$schyear-08-01"; # Aug 1 of this school year
my $prevstartdate = "$prevyear-08-01"; # August 1 of prev year.


my $q = CGI->new;
my %arr = $q->Vars;
print $q->header( -charset, $charset );


# Script Heading Descriptor
my $title = "LLI $lex{Report} 1";

# Print Page Heading

print "$doctype\n<html><head><title>$title</title>\n";
print "<link rel=\"stylesheet\" href=\"/division.css\" type=\"text/css\">\n";

if ( not $arr{page} ) { # calendar popup.
    print "<link rel=\"stylesheet\" type=\"text/css\" media=\"all\" ";
    print "href=\"/js/calendar-blue.css\" title=\"blue\">\n";
    print "<script type=\"text/javascript\" src=\"/js/calendar.js\"></script>\n";
    print "<script type=\"text/javascript\" src=\"/js/lang/calendar-en.js\"></script>\n";
    print "<script type=\"text/javascript\" src=\"/js/calendar-setup.js\"></script>\n";
}

print "$chartype\n</head><body style=\"padding:0.4em 2em;\">\n";
print "[ <a class=\"alt\" href=\"$homepage\">$lex{Main}</a> ]\n";
print "<h1>$title</h1>\n";

if ( not $arr{page} ) {
    showStartPage();

} elsif ( $arr{page} == 1 ) {
    delete $arr{page};
    assembleData();
}


#----------------
sub showStartPage {
#----------------

    print "<form action=\"$self\" method=\"post\">\n";
    print "<input type=\"hidden\" name=\"page\" value=\"1\">\n";

    print "<table cellpadding=\"3\" cellspacing=\"0\" border=\"0\" ";
    print "style=\"padding:0.5em;border:1px solid gray;\">\n";

    # Start Date
    print "<tr><td class=\"bra\">$lex{'Start Date'}</td><td class=\"la\"><input type=\"text\" ";
    print "name=\"startdate\" id=\"sdate\" size=\"10\" value=\"$currstartdate\">";
    print "<button type=\"reset\" id=\"start_trigger\">...</button>\n";
    print "</td></tr>\n";

    # End Date
    print "<tr><td class=\"bra\">$lex{'End Date'}</td><td class=\"la\"><input type=\"text\" ";
    print "name=\"enddate\" id=\"edate\" size=\"10\" value=\"$currdate\">";
    print "<button type=\"reset\" id=\"end_trigger\">...</button></td></tr>\n";

    print qq{<tr><td class="bra">Show Details</td>};
    print qq{<td class="la"><input type="checkbox" name="showdetails" value="1"></td></tr>\n};


=head
    # Databases
    print "<tr><td class=\"bra\">School Group</td><td class=\"la\">";
    print qq{<select name="confgroup">};
    print qq{<option value="nnec">NNEC</option>};
    print qq{<option value="batc">BATC</option>};
    print "</td></tr>\n";
=cut


    print "<tr><td></td><td class=\"la\"><input type=\"submit\" value=\"$lex{Continue}\"></td></tr>\n";
    print "</form>\n";

    print "<script type=\"text/javascript\">
     Calendar.setup({
        inputField     :    \"sdate\", 
        ifFormat       :    \"%Y-%m-%d\",
        button         :    \"start_trigger\",
        singleClick    :    false,
        step           :    1
    });

    Calendar.setup({
        inputField     :    \"edate\",
        ifFormat       :    \"%Y-%m-%d\",
        button         :    \"end_trigger\",
        singleClick    :    false,
        step           :    1
    });
    </script>\n";

    print "</body></html>\n";

    exit;

} # end of showStartPage



#---------------
sub assembleData {
#---------------

    # foreach my $key ( sort keys %arr ) { print "K:$key V:$arr{$key}<br>\n"; }

    my $startdate = $arr{startdate};
    my $enddate = $arr{enddate};
    print "<h3>Start:$startdate End:$enddate</h3>\n";


    my %datacount; # grade -> studcount, 
    my %datagain; # grade ->  year gain in reading level;
    my %datahours; # grade -> 
    my %datamin;
    my %datamax; 


    my $grandtotalstudents;

    @dbase = sort keys %alldbase; # global records.

    foreach my $database ( @dbase ) {

	my %localcount; # grade -> studcount, 
	my %localgain; # grade ->  year gain in reading level;
	my %localhours; # grade -> 
	my %localmin;
	my %localmax; 


	my $dsn = "DBI:$dbtype:dbname=$database";
	my $dbh = DBI->connect($dsn,$user,$password);

	my $schoolname = $alldbase{$database};
	print qq{<div style="color:red;font-size:150%;padding:0.5em 0;">$schoolname</div>\n};


	# Get Student Record
	$sth1 = $dbh->prepare("select * from lint_student where progid = ?");

	# Count student records
	$sth4 = $dbh->prepare("select count(*) from lint_student where progid = ?");

	# Get the test scores
	my $sth3 = $dbh->prepare("select sum(score), count(*) from read_test_score 
         where testid = ?");

	my $sth6 = $dbh->prepare("select lastname, firstname from studentall where studnum = ?");


	# id, programhours, groupdesc, startdate from lint_program 
	$sth = $dbh->prepare("select * from lint_program 
          where to_days(startdate) >= to_days('$startdate') and to_days(startdate) <= to_days('$enddate')");
	$sth->execute;
	if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }

	my $first = 1;
	my $totalstudents;

	# Program Loop
	while ( my $pref = $sth->fetchrow_hashref ) {
	    my %pr = %$pref;

	    if ( $first ) {
		print qq{<table cellpadding="3" cellspacing="0" border="1">\n};
		print qq{<tr><th>Description</th><th>Start Date</th><th>Students</th></tr>\n};
		$first = 0;
	    }


	    if ( $arr{showdetail} ) {
		print qq{<div style="color:blue;font-size:120%;padding:0.3em 0;">};
		print qq{$pr{groupdesc} - Start Date:$pr{startdate} ID:$pr{id}</div>\n};
	    }


	    my $prstartdate = $pr{startdate};
	    my $studcount;  # incremented through the loop

	    # Student Loop
	    $sth1->execute( $pr{id} );
	    if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }


	    # Loop through all students
STUDENT:    while ( my $ref = $sth1->fetchrow_hashref ) {
		my %r = %$ref;
#		print "Student:", %r, "<br>\n";
		if ( $r{dropped} ) { 
		    # Get Student Name
		    $sth6->execute( $r{studnum} );
		    if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr; }
		    my ($lastname, $firstname) = $sth6->fetchrow;
		    if ( $arr{showdetails} ) {
			print qq{<div style="color:green;font-weight:bold;">};
			print qq{Dropped: $firstname $lastname ($r{studnum})</div>\n};
		    }
		    next STUDENT;
		}


		my $sth2 = $dbh->prepare("select * from read_test where studnum = ? and
                 to_days( tdate ) <= to_days('$prstartdate') order by tdate desc ");

		$sth2->execute( $r{studnum} );
		if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr; }
		my $pretest_ref = $sth2->fetchrow_hashref;

		if ( not $pretest_ref ) { # move the date to look at first test WITHIN date range
		    my $sth5 = $dbh->prepare("select * from read_test where studnum = ? and
                   to_days( tdate ) > to_days('$prstartdate') and  to_days( tdate ) <= to_days('$pr{enddate}')
                   order by tdate "); # first one within this date range.
		    $sth5->execute( $sr{studnum} );
		    if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr; }
		    $pretest_ref = $sth5->fetchrow_hashref;
		}

		if ( not $pretest_ref->{id} ) {
		    # Get Student Name
		    $sth6->execute( $r{studnum} );
		    if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr; }
		    my ($lastname, $firstname) = $sth6->fetchrow;
		    if ( $arr{showdetails} ) {
			print qq{<div style="color:purple;font-weight:bold;">};
			print qq{No Pretest: $firstname $lastname ($r{studnum})</div>\n}; 
		    }
		    next STUDENT; 
		}

		my %pretest = %$pretest_ref;


		# foreach my $key ( sort keys %pretest ) { print "K:$key V:$pretest{$key}<br>\n"; }	

		# Get the pretest Scores
		$sth3->execute( $pretest{id} );
		if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr; }
		my ( $prescoretotal, $prescorecount ) = $sth3->fetchrow;
		my $prepossibletotal = $prescorecount * 4;

		my $equivgrade = scoreToGrade( $prescoretotal, $pretest{readlevel} );


		# Find the Posttest.
		$sth2 = $dbh->prepare("select * from read_test where studnum = ? and
                  to_days( tdate ) >= to_days( '$pr{enddate}' ) ");
		$sth2->execute( $r{studnum} );
		if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr; }
		my $posttest_ref = $sth2->fetchrow_hashref;

		if ( not $posttest_ref ) { # move the date to look at first test WITHIN date range
		    $sth5 = $dbh->prepare("select * from read_test where studnum = ? and
                      to_days( tdate ) <= to_days( '$pr{enddate}' ) and 
                      to_days( tdate ) >= to_days( '$pr{startdate}' )
                      order by tdate desc");
		    $sth5->execute( $sr{studnum} );
		    if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr; }
		    $posttest_ref = $sth5->fetchrow_hashref;
		    if ( $posttest_ref->{id} == $pretest_ref->{id} ) { # same test; bad!
			$posttest_ref = undef;
		    }
		}

		if ( not $posttest_ref->{id} ) { 
		    # Get Student Name
		    $sth6->execute( $r{studnum} );
		    if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr; }
		    my ($lastname, $firstname) = $sth6->fetchrow;
		    if ( $arr{showdetails} ) {
			print qq{<div style="color:purple;font-weight:bold;">};
			print qq{No Post Test: $firstname $lastname ($r{studnum})</div>\n}; 
		    }
		    next STUDENT; 
		}

#		print "REF:$posttest_ref<br>\n";
#		foreach my $key ( sort keys %{ $posttest_ref } ) {
#		    print "K:$key V:$posttest_ref->{$key}<br>\n";
#		}

		my %posttest = %$posttest_ref;

#		print "PostTEST:<br>", %posttest, "<br>\n"; 

		# Get the posttest Scores
		$sth3->execute( $posttest{id} );
		if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr; }
		my ( $postscoretotal, $postscorecount ) = $sth3->fetchrow;
		my $postpossibletotal = $postscorecount * 4;

		my $postequivgrade = scoreToGrade( $postscoretotal, $posttest{readlevel} );
		if ( not $postequivgrade or not $equivgrade ) { print "Missing Data - $r{studnum}<br>"; next STUDENT }

		my $delta = $postequivgrade - $equivgrade;
		my $grade = $posttest{tgrade};

#		print "Student:$r{studnum}  Initial:$equivgrade Final:$postequivgrade  Delta:$delta ";
#		print "Grade:$posttest{tgrade} <br>\n"; #Hours:$programhours<br>\n";

		my $programhours = $pr{programhours};
#		print "Hours:Program: $programhours ";
		if ( $r{programhours} ) {
		    $programhours = $r{programhours};
		}
#		print "- Actual:$programhours<br>\n";

		if ( not $grade) { 
		    print "Missing Grade for student:$studnum for postLLI test<br>\n";
		    next;
		}

		$datacount{$grade} += 1;
		$datagain{$grade} += $delta;
		$datahours{$grade} += $programhours;

		if ( not $datamin{$grade} ) { $datamin{$grade} = $delta; }
		if ( $delta < $datamin{$grade} ) { 
		    $datamin{$grade} = $delta; 
		}

		if ( $delta > $datamax{$grade} ) { 
		    $datamax{$grade} = $delta; 
		}

		$studcount++;   # we only increment if we get to bottom of the loop.


		# Local School Data
		$localcount{$grade} += 1;
		$localgain{$grade} += $delta;
		$localhours{$grade} += $programhours;

		if ( not $localmin{$grade} ) { $localmin{$grade} = $delta; }
		if ( $delta < $localmin{$grade} ) { 
		    $localmin{$grade} = $delta; 
		}

		if ( $delta > $localmax{$grade} ) { 
		    $localmax{$grade} = $delta; 
		}



	    } # end of students loop


	    $totalstudents += $studcount;

	    print qq{<tr><td>$pr{groupdesc}</td><td>$pr{startdate}</td><td>$studcount</td></tr>\n};


	} # end of programs loop

	if ( $first ) { 
	    print "<div>No Programs for this school found.</div>\n"; 
	} else { 
	    $grandtotalstudents += $totalstudents;
	    print qq{<tr><td colspan="5">Student Count:$totalstudents</td></tr></table>\n}; 
	}


#	foreach my $gr ( sort keys %datacount ) {
#	    print qq{Grade:$gr Count:$datacount{$gr} Gain:$datagain{$gr} Hours:$datahours{$gr}<br>\n};
#	}
#	print "<br>\n";


	print "<h3>$schoolname Summary Results</h3>\n";
	print "<table cellpadding=\"3\" cellspacing=\"0\" border=\"1\">\n";
	print qq{<tr><th>Grade</th><th>Number of<br>Students</th><th>Min Gain</th>\n};
	print qq{<th>Avg. Gain<br>(years)</th><th>Max Gain</th><th>Avg. Instr.<br>Hours</th></tr>\n};

	foreach my $grade ( sort {$a <=> $b} keys %localcount ) {
	    print qq{<tr><td class="bcn">$grade</td><td class="bcn">$localcount{$grade}</td>};

	    print qq{<td class="bcn">}. format_number( $localmin{$grade}, 2, 2 ). qq{</td>\n};
	    print qq{<td class="bcn">}. format_number( $localgain{$grade} / $localcount{$grade}, 2, 2 ). qq{</td>\n};
	    print qq{<td class="bcn">}. format_number( $localmax{$grade}, 2, 2 ). qq{</td>\n};
	    
	    print qq{<td class="bcn">}. format_number( $localhours{$grade} / $localcount{$grade}, 2, 2 ). qq{</td>\n};
	    print qq{</tr>\n};
	}

	my ($totcount, $totgain, $tothours);
	foreach my $grade ( keys %datacount ) {
	    $totcount += $localcount{$grade};
	    $totgain += $localgain{$grade};
	    $tothours += $localhours{$grade};
	}
	print qq{<tr style="background-color:#CCF;"><td class="bcn">Summary</td><td class="bcn">$totcount</td><td></td>};
	if ($totcount ) {
	    print qq{<td class="bcn">}. format_number( $totgain / $totcount, 2,2 ). qq{</td><td></td>\n};
	    print qq{<td class="bcn">}. format_number( $tothours / $totcount, 2,2 ). qq{</td>\n};
	}
	print qq{</tr>\n};    
	print "</table>\n";


    } # end of school loop

=head
    my $count;
    foreach my $key ( sort keys %datagain ) {
	print "K:$key V:$datagain{$key} $datacount{$key}<br>\n";
	$count += $datacount{$key};
    }
    print "Count:$count<br>\n";
=cut

    print "<h3>Overall Results</h3>\n";
    print "<table cellpadding=\"3\" cellspacing=\"0\" border=\"1\">\n";
    print qq{<tr><th>Grade</th><th>Number of<br>Students</th><th>Min Gain</th>\n};
    print qq{<th>Avg. Gain<br>(years)</th><th>Max Gain</th><th>Avg. Instr.<br>Hours</th></tr>\n};

    foreach my $grade ( sort {$a <=> $b} keys %datacount ) {
	print qq{<tr><td class="bcn">$grade</td><td class="bcn">$datacount{$grade}</td>};

	print qq{<td class="bcn">}. format_number( $datamin{$grade}, 2, 2 ). qq{</td>\n};
	print qq{<td class="bcn">}. format_number( $datagain{$grade} / $datacount{$grade}, 2, 2 ). qq{</td>\n};
	print qq{<td class="bcn">}. format_number( $datamax{$grade}, 2, 2 ). qq{</td>\n};

	print qq{<td class="bcn">}. format_number( $datahours{$grade} / $datacount{$grade}, 2, 2 ). qq{</td>\n};
	print qq{</tr>\n};
    }

    my ($totcount, $totgain, $tothours);
    foreach my $grade ( keys %datacount ) {
	$totcount += $datacount{$grade};
	$totgain += $datagain{$grade};
	$tothours += $datahours{$grade};
    }
    print qq{<tr style="background-color:#CCF;"><td class="bcn">Summary</td><td class="bcn">$totcount</td><td></td>};
    if ($totcount ) {
	print qq{<td class="bcn">}. format_number( $totgain / $totcount, 2,2 ). qq{</td><td></td>\n};
	print qq{<td class="bcn">}. format_number( $tothours / $totcount, 2,2 ). qq{</td>\n};
    }
    print qq{</tr>\n};    


    print "</table></body></html>\n";


    exit;

}



#---------------
sub scoreToGrade {
#---------------

    my ($score, $readinglevel) = @_;

    my %rd_LevelToGrade = ( '0' => 0.1, 1 => 0.3, 2 => 0.6, 3 => 1.0, 4 => 1.2,
			    6 => 1.4, 8 => 1.5, 10 => 1.5, 12 => 1.6, 14 => 1.7,
			    16 => 1.9, 18 => '2.0', 20 => 2.2, 24 => 2.5, 28 => 2.9,
			    30 => 3.3, 34 => 3.6, 38 => 3.9);


    my %rd_ScoreToGrade = (
	50 => 1.9, 51 => 1.9, 52 => 1.9, 53 => 1.9, 54 => '2.0',
	55 => 2.1, 56 => 2.2, 57 => 2.3, 58 => 2.4,
	59 => 2.5, 60 => 2.6, 
	61 => 2.7, 62 => 2.8, 63 => 2.9, 64 => '3.0', 
	65 => 3.1, 66 => 3.2, 67 => 3.3, 68 => 3.4,
	69 => 3.5, 70 => 3.6, 71 => 3.7, 72 => 3.8, 73 => 3.9, 
	74 => '4.0', 75 => 4.1, 76 => 4.2, 77 => 4.3, 78 => 4.4, 79 => 4.5,
	80 => 4.6, 81 => 4.7, 82 => 4.8, 83 => 4.9, 84 => '5.0', 85 => 5.1, 86 => 5.6, 
	87 => 5.3, 88 => 5.4, 89 => 5.5, 90 => 5.6, 91 => 5.7, 92 => 5.8, 93 => 5.9, 
	94 => '6.0', 95 => 6.1, 96 => 6.2, 97 => 6.3, 98 => 6.4, 99 => 6.5, '100' => '6.6',
	101 => 6.6, 102 => 6.8, 103 => 6.9, 
	104 => '7.0', 105 => 7.1, 106 => 7.2, 107 => 7.3, 108 => 7.4, 109 => 7.5, '110' => '7.6',
	111 => 7.7, 112 => 7.8, 113 => 7.9, 114 => '8.0', 115 => 8.1, 116 => 8.2, 117 => 8.3, 118 => 8.4, 
	119 => 8.5, 120 => 8.6, 121 => 8.7, 122 => 8.8, 123 => 8.9, 124 => 8.9, 125 => 8.9, 126 => 8.9,
	127 => 8.9, 128 => 8.9 );

    #foreach my $key ( sort {$a <=> $b} keys %rd_LevelToGrade ) { print "K:$key V:$rd_LevelToGrade{$key}<br>\n"; }

    if ( not $score ) { return undef; }

    if ( $readinglevel < 40 ) {

	foreach my $key ( sort { $b <=> $a } keys %rd_LevelToGrade ) {
	    # print "K:$key ";
	    if ( $readinglevel >= $key ) {
		# print "<br><b>Match: K:$key RL:$readinglevel</b><br>\n";
		return $rd_LevelToGrade{ $key };
	    }
	}

    } else { # else reading level >= 40

	$score += $readinglevel; # add both together for this 

	foreach my $key ( sort { $b <=> $a } keys %rd_ScoreToGrade ) {
	    # print "K:$key ";
	    if ( $score >= $key ) {
		# print "<b>Match: S:$score K:$key V:$rd_ScoreToGrade{$key}</b><br>\n";
		return $rd_ScoreToGrade{ $key };
	    }
	}

    }

    return undef;
}
