#! /usr/bin/perl
#  Copyright 2001-2014 Leslie Richardson

#  This file is part of Open Admin for Schools.

#  Open Admin for Schools is free software; you can redistribute it 
#  and/or modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2 of 
#  the License, or (at your option) any later version.

my %lex = ('Main' => 'Main',
	   'Continue' => 'Continue',
	   'Homeroom' => 'Homeroom',
	   'Grade' => 'Grade',
	   'Select by' => 'Select by',
	   'Error' => 'Error',
	   'Sort by' => 'Sort by',
	   'Name' => 'Name',

	   'Common Math Assessment' => 'Common Math Assessment',
	   'Report' => 'Report',
	   'OR' => 'OR',
	   'No Selection' => 'No Selection',
	   'School Year' => 'School Year',
	   'Select' => 'Select',
	   'School' => 'School',

	   );

my $self = 'cmaRpt1a.pl';

use DBI;
use CGI;
use Cwd;


eval require "../admin.conf";
if ( $@ ) {
    print $lex{Error}. " $@<br>\n";
    die $lex{Error}. " $@\n";
}

my $q = new CGI;
print $q->header( -charset, $charset );
my %arr = $q->Vars;


my ($sec, $min, $hour, $mday, $mon, $year, $wday, $yday, $iddst) = localtime(time);
$year = $year + 1900;
$wday++; $mon++;
my $currdate = "$dow[$wday], $month[$mon] $mday, $year";


# Get current dir so know what CSS to display;
if ( getcwd() =~ /tcgi/ ){ # we are in tcgi
    $css = $tchcss;
    $homepage = $tchpage;
}


my $title = "$lex{'Common Math Assessment'} $lex{Report} 1";
print "$doctype\n<html><head><title>$title</title>\n";
print "<link rel=\"stylesheet\" href=\"$css\" type=\"text/css\">\n";
print "$chartype\n</head><body style=\"padding:1em 5em;\">\n";
print "<div>[ <a href=\"$homepage\">$lex{Main}</a> ]</div>\n";

print "<h1>$title</h1>\n";


if ( not $arr{page} ) {
    showStartPage();

} elsif ( $arr{page} == 1 ) {
    delete $arr{page};
    selectStudents();

} elsif ( $arr{page} == 2 ) {
    delete $arr{page};
    showReport();
}


#-------------
sub showReport {
#-------------

    # foreach my $key ( sort keys %arr ) { print "K:$key V:$arr{$key}<br>\n"; }
    
    my $dbase = $arr{db};
    delete $arr{db};

    my $dsn = "DBI:$dbtype:dbname=$dbase";
    my $dbh = DBI->connect($dsn,$guser,$gpassword);
    $dbh->{mysql_enable_utf8} = 1;

    print qq{<h2>$alldbase{$dbase}</h2>\n};


    my ($select, $selectval);
    if ( $arr{grade} ) {
	$select = "where grade = ?";
	$selectval = $arr{grade};
	print qq{<h3>$lex{Grade} $arr{grade}</h3>\n};


    } elsif ( $arr{homeroom} ) {
	$select = "where homeroom = ?";
	$selectval = $arr{homeroom};
	print qq{<h3>$lex{Homeroom} $arr{homeroom}</h3>\n};

    } else {
	print qq{<h3>$lex{'No Selection'}</h3>\n};
	print qq{</body></html>\n};
	exit;
    }

    my $sth = $dbh->prepare("select lastname, firstname, studnum from student 
       $select order by lastname, firstname");
    $sth->execute( $selectval );
    if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }

    my %data; # holds all data $data{studnum}{outcome}{prepost} = score;
    my @students;  # provides order of students
    my %outcomes; # lists outcomes.
    my %names; # store student names;
    my %tdates; # store test dates for outcomes and pre/post  $tdates{outcome}{pretest/posttest};

    while ( my $ref = $sth->fetchrow_hashref ) {
	my %r = %$ref;

	my $studnum = $r{studnum};
	push @students, $studnum; 
	$names{$r{studnum}} = "<b>$r{lastname}</b>, $r{firstname}";

	# Get this student's data
	my $sth1 = $dbh->prepare("select prepost, outcome, score, tdate from mathca_scores
           where schoolyear = ? and studnum = ? order by tdate");
	$sth1->execute( $arr{schoolyear}, $studnum );
	if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }

	while ( my ($prepost, $outcome, $score, $tdate) = $sth1->fetchrow ) {
	    $outcomes{$outcome} = 1;
	    $data{$studnum}{$outcome}{$prepost} = $score;
	    $tdates{$outcome}{$prepost} = $tdate;
	}
    }


    # print the data structure as a table;
    print qq{<table cellpadding="3" cellspacing="0" border="1">\n};
    print qq{<caption>Pretest / Posttest (Hover on Outcomes to see Dates)</caption>\n};

    # header section.
    print qq{<tr><th></th>};
    foreach my $outcome ( sort keys %outcomes ) {
	my $predate = $tdates{$outcome}{'pretest'};
	my $postdate = $tdates{$outcome}{'posttest'};
	print qq{<th title="Pre:$predate Post:$postdate">$outcome</th>};
    }
    print qq{</tr>\n};


    foreach my $studnum ( @students ) {
	print qq{<tr><td>$names{$studnum}</td>\n};
	foreach my $outcome ( sort keys %outcomes ) {
	    my $pretest = $data{$studnum}{$outcome}{'pretest'};
	    my $posttest = $data{$studnum}{$outcome}{'posttest'};

	    # Check for Exceptions
	    if ( not $pretest ) {
		# Get Date of test to match against ssp_exceptions entry
		my $tdate = $tdates{$outcome}{'pretest'};

		my $sth2 = $dbh->prepare("select * from ssp_exceptions
                  where ssptype = 'cma' and tdate = ? and studnum = ?");
		$sth2->execute($tdate, $studnum);
		my $ref = $sth2->fetchrow_hashref;
		
		if ( not $ref ) {
		    $pretest = qq{<span style="color:red;" title="$tdate Missing Entry; No Exception">M</span>};
		} else { # display the exception
		    $pretest = qq{<span style="color:blue;" title="$tdate Exception:$ref->{reasoncode}">E</span>};
		}
	    }

	    if ( not $posttest ) {
		# Get Date of test to match against ssp_exceptions entry
		my $tdate = $tdates{$outcome}{'posttest'};

		my $sth2 = $dbh->prepare("select * from ssp_exceptions
                  where ssptype = 'cma' and tdate = ?");
		$sth2->execute($tdate);
		my $ref = $sth2->fetchrow_hashref;

		if ( not $ref ) {
		    $posttest = qq{<span style="color:red;" title="$tdate Missing Entry; No Exception">M</span>};
		} else { # display the exception
		    $posttest = qq{<span style="color:blue;" title="$tdate Exception:$ref->{reasoncode}">E</span>};
		}
	    }


	    print qq{<td>$pretest/$posttest</td>};
	}
	print qq{</tr>\n};
    }

    print qq{</table>\n};
    print qq{<div>M = Missing Entry; E = Missing Entry, Exception reason added</div>\n};

    print qq{</body></html>\n};

    exit;


} # end of showReport



#----------------
sub showStartPage { # Entry Values for Custom Script
#----------------

    print "<form action=\"$self\" method=\"post\">\n";
    print "<input type=\"hidden\" name=\"page\" value=\"1\">\n";

    print "<table cellpadding=\"3\" cellspacing=\"0\" border=\"0\">\n";


    print qq{<tr><td class="bla">$lex{Select} $lex{School}</td></tr>\n};
    foreach my $db ( sort keys %alldbase ) { 
	print qq{<tr><td class="la">};
	print qq{<input type="radio" name="db" value="$db" $checked>\n};
	print qq{$alldbase{$db} ($db)</td></tr>\n};
    }


    print qq{<tr><td class="la"><input type="submit" value="$lex{Continue}"></td></tr>\n};
    print qq{</table>\n};
    print qq{</form></body></html>\n};

    exit;

}


#----------------
sub selectStudents { # Entry Values for Custom Script
#----------------

    # foreach my $key ( sort keys %arr ) { print "K:$key V:$arr{$key}<br>\n"; }
    
    my $dbase = $arr{db};

    print qq{<h3>$alldbase{$dbase}</h3>\n};


    my $dsn = "DBI:$dbtype:dbname=$dbase";
    my $dbh = DBI->connect($dsn,$guser,$gpassword);
    $dbh->{mysql_enable_utf8} = 1;


    my (@homerooms, @grades, @schoolyears );
    # Get Homerooms
    my $sth = $dbh->prepare("select distinct homeroom from student 
      where homeroom is not NULL and homeroom != ''");
    $sth->execute;
    if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }
    while ( my $hr = $sth->fetchrow ) {
	push @homerooms, $hr;
    }
    @homerooms = sort {$a <=> $b} @homerooms;

    # Get Grades
    $sth = $dbh->prepare("select distinct grade from student 
      where grade is not NULL and grade != ''");
    $sth->execute;
    if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }
    while ( my $gr = $sth->fetchrow ) {
	push @grades, $gr;
    }
    @grades = sort {$a <=> $b} @grades;

    # Get School Years
    $sth = $dbh->prepare("select distinct schoolyear from mathca_scores");
    $sth->execute;
    if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }
    while ( my $yr = $sth->fetchrow ) {
	push @schoolyears, $yr;
    }
    @schoolyears = reverse sort @schoolyears;



    print qq{<form action="$self" method="post">\n};
    print qq{<input type="hidden" name="page" value="2">\n};
    print qq{<input type="hidden" name="db" value="$dbase">\n};


    print "<table cellpadding=\"3\" cellspacing=\"0\" border=\"0\">\n";


    # Select Grade
    print qq{<tr><td class="bra">$lex{'Select by'} $lex{Grade}</td>};
    print qq{<td><select name="grade"><option></option>\n};
    foreach my $gr ( @grades ) {
	print qq{<option>$gr</option>\n};
    }
    print qq{</select></td></tr>\n};

    # OR
    print qq{<tr><td colspan="2" style="text-align:center;">$lex{OR}</td></tr>\n};


    # Select Homeroom
    print qq{<tr><td class="bra">$lex{'Select by'} $lex{Homeroom}</td>};
    print qq{<td><select name="homeroom"><option></option>\n};
    foreach my $hr ( @homerooms ) {
	print qq{<option>$hr</option>\n};
    }
    print qq{</select></td></tr>\n};


    # School Year
    print qq{<tr><td class="bra">$lex{'School Year'}</td>};
    print qq{<td><select name="schoolyear">\n};
    foreach my $yr ( @schoolyears ) {
	print qq{<option>$yr</option>\n};
    }
    print qq{</select></td></tr>\n};


    print qq{<tr><td class="ra"><input type="submit" value="$lex{Continue}"></td></tr>\n};
    print qq{</table>\n};
    print qq{</form></body></html>\n};

    exit;

} # end of selectStudents

