#! /usr/bin/perl
#  Copyright 2001-2014 Leslie Richardson

#  This file is part of Open Admin for Schools.

#  Open Admin for Schools is free software; you can redistribute it 
#  and/or modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2 of 
#  the License, or (at your option) any later version.

my %lex = ('Main' => 'Main',
	   'Continue' => 'Continue',
	   'Homeroom' => 'Homeroom',
	   'Grade' => 'Grade',
	   'Select by' => 'Select by',
	   'Error' => 'Error',
	   'Sort by' => 'Sort by',
	   'Name' => 'Name',

	   'Common Math Assessment' => 'Common Math Assessment',
	   'Report' => 'Report',
	   'OR' => 'OR',
	   'No Selection' => 'No Selection',
	   'School Year' => 'School Year',
	   'Schools' => 'Schools',
	   'Select' => 'Select',
	   'Start Date' => 'Start Date',
	   'End Date' => 'End Date',
	   'Check All' => 'Check All',

	   );

my $self = 'cmaRpt5.pl';

my %strandmap = ('N' => 'Number',
		 'P' => 'Patterns and Relations',
		 'SS' => 'Shape and Space',
		 'SP' => 'Statistics and Probability');


use DBI;
use CGI;
use Cwd;
use Number::Format qw(:all);


eval require "../admin.conf";
if ( $@ ) {
    print $lex{Error}. ": $@<br>\n";
    die $lex{Error}. ": $@\n";
}

eval { require "$globdir/global.conf"; };
if ( $@ ) {
    print $lex{Error}. " $self: $@<br>\n";
    die $lex{Error}. "$self: $@\n";
}

my @dbase = sort keys %alldbase; # global records.


my ($sec, $min, $hour, $mday, $mon, $year, $wday, $yday, $iddst) = localtime(time);
$year = $year + 1900;
$wday++; $mon++;
my $currdate = qq{$year-$mon-$mday};


# Get current dir so know what CSS to display;
if ( getcwd() =~ /tcgi/ ){ # we are in tcgi
    $css = $tchcss;
    $homepage = $tchpage;
}


my $q = CGI->new;
my %arr = $q->Vars;
print $q->header( -charset, $charset );


my $title = "$lex{'Common Math Assessment'} &mdash; $lex{Report} 5";
print "$doctype\n<html><head><title>$title</title>\n";
print "<link rel=\"stylesheet\" href=\"$css\" type=\"text/css\">\n";

=head
if ( not $arr{page} ) { # calendar popup.
    print "<link rel=\"stylesheet\" type=\"text/css\" media=\"all\" ";
    print "href=\"/js/calendar-blue.css\" title=\"blue\">\n";
    print "<script type=\"text/javascript\" src=\"/js/calendar.js\"></script>\n";
    print "<script type=\"text/javascript\" src=\"/js/lang/calendar-en.js\"></script>\n";
    print "<script type=\"text/javascript\" src=\"/js/calendar-setup.js\"></script>\n";
}
=cut

print "$chartype\n</head><body style=\"padding:1em 5em;\">\n";
print "<div>[ <a href=\"$homepage\">$lex{Main}</a> ]</div>\n";

print "<h1>$title</h1>\n";


if ( not $arr{page} ) {
    showStartPage();

} elsif ( $arr{page} == 1 ) {
    delete $arr{page};
    showReport();
}


#-------------
sub showReport {
#-------------

    # foreach my $key ( sort keys %arr ) { print "K:$key V:$arr{$key}<br>\n"; }

    my $schoolyear = $arr{schoolyear};
    delete $arr{schoolyear};


    # Load all outcomes (objectives) at top of loop, one time.
    my %outcomes;
    my %strands;  # {grade}{strand}{seq} = oid;


    my %summary;
    my $outcomeload = 1;


    foreach my $dbase ( sort keys %arr ) {

	my $dsn = "DBI:$dbtype:dbname=$dbase";
	my $dbh = DBI->connect($dsn,$guser,$gpassword);
	$dbh->{mysql_enable_utf8} = 1;


	if ( $outcomeload ) { # load from this database
	    my $sth = $dbh->prepare("select oid, odesc from mathca_outcomes");
	    $sth->execute;
	    if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }
	    while ( my ($oid, $odesc ) = $sth->fetchrow ) {
		my ($val, $seq) = split('\.', $oid);
		my $tempgr = $val;
		$tempgr =~ s/N|P|SS|SP//g; # strip strands to leave grade.
		$val =~ s/[0-9]|K//g; # strip the digit (the grade).
		$outcomes{$oid}{'desc'} = $odesc;
		$outcomes{$oid}{'strand'} = $val;
		$outcomes{$oid}{'seq'} = $seq;

		$strands{$tempgr}{$val}{$seq} = $oid;
		
	    }
	    $outcomeload = 0;
	}

	print qq{<h3>$alldbase{$dbase}</h3>\n};
	my %score;

	# my $sth1 = $dbh->prepare("select lastname, firstname, sex from studentall where studnum = ?");

	my $first = 1;

	$sth = $dbh->prepare("select * from mathca_scores where schoolyear = ?");
	$sth->execute( $schoolyear );
	if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }
	while ( my $ref  = $sth->fetchrow_hashref ) {

	    my %r = %$ref;
	    $first = 0;

	    # Don't need  Gender
	    #$sth1->execute( $r{studnum} );
	    #if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }
	    # my ($ln, $fn, $gender ) = $sth1->fetchrow;


	    $score{ $r{tgrade} }{$r{outcome} }{ $r{prepost} }{'count'}++;
	    $score{ $r{tgrade} }{$r{outcome} }{ $r{prepost} }{'sum'} += $r{score};

	    $summary{ $r{tgrade} }{ $r{outcome} }{ $r{prepost} }{'count'}++;
	    $summary{ $r{tgrade} }{ $r{outcome} }{ $r{prepost} }{'sum'} += $r{score};

	}


	if ( $first ) { # no records from this school
	    print qq{<div>No records from $alldbase{$dbase}</div>\n};

	} else { # print out the record

	    print qq{<table cellspacing="0" cellpadding="3" border="1">\n};
	    print qq{<tr><th>Grade</th><th>Strand</th><th title="Outcomes Done">Outc Done<br>Pre/Post</th>};
	    print qq{<th>Pretest<br>Average</th><th>Posttest<br>Average</th><th>Diff</th></tr>\n};


	    foreach my $grade ( sort {$a <=> $b} keys %strands ) {
		foreach my $strand ( sort keys %{ $strands{$grade} } ) {

		    my ($preaverage, $precount, $presum, $postaverage, $postcount, $postsum);

		    # Loop over each objective/outcome
		    foreach my $seq ( sort keys %{ $strands{$grade}{$strand} } ) {

			my $outcome = $strands{$grade}{$strand}{$seq};

			$precount += $score{$grade}{$outcome}{'pretest'}{'count'};
			$presum += $score{$grade}{$outcome}{'pretest'}{'sum'};

			$postcount = $score{$grade}{$outcome}{'posttest'}{'count'};
			$postsum = $score{$grade}{$outcome}{'posttest'}{'sum'};

		    }


		    if  ( $precount ) {
			$preaverage =  format_number( $presum / $precount, 2) ;
		    }
		    if  ( $postcount ) {
			$postaverage =  format_number( $postsum / $postcount, 2) ;
		    }

		    # print "PRE:$precount - $presum, POST: $postcount - $postsum<br>";


		    my $diff;
		    if ( $preaverage and $postaverage ) {
			$diff = format_number($postaverage - $preaverage, 2);
		    }

		    print qq{<tr><td>$grade</td><td>$strandmap{$strand} ($strand)</td>};
		    print qq{<td>$precount / $postcount</td><td>$preaverage</td>};
		    print qq{<td>$postaverage</td><td>$diff</td></tr>\n};

		    } # strand loop
		} # grade loop

	    print qq{</table>\n};

	} # end of else.

    } # end of database (school) loop



    # Now do the summary
    print qq{<h3>Summary Values</h3>\n};
    print qq{<table cellspacing="0" cellpadding="3" border="1">\n};

    print qq{<tr><th>Grade</th><th>Strand</th><th>Units Done<br>Pre/Post</th>};
    print qq{<th>Pretest<br>Average</th><th>Posttest<br>Average</th><th>Diff</th></tr>\n};

    
    foreach my $grade ( sort {$a <=> $b} keys %strands ) {
	foreach my $strand ( sort keys %{ $strands{$grade} } ) {

	    my ($preaverage, $precount, $presum, $postaverage, $postcount, $postsum);

	    # Loop over each objective/outcome
	    foreach my $seq ( sort keys %{ $strands{$grade}{$strand} } ) {

		my $outcome = $strands{$grade}{$strand}{$seq};

		$precount += $summary{$grade}{$outcome}{'pretest'}{'count'};
		$presum += $summary{$grade}{$outcome}{'pretest'}{'sum'};

		$postcount = $summary{$grade}{$outcome}{'posttest'}{'count'};
		$postsum = $summary{$grade}{$outcome}{'posttest'}{'sum'};

	    }


	    if  ( $precount ) {
		$preaverage =  format_number( $presum / $precount, 2) ;
	    }
	    if  ( $postcount ) {
		$postaverage =  format_number( $postsum / $postcount, 2) ;
	    }

	    # print "PRE:$precount - $presum, POST: $postcount - $postsum<br>";


	    my $diff;
	    if ( $preaverage and $postaverage ) {
		$diff = format_number($postaverage - $preaverage, 2);
	    }

	    print qq{<tr><td>$grade</td><td>$strandmap{$strand} ($strand)</td>};
	    print qq{<td>$precount / $postcount</td><td>$preaverage</td>};
	    print qq{<td>$postaverage</td><td>$diff</td></tr>\n};

	} # strand loop
    } # grade loop

    print qq{</table>\n};


=head


#----------------------------
    print qq{<tr><th>Grade</th><th>Gender</th><th>Outcome</th><th>Number<br>Pre/Post</th>};
    print qq{<th>Pretest<br>Average</th><th>Posttest<br>Average</th><th>Diff</th></tr>\n};


    foreach my $grade ( sort {$a <=> $b} keys %summary ) {
	foreach my $gender ( sort keys %{ $summary{$grade} } ) {
	    foreach my $outcome ( sort keys %{ $summary{$grade}{$gender} } ) {

		my $preaverage; 
		my $precount = $summary{$grade}{$gender}{$outcome}{'pretest'}{'count'};
		my $presum = $summary{$grade}{$gender}{$outcome}{'pretest'}{'sum'};
		if  ( $precount ) {
		    $preaverage =  format_number( $presum / $precount, 2) ;
		}


		my $postaverage;
		my $postcount = $summary{$grade}{$gender}{$outcome}{'posttest'}{'count'};
		my $postsum = $summary{$grade}{$gender}{$outcome}{'posttest'}{'sum'};
		if  ( $postcount ) {
		    $postaverage =  format_number( $postsum / $postcount, 2) ;
		}

		my $diff;
		if ( $preaverage and $postaverage ) {
		    $diff = $postaverage - $preaverage;
		}

		print qq{<tr><td>$grade</td><td>$gender</td><td>$outcome</td>};
		print qq{<td>$precount / $postcount</td><td>$preaverage</td>};
		print qq{<td>$postaverage</td><td>$diff</td></tr>\n};
	    }
	}
    }
=cut


    print "</table>\n";


    print qq{</body></html>\n};

    exit;


} # end of showReport




#----------------
sub showStartPage { # Entry Values for Custom Script
#----------------

    my @schoolyears = qw(2014 2015 2016 2017);

    print qq{<form action="$self" method="post" style="display:inline;margin:1em;">\n};
    print qq{<input type="hidden" name="check" value="1">\n};
    print qq{<input type="submit" value="$lex{'Check All'}" style="display:inline;"></form>\n};

    my $checked;
    if ( $arr{check} ) {
	$checked = qq{checked="checked"};
    }



    print "<form action=\"$self\" method=\"post\">\n";
    print "<input type=\"hidden\" name=\"page\" value=\"1\">\n";

    print "<table cellpadding=\"3\" cellspacing=\"0\" border=\"0\">\n";


    # School Year
    print qq{<tr><td class="bra">$lex{'School Year'}</td>};
    print qq{<td><select name="schoolyear">\n};
    foreach my $yr ( @schoolyears ) {
	print qq{<option>$yr</option>\n};
    }
    print qq{</select></td></tr>\n};

    # Pre/Post Test
#    print qq{<tr><td class="bra">Test Type</td><td class="la">};
#    print qq{<select name="prepost"><option value="pretest">Pretest</option>\n};
#    print qq{<option value="posttest">Posttest</option></select></td></tr>\n};



    print qq{<tr><td class=\"bra\">$lex{Select} $lex{Schools}</td><td></td></tr>\n};
    foreach my $db ( @dbase ) { 
	print qq{<tr><td></td><td class=\"la\">};
	print qq{<input type="checkbox" name="$db" value="1" $checked>\n};
	print qq{$alldbase{$db} ($db)</td></tr>\n};
    }


    print qq{<tr><td class="ra"><input type="submit" value="$lex{Continue}"></td></tr>\n};
    print qq{</table>\n};
    print qq{</form></body></html>\n};

    exit;

}




#----------------
sub oldshowStartPage {
#----------------

    my $checked = qq{checked="checked"};

    # Start Date
    my $startdate;
    my ( $y,$m,$d) = split('-', $currdate);

    my $year; 
    if ( $m > 7 ) { $year = $y; } else { $year = $y - 1; } # prev year
    $startdate = qq{$year-08-01};


    print "<form action=\"$self\" method=\"post\">\n";
    print "<input type=\"hidden\" name=\"page\" value=\"1\">\n";

    print "<table cellpadding=\"3\" cellspacing=\"0\" border=\"0\" ";
    print "style=\"padding:0.5em;border:1px solid gray;\">\n";

    print qq{<tr><td class=\"bra\">$lex{Select} $lex{Schools}</td><td></td></tr>\n};
    foreach my $db ( @dbase ) { 
	print qq{<tr><td></td><td class=\"la\">};
	print qq{<input type="checkbox" name="$db" value="1" $checked>\n};
	print qq{$alldbase{$db} ($db)</td></tr>\n};
    }

    # Start Date
    print qq{<tr><td class="bra">$lex{'Start Date'}</td>};
    print qq{<td class="la"><input type="text" name="startdate" id="sdate" size="10" value="$startdate">};
    print qq{<button type="reset" id="start_trigger">...</button></td></tr>\n};


    # End Date
    print "<tr><td class=\"bra\">$lex{'End Date'}</td><td class=\"la\"><input type=\"text\" ";
    print "name=\"enddate\" id=\"edate\" size=\"10\" value=\"$currdate\">";
    print "<button type=\"reset\" id=\"end_trigger\">...</button></td></tr>\n";

    # Pre/Post Test
    print qq{<tr><td class="bra">Test Type</td><td class="la">};
    print qq{<select name="prepost"><option value="pretest">Pretest</option>\n};
    print qq{<option value="posttest">Posttest</option></select></td></tr>\n};


    print "<tr><td></td><td class=\"la\"><input type=\"submit\" value=\"$lex{Continue}\"></td></tr>\n";
    print "</table></form>\n";

    print "<script type=\"text/javascript\">
     Calendar.setup({
        inputField     :    \"sdate\", 
        ifFormat       :    \"%Y-%m-%d\",
        button         :    \"start_trigger\",
        singleClick    :    false,
        step           :    1
    });

    Calendar.setup({
        inputField     :    \"edate\",
        ifFormat       :    \"%Y-%m-%d\",
        button         :    \"end_trigger\",
        singleClick    :    false,
        step           :    1
    });
    </script>\n";

    print "</body></html>\n";

    exit;

} # end of showStartPage
