#! /usr/bin/perl
#  Copyright 2001-2015 Leslie Richardson

#  This file is part of Open Admin for Schools.

#  Open Admin for Schools is free software; you can redistribute it 
#  and/or modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2 of 
#  the License, or (at your option) any later version.

my %lex = ('Main' => 'Main',
	   'Continue' => 'Continue',
	   'Homeroom' => 'Homeroom',
	   'Grade' => 'Grade',
	   'Select by' => 'Select by',
	   'Error' => 'Error',
	   'Sort by' => 'Sort by',
	   'Name' => 'Name',

	   'Common Math Assessment' => 'Common Math Assessment',
	   'Report' => 'Report',
	   'OR' => 'OR',
	   'No Selection' => 'No Selection',
	   'School Year' => 'School Year',
	   'Show Withdrawn' => 'Show Withdrawn',

	   );

my $self = 'cmaRpt6.pl';

use DBI;
use CGI;
use Cwd;
use Number::Format qw(:all);

my @strands = qw(P N SS SP);
my %strandnames = ('P' => 'Patterns and Relations', 
		   'N' => 'Numbers and Operations',
		   'SS' => 'Shape and Space',
		   'SP' => 'Stats and Prob'
    );

%yearname = ('2014' => '2013-2014',
	     '2015' => '2014-2015',
	     '2016' => '2015-2016',
    );


eval require "../admin.conf";
if ( $@ ) {
    print $lex{Error}. " $@<br>\n";
    die $lex{Error}. " $@\n";
}

my @dbase = sort keys %alldbase; # global records.


my $q = new CGI;
print $q->header( -charset, $charset );
my %arr = $q->Vars;


my ($sec, $min, $hour, $mday, $mon, $year, $wday, $yday, $iddst) = localtime(time);
$year = $year + 1900;
$wday++; $mon++;
my $currdate = "$dow[$wday], $month[$mon] $mday, $year";

my $dsn = "DBI:$dbtype:dbname=$dbase";
my $dbh = DBI->connect($dsn,$user,$password);
$dbh->{mysql_enable_utf8} = 1;



my $title = "$lex{'Common Math Assessment'} $lex{Report} 6";
print "$doctype\n<html><head><title>$title</title>\n";
print "<link rel=\"stylesheet\" href=\"$css\" type=\"text/css\">\n";
print "$chartype\n</head><body style=\"padding:1em 5em;\">\n";
print "<div>[ <a href=\"$homepage\">$lex{Main}</a> ]</div>\n";

print "<h1>$title</h1>\n";

print qq{<p>CMA total Scores for all schools for all students for all
outcomes, broken into quartile scores (1-4) and including student counts</p>\n};

showReport();




#-------------
sub showReport {
#-------------

    # foreach my $key ( sort keys %arr ) { print "K:$key V:$arr{$key}<br>\n"; }

    my %quartile;  # $quartile{schoolyear}{quartile}
    my %yearcount; # yearcount{schoolyear} = # of kids.

    foreach my $schoolyear ( '2014','2015' ) { 

	foreach my $dbase ( sort keys %alldbase ) {

	    my $dsn = "DBI:$dbtype:dbname=$dbase";
	    my $dbh = DBI->connect($dsn,$guser,$gpassword);
	    $dbh->{mysql_enable_utf8} = 1;
	
	    # print qq{<h2>$alldbase{$dbase}</h2>\n};

	    my $sth = $dbh->prepare("select score from mathca_scores where schoolyear = ? 
              and score is not NULL and score != ''");
	    $sth->execute( $schoolyear);
	    if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; };
	    while ( my  $score = $sth->fetchrow ) {
		$quartile{$schoolyear}{$score}++;
	    }

	    my $sth1 = $dbh->prepare("select count(distinct studnum) from mathca_scores where schoolyear = ?");
	    $sth1->execute( $schoolyear);
	    if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; };
	    my  $studcount = $sth1->fetchrow;
	    $yearcount{$schoolyear} += $studcount;

	}
    }


    print qq{<table cellpadding="3" cellspacing="0" border="1">\n};
    print qq{<tr><th>School Year</th><th>Score 1</th><th>Score 2</th>};
    print qq{<th>Score 3</th><th>Score 4</th><th>Student #</th></tr>\n};

    foreach my $schoolyear ( sort keys %quartile ) {
	print qq{<tr><td>$yearname{$schoolyear} ($schoolyear)</td>\n};

	# Get Total Count for this year.
	my $total;
	foreach my $score ( sort keys %{ $quartile{$schoolyear} } ) {
	    $total += $quartile{$schoolyear}{$score};
	}


	foreach my $score ( sort keys %{ $quartile{$schoolyear} } ) {
	    my $count = $quartile{$schoolyear}{$score};
	    my $percent;
	    if ( $total ) {
		$percent = format_number( $count * 100 / $total, 2, 2);
	    }
	    print qq{<td>$count ($percent%)</td>\n};
	}
	print qq{<td>$yearcount{$schoolyear}</td></tr>\n};
    }

    print qq{</table>\n};
    print qq{</body></html>\n};

    return;

} # end of showReport



#----------------
sub showStartPage { # Entry Values for Custom Script
#----------------

    my @schoolyears = qw(2014 2015 2016 2017);

    my $checked = qq{checked="checked"};


    print "<form action=\"$self\" method=\"post\">\n";
    print "<input type=\"hidden\" name=\"page\" value=\"1\">\n";

    print "<table cellpadding=\"3\" cellspacing=\"0\" border=\"0\">\n";


    # School Year
    print qq{<tr><td class="bra">$lex{'School Year'}</td>};
    print qq{<td><select name="schoolyear">\n};
    foreach my $yr ( @schoolyears ) {
	print qq{<option>$yr</option>\n};
    }
    print qq{</select></td></tr>\n};

    # Pre/Post Test
#    print qq{<tr><td class="bra">Test Type</td><td class="la">};
#    print qq{<select name="prepost"><option value="pretest">Pretest</option>\n};
#    print qq{<option value="posttest">Posttest</option></select></td></tr>\n};



    print qq{<tr><td class=\"bra\">$lex{Select} $lex{Schools}</td><td></td></tr>\n};
    foreach my $db ( @dbase ) { 
	print qq{<tr><td></td><td class=\"la\">};
	print qq{<input type="checkbox" name="$db" value="1" $checked>\n};
	print qq{$alldbase{$db} ($db)</td></tr>\n};
    }


    print qq{<tr><td class="ra"><input type="submit" value="$lex{Continue}"></td></tr>\n};
    print qq{</table>\n};
    print qq{</form></body></html>\n};

    exit;

}

