#! /usr/bin/perl
#  Copyright 2001-2015 Leslie Richardson

#  This file is part of Open Admin for Schools.

#  Open Admin for Schools is free software; you can redistribute it 
#  and/or modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2 of 
#  the License, or (at your option) any later version.


my $self = 'compareconf.pl';

use DBI;
use CGI;


eval require "../admin.conf";
if ( $@ ) {
    print "Error: $@<br>\n";
    die "Error: $@\n";
}

my $q = new CGI;
print $q->header( -charset, $charset );
my %arr = $q->Vars;



my $title = "Compare Configuration Systems";
print "$doctype\n<html><head><title>$title</title>\n";
print "<link rel=\"stylesheet\" href=\"$css\" type=\"text/css\">\n";
print "$chartype\n</head><body style=\"padding:1em 5em;\">\n";
print "<div>[ <a href=\"$homepage\">Main</a> ]</div>\n";

print "<h1>$title</h1>\n";


if ( not $arr{page} ) {
    showStartPage();

} elsif ( $arr{page} == 1 ) {
    delete $arr{page};
    compareConfSystem();
}


#--------------------
sub compareConfSystem {
#--------------------

    # foreach my $key ( sort keys %arr ) { print "K:$key V:$arr{$key}<br>\n"; }
    

    # This is the base DB that others are compared to.
    my $dbase = $arr{db};

    my $dsn1 = "DBI:$dbtype:dbname=$dbase";
    my $dbh1 = DBI->connect($dsn1,$guser,$gpassword);
    $dbh->{mysql_enable_utf8} = 1;

    my $first = 1; # for displaying the keys


    foreach my $db2 ( sort keys %alldbase ) { 

	if ( $db2 eq $dbase ) { next; } # skip reference school


	my $dsn2 = "DBI:$dbtype:dbname=$db2";
	my $dbh2 = DBI->connect($dsn2,$guser,$gpassword);


	# Get  conf_system data - db1
	my (%conf1, %confdata1 );

	my $sth = $dbh1->prepare("select * from conf_system");
	$sth->execute;
	if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
	while ( my $ref = $sth->fetchrow_hashref ) {
	    $conf1{$ref->{dataname}} = $ref->{filename};
	    $confdata1{$ref->{dataname}} = $ref;
	}


	# Get  conf_system data - db2
	my (%conf2, %confdata2 );

	my $sth = $dbh2->prepare("select * from conf_system");
	$sth->execute;
	if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
	while ( my $ref = $sth->fetchrow_hashref ) {
	    $conf2{$ref->{dataname}} = $ref->{filename};
	    $confdata2{$ref->{dataname}} = $ref;
	}


	# Assemble master hash to iterate over
	my %master = %conf1;
	foreach my $key ( keys %conf2 ) {
	    if ( not $master{$key} ) {
		$master{$key} = $tempconf{$key};
	    }
	}


	my $count = 1;
	# Loop over all hash values
	foreach my $key ( sort keys %master ) {
	    if ( $conf1{$key} eq $conf2{$key} ) {

		# These are the variable names.
#		if ( $first ) {
#		    print qq{<div>$count. <b>$key</b></div>\n};
#		    $count++;
#		}

		# Compare sectionname, sequenceval, datatype
		if ( $confdata1{$key}->{sectionname} ne $confdata2{$key}->{sectionname} ) {
		    print qq{<div style="color:red;">$conf1{$key}: Sectionname differs: DB1: $confdata1{$key}->{sectionname} DB2: $confdata2{$key}->{sectionname}</div>\n};
		}
		if ( $confdata1{$key}->{sequenceval} ne $confdata2{$key}->{sequenceval} ) {
		    print qq{<div style="color:red;">$conf1{$key}: SequenceVal differs: DB1: $confdata1{$key}->{sequenceval} DB2: $confdata2{$key}->{sequenceval}</div>\n};
		}
		if ( $confdata1{$key}->{datatype} ne $confdata2{$key}->{datatype} ) {
		    print qq{<div style="color:red;">$conf1{$key}: Datatype differs: DB1: $confdata1{$key}->{datatype} DB2: $confdata2{$key}->{datatype}</div>\n};
		}

		# Remove identical records
		delete $conf1{$key};
		delete $conf2{$key};
	    }
	}

	$first = 0; # only print keys once.


	if ( %conf1 ) {
	    print "\n<div>Reference Remaining Configuration Keys:<br>\n";
	    foreach my $key ( sort keys %conf1 ) {
		print "<b>$key</b><br>\n";
	    }
	    print qq{</div>\n};
	} else {
	    print "<p>Reference (DB1) No Remaining Configuration Keys</p>\n";
	}
	

	if ( %conf2 ) {
	    print "\n<div><b>$alldbase{$db2}</b> ($db2) Remaining Configuration Keys:<br>\n";
	    foreach my $key ( sort keys %conf2 ) {
		print "<b>$key</b><br>\n";
	    }
	    print qq{</div>\n};
	} else {
	    print "<p><b>$alldbase{$db2}</b> ($db2) No Remaining Configuration Keys</p>\n";
	}

	print qq{<hr>\n};


    } # end of Database Loop
#    print qq{<table cellpadding="3" cellspacing="0" border="1">\n};
#    print qq{<caption>Pretest / Posttest (Hover on Outcomes to see Dates)</caption>\n};
#    print qq{</table>\n};


    print qq{</body></html>\n};

    exit;


} # end of compareConfSystem



#----------------
sub showStartPage { # Select Base School Database for Comparison
#----------------

    print "<form action=\"$self\" method=\"post\">\n";
    print "<input type=\"hidden\" name=\"page\" value=\"1\">\n";

    print "<table cellpadding=\"3\" cellspacing=\"0\" border=\"0\">\n";


    print qq{<tr><td class="bla">Select Base School for Comparison</td></tr>\n};
    foreach my $db ( sort keys %alldbase ) { 
	print qq{<tr><td class="la">};
	print qq{<input type="radio" name="db" value="$db" $checked>\n};
	print qq{$alldbase{$db} ($db)</td></tr>\n};
    }


    print qq{<tr><td class="la"><input type="submit" value="Continue"></td></tr>\n};
    print qq{</table>\n};
    print qq{</form></body></html>\n};

    exit;

}
