#!/usr/bin/perl
#  Copyright 2001-2014 Leslie Richardson

#  This file is part of Open Admin for Schools.

# Read Report - reference script that will read student data into data
# structure and then report by student, grade, or school. There can
# then be a variety of reports from this.


my %lex = ('Error' => 'Error',
	   'Main' => 'Main',
	   'Grade' => 'Grade',
	   'Blank=All' => 'Blank=All',
	   'Start Date' => 'Start Date',
	   'End Date' => 'End Date',
	   'Split' => 'Split',
	   'Date' => 'Date',
	   'Continue' => 'Continue',
	   'Mean' => 'Mean',
	   'Median' => 'Median',
	   'Range' => 'Range',
	   'StdDev' => 'StdDev',
	   'Previous' => 'Previous',
	   'Current' => 'Current',
	   'School' => 'School',
	   'Database' => 'Database',
	   'Reading' => 'Reading',
	   'Report' => 'Report',

	   );


my $self = 'rdRptRefGlobal.pl';

# Base Values for RL, irrespective of Kits
my %graphbasescore = qw(A 0  1 3.33  2 6.66  3 13.33  4 20  6 30  8 40  10 50  12 60  14 70  16 80  18 90  20 100
  24 110  28 120  30 130  34 140  38 150  40 120  50 120  60 120  70 120  80 120);

# RL Score Expectations, based on grade and part of year. 
my %gradeExpect = ('K-1' => 0, 'K-2' => 3.33, 'K-3' => 16.66, '1-1' => 16.66, '1-2' => 53.33, '1-3' => 83.33,
		   '2-1' => 83.33, '2-2' => 103.33, '2-3' => 123.33, '3-1' => 123.33, '3-2' => 140, '3-3' => 153.33,
		   '4-1' => 170, '4-2' => 173.33, '5-1' => 180, '5-2' => 183.33, '6-1' => 190, '6-2' => 193.33, 
		   '7-1' => 200, '7-2' => 203.33, '8-1' => 210, '8-2' => 213.33 );

use GD::Graph::bars;
use GD::Graph::colour qw(:colours);
use DBI;
use CGI;
use Cwd;
use Number::Format qw(:all);
use Time::JulianDay;

eval require "admin.conf";
if ( $@ ) {
    print $lex{Error}. ": $@<br>\n";
    die $lex{Error}. ": $@\n";
}

eval { require "$globdir/global.conf"; };
if ( $@ ) {
    print $lex{Error}. " $self: $@<br>\n";
    die $lex{Error}. "$self: $@\n";
}

@dbase = sort keys %alldbase; # global records.


my @time = localtime(time);
my $year = $time[5] + 1900;
my $month = $time[4] + 1;
my $currdate = "$year-$month-$time[3]";
my $schyear = $year;
if ( $month < 7 ){ $schyear = $schyear - 1; }
my $prevyear = $schyear - 1;

my $currstartdate = "$schyear-08-01"; # Aug 1 of this school year
my $prevstartdate = "$prevyear-08-01"; # August 1 of prev year.


my $q = CGI->new;
my %arr = $q->Vars;
print $q->header( -charset, $charset );

my ($dsn, $dbh );
if ( $arr{db} ) {
    my $database = $arr{db};
    $dsn = "DBI:$dbtype:dbname=$database";
    $dbh = DBI->connect($dsn,$guser,$gpassword);
}
my $schoolname = $alldbase{"$arr{db}"};

# Script Heading Descriptor
my $scriptheading = "$schoolname $lex{Reading} $lex{Report}";

# Print Page Heading
print "$doctype\n<html><head><title>$scriptheading</title>\n";
print "<link rel=\"stylesheet\" href=\"/division.css\" type=\"text/css\">\n";

if ( not $arr{page} ) { # calendar popup.
    print "<link rel=\"stylesheet\" type=\"text/css\" media=\"all\" ";
    print "href=\"/js/calendar-blue.css\" title=\"blue\">\n";
    print "<script type=\"text/javascript\" src=\"/js/calendar.js\"></script>\n";
    print "<script type=\"text/javascript\" src=\"/js/lang/calendar-en.js\"></script>\n";
    print "<script type=\"text/javascript\" src=\"/js/calendar-setup.js\"></script>\n";
}

print "$chartype\n</head><body style=\"padding:0.4em 2em;\">\n";
print "[ <a class=\"alt\" href=\"$homepage\">$lex{Main}</a> ]\n";
print "<h1>$scriptheading</h1>\n";

if ( not $arr{page} ) {
    showStartPage();

} elsif ( $arr{page} == 1 ) {
    delete $arr{page};
    assembleData();
}



#---------------
sub assembleData {
#---------------

#    foreach my $key ( sort keys %arr ) { print "K:$key V:$arr{$key}<br>\n"; }

#    my @grades = ();
#    if ( $arr{grade} ) {
#	@grades = split(/\s/, $arr{grade});
#    }
#    my $currgrade = $grades[0];  # for now, only single grade.


    # Group the tests into date ranges: Before, Prev, Curr, After
    # based on values in startdate, splitdate, enddate.
    my $startjd = julian_day( split(/-/,$arr{startdate}) );
    my $endjd = julian_day( split(/-/,$arr{enddate}) );
    my $splitjd = julian_day( split(/-/,$arr{splitdate}) );
    my @before = ();
    my @prev = ();
    my @curr = ();
    my @after = ();
    my %prev = ();
    my %curr = ();

    $sth = $dbh->prepare("select id, tdate, tgrade from read_test order by tdate"); 
    $sth->execute;
    if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }

    while ( my ( $id, $tdate, $tgrade ) = $sth->fetchrow ) {

	my $tdatejd = julian_day( split(/-/,$tdate) );
	if ( $tdatejd < $startjd ) {
	    push @before, $id;
	} elsif ( $tdatejd >= $startjd and $tdatejd < $splitjd ) {
	    push @prev, $id;
	    push @{$prev{$tgrade}},$id; 
	} elsif ( $tdatejd >= $splitjd and $tdatejd <= $endjd ) {
	    push @curr, $id;
	    push @{$curr{$tgrade}},$id; 
	} elsif ( $tdatejd > $endjd ) {
	    push @after, $id;
	}
    }
    # We now have all test id's in 4 arrays; @prev and @curr are the ones of interest.

    foreach my $gr ( sort {$a <=> $b} keys %prev ) {

	print "<div style=\"border:2px solid gray;background-color:#DDD;padding:0 0.5em 1em;margin:0.5em;\">";
	print "<h3>Reading Scores - Grade: $gr</h3>\n";
        print "<table cellpadding=\"3\" cellspacing=\"0\" border=\"1\">\n";
	
	print "<tr><td></td><td colspan=\"4\" class=\"bcn\">Reading Level Scores</td>\n";
	print "<td colspan=\"4\" class=\"bcn\">Normalized Scores</td></tr>\n";
	
	print "<tr><th></th><th>$lex{Median}</th><th>$lex{Range}</th>";
	print "<th>$lex{Mean}</th><th>$lex{StdDev}</th>\n";
	print "<th>$lex{Median}</th><th>$lex{Range}</th>";
	print "<th>$lex{Mean}</th><th>$lex{StdDev}</th></tr>\n";

	print "<tr><td>$lex{Previous}</td>";
	showScores( @{ $prev{$gr} } );
	print "</tr>\n";

#~~	print "Separator<br>\n";

	print "<tr><td>$lex{Current}</td>";
	showScores( @{ $curr{$gr} } );
	print "</tr>\n";

	print "</table></div>\n";

    }



    # Overall Stats

    print "<h2>Summary Stats</h2>\n";
    print "<h3>Previous Year Scores</h3>\n";
    print "<table cellpadding=\"3\" cellspacing=\"0\" border=\"1\">\n";

    print "<tr><td colspan=\"4\" class=\"bcn\">Reading Level Scores</td>\n";
    print "<td colspan=\"4\" class=\"bcn\">Normalized Scores</td></tr>\n";

    print "<tr><th>$lex{Median}</th><th>$lex{Range}</th>";
    print "<th>$lex{Mean}</th><th>$lex{StdDev}</th>\n";
    print "<th>$lex{Median}</th><th>$lex{Range}</th>";
    print "<th>$lex{Mean}</th><th>$lex{StdDev}</th></tr>\n";

    showScores( @prev );
    print "</table>\n";

    print "<h3>Current Year Scores</h3>\n";

    print "<table cellpadding=\"3\" cellspacing=\"0\" border=\"1\">\n";
    print "<tr><td colspan=\"4\" class=\"bcn\">Reading Level Scores</td>\n";
    print "<td colspan=\"4\" class=\"bcn\">Normalized Scores</td></tr>\n";

    print "<tr><th>$lex{Median}</th><th>$lex{Range}</th>";
    print "<th>$lex{Mean}</th><th>$lex{StdDev}</th>\n";
    print "<th>$lex{Median}</th><th>$lex{Range}</th>";
    print "<th>$lex{Mean}</th><th>$lex{StdDev}</th></tr>\n";

    showScores( @curr );
    print "</table>\n";


    print "</body></html>\n";

    exit;

}


#-------------
sub showScores {
#-------------

    my @val = @_;

    # Previous Values
    my @rlscore = ();
    my @normscore = ();
    foreach my $id ( @val ) {
	my ( $rlscore, $normscore ) = split(/:/, findScores( $id ));

	my $sth = $dbh->prepare("select readlevel, tgrade, tdate from read_test where id = ? ");
	$sth->execute( $id );
	if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
	my ( $readlevel, $tgrade, $tdate ) = $sth->fetchrow;
#	print "RL:$readlevel  ID:$id GR:$tgrade \n";
#	print " RLScore:$rlscore  NormScore:$normscore<br>\n";
#~~

	push @rlscore, $rlscore;
	push @normscore, $normscore;
    }
    @rlscore = sort { $a <=> $b } @rlscore;
    @normscore = sort { $a <=> $b } @normscore;

    # Reading Level Scores
    my $result = calcStats( @rlscore );
    @rslt = split(/:/, $result);
    foreach my $v ( @rslt ) {
	$v = format_number($v, 2, 2);
	print "<td>$v</td>";
    }

    # Normalized Scores
    my $result = calcStats( @normscore );
    @rslt = split(/:/, $result);
    foreach my $v ( @rslt ) {
	$v = format_number($v, 2, 2);
	print "<td>$v</td>";
    }
}



#------------
sub calcStats {
#------------

    my @val = @_;

    my ( $median, $range, $mean, $sd );

    my $count = scalar @val;
    if ( not $count ) { return undef; }

    # Find median.
    if ( $count % 2 ) { # we have an odd number of elements
	$median = $val[ int($count/2) ];
    } else { # even number of elements
	$median = ( $val[ $count/2 ] + $val[ $count/2 - 1 ] ) / 2;
    }

    # Find Mean (Average)
    my $sum;
    foreach ( @val ) { $sum += $_; }
    $mean = $sum / $count;

    # Find range
    $range = $val[-1] - $val[0];

    # Find SD;
    my $sos; # sum of squares;
    foreach my $v ( @val ) {
	$sos += ($v - $mean)**2;
    }
    if ( $count > 1 ) {
	$sd = sqrt( $sos / ($count - 1) );
    } else {
	$sd = 0;
    }

    return "$median:$range:$mean:$sd";



}



#----------------
sub showStartPage {
#----------------

    print "<form action=\"$self\" method=\"post\">\n";
    print "<input type=\"hidden\" name=\"page\" value=\"1\">\n";

    print "<table cellpadding=\"3\" cellspacing=\"0\" border=\"0\" ";
    print "style=\"padding:0.5em;border:1px solid gray;\">\n";

    print "<tr><td class=\"bra\">$lex{School} $lex{Database}</td><td class=\"la\">";
    print "<select name=\"db\"><option></option>\n";
    foreach my $db ( @dbase ) { print "<option>$db</option>"; }
    print "</select></td></tr>\n";


    print "<tr><td class=\"bra\">$lex{'Start Date'}</td><td class=\"la\"><input type=\"text\" ";
    print "name=\"startdate\" id=\"sdate\" size=\"10\" value=\"$prevstartdate\">";
    print "<button type=\"reset\" id=\"start_trigger\">...</button>\n";
    print "</td></tr>\n";

    print "<tr><td class=\"bra\">$lex{Split} $lex{Date}</td><td class=\"la\"><input type=\"text\" ";
    print "name=\"splitdate\" id=\"splitdate\" size=\"10\" value=\"$currstartdate\">";
    print "<button type=\"reset\" id=\"split_trigger\">...</button>\n";
    print "</td></tr>\n";

    print "<tr><td class=\"bra\">$lex{'End Date'}</td><td class=\"la\"><input type=\"text\" ";
    print "name=\"enddate\" id=\"edate\" size=\"10\" value=\"$currdate\">";
    print "<button type=\"reset\" id=\"end_trigger\">...</button></td></tr>\n";

    print "<tr><td></td><td class=\"la\"><input type=\"submit\" value=\"$lex{Continue}\"></td></tr>\n";
    print "</form>\n";

    print "<script type=\"text/javascript\">
     Calendar.setup({
        inputField     :    \"sdate\", 
        ifFormat       :    \"%Y-%m-%d\",
        button         :    \"start_trigger\",
        singleClick    :    false,
        step           :    1
    });

     Calendar.setup({
        inputField     :    \"splitdate\", 
        ifFormat       :    \"%Y-%m-%d\",
        button         :    \"split_trigger\",
        singleClick    :    false,
        step           :    1
    });

    Calendar.setup({
        inputField     :    \"edate\",
        ifFormat       :    \"%Y-%m-%d\",
        button         :    \"end_trigger\",
        singleClick    :    false,
        step           :    1
    });
    </script>\n";

    print "</body></html>\n";

    exit;

} # end of showStartPage



#----------
sub calcAge {
#----------

    # Passed (birthdate, $currdate)
    my ( $birthdate, $currdate ) = @_;
    my ($byear,$bmonth,$bday) = split /-/,$birthdate;
    my ($cyear,$cmonth,$cday) = split /-/,$currdate;
    my $age = $cyear - $byear;
    my $month = $cmonth - $bmonth;
    #print "BD: $birthdate CD: $currdate Age: $age MO:$month<br>\n";

    if ($cmonth < $bmonth){
	$month = $month + 12;
	if ($cday < $bday){ $month--;}
	$age--;
    }
    elsif ($cmonth == $bmonth and $cday < $bday){
	$age--; 
	$month = 11;
    } elsif ($cmonth > $bmonth and $cday < $bday) {
	$month--;
    }

    if ( $age < 0 or $age > 100 ) { $age = 0; $month = 0; }

    return "$age:$month";

}

#------------
sub findScores {
#------------

    # passed test id, return rlscore, and normscore relative to expectations.

    my $id = shift;

    my $sth = $dbh->prepare("select readlevel, tgrade, tdate from read_test where id = ? ");
    $sth->execute( $id );
    if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
    my ( $readlevel, $tgrade, $tdate ) = $sth->fetchrow;
    my $rl = $readlevel;
    if ( $readlevel ne 'A' ) { 
	$readlevel =~ s/[A-Za-z]//g; # strip trailing letters.
    }

    my ($tyear, $tmonth, $tday ) = split(/-/, $tdate);
    $tmonth =~ s/^0//; # strip 0.
    my $blockSelect;

    if ( $readlevel > 38 ) { # 4-8 kit, 2 period/year
	my $block;
	if ( $tmonth > 7 or $tmonth < 2 ) { # Aug to end Jan
	    $block = 1;
	} else { 
	    $block = 2;
	}
	$blockSelect = $tgrade. q{-}. $block;
    } else { # k-3 kit 3 periods /year
	my $block;
	if ( $tmonth > 7 and $tmonth < 12 ) { # Aug to end Nov
	    $block = 1;
	} elsif ( $tmonth < 4 or $tmonth > 11 ) { # Aug to end Nov { 
	    $block = 2;
	} else {
	    $block = 3;
	}
	$blockSelect = $tgrade. q{-}. $block;
    }

    my $expect = $gradeExpect{ $blockSelect };

    # Now get the score
    my $sth = $dbh->prepare("select sum(score) from read_test_score 
     where testid = ?");
    $sth->execute( $id );
    if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
    my $sum = $sth->fetchrow;

    my $score;
    if ( $readlevel < 40 ) { # K-3 kit

	# get average of each category and add. It will be out of 12 (3 categories; max 4)
	my $sth1 = $dbh->prepare("select distinct category, avg(score) from read_test_score 
         where testid = ? group by category");
	$sth1->execute( $id );
	if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }

	my $catsum;
	while ( my ( $category, $avg ) = $sth1->fetchrow ) {
	    # print "Cat:$category Avg:$avg<br>\n";
	    $catsum += $avg;
	}
	$catsum = $catsum * 0.8333333; # convert from score/12 to score/10. (x5/6)

	my $base = $graphbasescore{$readlevel};
	if (not defined $base) { print "No Base for Reading Level: $readlevel ($rl)<br>\n"; }
#~~
#	print "ID:$id Sum:$catsum Base:$base Block:$blockSelect Expect:$expect<br>\n";

	$score = $base + $catsum;

    } else { # 4-8 Kit

	my $sth = $dbh->prepare("select sum(score) from read_test_score 
         where testid = ?");
	$sth->execute( $id );
	if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
	my $sum = $sth->fetchrow;
#~~
#	print "ID:$id Sum:$sum Base:$base Block:$blockSelect Expect:$expect<br>\n";

	$score = $sum + 120;
    }

    my $normscore = format_number($score - $expect, 1, 1);
    my $rlscore = format_number($score);

#    print "Score:". round($score). " Exp:". round($expect). " Delta:". round($delta). "<br>\n";

    return "$rlscore:$normscore"

}

