#!/usr/bin/perl
#  Copyright 2001-2014 Leslie Richardson

#  This file is part of Open Admin for Schools.

# Read Report - read all reading tests from all schools, calc the
# Equiv Grade and then compare to the actual grade and get a
# delta. Compile into per grade, per school and overall stats.

my %lex = ('Error' => 'Error',
	   'Main' => 'Main',
	   'Start Date' => 'Start Date',
	   'End Date' => 'End Date',
	   'Date' => 'Date',
	   'Continue' => 'Continue',
	   'Schools' => 'Schools',
	   'Database' => 'Database',
	   'Reading' => 'Reading',
	   'Report' => 'Report',
	   'Global' => 'Global',
	   'Select' => 'Select',
	   'Split' => 'Split',
	   'Grade' => 'Grade',
	   'Current' => 'Current',
	   'Output' => 'Output',
	   'Combine failed on input' => 'Combine failed on input',
	   'Download CSV File' => 'Download CSV File',
	   'Test' => 'Test',
	   'Group' => 'Group',
	   'Division' => 'Division',
	   'Paper Size' => 'Paper Size',
	   'Letter' => 'Letter',
	   'Legal' => 'Legal',
	   'A4' => 'A4',
	   'Season' => 'Season',

	   );


my $self = 'rdRptSeason.pl';

# Maps Grades to Grade Divisions I, II, III, IV.
my %divmap = ( 'K' => 1, 1 => 1, 2 => 1, 3 => 1, 
	       4 => 2, 5 => 2, 6 => 2, 
	       7 => 3, 8 => 3, 9 => 3,
	       10 => 4, 11 => 4, 12 => 4 );


# use GD::Graph::bars;
# use GD::Graph::colour qw(:colours);
use DBI;
use CGI;
use Number::Format qw(:all);
use Time::JulianDay;
use Text::CSV_XS;
use Statistics::Basic qw(:all);

#use strict;

eval require "admin.conf";
if ( $@ ) {
    print $lex{Error}. ": $@<br>\n";
    die $lex{Error}. ": $@\n";
}

eval { require "$globdir/global.conf"; };
if ( $@ ) {
    print $lex{Error}. " $self: $@<br>\n";
    die $lex{Error}. "$self: $@\n";
}

@dbase = sort keys %alldbase; # global records.


my @time = localtime(time);
my $year = $time[5] + 1900;
my $month = $time[4] + 1;
my $currdate = "$year-$month-$time[3]";
my $schyear = $year;
if ( $month < 7 ){ $schyear = $schyear - 1; }
my $prevyear = $schyear - 1;

my $currstartdate = "$schyear-08-01"; # Aug 1 of this school year
my $prevstartdate = "$prevyear-08-01"; # August 1 of prev year.


my $q = CGI->new;
my %arr = $q->Vars;
print $q->header( -charset, $charset );


# Script Heading Descriptor
my $title = "$lex{Season} $lex{Reading} $lex{Report}";

# Print Page Heading
print "$doctype\n<html><head><title>$title</title>\n";
print "<link rel=\"stylesheet\" href=\"/division.css\" type=\"text/css\">\n";

=head
if ( not $arr{page} ) { # calendar popup.
    print "<link rel=\"stylesheet\" type=\"text/css\" media=\"all\" ";
    print "href=\"/js/calendar-blue.css\" title=\"blue\">\n";
    print "<script type=\"text/javascript\" src=\"/js/calendar.js\"></script>\n";
    print "<script type=\"text/javascript\" src=\"/js/lang/calendar-en.js\"></script>\n";
    print "<script type=\"text/javascript\" src=\"/js/calendar-setup.js\"></script>\n";
}
=cut

print "$chartype\n</head><body style=\"padding:0.4em 2em;\">\n";
print "[ <a class=\"alt\" href=\"$homepage\">$lex{Main}</a> ]\n";
print "<h1>$title</h1>\n";

if ( not $arr{page} ) {
    showStartPage();

} elsif ( $arr{page} == 1 ) {
    delete $arr{page};

#    if ( $arr{output} eq 'html' or $arr{output} eq 'pdf' ) {
#	delete $arr{output};
	selectReading();

#    } elsif ( $arr{output} eq 'csv' ) {
#	delete $arr{output};
#	exportCSV();
#    }
}



#----------------
sub showStartPage {
#----------------

    print "<form action=\"$self\" method=\"post\">\n";
    print "<input type=\"hidden\" name=\"page\" value=\"1\">\n";

    print "<table cellpadding=\"3\" cellspacing=\"0\" border=\"0\" ";
    print "style=\"padding:0.5em;border:1px solid gray;\">\n";

    print "<tr><td class=\"bla\">$lex{Select} $lex{Schools}</td></tr>\n";
    foreach my $db ( @dbase ) { 
	print "<tr><td class=\"la\">";
	print "<input type=\"checkbox\" name=\"$db\" value=\"1\" checked=\"checked\">\n";
	print "$alldbase{$db} ($db)</td></tr>\n";
    }

=head
    # Start Date
    print "<tr><td class=\"bra\">$lex{'Start Date'}</td><td class=\"la\"><input type=\"text\" ";
    print "name=\"startdate\" id=\"sdate\" size=\"10\" value=\"$currstartdate\">";
    print "<button type=\"reset\" id=\"start_trigger\">...</button>\n";
    print "</td></tr>\n";


    # End Date
    print "<tr><td class=\"bra\">$lex{'End Date'}</td><td class=\"la\"><input type=\"text\" ";
    print "name=\"enddate\" id=\"edate\" size=\"10\" value=\"$currdate\">";
    print "<button type=\"reset\" id=\"end_trigger\">...</button></td></tr>\n";


    # Grouping 
    print "<tr><td class=\"bra\">$lex{Group}</td><td class=\"la\">";
    print "<select name=\"grouping\"><option value=\"division\">$lex{Division}</option>\n";
    print "<option value=\"grade\">$lex{Grade}</option>";
    print "</select></td></tr>\n";


    # Output Type
    print "<tr><td class=\"bra\">$lex{Output}</td><td class=\"la\">";
    print "<select name=\"output\"><option value=\"html\">HTML</option>\n";
    print "<option value=\"pdf\">PDF</option>";
    print "<option value=\"csv\">CSV</option>";
    print "</select></td></tr>\n";

    # Paper Size, if doing PDF.
    print "<tr><td class=\"bra\">$lex{'Paper Size'}</td>\n<td>";
    print "<select name=\"papersize\"><option value=\"letter\">$lex{Letter}</option>\n";
    print "<option value=\"a4\">$lex{A4}</option><option value=\"legal\">$lex{Legal}</option>";
    print "</select></td></tr>\n ";
=cut

    print "<tr><td class=\"la\"><input type=\"submit\" value=\"$lex{Continue}\"></td></tr>\n";
    print "</table></form>\n";

=head
    print "<script type=\"text/javascript\">
     Calendar.setup({
        inputField     :    \"sdate\", 
        ifFormat       :    \"%Y-%m-%d\",
        button         :    \"start_trigger\",
        singleClick    :    false,
        step           :    1
    });

    Calendar.setup({
        inputField     :    \"edate\",
        ifFormat       :    \"%Y-%m-%d\",
        button         :    \"end_trigger\",
        singleClick    :    false,
        step           :    1
    });
    </script>\n";
=cut

    print "</body></html>\n";

    exit;

} # end of showStartPage



#----------------
sub selectReading {
#----------------

#     foreach my $key ( sort keys %arr ) { print "K:$key V:$arr{$key}<br>\n"; }

#    my $sdate = fmtDate( $arr{startdate} );
#    my $startdate = $arr{startdate};
#    delete $arr{startdate}; 

#    my $edate = fmtDate( $arr{enddate} );
#    my $enddate = $arr{enddate};
#    delete $arr{enddate};

#    my $grouping = $arr{grouping};
#    delete $arr{grouping};
#    my $papersize = $arr{papersize};
#    delete $arr{papersize};


    my %overall = ();
    my %overallgrade = ();

    foreach my $db ( sort keys %arr ) {

	my $database = $db;
	my $dsn = "DBI:$dbtype:dbname=$database";
	my $dbh = DBI->connect($dsn,$guser,$gpassword);
	my $schoolname = $alldbase{"$db"};

	print "<h2>$schoolname</h2>\n";

	# Get all students who have tests into a @students array (may not be current)
	my @students = ();
	my $sth = $dbh->prepare("select distinct studnum from read_test");
	$sth->execute;
	if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
	while ( my $studnum = $sth->fetchrow ) {
	    push @students, $studnum;
	}

	if ( not @students ) {
	    print "<h3>No Student Tests Found</h3>\n";
	    next;
	}


	# Now loop through all students
	my $sth1 = $dbh->prepare("select id, tdate, readlevel, tgrade from read_test 
         where studnum = ? order by tdate desc");
	if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }

	my $sth2 = $dbh->prepare("select sum(score) from read_test_score where testid = ?");
	my $sth3 = $dbh->prepare("select lastname, firstname from studentall where studnum = ?");	


	my %schooldata = ();
	my %gradedata = ();

	foreach my $studnum ( @students ) {

	    # Get Tests
	    $sth1->execute( $studnum );
	    if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }

	    my $trows = $sth1->rows;
	    # if $onlylatest, limit to 1 test
	    # if ( $onlylatest and $trows > 1 ) { $trows = 1; } 
	    # if ( $trows > 1 ) { $trows = 1; }


	    for ( 1 .. $trows ) {

		my ( $testid, $tdate, $readlevel, $tgrade ) = $sth1->fetchrow;

		# Get the score total;
		$sth2->execute( $testid );
		if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
		my $total = $sth2->fetchrow;

		# Equivalent Grade
		my $equivgrade = scoreToGrade($total, $readlevel );

		my $grdelta;
		if ( not $tgrade ) {
		    $sth3->execute( $studnum );
		    my ( $lastname, $firstname ) = $sth3->fetchrow;		    
		    print "Grade Missing: $lastname, $firstname ($studnum) - $tdate<br>\n";
		    next;
		
		} else { # Find grade delta
		    $grdelta = $equivgrade - $tgrade;
		}

		#if ( $grdelta >= 0 ) { $grdelta = int $grdelta; } # truncate.
		#else { $grdelta = int $grdelta - 1; }

		my $level;
		if ( $grdelta < -2 ) { 
		    $level = 1;
		} elsif ( $grdelta < 0 ) {
		    $level = 2;
		} else {
		    $level = 3;
		}

		my ($y,$m,$d) = split(/-/, $tdate);
		my $season;
		if ( $m > 6 ) { # last half of year
		    $season = $y.'2'; # 2013-2 format.
		} else { # spring
		    $season = $y.'1'; # 2013-1 format.
		}
		
		# if ( $grdelta >= 0 ) { $grdelta = int $grdelta; } # truncate.
		# else { $grdelta = int $grdelta - 1; }

		$schooldata{$season}{$level}++;
		$overall{$season}{$level}++;

		$gradedata{$season} += $grdelta; # running total of all grdelta's for this season.
		$overallgrade{$season} += $grdelta;

	    } # end of trows loop

	} # end of student loop


	# Skip if no tests in this date range
	if ( not %schooldata ) { 
	    print "<h3>No tests in this Date Range</h3>\n";
	    next; 
	}


=head

	# Get MIN, MAX values of Delta Grade (How far above/below grade level.
	my ($max, $min, $studcount );
	my %dcount = ();
	foreach my $grade ( sort keys %schooldata ) {
	    foreach my $delta ( sort keys %{ $schooldata{$grade} } ) {
		if  ( $delta > $max ) { $max = $delta; }
		if  ( $delta < $min ) { $min = $delta; }
		$studcount += $schooldata{$grade}{$delta};
		$dcount{$delta} += $schooldata{$grade}{$delta};
		# print "G:$grade D:$delta Count:$schooldata{$grade}{$delta}<br>\n";
	    }
	}


	my %divdata = ();  # Grade Divisions Data

	if ( $grouping eq 'division' ) { # populate this structure
	    foreach my $grade ( sort {$a <=> $b} keys %schooldata ) {
		my $div = $divmap{$grade};
		if ( not $div ) { 
		    print "$lex{Error}: No Div for $lex{Grade}:$grade<br>\n";
		    next;
		}

		foreach my $delta ( $min .. $max ) {
		    $divdata{$div}{$delta} += $schooldata{$grade}{$delta};
		    # print "D:$delta DIV:$div GRD:$grade -- $divdata{$div}{$delta} -- $schooldata{$grade}{$delta}<br>\n";
		}
	    }
	}

	# Division Level Output
	if ( $grouping eq 'division' ) { 
	    print "<table cellspacing=\"0\" cellpadding=\"3\" border=\"1\">\n";
	    # Table Heading
	    print "<tr><th>$lex{Division}</th>";
	    foreach my $delta ( $min .. $max ) {
		print "<th>$delta</th>";
	    }
	    print "<th>$lex{Test}#</th><th>Wt. Avg</th></tr>\n";


	    foreach my $div ( sort {$a <=> $b} keys %divdata ) {
		print "<tr><td>$div</td>";
		my ($total,$partavg);
		foreach my $delta ( $min .. $max ) {
		    my $count = $divdata{$div}{$delta};
		    print "<td>$divdata{$div}{$delta}</td>";
		    $total += $count;
		    $partavg += ( $count * $delta );
		}

		my $avg;
		if ( $total ) {
		    $avg = $partavg / $total;
		} else { $avg = 0; }
		
		$avg = round($avg, 2);
		print "<td>$total</td><td>$avg</td>";
		print "</tr>\n";
	    }
	    # print "</table><p></p>\n";
	    # End - Division Level Output

	} else {
	# Grade Level Output    
=cut

	# Now Printing Grade Level Output
	print "<table cellspacing=\"0\" cellpadding=\"3\" border=\"1\">\n";


	# Table Heading

	print "<tr><th></th>";
	print qq{<th colspan="3">Percentage of Students vs Reading Level Category</th>};
	print qq{<th colspan="2"> </th></tr>\n};

	print "<tr><th>$lex{Season}</th><th>&lt;=-3</th><th>-2,-1</th><th>&gt;=0</th>";
	print "<th>#$lex{Test}</th><th>Avg</th>";
	print "</tr>\n";


	foreach my $season ( sort {$a <=> $b} keys %schooldata ) {
	    my ($yr,$s ) = unpack("A4,A1", $season);
	    if ( $s == 1 ) { $s = 'Spring'; } else { $s = 'Fall'; }
	    my $useason = "$yr-$s";

	    print "<tr><td>$useason</td>";

	    # Get Total tests
	    my $total;
	    foreach my $level ( 1 .. 3 ) {
		my $count = $schooldata{$season}{$level};
		$total += $count;
	    }

	    # Print Level %
	    foreach my $level ( 1 .. 3 ) {
		my $percent;
		if ( $total ) {
		    $percent = $schooldata{$season}{$level} / $total * 100;
		    $percent = round($percent, 2);
		} else { $percent = 0; }
		print "<td>$percent</td>";
	    }

	    # Do EQ Grade Average
	    my $avg;
	    if ( $total ) {
		$avg = $gradedata{$season} / $total;
	    } else { $avg = 0; }
		
	    $avg = round($avg, 2);
	    print "<td>$total</td><td>$avg</td>";
	    print "</tr>\n";
	} # end of season loop

	
=head
	# Summary Row
	print "<tr><th>Average</th>";
	my $partavg;
	foreach my $delta ( $min .. $max ) {
	    $partavg += ($delta * $dcount{$delta} );
	    my $avg;
	    if ( $studcount ) {
		$avg = round( $dcount{$delta} / $studcount * 100, 2);
	    }
	    
	    print "<td>$dcount{$delta}/$studcount<br>$avg%</td>";
	}

	my $avg = $partavg / $studcount;
	$avg = round($avg,2);
	print "<th>$studcount</th><th>$avg</th></tr>\n";
=cut

	print "</table><p></p>\n";



    } # database loop


    # All Schools Combined.
    print "<h2>Combined Schools Result</h2>\n";

    $max = 0, 
    $min = 0, 
    $studcount = 0;
    %dcount = ();


    print "<table cellspacing=\"0\" cellpadding=\"3\" border=\"1\">\n";

    # Heading.
    print "<tr><th></th>";
    print qq{<th colspan="3">Percentage of Students vs Reading Level Category</th>};
    print qq{<th colspan="2"> </th></tr>\n};

    print "<tr><th>$lex{Season}</th><th>&lt;=-3</th><th>-2,-1</th><th>&gt;=0</th>";
    print "<th>#$lex{Test}</th><th>Avg</th>";
    print "</tr>\n";


    # Data
    foreach my $season ( sort {$a <=> $b} keys %overall ) {
	my ($yr,$s ) = unpack("A4,A1", $season);
	if ( $s == 1 ) { $s = 'Spring'; } else { $s = 'Fall'; }
	my $useason = "$yr-$s";
	
	print "<tr><td>$useason</td>";

	# Get Total tests
	my $total;
	foreach my $level ( 1 .. 3 ) {
	    my $count = $overall{$season}{$level};
	    $total += $count;
	}

	# Print Level %
	foreach my $level ( 1 .. 3 ) {
	    my $percent;
	    if ( $total ) {
		$percent = $overall{$season}{$level} / $total * 100;
		$percent = round($percent, 2);
	    } else { $percent = 0; }
	    print "<td>$percent</td>";
	}

	# Do EQ Grade Average
	my $avg;
	if ( $total ) {
	    $avg = $overallgrade{$season} / $total;
	} else { $avg = 0; }
	
	$avg = round($avg, 2);
	print "<td>$total</td><td>$avg</td>";
	print "</tr>\n";

    } # end of season loop

=head
    print "<tr><th>Average</th>";
    my $partavg;
    foreach my $delta ( $min .. $max ) {
	$partavg += ($delta * $dcount{$delta} );
	my $avg;
	if ( $studcount ) {
	    $avg = round( $dcount{$delta} / $studcount * 100, 2);
	}
	print "<td>$dcount{$delta}/$studcount<br>$avg%</td>";
    }

    my $avg = $partavg / $studcount;
    $avg = round($avg,2);
    print "<th>$studcount</th><th>$avg</th></tr>\n";
=cut


    print "</table><p></p>\n";

    exit;
}






#-----------
sub printPDF { # print one table (one school or summary )
#-----------

    my ($mode, $dataref ) = @_;
    # mode is either grade or division (I,II,etc)
    my %data = %$dataref;

}


#-----------
sub startPDF {  # print the setup for PDF
#-----------

    my ($filename,$papersize) = shift;

    # Preamble and Head of Document
    my ( $papersize, $textwidth, $textheight );

    if ( $arr{papersize} eq 'letter' ) {
	$papersize = 'letterpaper';
	$textwidth = $g_letterpaper_textwidth;
	$textheight = $g_letterpaper_textheight;
    } elsif ( $arr{papersize} eq 'legal' ) {
	$papersize = 'legalpaper';
	$textwidth = $g_legalpaper_textwidth;
	$textheight = $g_legalpaper_textheight;
    } elsif ( $arr{papersize} eq 'a4' ) {
	$papersize = 'a4paper';
	$textwidth = $g_a4paper_textwidth;
	$textheight = $g_a4paper_textheight;
    }

    my $shortname = "readprogress$$";
    my $filename = "$shortname.tex";

    open(TEX,">$filename") || die "Can't open tex file";


    print TEX <<"EOS1";  
\\documentclass[ 12pt, $papersize, oneside]{article}
\\usepackage{array,graphicx,multicol,colortbl,inputenc,fancyhdr}
\\inputencoding{latin1}
\\renewcommand{\\familydefault}{\\sfdefault}

\\setlength{\\hoffset}{-20mm}
\\setlength{\\voffset}{-20mm}

\\setlength{\\headsep}{10pt}
\\setlength{\\headheight}{14pt}
\\setlength{\\topmargin}{0mm}

\\setlength{\\textwidth}{ $textwidth }
\\setlength{\\textheight}{ $textheight }

\\setlength{\\parindent}{0pt}
\\setlength{\\evensidemargin}{0mm}
\\setlength{\\oddsidemargin}{0mm}

\\begin{document}
EOS1


    # Print Data Structure





    print TEX "\\end{document}\n";
    close TEX;

    system("$pdflatex $filename >pdflog$$.txt");
   
    system("mv $shortname.pdf $downloaddir");
    system("mv pdflog$$.txt $downloaddir");
    system("rm -f $shortname.*");

    print "<h1><a href=\"$webdownloaddir/$shortname.pdf\">";
    print "$lex{'View/Download'} $lex{'Progress Report'}</a></h1>\n";
    print "<p>[ <a href=\"$homepage\">$lex{Main}</a> |\n <a href=\"$webdownloaddir/pdflog$$.txt\">";
    print "$lex{'View Log File'}</a>\n ]</p>\n";
    print "</body></html>\n";

} # end of printPDF



#-----------
sub writeCSV {
#-----------

    my $ref = shift;

    # print "Val:", @val, " Ref:$ref<br>\n";
    
    if ( $csv->combine( @$ref ) ) {
	my $record = $csv->string;
	print EX $record, "\r\n";

    } else {
	my $err = $csv->error_input;
	print "$lex{'Combine failed on input'}: $err<br>\n";
    }

    return;

} # end writeCSV


#------------
sub exportCSV {
#------------

    # foreach my $key ( sort keys %arr ) { print "K:$key V:$arr{$key}<br>\n"; }

    # Remove the extra:
    delete $arr{grouping};
    delete $arr{papersize};


    $csv = Text::CSV_XS->new( {binary => 1} );

    # Open output file
    my $filename = "expReadProgress$$.csv";
    open (EX,">$filename") || die $lex{'Cannot open Export file'}. " $fileName";


    my $sdate = $arr{startdate};
    delete $arr{startdate};
    my $edate = $arr{enddate};
    delete $arr{enddate};

    # Write date range.
    my @data = ();
    #for my $i (1..20){ $data[$i] = ' ';}
    $data[0] = "$lex{'Start Date'} $sdate";
    $data[2] = "$lex{'End Date'} $edate";
    writeCSV(\@data);


    my $startjd = julian_day( split(/-/, $sdate ));
    my $endjd = julian_day( split(/-/, $edate ));

    my %overall = ();

    foreach my $db ( sort keys %arr ) {
	
	my $database = $db;
	my $dsn = "DBI:$dbtype:dbname=$database";
	my $dbh = DBI->connect($dsn,$guser,$gpassword);
	my $schoolname = $alldbase{"$db"};

	# Write the school name
	my @data = ();
	#for my $i (1..20){ $data[$i] = ' ';}
	$data[0] = $schoolname;
	writeCSV(\@data);

	
	# Get all students in the school who have tests into a @students array
	my @students = ();
	my $sth = $dbh->prepare("select distinct studnum from read_test");
	$sth->execute;
	if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
	while ( my $studnum = $sth->fetchrow ) {
	    push @students, $studnum;
	}

	if ( not @students ) {

	    my @data = ();
	    for my $i (1..20){ $data[$i] = ' ';}
	    $data[0] = "No Student Tests Found";
	    writeCSV(\@data);

	    # Write blank line.
	    my @data = ();
	    $data[0] = ' ';
	    writeCSV(\@data);
	    
	    next;
	}


	# Now loop through all students printing results; setup queries first
	my $sth1 = $dbh->prepare("select id, tdate, readlevel, tgrade from read_test 
         where studnum = ? order by tdate desc");

	my $sth2 = $dbh->prepare("select sum(score) from read_test_score where testid = ?");
	
	my %schooldata = ();
	my %gradedata = ();

	foreach my $studnum ( @students ) {

	    # Get Tests
	    $sth1->execute( $studnum );
	    if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }

	    my $trows = $sth1->rows;
	    # if $onlylatest, limit to 1 test
	    #if ( $onlylatest and $trows > 1 ) { $trows = 1; } 
	    $trows = 1;
	    for ( 1 .. $trows ) {

		my ( $testid, $tdate, $readlevel, $tgrade ) = $sth1->fetchrow;

		# Get the score total;
		$sth2->execute( $testid );
		if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
		my $total = $sth2->fetchrow;

		# Equivalent Grade print
		my $equivgrade = scoreToGrade($total, $readlevel );

		my $grdelta;
		if ( not $tgrade ) {
		    print "Grade Missing: $studnum - $readlevel - $tdate<br>\n";
		    next;
		} else { # Find grade delta
		    $grdelta = $equivgrade - $tgrade;
		}

		my $level;
		if ( $grdelta < -2 ) { 
		    $level = 1;
		} elsif ( $grdelta < 0 ) {
		    $level = 2;
		} else {
		    $level = 3;
		}


		my ($y,$m,$d) = split(/-/, $tdate);
		my $season;
		if ( $m > 6 ) { # last half of year
		    $season = $y.'-Fall'; # 2013-2 format.
		} else { # spring
		    $season = $y.'-Spring'; # 2013-1 format.
		}
		

		# if ( $grdelta >= 0 ) { $grdelta = int $grdelta; } # truncate.
		# else { $grdelta = int $grdelta - 1; }

		$schooldata{$season}{$level}++;
		$overall{$season}{$level}++;

		$gradedata{$season} += $grdelta; # running total of all grdelta's for this season.

	    } # end of trows loop

	} # end of student loop


	my ($max, $min, $studcount );
	my %dcount = ();
	my @stats = ();
	foreach my $grade ( sort keys %schooldata ) {
	    foreach my $delta ( sort keys %{ $schooldata{$grade} } ) {
		# Set Min/Max values
		if  ( $delta > $max ) { $max = $delta; }
		if  ( $delta < $min ) { $min = $delta; }

		# Get Count and Delta Counts.
		$studcount += $schooldata{$grade}{$delta};
		$dcount{$delta} += $schooldata{$grade}{$delta};
		for (1..$dcount{$delta} ) { push @stats, $delta; } # put delta values into @stats
		# print "G:$grade D:$delta Count:$schooldata{$grade}{$delta}<br>\n";
	    }
	}



	# Table Heading
	@data = ();
	$data[0] = "$lex{Grade}";
	foreach my $delta ( $min .. $max ) {
	    push @data, $delta;
	}
	push @data, "SDev";
	push @data, "Var";

	writeCSV(\@data);


	# Write each grade row.
	foreach my $grade ( sort {$a <=> $b} keys %schooldata ) {
	    my @data = ();
	    my @grstats = ();
	    $data[0] = $grade;
	    foreach my $delta ( $min .. $max ) {
		push @data, $schooldata{$grade}{$delta};
		for (1..$schooldata{$grade}{$delta}) { push @grstats, $delta; }
	    }

	    push @data, stddev(\@grstats);
	    push @data, variance(\@grstats);

	    writeCSV(\@data);
	}


	# Write average Row
	my @data = ();
	my @data1 = ();
	$data[0] = 'Average';
	$data1[0] = 'Count';
	foreach my $delta ( $min .. $max ) {
	    my $avg;
	    if ( $studcount ) {
		$avg = round( $dcount{$delta} / $studcount * 100, 2);
	    }
	    push @data, $avg;
	    push @data1, $dcount{$delta}
	    # print "<td>$dcount{$delta}/$studcount<br>$avg%</td>";
	}

	push @data, stddev(\@stats);
	push @data, variance(\@stats);

	writeCSV(\@data); # average
	writeCSV(\@data1); # count


	# Duplicate of the studcount row...
	# Write Column Sum
#	my @data = ();
#	$data[0] = 'Column Sum';
#	foreach my $delta ( $min .. $max ) {
#	    push @data,	$dcount{$delta}
#	}
#	writeCSV(\@data);


	# Write studcount Row
	my @data = ();
	$data[0] = 'S Count';
	foreach my $delta ( $min .. $max ) {
	    push @data, $studcount;
	}
	writeCSV(\@data);

	# Write blank line.
	my @data = ();
	$data[0] = ' ';
	writeCSV(\@data);


    } # database loop



    # All Schools Combined.
    my @data = ();
    $data[0] = 'Combined Schools Result';
    writeCSV(\@data);


    # Next Line
    $max = 0, 
    $min = 0, 
    $studcount = 0;
    %dcount = ();
    @stats = ();

    foreach my $grade ( sort keys %overall ) {
	foreach my $delta ( sort keys %{ $overall{$grade} } ) {
	    if  ( $delta > $max ) { $max = $delta; }
	    if  ( $delta < $min ) { $min = $delta; }
	    $studcount += $overall{$grade}{$delta};
	    $dcount{$delta} += $overall{$grade}{$delta};
	    for (1..$overall{$grade}{$delta}) { push @stats, $delta; }
	    # print "G:$grade D:$delta Count:$overall{$grade}{$delta}<br>\n";
	}
    }

    # Write Heading Line
    my @data = ();
    $data[0] = $lex{Grade};
    foreach my $delta ( $min .. $max ) {
	push @data, $delta;
    }
    push @data, "SDev";
    push @data, "Var";

    writeCSV(\@data);


    # Write all grade rows.
    foreach my $grade ( sort {$a <=> $b} keys %overall ) {
	my @data = ();
	my @grstats = ();
	$data[0] = $grade;
	foreach my $delta ( $min .. $max ) {
	    push @data, $overall{$grade}{$delta};
	    for (1..$overall{$grade}{$delta}) { push @grstats, $delta; }
	}

	push @data, stddev(\@grstats);
	push @data, variance(\@grstats);

	writeCSV(\@data);
    }


    # Write average Row
    my @data = ();
    $data[0] = 'Average';
    foreach my $delta ( $min .. $max ) {
	my $avg;
	if ( $studcount ) {
	    $avg = round( $dcount{$delta} / $studcount * 100, 2);
	}
	push @data, $avg;
	# print "<td>$dcount{$delta}/$studcount<br>$avg%</td>";
    }

    push @data, stddev(\@stats);
    push @data, variance(\@stats);

    writeCSV(\@data);


    # Write Column Sum Row
    my @data = ();
    $data[0] = 'Count';
    foreach my $delta ( $min .. $max ) {
	push @data,	$dcount{$delta}
    }
    writeCSV(\@data);


    # Write studcount Row
    my @data = ();
    $data[0] = 'Total';
    foreach my $delta ( $min .. $max ) {
	push @data, $studcount;
    }
    writeCSV(\@data);


    close EX;
    system("mv $filename $downloaddir");
    
    print "<p><a href=\"$webdownloaddir/$filename\">\n";
    print "<span style=\"font-size:1.5em;\">$lex{'Download CSV File'}</span></a></p>\n";
    print "</body></html>\n";

    exit;

} # end of exportCSV



#----------
sub fmtDate {
#----------

    my ( $year, $mon, $day ) = split /-/, shift;
    return "$year-$s_month[$mon]-$day";
}



#---------------
sub scoreToGrade {
#---------------

    my ($score, $readinglevel) = @_;

    my %rd_LevelToGrade = ( '0' => 0.2, 1 => 0.5, 2 => 0.8, 3 => 1.1, 4 => 1.2,
			    6 => 1.3, 8 => 1.4, 10 => 1.5, 12 => 1.6, 14 => 1.7,
			    16 => 1.8, 18 => 1.9, 20 => '2.0', 24 => 2.5, 28 => 2.9,
			    30 => '3.0', 34 => 3.5, 38 => 3.9);


    my %rd_ScoreToGrade = (
	50 => 1.9, 52 => '2.0', 53 => 2.1, 54 => 2.2,
	55 => 2.3, 56 => 2.5, 57 => 2.7, 58 => 2.8,
	61 => 2.9, 62 => '3.0', 63 => 3.1, 64 => 3.2, 
	65 => 3.3, 66 => 3.5, 67 => 3.6, 68 => 3.7,
	69 => 3.8, 70 => 3.9, 74 => 4.3, 76 => 4.6,
	80 => 4.9, 84 => 5.3, 86 => 5.6, 90 => 5.9,
	94 => 6.3, 96 => 6.6, '100' => '6.9',
	'104' => '7.3', '106' => '7.6','110' => '7.9',
	'114' => '8.3', '119' => '8.6','124' => '8.9');


    if ( $readinglevel < 40 ) {

	foreach my $key ( sort { $b <=> $a } keys %rd_LevelToGrade ) {
	    # print "K:$key ";
	    if ( $readinglevel >= $key ) {
		# print "<br><b>Match: K:$key RL:$readinglevel</b><br>\n";
		return $rd_LevelToGrade{ $key };
	    }
	}

    } else { # else reading level >= 40

	$score += $readinglevel; # add both together for this 

	foreach my $key ( sort { $b <=> $a } keys %rd_ScoreToGrade ) {
	    # print "K:$key ";
	    if ( $score >= $key ) {
		# print "<b>Match: S:$score K:$key V:$rd_ScoreToGrade{$key}</b><br>\n";
		return $rd_ScoreToGrade{ $key };
	    }
	}

    }

    return 'NA';
}



