#!/usr/bin/perl
#  Copyright 2001-2014 Leslie Richardson

#  This file is part of Open Admin for Schools.

# Dump data for DRA reading to include school, grade, gender, totalcount, and passcount (based on prov values for reading level)

my $startdate = '2014-01-01';
my $enddate = '2014-06-30';

my %lex = ('Dump Reading Data' => 'Dump Reading Data',
	   'Combine failed on input' => 'Combine failed on input',
	   'Download CSV File' => 'Download CSV File',
	   'Main' => 'Main',

	   );

# grade to reading level pass values
my %passlevel = ( 1 => 12,
		  2 => 24,
		  3 => 36,
		  4 => 40,
		  5 => 50,
		  6 => 60,
		  7 => 70,
		  8 => 80,
		  9 => 80,
		  10 => 80
    );



use DBI;
use CGI;
use Text::CSV_XS;


my $q = new CGI;
print $q->header; 
my %arr = $q->Vars;


# Read config variables
eval require "../admin.conf";
if ( $@ ) {
    print $lex{Error}. " $self: $@<br>\n";
    die $lex{Error}. "$self: $@\n";
}


eval require "$globdir/global.conf";
if ( $@ ) {
    print $lex{Error}. " $self: $@<br>\n";
    die $lex{Error}. "$self: $@\n";
}

=head
eval require "$batcglobdir/global.conf";
if ( $@ ) {
    print $lex{Error}. " $self: $@<br>\n";
    die $lex{Error}. "$self: $@\n";
}
=cut

my ($sec, $min, $hour, $mday, $mon, $year, $wday, $yday, $iddst) = localtime(time);
$year = $year + 1900;
$wday++;
$mon++;
my $currdate = "$dow[$wday], $month[$mon] $mday, $year";

my $csv = Text::CSV_XS->new( {binary => 1} );


# HTML Heading
my $title = "Reading Data Performance 2012-2013";
print "$doctype\n<html><head><title>$title</title>\n";
print "<link rel=\"stylesheet\" href=\"/division.css\" type=\"text/css\">\n";
print "<style type=\"text/css\">th.h {background-color:#99F;}</style>\n";
print "$chartype\n</head><body style=\"padding:1em 2em;\">\n";

print "[ <a href=\"$homepage\">$lex{Main}</a> ]\n";

print "<h1>$title</h1><h3>$currdate</h3>\n";

print "<h3>Start: $startdate</h3>\n";
print "<h3>End: $enddate</h3>\n";

# Open output file
my $filename = "readperformance$$.csv";
open (EX,">$filename") || die "Can't open Export file $filename";

my @fields = qw( School Grade Gender Total Pass);
if ( $csv->combine(@fields) ) {
    my $record = $csv->string;
    print EX $record, "\r\n";
} else {
    print "<h3>Header Error</h3>\n";
    exit;
}

foreach my $i (1..2) {
    if ( $i == 2 ) { # load batc data
	eval require "$batcglobdir/global.conf";
	if ( $@ ) {
	    print $lex{Error}. " $self: $@<br>\n";
	    die $lex{Error}. "$self: $@\n";
	}
    }

    foreach $database ( keys %alldbase ) { #defined in global config file.

	my $dsn = "DBI:mysql:$database";
	my $dbh = DBI->connect($dsn,$guser,$gpassword);

	my $school = $alldbase{$database};
	print "School:$school<br>\n";

	# Read each test and then scores in turn.
	my $sth1 = $dbh->prepare("select readlevel, tgrade from read_test where studnum = ? order by tdate desc"); 
	my $sth2 = $dbh->prepare("select sex from studentall where studnum = ?");
	
	my $sth = $dbh->prepare("select distinct studnum from read_test where to_days(tdate) < to_days('$enddate') and
         to_days(tdate) > to_days('$startdate')"); 
	$sth->execute;
	if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr; }

	my %students;  # students{grade}{gender}{pass/fail}
	# loop through all students
	while ( my $studnum = $sth->fetchrow ) {



	    # Get most recent test.
	    $sth1->execute($studnum);
	    if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr; }
	    my ($readlevel, $tgrade) = $sth1->fetchrow;
	    $readlevel =~ s/\W$//; # strip any trailing characters.

	    # Get Gender
	    $sth2->execute($studnum);
	    if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr; }
	    my $gender = $sth2->fetchrow;

	    # Get Pass/Fail
	    my $passlevel = $passlevel{$tgrade};
	    if ( $readlevel >= $passlevel ) { # pass
		$students{$tgrade}{$gender}{'pass'} += 1;
	    } else {
		$students{$tgrade}{$gender}{'fail'} += 1;
	    }
	}

=head
    foreach my $grade ( sort {$a <=> $b} keys %students ) {
	foreach my $gender ( sort keys %{ $students{$grade} } ) {
	    foreach my $type ( sort keys %{ $students{$grade}{$gender} } ) {
		print "GR:$grade GEN:$gender Type:$type Count: $students{$grade}{$gender}{$type}\n";
	    }
	}
    }
=cut

	foreach my $grade ( sort {$a <=> $b} keys %students ) {
	    foreach my $gender ( sort keys %{ $students{$grade} } ) {

		my @fields;
		$fields[0] = $database;
		$fields[1] = $grade;
		$fields[2] = $gender;
		my $pass = $students{$grade}{$gender}{'pass'};
		my $total = $students{$grade}{$gender}{'fail'} + $pass;
		$fields[3] = $total;
		$fields[4] = $pass;

		# convert record to csv
		if ( $csv->combine(@fields) ) {
		    my $record = $csv->string;
		    print EX $record, "\r\n";
		} else {
		    my $err = $csv->error_input;
		    print $lex{'Combine failed on input'}. ":$err\n\n";
		}
	    }
	}
	
    } # End of Database reading Loop.
} # end of multiple database groups

close EX;
system("mv $filename $downloaddir");

print "<p><a href=\"$webdownloaddir/$filename\"><span style=\"font-size:130%\">$lex{'Download CSV File'}</span></a></p>\n";
print "</center></body></html>\n";
