#!/usr/bin/perl
#  Copyright 2001-2010 Leslie Richardson

#  This file is part of Open Admin for Schools.

#  Open Admin for Schools is free software; you can redistribute it 
#  and/or modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2 of 
#  the License, or (at your option) any later version.

my %lex = ('Main' => 'Main',
	   'Error' => 'Error',
	   'View Reading Levels' => 'View Reading Levels',
	   'Delete' => 'Delete',
	   'Delete Selected Records' => 'Delete Selected Records',
	   'Record Deleted' => 'Record Deleted',
	   'Save Record' => 'Save Record',
	   'Save' => 'Save',
	   'Reading Level' => 'Reading Level',
	   'Printed Language Concepts' => 'Printed Language Concepts',
	   'Oral Fluency' => 'Oral Fluency',
	   'Reading Engagement' => 'Reading Engagement',
	   'Comprehension' => 'Comprehension',
	   'Category' => 'Category',
	   'Objective' => 'Objective',
	   'No Records Found' => 'No Records Found',
	   'Existing Records' => 'Existing Records',
	   'Separate by 10s' => 'Separate by 10s',
	   'Sequence Number' => 'Sequence Number',
	   'Sequence' => 'Sequence',
	   'Continue' => 'Continue',
	   'Book Title' => 'Book Title',
	   'The record is now stored' => 'The record is now stored',
	   'There was an error storing your data' => 'There was an error storing your data',
	   'Contact' => 'Contact',
	   'Complete Help List' => 'Complete Help List',
	   'No Match' => 'No Match',

	   );

use DBI;
use CGI;

my $self = 'readlvlview.pl';

eval require "../admin.conf";
if ( $@ ) {
    print $lex{Error}. " $self: $@<br>\n";
    die $lex{Error}. "$self: $@\n";
}

my $q = new CGI;
print $q->header; 
my %arr = $q->Vars;


# Page Header
print "$doctype\n<html><head><title>". $lex{'View Reading Levels'};
print "</title><link rel=\"stylesheet\" href=\"/division.css\" type=\"text/css\">\n";
print "<style type=\"text/css\">th.h {background-color:#99F;}</style>\n";
print "$chartype\n</head><body style=\"padding:1em 2em;\">\n";

print "[ <a href=\"$homepage\">". $lex{Main}. "</a> ]\n";
print "<h1>". $lex{'View Reading Levels'}. "</h1>\n";

my $dsn = "DBI:$dbtype:dbname=$dbase";
my $dbh = DBI->connect($dsn,$user,$password);

if ( not $arr{page} ) {
    showMainPage();
}
if ( $arr{page} == 1 ) {
    delete $arr{page};
    deleteRecords();
} 


#----------------
sub showMainPage {
#----------------


    # Display Existing Values (if any)
    my %readlev = ();
    my $sth = $dbh->prepare("select distinct readlevel from read_level
      where readlevel is not NULL and readlevel != ''");
    $sth->execute;
    if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
    while ( my $rl = $sth->fetchrow ) {
	$readlev{ $rl } = 1;
    }


    # Print Form Start
    print "<form action=\"$self\" method=\"post\"> \n";
    print "<input type=\"hidden\" name=\"page\" value=\"1\">\n\n";

    print "<input type=\"submit\" value=\"". $lex{'Delete Selected Records'}. "\">\n";


    my $first = 1;
    my $currlevel = -1;
    my $oldlevel;

    foreach my  $rl ( sort { $a <=> $b } keys %readlev ) {


	# Display Existing Values (if any)
	my $sth = $dbh->prepare("select * from read_level where readlevel = ? 
         order by seq");
	$sth->execute( $rl );
	if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }

	while ( my ( $id, $readlevel, $category, $name, $seq, $help1, $help2, $help3  ) 
		= $sth->fetchrow ) {

	    $oldlevel = $currlevel;
	    $currlevel = $readlevel;
	
	    if ( $currlevel ne $oldlevel ) {
		if ( not $first ) { 
		    print "</table><p>";
		    print "<input type=\"submit\" value=\"";
		    print $lex{'Delete Selected Records'}. "\"></p>\n";
		} else { $first = 0; }

		print "<div style=\"font-size:150%; font-weight:bold;\">";
		print $lex{'Reading Level'};
		print " $currlevel</div>\n";

		print "<table cellpadding=\"3\" cellspacing=\"0\" border=\"1\">\n";
		print "<tr><th>". $lex{Category}. "</th><th>". $lex{Objective};
		print "</th><th>". $lex{Sequence}. "</th>";
		print "<th>". $lex{Delete}. "</th></tr>\n";

	    }

	    print "<tr><td>$category</td><td>$name</td><td>$seq</td>\n";
	    print "<td><input type=\"checkbox\" name=\"$id\" value=\"1\"></td></tr>\n";

	    # Show the helps
	    if ( $help1 or $help2 or $help3 ) {
		print "<tr><td colspan=\"4\" style=\"background-color:#DDD\"><ul>\n";
		print "$help1</ul><ul>$help2</ul><ul>$help3</ul></td></tr>\n";
	    }

	}
    }

    if ( $first ) { print "<tr><td>". $lex{'No Records Found'}. "</td></tr>\n"; }

    print "</table></form>\n";

#    print "<h1>". $lex{'Complete Help List'}. "</h1>\n";

#    $sth = $dbh->prepare("select distinct readlevel from read_level order by readlevel");
#    $sth->execute;
#    if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
#    my $readref = $sth->fetchall_hashref(readlevel);

#    foreach my $readlevel ( sort { $a <=> $b } keys %{$readref} ) {
#	showHelp( $readlevel );
#    }

    print "</center></body></html>\n";

    exit;

}


#----------------
sub deleteRecords {
#----------------


    #foreach my $key ( sort keys %arr ) { print "K:$key V:$arr{$key}<br>\n"; }

    my $sth = $dbh->prepare("delete from read_level where id = ?");

    foreach my $id ( keys %arr ) { 

	$sth->execute( $id );

	if ( not $DBI::errstr ) {
	    print '<h3>'. $lex{'Record Deleted'}. ".</h3>\n";
	} else { 
	    print '<h3>'. $lex{Error}. ". \n";
	    print $lex{Contact}. " $adminname ";
	    print "(<a href=\"mailto:$adminemail\">$adminemail</a>)<br>\n";
	    print " $DBI::errstr</h3>\n";
	}

    }

    print "<p>[ <a href=\"$self\">". $lex{'View Reading Levels'}. "</a> |\n";
    print "<a href=\"$homepage\">". $lex{Main};
    print "</a> ]</p></center></body></html>\n";

    exit;

}


#-----------
sub showHelp {
#-----------

    my $readlevel = shift;

    my $sth = $dbh->prepare("select distinct category, objective
      from read_help where readlevel = ?");
    $sth->execute( $readlevel );
    if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
    my $rows = $sth->rows;

    if ( $rows ) {
	print "<table cellpadding=\"3\" cellspacing=\"0\" border=\"1\">\n";
	print "<tr><th class=\"h\">". $lex{'Reading Level'}. q{ &ndash; }. $lex{Category};
	print q{ &ndash; }. $lex{Objective}. "</th></tr>\n";
    }

    my $sth1 = $dbh->prepare("select teach_strategy, book_title from read_help 
      where readlevel = ? and category = ? and objective = ?");

    my $sth2 = $dbh->prepare("select count(*) from read_level 
      where readlevel = ? and category = ? and name = ?");

    while ( my ( $category, $objective ) = $sth->fetchrow ) {

	# Check for matching records in reading level.... alway should be a match
	$sth2->execute( $readlevel, $category, $objective );
	if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
	my $count = $sth2->fetchrow;
	
	# Load help records
	$sth1->execute( $readlevel, $category, $objective );
	if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }

	print "<tr><td style=\"font-weight:bold;\"> $readlevel - $category - $objective";
	if ( not $count ) {
	    print "&nbsp;&nbsp;<span style=\"background-color:red;\"> ". $lex{'No Match'}. " </span>\n";
	}
	print "</td></tr>\n";
	print "<tr><td style=\"background-color:#DDD\"><ul>\n";

	while ( my ( $teach_strategy, $book_title ) = $sth1->fetchrow ) {
	    print "<li>$teach_strategy  <i>$book_title</i></li>\n";
	}
	print "</ul></td></tr>\n";

    }
    if ( $rows ) {
	print "</table><p></p>\n";
    }

}


#-----------
sub showTinyHelp {
#-----------

    my ( $readlevel,$category,$objective )  = @_;

#    my $sth = $dbh->prepare("select teach_strategy, book_title from read_help 
#      where readlevel = ? and category = ? and objective = ?");

#    $sth->execute( $readlevel, $category, $objective );
    if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
    my $rows = $sth->rows;

    if ( $rows ) {
	print "<tr><td colspan=\"4\" style=\"background-color:#DDD\"><ul>\n";
    
	while ( my ( $teach_strategy, $book_title ) = $sth->fetchrow ) {
	    print "<li>$teach_strategy &ndash; <i>$book_title</i></li>\n";
	}
	print "</ul></td></tr>\n";
    }

}
