#!/usr/bin/perl
#  Copyright 2001-2014 Leslie Richardson

#  This file is part of Open Admin for Schools.

# DRA Summary Reading Report - reports on multiple years.

# Read Report - read all reading tests from all schools, calc the
# Equiv Grade and then compare to the actual grade and get a
# delta. Compile into per grade, per school and overall stats.


my %lex = ('Error' => 'Error',
	   'Main' => 'Main',
	   'Start Date' => 'Start Date',
	   'End Date' => 'End Date',
	   'Date' => 'Date',
	   'Continue' => 'Continue',
	   'School' => 'School',
	   'Database' => 'Database',
	   'Reading' => 'Reading',
	   'Report' => 'Report',
	   'Global' => 'Global',
	   'Select' => 'Select',
	   'Split' => 'Split',
	   'Grade' => 'Grade',
	   'Current' => 'Current',
	   'Output' => 'Output',
	   'Combine failed on input' => 'Combine failed on input',
	   'Download CSV File' => 'Download CSV File',
	   'Test' => 'Test',
	   'Group' => 'Group',
	   'Division' => 'Division',
	   'Paper Size' => 'Paper Size',
	   'Letter' => 'Letter',
	   'Legal' => 'Legal',
	   'A4' => 'A4',
	   'Check All' => 'Check All',
	   'Summary' => 'Summary',
	   'Start Year' => 'Start Year',
	   'Spring' => 'Spring',
	   'Fall' => 'Fall',
	   'Season' => 'Season',
	   'End Year' => 'End Year',
	   'Missing' => 'Missing',
	   'Value' => 'Value',

	   );


my $self = 'readsummary.pl';


# Maps Grades to Grade Divisions I, II, III, IV.
my %divmap = ( 'K' => 1, 1 => 1, 2 => 1, 3 => 1, 
	       4 => 2, 5 => 2, 6 => 2, 
	       7 => 3, 8 => 3, 9 => 3,
	       10 => 4, 11 => 4, 12 => 4 );


use GD::Graph::bars;
use GD::Graph::colour qw(:colours);
use DBI;
use CGI;
use Number::Format qw(:all);
use Time::JulianDay;
use Text::CSV_XS;
use Statistics::Basic qw(:all);
use Math::Round qw(nhimult);


eval require "../admin.conf";
if ( $@ ) {
    print $lex{Error}. ": $@<br>\n";
    die $lex{Error}. ": $@\n";
}

eval { require "$globdir/global.conf"; };
if ( $@ ) {
    print $lex{Error}. " $self: $@<br>\n";
    die $lex{Error}. "$self: $@\n";
}


@dbase = sort keys %alldbase; # global records.


my @time = localtime(time);
my $year = $time[5] + 1900;
my $month = $time[4] + 1;
my $currdate = "$year-$month-$time[3]";

my $schyear = $year;
if ( $month > 7 ){ $schyear++; }


my $q = CGI->new;
my %arr = $q->Vars;
print $q->header( -charset, $charset );


# Script Heading Descriptor
my $title = "$lex{Summary} $lex{Reading} $lex{Report}";
my $title1 = "Percentage of Students vs Differential Reading Level by Year";


# Print Page Heading
print "$doctype\n<html><head><title>$title</title>\n";
print "<link rel=\"stylesheet\" href=\"/division.css\" type=\"text/css\">\n";


print qq{$chartype\n</head><body style="padding:0.4em 2em;">\n};
print qq{[ <a class="alt" href="$homepage">$lex{Main}</a> ]\n};
print qq{<h1>$title</h1>\n};
print qq{<h3>$title1</h3>\n};


if ( not $arr{page} ) {
    showStartPage();

} elsif ( $arr{page} == 1 ) {
    delete $arr{page};
    selectYears();

} elsif ( $arr{page} == 2 ) {
    delete $arr{page};
    createReport();

}


#----------------
sub categoryTable { # display results by -2 or below, -1, at or above.
#----------------

    my $ref = shift;

    # %data is {year}{delta} = testcount;
    my %data = %$ref;

    print qq{<h3>Student Reading Achievement by Relative Reading Level</h3>\n};

    # Start Table
    print qq{<table cellspacing="0" cellpadding="3" border="1">\n};
    print qq{<tr><th>Year</th><th>Significantly Below<br>Expected Level<br>(-2 years or more)</th>\n};
    print qq{<th>Below Expected<br>Level (-1 year)</th>};
    print qq{<th>At or Above Expected<br>Reading Level</th></tr>\n};


    foreach my $year ( sort keys %data ) {
	my ( $minus2, $minus1, $normal );
	foreach my $delta ( keys %{ $data{$year} } ) {
	    if ( $delta <= -2 ) {
		$minus2 += $data{$year}{$delta};
	    } elsif ( $delta == -1 ) {
		$minus1 += $data{$year}{$delta};
	    } else {
		$normal += $data{$year}{$delta};
	    }
	}
	my $total = $minus2 + $minus1 + $normal;

	my ($minus2avg, $minus2avg, $normalavg);
	if ( $total ) { 
	    $minus2avg = round( $minus2 / $total * 100, 1);
	    $minus1avg = round( $minus1 / $total * 100, 1);
	    $normalavg = round( $normal / $total * 100, 1);
	} 

	print qq{<tr><td class="la">$year</td>};
	print qq{<td class="cn">$minus2avg%</td>};
	print qq{<td class="cn">$minus1avg%</td>};
	print qq{<td class="cn">$normalavg%</td>};
	print qq{</tr>\n};

    }

    print qq{</table>\n};

    return;

}



#----------------
sub generateGraph {
#----------------

    my $ref = shift;

    # %data is {year}{delta} = testcount;
    my %data = %$ref;

    my @years = sort keys %data;
    shift @years until scalar @years < 4;


    # Get MIN, MAX values of Delta Grade (How far above/below grade level.
    my ($max, $min, $studcount ); # min and max delta values and total test count;
    my %dcount; # overall count of tests for each delta.
    foreach my $year ( sort keys %data ) {
	foreach my $delta ( sort keys %{ $data{$year} } ) {
	    if  ( $delta > $max ) { $max = $delta; }
	    if  ( $delta < $min ) { $min = $delta; }
	    $studcount += $data{$year}{$delta};
	    $dcount{$delta} += $data{$year}{$delta};
	    # print "G:$year D:$delta Count:$schooldata{$year}{$delta}<br>\n";
	}
    }

    # Setup the Delta array for x coords.
    my @delta;
    for my $idx ($min..$max) {
	push @delta, $idx;
    }

    my $testmax; # maximum y value

    my %yeararray;
    foreach my $year ( keys %data ) {
	my @temp;
	foreach my $delta ( $min..$max ) {
	    push @temp, $data{$year}{$delta};

	    if ( $data{$year}{$delta} > $testmax ) {
		$testmax = $data{$year}{$delta};
	    }
	}
	$yeararray{$year} = \@temp;
    }



    $testmax = nhimult(5, $testmax);


    my @spacer;
    foreach my $year ( keys %data ) {
	push @spacer, '';
    }


    my $graph = GD::Graph::bars->new(800, 600);
    $graph->set( dclrs => [ qw{blue green red dblue dgreen dred} ]);    


#    my @data = ( \@delta, \@year1, \@year2, \@year3 );
    
    my @data;
    push @data, \@delta;

    my $count = 1;
    foreach my $year ( @years ) { # reverse sort keys %yeararray ) {
	push @data, \@{ $yeararray{$year} };
	$count++;
	if ($count > 3) { last; }
    }

    push @data, \@spacer;


    $graph->set_legend_font(gdLargeFont); 
    $graph->set_legend( @years );

    $graph->set(
		x_label => 'Relative Reading Level',
		y_label => 'Percent Student',
		title  => "Summary Reading Levels",
		y_max_value => $testmax,
		x_max_value => $max,
		) or die $graph->error;




    my $gd = $graph->plot(\@data) or die $graph->error;


    my $filename = "drasumm-$$.png";
    open(IMG,">$filename");
    binmode IMG;
    print IMG $gd->png;
    close IMG;

    system("mv $filename $downloaddir");

    return $filename;

}



#-----------
sub printPDF { # print one table (one school or summary )
#-----------

    my ($mode, $dataref ) = @_;
    # mode is either grade or division (I,II,etc)
    my %data = %$dataref;

}



#-----------
sub startPDF {  # print the setup for PDF
#-----------

    my ($filename,$papersize) = shift;

    # Preamble and Head of Document
    my ( $papersize, $textwidth, $textheight );

    if ( $arr{papersize} eq 'letter' ) {
	$papersize = 'letterpaper';
	$textwidth = $g_letterpaper_textwidth;
	$textheight = $g_letterpaper_textheight;
    } elsif ( $arr{papersize} eq 'legal' ) {
	$papersize = 'legalpaper';
	$textwidth = $g_legalpaper_textwidth;
	$textheight = $g_legalpaper_textheight;
    } elsif ( $arr{papersize} eq 'a4' ) {
	$papersize = 'a4paper';
	$textwidth = $g_a4paper_textwidth;
	$textheight = $g_a4paper_textheight;
    }

    my $shortname = "readprogress$$";
    my $filename = "$shortname.tex";

    open(TEX,">$filename") || die "Can't open tex file";


    print TEX <<"EOS1";  
\\documentclass[ 12pt, $papersize, oneside]{article}
\\usepackage{array,graphicx,multicol,colortbl,inputenc,fancyhdr}
\\inputencoding{latin1}
\\renewcommand{\\familydefault}{\\sfdefault}

\\setlength{\\hoffset}{-20mm}
\\setlength{\\voffset}{-20mm}

\\setlength{\\headsep}{10pt}
\\setlength{\\headheight}{14pt}
\\setlength{\\topmargin}{0mm}

\\setlength{\\textwidth}{ $textwidth }
\\setlength{\\textheight}{ $textheight }

\\setlength{\\parindent}{0pt}
\\setlength{\\evensidemargin}{0mm}
\\setlength{\\oddsidemargin}{0mm}

\\begin{document}
EOS1


    # Print Data Structure


    print TEX "\\end{document}\n";
    close TEX;

    system("$pdflatex $filename >pdflog$$.txt");
   
    system("mv $shortname.pdf $downloaddir");
    system("mv pdflog$$.txt $downloaddir");
    system("rm -f $shortname.*");

    print "<h1><a href=\"$webdownloaddir/$shortname.pdf\">";
    print "$lex{'View/Download'} $lex{'Progress Report'}</a></h1>\n";
    print "<p>[ <a href=\"$homepage\">$lex{Main}</a> |\n <a href=\"$webdownloaddir/pdflog$$.txt\">";
    print "$lex{'View Log File'}</a>\n ]</p>\n";
    print "</body></html>\n";



} # end of printPDF



#-----------
sub writeCSV {
#-----------

    my $ref = shift;

    # print "Val:", @val, " Ref:$ref<br>\n";
    
    if ( $csv->combine( @$ref ) ) {
	my $record = $csv->string;
	print EX $record, "\r\n";

    } else {
	my $err = $csv->error_input;
	print "$lex{'Combine failed on input'}: $err<br>\n";
    }

    return;

} # end writeCSV




#------------
sub exportCSV {
#------------

    # foreach my $key ( sort keys %arr ) { print "K:$key V:$arr{$key}<br>\n"; }

    # Remove the extra:
    delete $arr{grouping};
    delete $arr{papersize};


    $csv = Text::CSV_XS->new( {binary => 1} );

    # Open output file
    my $filename = "expReadProgress$$.csv";
    open (EX,">$filename") || die $lex{'Cannot open Export file'}. " $fileName";


    my $sdate = $arr{startdate};
    delete $arr{startdate};
    my $edate = $arr{enddate};
    delete $arr{enddate};

    # Write date range.
    my @data = ();
    #for my $i (1..20){ $data[$i] = ' ';}
    $data[0] = "$lex{'Start Date'} $sdate";
    $data[2] = "$lex{'End Date'} $edate";
    writeCSV(\@data);


    my $startjd = julian_day( split(/-/, $sdate ));
    my $endjd = julian_day( split(/-/, $edate ));

    my %overall = ();

    foreach my $db ( sort keys %arr ) {
	
	my $database = $db;
	my $dsn = "DBI:$dbtype:dbname=$database";
	my $dbh = DBI->connect($dsn,$guser,$gpassword);
	my $schoolname = $alldbase{"$db"};

	# Write the school name
	my @data = ();
	#for my $i (1..20){ $data[$i] = ' ';}
	$data[0] = $schoolname;
	writeCSV(\@data);

	
	# Get all students in the school who have tests into a @students array
	my @students = ();
	my $sth = $dbh->prepare("select distinct studnum from read_test");
	$sth->execute;
	if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
	while ( my $studnum = $sth->fetchrow ) {
	    push @students, $studnum;
	}

	if ( not @students ) {

	    my @data = ();
	    for my $i (1..20){ $data[$i] = ' ';}
	    $data[0] = "No Student Tests Found";
	    writeCSV(\@data);

	    # Write blank line.
	    my @data = ();
	    $data[0] = ' ';
	    writeCSV(\@data);
	    
	    next;
	}


	# Now loop through all students printing results; setup queries first
	my $sth1 = $dbh->prepare("select id, tdate, readlevel, tgrade from read_test 
         where studnum = ? order by tdate desc");

	my $sth2 = $dbh->prepare("select sum(score) from read_test_score where testid = ?");
	
	my %schooldata = ();

	foreach my $studnum ( @students ) {

	    # Get Tests
	    $sth1->execute( $studnum );
	    if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }

	    # Only use latest value for the student.
	    my $trows = $sth1->rows;
	    # if $onlylatest, limit to 1 test
	    #if ( $onlylatest and $trows > 1 ) { $trows = 1; } 
	    $trows = 1;
	    for ( 1 .. $trows ) {

		my ( $testid, $tdate, $readlevel, $tgrade ) = $sth1->fetchrow;

#	    print "JD:$testjd  Start:$startjd  \n";
#	    if ( $testjd > $startjd ) { print "<br>GREATER<br>\n"; }
#	    if ( $testjd < $endjd ) { print "<br>GREATER<br>\n"; }

		# skip out of date range.
		my $testjd = julian_day( split(/-/, $tdate));
		if ( $testjd < $startjd or $testjd > $endjd ) { next; } 

		# Get the score total;
		$sth2->execute( $testid );
		if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
		my $total = $sth2->fetchrow;

		# Equivalent Grade print
		my $equivgrade = scoreToGrade($total, $readlevel );

#	    print "<div>$testid - $total - $equivgrade</div>\n";

		my $grdelta;
		if ( not $tgrade ) {
		    print "Grade Missing: $studnum - $readlevel - $tdate<br>\n";
		    next;
		} else { # Find grade delta
		    $grdelta = $equivgrade - $tgrade;
		}

		if ( $grdelta >= 0 ) { $grdelta = int $grdelta; } # truncate.
		else { $grdelta = int $grdelta - 1; }

		$schooldata{$tgrade}{$grdelta}++;
		$overall{$tgrade}{$grdelta}++;

	    } # end of trows loop

	} # end of student loop


	my ($max, $min, $studcount );
	my %dcount = ();
	my @stats = ();
	foreach my $grade ( sort keys %schooldata ) {
	    foreach my $delta ( sort keys %{ $schooldata{$grade} } ) {
		# Set Min/Max values
		if  ( $delta > $max ) { $max = $delta; }
		if  ( $delta < $min ) { $min = $delta; }

		# Get Count and Delta Counts.
		$studcount += $schooldata{$grade}{$delta};
		$dcount{$delta} += $schooldata{$grade}{$delta};
		for (1..$dcount{$delta} ) { push @stats, $delta; } # put delta values into @stats
		# print "G:$grade D:$delta Count:$schooldata{$grade}{$delta}<br>\n";
	    }
	}



	# Table Heading
	@data = ();
	$data[0] = "$lex{Grade}";
	foreach my $delta ( $min .. $max ) {
	    push @data, $delta;
	}
	push @data, "SDev";
	push @data, "Var";

	writeCSV(\@data);


	# Write each grade row.
	foreach my $grade ( sort {$a <=> $b} keys %schooldata ) {
	    my @data = ();
	    my @grstats = ();
	    $data[0] = $grade;
	    foreach my $delta ( $min .. $max ) {
		push @data, $schooldata{$grade}{$delta};
		for (1..$schooldata{$grade}{$delta}) { push @grstats, $delta; }
	    }

	    push @data, stddev(\@grstats);
	    push @data, variance(\@grstats);

	    writeCSV(\@data);
	}


	# Write average Row
	my @data = ();
	my @data1 = ();
	$data[0] = 'Average';
	$data1[0] = 'Count';
	foreach my $delta ( $min .. $max ) {
	    my $avg;
	    if ( $studcount ) {
		$avg = round( $dcount{$delta} / $studcount * 100, 2);
	    }
	    push @data, $avg;
	    push @data1, $dcount{$delta}
	    # print "<td>$dcount{$delta}/$studcount<br>$avg%</td>";
	}

	push @data, stddev(\@stats);
	push @data, variance(\@stats);

	writeCSV(\@data); # average
	writeCSV(\@data1); # count


	# Duplicate of the studcount row...
	# Write Column Sum
#	my @data = ();
#	$data[0] = 'Column Sum';
#	foreach my $delta ( $min .. $max ) {
#	    push @data,	$dcount{$delta}
#	}
#	writeCSV(\@data);


	# Write studcount Row
	my @data = ();
	$data[0] = 'S Count';
	foreach my $delta ( $min .. $max ) {
	    push @data, $studcount;
	}
	writeCSV(\@data);

	# Write blank line.
	my @data = ();
	$data[0] = ' ';
	writeCSV(\@data);


    } # database loop



    # All Schools Combined.
    my @data = ();
    $data[0] = 'Combined Schools Result';
    writeCSV(\@data);


    # Next Line
    $max = 0, 
    $min = 0, 
    $studcount = 0;
    %dcount = ();
    @stats = ();

    foreach my $grade ( sort keys %overall ) {
	foreach my $delta ( sort keys %{ $overall{$grade} } ) {
	    if  ( $delta > $max ) { $max = $delta; }
	    if  ( $delta < $min ) { $min = $delta; }
	    $studcount += $overall{$grade}{$delta};
	    $dcount{$delta} += $overall{$grade}{$delta};
	    for (1..$overall{$grade}{$delta}) { push @stats, $delta; }
	    # print "G:$grade D:$delta Count:$overall{$grade}{$delta}<br>\n";
	}
    }

    # Write Heading Line
    my @data = ();
    $data[0] = $lex{Grade};
    foreach my $delta ( $min .. $max ) {
	push @data, $delta;
    }
    push @data, "SDev";
    push @data, "Var";

    writeCSV(\@data);


    # Write all grade rows.
    foreach my $grade ( sort {$a <=> $b} keys %overall ) {
	my @data = ();
	my @grstats = ();
	$data[0] = $grade;
	foreach my $delta ( $min .. $max ) {
	    push @data, $overall{$grade}{$delta};
	    for (1..$overall{$grade}{$delta}) { push @grstats, $delta; }
	}

	push @data, stddev(\@grstats);
	push @data, variance(\@grstats);

	writeCSV(\@data);
    }


    # Write average Row
    my @data = ();
    $data[0] = 'Average';
    foreach my $delta ( $min .. $max ) {
	my $avg;
	if ( $studcount ) {
	    $avg = round( $dcount{$delta} / $studcount * 100, 2);
	}
	push @data, $avg;
	# print "<td>$dcount{$delta}/$studcount<br>$avg%</td>";
    }

    push @data, stddev(\@stats);
    push @data, variance(\@stats);

    writeCSV(\@data);


    # Write Column Sum Row
    my @data = ();
    $data[0] = 'Count';
    foreach my $delta ( $min .. $max ) {
	push @data,	$dcount{$delta}
    }
    writeCSV(\@data);


    # Write studcount Row
    my @data = ();
    $data[0] = 'Total';
    foreach my $delta ( $min .. $max ) {
	push @data, $studcount;
    }
    writeCSV(\@data);


    close EX;
    system("mv $filename $downloaddir");
    
    print "<p><a href=\"$webdownloaddir/$filename\">\n";
    print "<span style=\"font-size:1.5em;\">$lex{'Download CSV File'}</span></a></p>\n";
    print "</body></html>\n";

    exit;

} # end of exportCSV




#---------------
sub createReport {
#---------------

    # foreach my $key ( sort keys %arr ) { print "K:$key V:$arr{$key}<br>\n"; }

    if ( not $arr{season} or not $arr{startyear} or not $arr{endyear} ) {
	print qq{<h3>$lex{Missing} $lex{Value}</h3>\n};
	print qq{</body></html>\n};
	exit;
    }

    # sdate and edate are formatted dates.
    # print "<h3>$lex{'Start Date'} $sdate | $lex{'End Date'} $edate</h3>\n";

    my $db = $arr{db};
    my $dsn = "DBI:$dbtype:dbname=$db";
    my $dbh = DBI->connect($dsn,$guser,$gpassword);
    my $schoolname = $alldbase{"$db"};

    print "<h3>$schoolname</h3>\n";

    my %dates = ( 'spring' => 'Feb 1 - Jun 30', 'fall' => 'Aug 1 - Jan 31' );

    my $season = ucfirst $arr{season};
    print qq{<p><b>$lex{Season}</b>: $season <b>Dates</b>: $dates{"$arr{season}"}</p>\n};


    # Get Start and End dates and also seasonality
    my $startyear = $arr{startyear};
    my $endyear = $arr{endyear};


    my %overall;
    my %schooldata;


    # Get all students who have tests into a @students array (may not be current)
    my @students = ();
    my $sth = $dbh->prepare("select distinct studnum from read_test");
    $sth->execute;
    if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
    while ( my $studnum = $sth->fetchrow ) {
	push @students, $studnum;
    }

    if ( not @students ) {
	print "<h3>No Student Tests Found</h3>\n";
	next;
    }



    foreach my $year ( $startyear..$endyear ) {

	# Get Start/End Dates.
	my ($startdate, $enddate);
	my $tmpyear = $year - 1; # since doing fall of prev calendar year
	if ( $arr{season} eq 'fall' ) {
	    $startdate = $tmpyear. '-08-01';
	    $enddate = $year. '-01-31';
	} else { # spring
	    $startdate = $year. '-02-01';
	    $enddate = $year. '-06-30';
	}


	# Now loop through all students printing results; setup queries first
	my $sth1 = $dbh->prepare("select id, tdate, readlevel, tgrade from read_test 
         where studnum = ? and to_days(tdate) >= to_days('$startdate') and 
          to_days(tdate) <= to_days('$enddate') order by tdate desc");
	if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }

	my $sth2 = $dbh->prepare("select sum(score) from read_test_score where testid = ?");
	my $sth3 = $dbh->prepare("select lastname, firstname from studentall where studnum = ?");	


	foreach my $studnum ( @students ) {

	    # Get Tests
	    $sth1->execute( $studnum );
	    if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }

	    # Only use latest value for the student.
	    my $trows = $sth1->rows;
	    # if $onlylatest, limit to 1 test
	    # if ( $onlylatest and $trows > 1 ) { $trows = 1; } 
	    if ( $trows > 1 ) { $trows = 1; }


	    for ( 1 .. $trows ) {

		my ( $testid, $tdate, $readlevel, $tgrade ) = $sth1->fetchrow;


		# Get the score total;
		$sth2->execute( $testid );
		if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
		my $total = $sth2->fetchrow;

		# Equivalent Grade print
		my $equivgrade = scoreToGrade($total, $readlevel );

		my $grdelta;
		if ( not $tgrade ) {
		    $sth3->execute( $studnum );
		    my ( $lastname, $firstname ) = $sth3->fetchrow;		    
		    print "Grade Missing: $lastname, $firstname ($studnum) - $tdate<br>\n";
		    next;
		
		} else { # Find grade delta
		    $grdelta = $equivgrade - $tgrade;
		}

		if ( $grdelta >= 0 ) { $grdelta = int $grdelta; } # truncate.
		else { $grdelta = int $grdelta - 1; }

		# Original
		# $schooldata{$tgrade}{$grdelta}++;
		# $overall{$tgrade}{$grdelta}++;

		$schooldata{$year}{$grdelta}++;
		$overall{$year}{$grdelta}++;


	    } # end of trows loop

	} # end of student loop


    } # year loop


    # Skip if no tests in this date range
    if ( not %schooldata ) { 
	print "<h3>No tests in this Date Range</h3>\n";
	next; 
    }


    # Get MIN, MAX values of Delta Grade (How far above/below grade level.
    my ($max, $min, $studcount );
    my %dcount = ();
    foreach my $year ( sort keys %schooldata ) {
	foreach my $delta ( sort keys %{ $schooldata{$year} } ) {
	    if  ( $delta > $max ) { $max = $delta; }
	    if  ( $delta < $min ) { $min = $delta; }
	    $studcount += $schooldata{$year}{$delta};
	    $dcount{$delta} += $schooldata{$year}{$delta};
	    # print "G:$year D:$delta Count:$schooldata{$year}{$delta}<br>\n";
	}
    }



    # Start Table
    print qq{<table cellspacing="0" cellpadding="3" border="1">\n};

    # Heading.
    print "<tr><th>$lex{Year}</th>";
    foreach my $delta ( $min .. $max ) {
	print "<th>$delta</th>";
    }
    print "<th>No. Tested</th><th>Average<br>Difference</th>";
    print "</tr>\n";


    foreach my $year ( $startyear..$endyear ) {

	my %dcount;
	my $studcount;
	foreach my $delta ( sort keys %{ $schooldata{$year} } ) {
	    if  ( $delta > $max ) { $max = $delta; }
	    if  ( $delta < $min ) { $min = $delta; }
	    $studcount += $schooldata{$year}{$delta};
	    $dcount{$delta} += $schooldata{$year}{$delta};
	    # print "G:$year D:$delta Count:$schooldata{$year}{$delta}<br>\n";
	}


	print "<tr><th>$year</th>";
	my $partavg;
	foreach my $delta ( $min .. $max ) {
	    $partavg += ($delta * $dcount{$delta} );
	    my $avg;
	    if ( $studcount ) {
		$avg = round( $dcount{$delta} / $studcount * 100, 1);
	    }
	    
	    if ( $dcount{$delta} ) {
#		print "<td>$dcount{$delta}/$studcount<br>$avg%</td>";
		print "<td>$avg%</td>";
	    } else { print qq{<td></td>}; }

	}

	my $avg;
	if ( $studcount ) {
	    $avg = $partavg / $studcount;
	} else { $avg = 0; }

	$avg = round($avg,2);
	print "<th>$studcount</th><th>$avg</th></tr>\n";

    }



=head
	# Summary Row
	print "<tr><th>$year</th>";
	my $partavg;
	foreach my $delta ( $min .. $max ) {
	    $partavg += ($delta * $dcount{$delta} );
	    my $avg;
	    if ( $studcount ) {
		$avg = round( $dcount{$delta} / $studcount * 100, 2);
	    }
	    
	    print "<td>$dcount{$delta}/$studcount<br>$avg%</td>";
	}

	my $avg = $partavg / $studcount;
	$avg = round($avg,2);
	print "<th>$studcount</th><th>$avg</th></tr>\n";

#---------------------
    $max = 0, 
    $min = 0, 
    $studcount = 0;
    %dcount = ();

    foreach my $grade ( sort keys %overall ) {
	foreach my $delta ( sort keys %{ $overall{$grade} } ) {
	    if  ( $delta > $max ) { $max = $delta; }
	    if  ( $delta < $min ) { $min = $delta; }
	    $studcount += $overall{$grade}{$delta};
	    $dcount{$delta} += $overall{$grade}{$delta};
	    # print "G:$grade D:$delta Count:$overall{$grade}{$delta}<br>\n";
	}
    }


    print "<table cellspacing=\"0\" cellpadding=\"3\" border=\"1\">\n";

    # Heading.
    print "<tr><th>$lex{Grade}</th>";
    foreach my $delta ( $min .. $max ) {
	print "<th>$delta</th>";
    }
    print "<th>$lex{Test}#</th><th>Wt. Avg</th>";
    print "</tr>\n";

    # Data
    foreach my $grade ( sort {$a <=> $b} keys %overall ) {
	print "<tr><td>$grade</td>";
	my ($total,$partavg);
	foreach my $delta ( $min .. $max ) {
	    my $count = $overall{$grade}{$delta};
	    print "<td>$overall{$grade}{$delta}</td>";
	    $total += $count;
	    $partavg += ( $count * $delta );
	}

	my $avg;
	if ( $total ) {
	    $avg = $partavg / $total;
	} else { $avg = 0; }

	$avg = round($avg, 2);
	print "<td>$total</td><td>$avg</td>";
	print "</tr>\n";

    }


    print "<tr><th>Average</th>";
    my $partavg;
    foreach my $delta ( $min .. $max ) {
	$partavg += ($delta * $dcount{$delta} );
	my $avg;
	if ( $studcount ) {
	    $avg = round( $dcount{$delta} / $studcount * 100, 2);
	}
	print "<td>$dcount{$delta}/$studcount<br>$avg%</td>";
    }

    my $avg = $partavg / $studcount;
    $avg = round($avg,2);
    print "<th>$studcount</th><th>$avg</th></tr>\n";
=cut

    print "</table><p></p>\n";

    my $filename = generateGraph(\%schooldata);
    my $src = $webdownloaddir. '/'. $filename;

    print qq{<img src="$src"><br clear="left">\n};
    

    categoryTable(\%schooldata);


    exit;
}



#----------
sub fmtDate {
#----------

    my ( $year, $mon, $day ) = split /-/, shift;
    return "$year-$s_month[$mon]-$day";
}



#---------------
sub scoreToGrade {
#---------------

    my ($score, $readinglevel) = @_;

    my %rd_LevelToGrade = ( '0' => 0.2, 1 => 0.5, 2 => 0.8, 3 => 1.1, 4 => 1.2,
			    6 => 1.3, 8 => 1.4, 10 => 1.5, 12 => 1.6, 14 => 1.7,
			    16 => 1.8, 18 => 1.9, 20 => '2.0', 24 => 2.5, 28 => 2.9,
			    30 => '3.0', 34 => 3.5, 38 => 3.9);


    my %rd_ScoreToGrade = (
	50 => 1.9, 52 => '2.0', 53 => 2.1, 54 => 2.2,
	55 => 2.3, 56 => 2.5, 57 => 2.7, 58 => 2.8,
	61 => 2.9, 62 => '3.0', 63 => 3.1, 64 => 3.2, 
	65 => 3.3, 66 => 3.5, 67 => 3.6, 68 => 3.7,
	69 => 3.8, 70 => 3.9, 74 => 4.3, 76 => 4.6,
	80 => 4.9, 84 => 5.3, 86 => 5.6, 90 => 5.9,
	94 => 6.3, 96 => 6.6, '100' => '6.9',
	'104' => '7.3', '106' => '7.6','110' => '7.9',
	'114' => '8.3', '119' => '8.6','124' => '8.9');


    if ( $readinglevel < 40 ) {

	foreach my $key ( sort { $b <=> $a } keys %rd_LevelToGrade ) {
	    # print "K:$key ";
	    if ( $readinglevel >= $key ) {
		# print "<br><b>Match: K:$key RL:$readinglevel</b><br>\n";
		return $rd_LevelToGrade{ $key };
	    }
	}

    } else { # else reading level >= 40

	$score += $readinglevel; # add both together for this 

	foreach my $key ( sort { $b <=> $a } keys %rd_ScoreToGrade ) {
	    # print "K:$key ";
	    if ( $score >= $key ) {
		# print "<b>Match: S:$score K:$key V:$rd_ScoreToGrade{$key}</b><br>\n";
		return $rd_ScoreToGrade{ $key };
	    }
	}

    }

    return 'NA';
}




#----------------
sub showStartPage {
#----------------

    print "<form action=\"$self\" method=\"post\">\n";
    print "<input type=\"hidden\" name=\"page\" value=\"1\">\n";

    print "<table cellpadding=\"3\" cellspacing=\"0\" border=\"0\" ";
    print "style=\"padding:0.5em;border:1px solid gray;\">\n";

    print qq{<tr><td class="bla">$lex{Select} $lex{School}</td></tr>\n};
    foreach my $db ( @dbase ) { 
	print qq{<tr><td class="la">};
	print qq{<input type="radio" name="db" value="$db">\n};
	print qq{$alldbase{$db} ($db)</td></tr>\n};
    }

    print qq{<tr><td class="la"><input type="submit" value="$lex{Continue}"></td></tr>\n};
    print qq{</table></form>\n};


    print qq{</body></html>\n};

    exit;

} # end of showStartPage


#--------------
sub selectYears {
#--------------

    # foreach my $key ( sort keys %arr ) { print "K:$key V:$arr{$key}<br>\n"; }

    my $database = $arr{db};
    my $dsn = "DBI:$dbtype:dbname=$database";
    my $dbh = DBI->connect($dsn,$guser,$gpassword);
    my $schoolname = $alldbase{"$db"};

    # Get all school years in the read_test table.
    my $sth = $dbh->prepare("select distinct tdate from read_test");
    $sth->execute;
    if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
    my $ref = $sth->fetchall_hashref(tdate);

    my %years; # only store second part of school year (ie. 2014-2015 = 2015; same as grad year)


    foreach my $key ( sort keys %{ $ref } ) {
	my ($y,$m,$d) = split('-', $key);
	if ( $m <= 7 ) { # split on July
	    $years{$y} = 1;
	} else {
	    my $nextyr = $y + 1;
	    $years{$nextyr} = 1;
	}

    }

    print "<form action=\"$self\" method=\"post\">\n";
    print qq{<input type="hidden" name="page" value="2">\n};
    print qq{<input type="hidden" name="db" value="$arr{db}">\n};

    print "<table cellpadding=\"3\" cellspacing=\"0\" border=\"0\" ";
    print "style=\"padding:0.5em;border:1px solid gray;\">\n";

    print qq{<tr><td class="bla">$lex{Select} $lex{'Start Year'}</td></tr>\n};
    foreach my $yr ( sort keys %years ) {
	my $prev = $yr - 1;
	my $fullyear = $prev. '-'. $yr;

	print qq{<tr><td class="la">};
	print qq{<input type="radio" name="startyear" value="$yr">};
	print qq{$fullyear</td></tr>\n};
    }

    print qq{<tr><td colspan="2"></td></tr>\n};

    # Season.
    print qq{<tr><td class="bla">$lex{Select} $lex{Season}</td></tr>\n};
    print qq{<td class="la">};
    print qq{<input type="radio" name="season" value="spring"> $lex{Spring}</td></tr>\n};
    print qq{<td class="la">};
    print qq{<input type="radio" name="season" value="fall"> $lex{Fall}</td></tr>\n};



    print qq{<tr><td colspan="2"></td></tr>\n};

    print qq{<tr><td class="bla">$lex{Select} $lex{'End Year'}</td></tr>\n};

    my $tmp = $schyear - 1;
    my $disyear = "$tmp-$schyear";
    print qq{<tr><td class="la">};
    print qq{<input type="radio" name="endyear" value="$schyear">};
    print qq{$disyear</td></tr>\n};

    my $prevyear = $schyear - 1;
    $tmp = $prevyear - 1;
    $disyear = "$tmp-$prevyear";
    print qq{<tr><td class="la">};
    print qq{<input type="radio" name="endyear" value="$prevyear">};
    print qq{$disyear</td></tr>\n};

    print qq{<tr><td colspan="2"></td></tr>\n};


    print qq{<tr><td class="la"><input type="submit" value="$lex{Continue}"></td></tr>\n};
    print qq{</table></form>\n};

    print qq{</body></html>\n};

    exit;

} # end of selectYears


