#!/usr/bin/perl
#  Copyright 2001-2015 Leslie Richardson

#  This file is part of Open Admin for Schools.

# DRA Summary Reading Report - reports on multiple years.

# Read Report - read all reading tests from all schools, calc the
# Equiv Grade and then compare to the actual grade and get a
# delta. Compile into per grade, per school and overall stats.


my %lex = ('Error' => 'Error',
	   'Main' => 'Main',
	   'Start Date' => 'Start Date',
	   'End Date' => 'End Date',
	   'Date' => 'Date',
	   'Continue' => 'Continue',
	   'School' => 'School',
	   'Database' => 'Database',
	   'Reading' => 'Reading',
	   'Report' => 'Report',
	   'Global' => 'Global',
	   'Select' => 'Select',
	   'Split' => 'Split',
	   'Grade' => 'Grade',
	   'Current' => 'Current',
	   'Output' => 'Output',
	   'Combine failed on input' => 'Combine failed on input',
	   'Download CSV File' => 'Download CSV File',
	   'Test' => 'Test',
	   'Group' => 'Group',
	   'Division' => 'Division',
	   'Paper Size' => 'Paper Size',
	   'Letter' => 'Letter',
	   'Legal' => 'Legal',
	   'A4' => 'A4',
	   'Check All' => 'Check All',
	   'Summary' => 'Summary',
	   'Start Year' => 'Start Year',
	   'Spring' => 'Spring',
	   'Fall' => 'Fall',
	   'Season' => 'Season',
	   'End Year' => 'End Year',
	   'Missing' => 'Missing',
	   'Value' => 'Value',

	   );


my $self = 'tsecreadsummary.pl';


# Maps Grades to Grade Divisions I, II, III, IV.
my %divmap = ( 'K' => 1, 1 => 1, 2 => 1, 3 => 1, 
	       4 => 2, 5 => 2, 6 => 2, 
	       7 => 3, 8 => 3, 9 => 3,
	       10 => 4, 11 => 4, 12 => 4 );

my %dates = ( 'spring' => 'Feb 1 - Jun 30', 'fall' => 'Aug 1 - Jan 31' );


use GD::Graph::bars;
use GD::Graph::colour qw(:colours);
use DBI;
use CGI;
use Number::Format qw(:all);
use Time::JulianDay;
use Text::CSV_XS;
use Statistics::Basic qw(:all);
use Math::Round qw(nhimult);


eval require "../admin.conf";
if ( $@ ) {
    print $lex{Error}. ": $@<br>\n";
    die $lex{Error}. ": $@\n";
}

eval { require "$globdir/global.conf"; };
if ( $@ ) {
    print $lex{Error}. " $self: $@<br>\n";
    die $lex{Error}. "$self: $@\n";
}


# Create restricted dbase array of only tsec schools.
#@dbase = sort keys %alldbase; # global records.
my @exclude = qw(eagleview chieftaylor pewasenakwan khwc chiefnapew thunderchild);
my %exclude = map {$_ => 1} @exclude;

my @dbase;
foreach my $db ( keys %alldbase ) {
    if ( $exclude{$db} ) { next; }
    push @dbase, $db;
}

#print "Dbase:", @dbase, "<br>\n";


my @time = localtime(time);
my $year = $time[5] + 1900;
my $month = $time[4] + 1;
my $currdate = "$year-$month-$time[3]";

my $schyear = $year;
if ( $month > 7 ){ $schyear++; }


my $q = CGI->new;
my %arr = $q->Vars;
print $q->header( -charset, $charset );


# Script Heading Descriptor
my $title = "TSEC $lex{Summary} $lex{Reading} $lex{Report}";
my $title1 = "Percentage of Students vs Differential Reading Level by Year";


# Print Page Heading
print "$doctype\n<html><head><title>$title</title>\n";
print "<link rel=\"stylesheet\" href=\"/division.css\" type=\"text/css\">\n";


print qq{$chartype\n</head><body style="padding:0.4em 2em;">\n};
print qq{[ <a class="alt" href="$homepage">$lex{Main}</a> ]\n};
print qq{<h1>$title</h1>\n};
print qq{<h3>$title1</h3>\n};


if ( not $arr{page} ) {
    showStartPage();

} elsif ( $arr{page} == 1 ) {
    delete $arr{page};
    createReport();
}


#----------------
sub categoryTable { # display results by -2 or below, -1, at or above.
#----------------

    my $ref = shift;

    # %data is {year}{delta} = testcount;
    my %data = %$ref;

    print qq{<h3>Student Reading Achievement by Relative Reading Level</h3>\n};

    # Start Table
    print qq{<table cellspacing="0" cellpadding="3" border="1">\n};
    print qq{<tr><th>Year</th><th>Significantly Below<br>Expected Level<br>(-2 years or more)</th>\n};
    print qq{<th>Below Expected<br>Level (-1 year)</th>};
    print qq{<th>At or Above Expected<br>Reading Level</th></tr>\n};


    foreach my $year ( sort keys %data ) {
	my ( $minus2, $minus1, $normal );
	foreach my $delta ( keys %{ $data{$year} } ) {
	    if ( $delta <= -2 ) {
		$minus2 += $data{$year}{$delta};
	    } elsif ( $delta == -1 ) {
		$minus1 += $data{$year}{$delta};
	    } else {
		$normal += $data{$year}{$delta};
	    }
	}
	my $total = $minus2 + $minus1 + $normal;

	my ($minus2avg, $minus2avg, $normalavg);
	if ( $total ) { 
	    $minus2avg = round( $minus2 / $total * 100, 1);
	    $minus1avg = round( $minus1 / $total * 100, 1);
	    $normalavg = round( $normal / $total * 100, 1);
	} 

	print qq{<tr><td class="la">$year</td>};
	print qq{<td class="cn">$minus2avg%</td>};
	print qq{<td class="cn">$minus1avg%</td>};
	print qq{<td class="cn">$normalavg%</td>};
	print qq{</tr>\n};

    }

    print qq{</table>\n};

    return;

}



#----------------
sub generateGraph {
#----------------

    my $ref = shift;

    # %data is {year}{delta} = testcount;
    my %data = %$ref;

    my @years = sort keys %data;
    shift @years until scalar @years < 4;


    # Get MIN, MAX values of Delta Grade (How far above/below grade level.
    my ($max, $min, $studcount ); # min and max delta values and total test count;
    my %dcount; # overall count of tests for each delta.
    foreach my $year ( sort keys %data ) {
	foreach my $delta ( sort keys %{ $data{$year} } ) {
	    if  ( $delta > $max ) { $max = $delta; }
	    if  ( $delta < $min ) { $min = $delta; }
	    $studcount += $data{$year}{$delta};
	    $dcount{$delta} += $data{$year}{$delta};
	    # print "G:$year D:$delta Count:$schooldata{$year}{$delta}<br>\n";
	}
    }

    # Setup the Delta array for x coords.
    my @delta;
    for my $idx ($min..$max) {
	push @delta, $idx;
    }

    my $testmax; # maximum y value

    my %yeararray;
    foreach my $year ( keys %data ) {
	my @temp;
	foreach my $delta ( $min..$max ) {
	    push @temp, $data{$year}{$delta};

	    if ( $data{$year}{$delta} > $testmax ) {
		$testmax = $data{$year}{$delta};
	    }
	}
	$yeararray{$year} = \@temp;
    }



    $testmax = nhimult(5, $testmax);


    my @spacer;
    foreach my $year ( keys %data ) {
	push @spacer, '';
    }


    my $graph = GD::Graph::bars->new(800, 600);
    $graph->set( dclrs => [ qw{blue green red dblue dgreen dred} ]);    


#    my @data = ( \@delta, \@year1, \@year2, \@year3 );
    
    my @data;
    push @data, \@delta;

    my $count = 1;
    foreach my $year ( @years ) { # reverse sort keys %yeararray ) {
	push @data, \@{ $yeararray{$year} };
	$count++;
	if ($count > 3) { last; }
    }

    push @data, \@spacer;


    $graph->set_legend_font(gdLargeFont); 
    $graph->set_legend( @years );

    $graph->set(
		x_label => 'Relative Reading Level',
		y_label => 'Percent Student',
		title  => "Summary Reading Levels",
		y_max_value => $testmax,
		x_max_value => $max,
		) or die $graph->error;




    my $gd = $graph->plot(\@data) or die $graph->error;


    my $filename = "drasumm-$$.png";
    open(IMG,">$filename");
    binmode IMG;
    print IMG $gd->png;
    close IMG;

    system("mv $filename $downloaddir");

    return $filename;

}


#---------------
sub createReport {
#---------------

    # foreach my $key ( sort keys %arr ) { print "K:$key V:$arr{$key}<br>\n"; }

    if ( not $arr{season} or not $arr{startyear} or not $arr{endyear} ) {
	print qq{<h3>$lex{Missing} $lex{Value}</h3>\n};
	print qq{</body></html>\n};
	exit;
    }

    # sdate and edate are formatted dates.
    # print "<h3>$lex{'Start Date'} $sdate | $lex{'End Date'} $edate</h3>\n";

    my $season = ucfirst $arr{season};
    print qq{<p><b>$lex{Season}</b>: $season <b>Dates</b>: $dates{"$arr{season}"}</p>\n};

    # Get Start and End dates and also seasonality
    my $startyear = $arr{startyear};
    my $endyear = $arr{endyear};

    my %overall;
    my %schooldata;


    foreach my $db ( @dbase ) {

	my $dsn = "DBI:$dbtype:dbname=$db";
	my $dbh = DBI->connect($dsn,$guser,$gpassword);

	# Get all students who have tests into a @students array (may not be current)
	my @students;
	my $sth = $dbh->prepare("select distinct studnum from read_test");
	$sth->execute;
	if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
	while ( my $studnum = $sth->fetchrow ) {
	    push @students, $studnum;
	}


	foreach my $year ( $startyear..$endyear ) {

	    # Get Start/End Dates.
	    my ($startdate, $enddate);
	    my $tmpyear = $year - 1; # since doing fall of prev calendar year
	    if ( $arr{season} eq 'fall' ) {
		$startdate = $tmpyear. '-08-01';
		$enddate = $year. '-01-31';
	    } else { # spring
		$startdate = $year. '-02-01';
		$enddate = $year. '-06-30';
	    }


	    # Now loop through all students printing results; setup queries first
	    my $sth1 = $dbh->prepare("select id, tdate, readlevel, tgrade from read_test 
             where studnum = ? and to_days(tdate) >= to_days('$startdate') and 
             to_days(tdate) <= to_days('$enddate') order by tdate desc");
	    if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }

	    my $sth2 = $dbh->prepare("select sum(score) from read_test_score where testid = ?");
	    my $sth3 = $dbh->prepare("select lastname, firstname from studentall where studnum = ?");	


	    foreach my $studnum ( @students ) {

		# Get Tests
		$sth1->execute( $studnum );
		if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }

		# Only use latest value for the student.
		my $trows = $sth1->rows;
		# if $onlylatest, limit to 1 test
		# if ( $onlylatest and $trows > 1 ) { $trows = 1; } 
		if ( $trows > 1 ) { $trows = 1; }

		for ( 1 .. $trows ) {

		    my ( $testid, $tdate, $readlevel, $tgrade ) = $sth1->fetchrow;


		    # Get the score total;
		    $sth2->execute( $testid );
		    if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
		    my $total = $sth2->fetchrow;

		    # Equivalent Grade print
		    my $equivgrade = scoreToGrade($total, $readlevel );

		    my $grdelta;
		    if ( not $tgrade ) {
			$sth3->execute( $studnum );
			my ( $lastname, $firstname ) = $sth3->fetchrow;		    
			print "Grade Missing: $lastname, $firstname ($studnum) - $tdate<br>\n";
			next;
		
		    } else { # Find grade delta
			$grdelta = $equivgrade - $tgrade;
		    }

		    if ( $grdelta >= 0 ) { $grdelta = int $grdelta; } # truncate.
		    else { $grdelta = int $grdelta - 1; }

		    # Original
		    # $schooldata{$tgrade}{$grdelta}++;
		    # $overall{$tgrade}{$grdelta}++;

		    $schooldata{$year}{$grdelta}++;
		    $overall{$year}{$grdelta}++;


		} # end of trows loop
	    } # end of student loop
	} # year loop
    } # database loop

    # Skip if no tests in this date range
    if ( not %schooldata ) { 
	print "<h3>No tests in this Date Range</h3>\n";
	next; 
    }


    # Get MIN, MAX values of Delta Grade (How far above/below grade level.
    my ($max, $min, $studcount );
    my %dcount = ();
    foreach my $year ( sort keys %schooldata ) {
	foreach my $delta ( sort keys %{ $schooldata{$year} } ) {
	    if  ( $delta > $max ) { $max = $delta; }
	    if  ( $delta < $min ) { $min = $delta; }
	    $studcount += $schooldata{$year}{$delta};
	    $dcount{$delta} += $schooldata{$year}{$delta};
	    # print "G:$year D:$delta Count:$schooldata{$year}{$delta}<br>\n";
	}
    }



    # Start Table
    print qq{<table cellspacing="0" cellpadding="3" border="1">\n};

    # Heading.
    print "<tr><th>$lex{Year}</th>";
    foreach my $delta ( $min .. $max ) {
	print "<th>$delta</th>";
    }
    print "<th>No. Tested</th><th>Average<br>Difference</th>";
    print "</tr>\n";


    foreach my $year ( $startyear..$endyear ) {

	my %dcount;
	my $studcount;
	foreach my $delta ( sort keys %{ $schooldata{$year} } ) {
	    if  ( $delta > $max ) { $max = $delta; }
	    if  ( $delta < $min ) { $min = $delta; }
	    $studcount += $schooldata{$year}{$delta};
	    $dcount{$delta} += $schooldata{$year}{$delta};
	    # print "G:$year D:$delta Count:$schooldata{$year}{$delta}<br>\n";
	}


	print "<tr><th>$year</th>";
	my $partavg;
	foreach my $delta ( $min .. $max ) {
	    $partavg += ($delta * $dcount{$delta} );
	    my $avg;
	    if ( $studcount ) {
		$avg = round( $dcount{$delta} / $studcount * 100, 1);
	    }
	    
	    if ( $dcount{$delta} ) {
#		print "<td>$dcount{$delta}/$studcount<br>$avg%</td>";
		print "<td>$avg%</td>";
	    } else { print qq{<td></td>}; }

	}

	my $avg;
	if ( $studcount ) {
	    $avg = $partavg / $studcount;
	} else { $avg = 0; }

	$avg = round($avg,2);
	print "<th>$studcount</th><th>$avg</th></tr>\n";

    }



=head
	# Summary Row
	print "<tr><th>$year</th>";
	my $partavg;
	foreach my $delta ( $min .. $max ) {
	    $partavg += ($delta * $dcount{$delta} );
	    my $avg;
	    if ( $studcount ) {
		$avg = round( $dcount{$delta} / $studcount * 100, 2);
	    }
	    
	    print "<td>$dcount{$delta}/$studcount<br>$avg%</td>";
	}

	my $avg = $partavg / $studcount;
	$avg = round($avg,2);
	print "<th>$studcount</th><th>$avg</th></tr>\n";

#---------------------
    $max = 0, 
    $min = 0, 
    $studcount = 0;
    %dcount = ();

    foreach my $grade ( sort keys %overall ) {
	foreach my $delta ( sort keys %{ $overall{$grade} } ) {
	    if  ( $delta > $max ) { $max = $delta; }
	    if  ( $delta < $min ) { $min = $delta; }
	    $studcount += $overall{$grade}{$delta};
	    $dcount{$delta} += $overall{$grade}{$delta};
	    # print "G:$grade D:$delta Count:$overall{$grade}{$delta}<br>\n";
	}
    }


    print "<table cellspacing=\"0\" cellpadding=\"3\" border=\"1\">\n";

    # Heading.
    print "<tr><th>$lex{Grade}</th>";
    foreach my $delta ( $min .. $max ) {
	print "<th>$delta</th>";
    }
    print "<th>$lex{Test}#</th><th>Wt. Avg</th>";
    print "</tr>\n";

    # Data
    foreach my $grade ( sort {$a <=> $b} keys %overall ) {
	print "<tr><td>$grade</td>";
	my ($total,$partavg);
	foreach my $delta ( $min .. $max ) {
	    my $count = $overall{$grade}{$delta};
	    print "<td>$overall{$grade}{$delta}</td>";
	    $total += $count;
	    $partavg += ( $count * $delta );
	}

	my $avg;
	if ( $total ) {
	    $avg = $partavg / $total;
	} else { $avg = 0; }

	$avg = round($avg, 2);
	print "<td>$total</td><td>$avg</td>";
	print "</tr>\n";

    }


    print "<tr><th>Average</th>";
    my $partavg;
    foreach my $delta ( $min .. $max ) {
	$partavg += ($delta * $dcount{$delta} );
	my $avg;
	if ( $studcount ) {
	    $avg = round( $dcount{$delta} / $studcount * 100, 2);
	}
	print "<td>$dcount{$delta}/$studcount<br>$avg%</td>";
    }

    my $avg = $partavg / $studcount;
    $avg = round($avg,2);
    print "<th>$studcount</th><th>$avg</th></tr>\n";
=cut

    print "</table><p></p>\n";

    my $filename = generateGraph(\%schooldata);
    my $src = $webdownloaddir. '/'. $filename;

    print qq{<img src="$src"><br clear="left">\n};
    

    categoryTable(\%schooldata);


    exit;
}



#----------
sub fmtDate {
#----------

    my ( $year, $mon, $day ) = split /-/, shift;
    return "$year-$s_month[$mon]-$day";
}



#---------------
sub scoreToGrade {
#---------------

    my ($score, $readinglevel) = @_;

    my %rd_LevelToGrade = ( '0' => 0.2, 1 => 0.5, 2 => 0.8, 3 => 1.1, 4 => 1.2,
			    6 => 1.3, 8 => 1.4, 10 => 1.5, 12 => 1.6, 14 => 1.7,
			    16 => 1.8, 18 => 1.9, 20 => '2.0', 24 => 2.5, 28 => 2.9,
			    30 => '3.0', 34 => 3.5, 38 => 3.9);


    my %rd_ScoreToGrade = (
	50 => 1.9, 52 => '2.0', 53 => 2.1, 54 => 2.2,
	55 => 2.3, 56 => 2.5, 57 => 2.7, 58 => 2.8,
	61 => 2.9, 62 => '3.0', 63 => 3.1, 64 => 3.2, 
	65 => 3.3, 66 => 3.5, 67 => 3.6, 68 => 3.7,
	69 => 3.8, 70 => 3.9, 74 => 4.3, 76 => 4.6,
	80 => 4.9, 84 => 5.3, 86 => 5.6, 90 => 5.9,
	94 => 6.3, 96 => 6.6, '100' => '6.9',
	'104' => '7.3', '106' => '7.6','110' => '7.9',
	'114' => '8.3', '119' => '8.6','124' => '8.9');


    if ( $readinglevel < 40 ) {

	foreach my $key ( sort { $b <=> $a } keys %rd_LevelToGrade ) {
	    # print "K:$key ";
	    if ( $readinglevel >= $key ) {
		# print "<br><b>Match: K:$key RL:$readinglevel</b><br>\n";
		return $rd_LevelToGrade{ $key };
	    }
	}

    } else { # else reading level >= 40

	$score += $readinglevel; # add both together for this 

	foreach my $key ( sort { $b <=> $a } keys %rd_ScoreToGrade ) {
	    # print "K:$key ";
	    if ( $score >= $key ) {
		# print "<b>Match: S:$score K:$key V:$rd_ScoreToGrade{$key}</b><br>\n";
		return $rd_ScoreToGrade{ $key };
	    }
	}

    }

    return 'NA';
}




#----------------
sub showStartPageOLD {
#----------------

    print "<form action=\"$self\" method=\"post\">\n";
    print "<input type=\"hidden\" name=\"page\" value=\"1\">\n";

    print "<table cellpadding=\"3\" cellspacing=\"0\" border=\"0\" ";
    print "style=\"padding:0.5em;border:1px solid gray;\">\n";

    print qq{<tr><td class="bla">$lex{Select} $lex{School}</td></tr>\n};
    foreach my $db ( @dbase ) { 
	print qq{<tr><td class="la">};
	print qq{<input type="radio" name="db" value="$db">\n};
	print qq{$alldbase{$db} ($db)</td></tr>\n};
    }

    print qq{<tr><td class="la"><input type="submit" value="$lex{Continue}"></td></tr>\n};
    print qq{</table></form>\n};


    print qq{</body></html>\n};

    exit;

} # end of showStartPage


#--------------
sub showStartPage { # was selectYears
#--------------

    # foreach my $key ( sort keys %arr ) { print "K:$key V:$arr{$key}<br>\n"; }

    my %years; # only store second part of school year (ie. 2014-2015 = 2015; same as grad year)

    foreach my $db ( @dbase ) {

	my $database = $db;
	my $dsn = "DBI:$dbtype:dbname=$database";
	my $dbh = DBI->connect($dsn,$guser,$gpassword);
	my $schoolname = $alldbase{"$db"};

	# Get all school years in the read_test table.
	my $sth = $dbh->prepare("select distinct tdate from read_test");
	$sth->execute;
	if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
	my $ref = $sth->fetchall_hashref(tdate);


	foreach my $key ( sort keys %{ $ref } ) {
	    my ($y,$m,$d) = split('-', $key);
	    if ( $y eq '0000' ) { 
		print "Date Error: $key - $db<br>\n";
		next; 
	    }

	    if ( $m <= 7 ) { # split on July
		$years{$y} = 1;
	    } else {
		my $nextyr = $y + 1;
		$years{$nextyr} = 1;
	    }
	    
	}
    } # end of dbase loop


    print "<form action=\"$self\" method=\"post\">\n";
    print qq{<input type="hidden" name="page" value="1">\n};

    print "<table cellpadding=\"3\" cellspacing=\"0\" border=\"0\" ";
    print "style=\"padding:0.5em;border:1px solid gray;\">\n";

    print qq{<tr><td class="bla">$lex{Select} $lex{'Start Year'}</td></tr>\n};
    foreach my $yr ( sort keys %years ) {
	my $prev = $yr - 1;
	my $fullyear = $prev. '-'. $yr;


	print qq{<tr><td class="la">};
	print qq{<input type="radio" name="startyear" value="$yr">};
	print qq{$fullyear</td></tr>\n};
    }

    print qq{<tr><td colspan="2"></td></tr>\n};

    # Season.
    print qq{<tr><td class="bla">$lex{Select} $lex{Season}</td></tr>\n};
    print qq{<tr><td class="la">};
    print qq{<input type="radio" name="season" value="spring"> $lex{Spring}</td></tr>\n};
    print qq{<tr><td class="la">};
    print qq{<input type="radio" name="season" value="fall"> $lex{Fall}</td></tr>\n};



    print qq{<tr><td colspan="2"></td></tr>\n};

    print qq{<tr><td class="bla">$lex{Select} $lex{'End Year'}</td></tr>\n};

    my $tmp = $schyear - 1;
    my $disyear = "$tmp-$schyear";
    print qq{<tr><td class="la">};
    print qq{<input type="radio" name="endyear" value="$schyear">};
    print qq{$disyear</td></tr>\n};

    my $prevyear = $schyear - 1;
    $tmp = $prevyear - 1;
    $disyear = "$tmp-$prevyear";
    print qq{<tr><td class="la">};
    print qq{<input type="radio" name="endyear" value="$prevyear">};
    print qq{$disyear</td></tr>\n};

    print qq{<tr><td colspan="2"></td></tr>\n};


    print qq{<tr><td class="la"><input type="submit" value="$lex{Continue}"></td></tr>\n};
    print qq{</table></form>\n};

    print qq{</body></html>\n};

    exit;

} # end of showStartPage  # was selectYears


