#!/usr/bin/perl
#  Copyright 2001-2011 Leslie Richardson

#  This file is part of Open Admin for Schools.

#  Open Admin for Schools is free software; you can redistribute it 
#  and/or modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2 of 
#  the License, or (at your option) any later version.

my %lex = ('Main' => 'Main',
	   'Error' => 'Error',
	   'Class Reading Report' => 'Class Reading Report',
	   'Reading Level' => 'Reading Level',
	   'Name' => 'Name',
	   'Date' => 'Date',
	   'Continue' => 'Continue',
	   'Select' => 'Select',
	   'Student' => 'Student',
	   'Homeroom' => 'Homeroom',
	   'Grade' => 'Grade',
	   'Blank=All' => 'Blank=All',
	   'No Students Found' => 'No Students Found',
	   'Show Only Latest Test' => 'Show Only Latest Test',
	   'Gr' => 'Gr',
	   'Levels' => 'Levels',
	   'Avg' => 'Avg',
	   'Start Date' => 'Start Date',
	   'End Date' => 'End Date',
	   'EGr' => 'EGr',

	   );


# Grade => Start:End Reading Level
my %levelExpect = ( K => '0:3', 1 => '3:16', 2 => '16:28', 3 => '28:38', 4 => '40:50', 
		    5 => '50:60', 6 => '60:70', 7 => '70:80', 8 => '80:80' ); 


my %shortCategory = ( 'Oral Fluency' => 'OrFlu', 'Reading Engagement' =>'RdEg',
		      'Comprehension' => 'Comp', 'Printed Language Concepts' => 'pLgCcp');


use DBI;
use CGI;
use Number::Format qw(:all);
use Cwd;


my $self = 'readRptClass.pl';

my @time = localtime(time);
my $year = $time[5] + 1900;
my $month = $time[4] + 1;
my $currdate = "$year-$month-$time[3]";

eval require "../../etc/admin.conf";
if ( $@ ) {
    print $lex{Error}. " $self: $@<br>\n";
    die $lex{Error}. "$self: $@\n";
}

# Get current dir so know what CSS to display and shift to teacher settings.
if ( getcwd() !~ /tcgi/ ) { # we are in cgi

    $tchcss = $css;
    $tchpage = $homepage;
    $tchdownloaddir = $downloaddir;
    $tchwebdownloaddir = $webdownloaddir;
}


my $q = new CGI;
print $q->header; 
my %arr = $q->Vars;

my $dsn = "DBI:$dbtype:dbname=$dbase";
my $dbh = DBI->connect($dsn,$user,$password);

# Page Header
print "$doctype\n<html><head><title>". $lex{'Class Reading Report'};
print "</title><link rel=\"stylesheet\" href=\"$tchcss\" type=\"text/css\">
<style type=\"text/css\">
td.ra {text-align:right;} 
td.la {text-align:left;} 
th.fs6{font-size:60%;}
td { text-align:center; }
td.r { background-color:#822;color:white;font-size:120%;font-weight:bold; }
td.y { background-color:#BB1;color:white;font-size:120%;font-weight:bold; }
td.b { background-color:#228;color:white;font-size:120%;font-weight:bold; }
td.g { background-color:#282;color:white;font-size:120%;font-weight:bold; }
a {color:white; }
a.alt {color:blue; }
</style>\n";

if ( not $arr{page} ) { # calendar popup.
    print "<link rel=\"stylesheet\" type=\"text/css\" media=\"all\" ";
    print "href=\"/js/calendar-blue.css\" title=\"blue\">\n";
    print "<script type=\"text/javascript\" src=\"/js/calendar.js\"></script>\n";
    print "<script type=\"text/javascript\" src=\"/js/lang/calendar-en.js\"></script>\n";
    print "<script type=\"text/javascript\" src=\"/js/calendar-setup.js\"></script>\n";
}

print "<script language=\"javascript\" type=\"text/javascript\">
function showhelp(type) {
  winName=window.open('/tcgi-bin/reading/showhelp.pl?id=' + type,'helpWindow',
  'height=300,width=700,screenX=100,screenY=100,resizeable');
  winName.focus();
}
</script>
$chartype\n</head><body style=\"padding:1em 6em 0 0.5em;\">\n";


# Show Grade and Reading Levels Expected.
print "<div style=\"position:absolute;top:0;right:0;\">\n";
print "<table cellspacing=\"0\" cellpadding=\"3\" border=\"1\">\n";
print "<tr><th>". $lex{Gr}. "</th><th>". $lex{'Levels'}. "</th></tr>\n";
foreach my $key ( sort keys %levelExpect ) {
    my ($start, $end ) = split /:/, $levelExpect{$key};
    if ( $start == '0' ) { $start = 'A'; }
    print "<tr><td>$key</td><td>$start &ndash; $end</td></tr>\n";
}
print "</table></div>\n";


print "[ <a class=\"alt\" href=\"$homepage\">$lex{Main}</a> ]\n";
print "<h1 style=\"text-align:left;margin:0;padding:0.5em;\">";
print $lex{'Class Reading Report'};
print " &ndash; $schoolname &ndash; ". fmtDate($currdate). "</h1>\n";


if ( not $arr{page} ) {
    showStartPage();

} elsif ( $arr{page} == 1 ) {
    delete $arr{page};
    showReport();
} 



#----------
sub fmtDate {
#----------

    my ( $year, $mon, $day ) = split /-/, shift;
    return "$year-$s_month[$mon]-$day";
}



#---------------
sub scoreToGrade {
#---------------

    my ($score, $readinglevel) = @_;

    my %rd_LevelToGrade = ( '0' => 0.2, 1 => 0.5, 2 => 0.8, 3 => 1.1, 4 => 1.2,
			    6 => 1.3, 8 => 1.4, 10 => 1.5, 12 => 1.6, 14 => 1.7,
			    16 => 1.8, 18 => 1.9, 20 => '2.0', 24 => 2.5, 28 => 2.9,
			    30 => '3.0', 34 => 3.5, 38 => 3.9);


    my %rd_ScoreToGrade = (
	50 => 1.9, 52 => '2.0', 53 => 2.1, 54 => 2.2,
	55 => 2.3, 56 => 2.5, 57 => 2.7, 58 => 2.8,
	61 => 2.9, 62 => '3.0', 63 => 3.1, 64 => 3.2, 
	65 => 3.3, 66 => 3.5, 67 => 3.6, 68 => 3.7,
	69 => 3.8, 70 => 3.9, 74 => 4.3, 76 => 4.6,
	80 => 4.9, 84 => 5.3, 86 => 5.6, 90 => 5.9,
	94 => 6.3, 96 => 6.6, '100' => '6.9',
	'104' => '7.3', '106' => '7.6','110' => '7.9',
	'114' => '8.3', '119' => '8.6','124' => '8.9');


    if ( $readinglevel < 40 ) {

	foreach my $key ( sort { $b <=> $a } keys %rd_LevelToGrade ) {
	    # print "K:$key ";
	    if ( $readinglevel >= $key ) {
		# print "<br><b>Match: K:$key RL:$readinglevel</b><br>\n";
		return $rd_LevelToGrade{ $key };
	    }
	}

    } else { # else reading level >= 40

	$score += $readinglevel; # add both together for this 

	foreach my $key ( sort { $b <=> $a } keys %rd_ScoreToGrade ) {
	    # print "K:$key ";
	    if ( $score >= $key ) {
		# print "<b>Match: S:$score K:$key V:$rd_ScoreToGrade{$key}</b><br>\n";
		return $rd_ScoreToGrade{ $key };
	    }
	}

    }

    return 'NA';
}



#----------------
sub showStartPage {
#----------------

    # Get global reading levels.
    my %readinglevel = ();
    my @readinglevel = ();
    my $sth = $dbh->prepare("select distinct readlevel from read_level
      where readlevel is not NULL");
    $sth->execute;
    if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr; }
    while ( my $rl = $sth->fetchrow ) {
	$readinglevel{$rl} = 1;
    }
    @readinglevel = sort { $a <=> $b } keys %readinglevel;


    # Form Start - Student Group
    print "<form action=\"$self\" method=\"post\"> \n";
    print "<input type=\"hidden\" name=\"page\" value=\"1\">\n";

    print "<table cellpadding=\"3\" cellspacing=\"0\" border=\"0\" ";
    print "style=\"padding:0.5em;border:1px solid gray;\">\n";

    print "<tr><td class=\"ra\">$lex{Select} $lex{Student}</td>";
    print "<td><select name=\"groupname\">";
    print "<option>$lex{Grade}</option>\n";
    print "<option>$lex{Homeroom}</option>\n";
    print "</select>\n";
    print "<input type=\"text\" name=\"groupvalue\" size=\"10\"> ";
    print "</td></tr>\n\n";

    print "<tr><td class=\"ra\">$lex{'Reading Level'}</td>\n<td class=\"la\">";
    print "<select name=\"readlevel\"><option></option>\n";
    foreach my $rl ( @readinglevel ) {
	print "<option>$rl</option>";
    }
    print "\n</select> $lex{'Blank=All'}</td></tr>\n\n";

    print "<tr><td class=\"ra\">\n";
    print "<input type=\"checkbox\" name=\"onlylatest\" value=\"1\">\n";
    print "<td  class=\"la\">$lex{'Show Only Latest Test'}\n";
    print "</td></tr>\n";

    print "<tr><td class=\"ra\">$lex{'Start Date'}</td><td class=\"la\"><input type=\"text\" ";
    print "name=\"startdate\" id=\"sdate\" size=\"10\" value=\"$schoolstart\">";
    print "<button type=\"reset\" id=\"start_trigger\">...</button>\n";
    print "</td></tr>\n";

    print "<tr><td class=\"ra\">$lex{'End Date'}</td><td class=\"la\"><input type=\"text\" ";
    print "name=\"enddate\" id=\"edate\" size=\"10\" value=\"$currdate\">";
    print "<button type=\"reset\" id=\"end_trigger\">...</button></td></tr>\n";


    print "<tr><td></td><td class=\"la\"><input type=\"submit\" value=\"$lex{Continue}\"></td></tr>\n";
    print "</table></form>\n";

    print "<script type=\"text/javascript\">
     Calendar.setup({
        inputField     :    \"sdate\", 
        ifFormat       :    \"%Y-%m-%d\",
        button         :    \"start_trigger\",
        singleClick    :    false,
        step           :    1
    });

    Calendar.setup({
        inputField     :    \"edate\",
        ifFormat       :    \"%Y-%m-%d\",
        button         :    \"end_trigger\",
        singleClick    :    false,
        step           :    1
    });
   </script>\n";

    print "</body></html>\n";

    exit;

} # end of showStartPage



#-------------
sub showReport {
#-------------

    #foreach my $key ( sort keys %arr ) { print "K:$key V:$arr{$key}<br>\n"; }

    my $group;
    my @students = ();

    my $startdate = $arr{startdate};
    my $enddate = $arr{enddate};
    delete $arr{startdate};
    delete $arr{enddate};   


    if ( $arr{groupvalue} ) {
	print "<h3 style=\"text-align:left;font-size:120%;margin:0;padding:0.3em;\">";
	print "$arr{groupname} $arr{groupvalue}\n";
    }

    if ( $arr{groupname} eq $lex{Homeroom} and $arr{groupvalue} ) {
	my $sth = $dbh->prepare("select lastname, firstname from staff as s, staff_multi as sm
          where sm.userid = s.userid and field_name = 'homeroom' and field_value = ?");
        $sth->execute( $arr{groupvalue} );
        if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
	my ( $lastname, $firstname ) = $sth->fetchrow;
	print " &ndash; $firstname $lastname ";
    }
    print "</h3>\n";


    # look for kids in a particular group with particular reading level
    if ( $arr{groupvalue} and $arr{readlevel} ) {  

	if ( $arr{groupname} eq $lex{Homeroom} ) {
	    $group = 'homeroom';
	} else {
	    $group = 'grade';
	}

	# Get the students, and then get reading level values
	my $sth = $dbh->prepare("select studnum from student where $group = ? 
          order by lastname, firstname");
        $sth->execute( $arr{groupvalue} );
        if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }

        my $sth1 = $dbh->prepare("select distinct readlevel from read_test 
           where studnum = ?"); 
			    
	while ( my $studnum = $sth->fetchrow ) {
	    $sth1->execute( $studnum );
	    if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
	    my $readlevel = $sth1->fetchrow;
	    if ( $readlevel eq $arr{readlevel} ) {
		push @students, $studnum;
	    }
	}

	if ( not @students ) { 
	    print "<div style=\"border:1px solid gray;padding:1em;";
	    print "background-color:red;font-weight:bold;font-size:120%;\">\n";
	    print $lex{'No Students Found'};
	    print "</div></body></html>\n";
	    exit;
	}

	# Only have to print once for this group all in one reading level.
	showReadingLevel( $arr{readlevel}, $arr{onlylatest}, \@students, $startdate, $enddate );


    } elsif ( $arr{groupvalue} ) { # group only

	# Find students in this group and order by readinglevel, then testdates.
	if ( $arr{groupname} eq $lex{Homeroom} ) {
	    $group = 's.homeroom';
	} else {
	    $group = 's.grade';
	}

	# Find the reading levels
	my @readinglevels = ();
	my %readinglevel = ();
	my $sth = $dbh->prepare("select distinct r.readlevel from read_test as r, student as s
          where $group = ? and r.studnum = s.studnum order by r.readlevel");
        $sth->execute( $arr{groupvalue} );
        if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
	while ( my $rl = $sth->fetchrow ) { $readinglevel{$rl} = 1; }
	@readinglevels = sort { $a <=> $b } keys %readinglevel;


	if ( not @readinglevels ) { 
	    print "<div style=\"border:1px solid gray;padding:1em;";
	    print "background-color:red;font-weight:bold;font-size:120%;\">\n";
	    print $lex{'No Students Found'};
	    print "</div></body></html>\n";
	    exit;
	}


	foreach my $readinglevel ( @readinglevels ) {

 	    # Find students in group / reading levels
	    my @students = ();
	    my $sth = $dbh->prepare("select distinct s.studid,s.studnum from read_test as r, student as s
               where $group = ? and r.studnum = s.studnum and r.readlevel = ?
               order by s.lastname, s.firstname");
	    $sth->execute( $arr{groupvalue}, $readinglevel );
	    if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
	    while ( my ($id, $studnum) = $sth->fetchrow ) {
		push @students, $studnum;
	    }

	    showReadingLevel( $readinglevel, $arr{onlylatest}, \@students, $startdate, $enddate );
	}

    } else { # only reading level.... all students in same reading level.

	# Only have to do this once....
	# Find students in group / reading levels

	my @students = ();

	my $sth = $dbh->prepare("select distinct s.studnum from read_test as r, student as s
          where r.studnum = s.studnum and r.readlevel = ? order by s.lastname, s.firstname");
	$sth->execute( $arr{readlevel} );
	if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
	while ( my $studnum = $sth->fetchrow ) {
	    push @students, $studnum;
	}

	if ( not @students ) { 
	    print "<div style=\"border:1px solid gray;padding:1em;";
	    print "background-color:red;font-weight:bold;font-size:120%;\">\n";
	    print $lex{'No Students Found'};
	    print "</div></body></html>\n";
	    exit;
	}

	showReadingLevel( $arr{readlevel}, $arr{onlylatest}, \@students, $startdate, $enddate );

    }

    print "</body></html>\n";

    exit;

}



#-------------------
sub showReadingLevel {
#-------------------
    my ( $readinglevel, $onlylatest, $stud_ref, $startdate, $enddate ) = @_;

    my @students = @$stud_ref;


    # Check that tests exist
    my $sth = $dbh->prepare("select id from read_test 
     where studnum = ? and readlevel = ? and to_days( tdate ) >= to_days( '$startdate' ) 
     and to_days( tdate ) <= to_days( '$enddate')"); # get tests

    my $testflag = 0;
    foreach my $studnum ( @students ) {
	# Check for Tests
	$sth->execute( $studnum, $readinglevel );
	if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
	my $trows = $sth->rows;
	if ( $trows > 0 ) { $testflag = 1; }
    }
    if ( not $testflag ) { return; }


    # Setup for Reading Level Heading, Get Category and Name
    $sth = $dbh->prepare("select distinct rs.name, rs.category, rs.seq
     from read_test_score as rs, read_test as rt
     where rs.testid = rt.id and readlevel = ?");

    my %objectives = ();
    my %catcount = ();


    $sth->execute( $readinglevel );
    if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }

    while ( my ( $name, $category, $seq ) = $sth->fetchrow ) {
	$catcount{ $category } = $catcount{$category} + 1;
	$objectives{ $seq } = $category. '::'. $name;
    }


    # Set Index values for all objectives
    my $count = 0;
    my $startcount;
    my %skip = ();
    my $currcat = '';

   # Print out First Heading Line
    my $colspan = 4 + %catcount; # %catcount holds categories and number of types in each.
    print "<table cellpadding=\"3\" cellspacing=\"0\" border=\"1\">\n";
    print "<tr><th colspan=\"$colspan\">$lex{'Reading Level'} $readinglevel</th>\n";

    my @catorder = (); # ordering of categories
    my %temp = (); # used to track cat in loop

    foreach my $key ( sort { $a <=> $b } keys %objectives ) { # numeric sort 

	my ( $category, $name ) = split(/::/, $objectives{$key} );

	if ( not $temp{$category} ) {
	    push @catorder, $category;
	    $temp{$category} = 1;
	}

	$oldcat = $currcat;
	$currcat = $category;

	if ( $oldcat ne $currcat and $count ) {
	    $diff = $count - $startcount;
	    print "<th colspan=\"$diff\" style=\"font-size:100%;\">$oldcat</th>\n";
	    $skip{$oldcat} = $diff;
	    $startcount = $count;
	}
	
	$count++;

    }
    $diff = $count - $startcount;
    $skip{$currcat} = $diff;
    print "<th colspan=\"$diff\" style=\"font-size:100%;\">$currcat</th></tr>\n";


    # Print Out Second Heading Line
    print "<th>$lex{Name}</th><th>$lex{Date}</th><th>$lex{EGr}</th>\n";


    foreach my $key ( @catorder ) {
#    foreach my $key ( sort keys %catcount ) {
# old - remove when done testing.

	print "<th>$shortCategory{$key}<br>$lex{Score}</th>";
    }
    print "<th>Overall<br>Score</th>\n";


    # SQL Query of global reading level table.
    my $sth1 = $dbh->prepare("select id, help1 from read_level where readlevel = ? and
      category = ? and name = ?");

    foreach my $key ( sort { $a <=> $b } keys %objectives ) { # numeric sort 
	my ( $category, $name ) = split(/::/, $objectives{$key} );
	
	# Get the record id and help for this...
	$sth1->execute( $readinglevel, $category, $name );
	if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
	my ($id, $help1 ) = $sth1->fetchrow;

	print "<th class=\"fs6\">";
	if ( $help1 ) {
	    print "<a href=\"javascript:showhelp($id)\">$name</a>";

	} else {
	    print "$name";
	}
	print "</th>\n";

    }
    print "</tr>\n";



    # Now loop through all students printing results; setup queries first
    $sth = $dbh->prepare("select lastname, firstname from studentall 
       where studnum = ?");

    $sth1 = $dbh->prepare("select id, tdate from read_test 
     where studnum = ? and readlevel = ? and to_days( tdate ) >= to_days( '$startdate' ) 
     and to_days( tdate ) <= to_days( '$enddate') order by tdate desc"); # get tests

    my $sth2 = $dbh->prepare("select category, name, score, seq from read_test_score 
      where testid = ? order by seq");


    foreach my $studnum ( @students ) {

        # Get Name
	$sth->execute( $studnum );
	my ( $lastname, $firstname ) = $sth->fetchrow;
	if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }

	# Get Tests
	$sth1->execute( $studnum, $readinglevel );
	if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
	my $trows = $sth1->rows;

	# if $onlylatest, limit to 1 test
	if ( $onlylatest and $trows > 1 ) { $trows = 1; } 

	for ( 1 .. $trows ) {

	    my ( $testid, $tdate ) = $sth1->fetchrow;

	    print "<tr><td class=\"la\"><b>$lastname</b>, $firstname</td><td>". fmtDate($tdate). "</td>\n";
	    
	    # Loop through all the items.
	    $sth2->execute( $testid );
	    if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
	    my @studata = ();
	    my $total = 0;
	    my $count = 0;
	    my %cattotal = ();
	    my %catcount = ();



	    while ( my ( $category, $name, $score, $seq ) = $sth2->fetchrow ) {
		# Check objective
	        my ( $tcat, $tname ) = split(/::/, $objectives{ $seq } );
		if ( $tcat ne $category or $tname ne $name ) { # error
		    print "<div>$lex{Error} Mismatch: $tcat - $category OR $tname - $name</div>\n";
		    next;
		}

		$cattotal{$category} += $score;
		$catcount{$category} += 1;

		$studata[$seq] = $score;
		$total += $score;
		$count++;
	    }


	    # Equivalent Grade print
	    my $equivgrade = scoreToGrade($total, $readinglevel );
	    print "<td>$equivgrade</td>\n";
 
	    # Category Scores
	    my $totalscore;
	    my $totalpossible;
#	    foreach my $cat ( sort keys %catcount ) {
# Old 
	    foreach my $cat ( @catorder ) {
		my $tot = $catcount{$cat} * 4;
		print "<td>$cattotal{$cat}/$tot</td>";
		$totalscore += $cattotal{$cat};
		$totalpossible += $catcount{$cat};
	    }
	    $totalpossible *= 4;
	    print "<td>$totalscore/$totalpossible</td>";


	    # Now do the numeric print.
	    my %class = ( 1 => 'r', 2 => 'y', 3 => 'b', 4 => 'g' );
	    foreach my $key ( sort { $a <=> $b } keys %objectives ) { # numeric sort 
		my $val = $studata[ $key ];
		my $cl = $class{ $val };
		print "<td class=\"$cl\">$val</td>";
	    }
	    print "</tr>\n";
	} # end of test print loop

    } # end of student loop.

    print "</table><p></p>\n";
    return;

}
