#! /usr/bin/perl
#  Copyright 2001-2014 Leslie Richardson

#  This file is part of Open Admin for Schools.

#  Open Admin for Schools is free software; you can redistribute it 
#  and/or modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2 of 
#  the License, or (at your option) any later version.

# Global version of this report
# INAC Report 2 - by Grade only, not band. Uses new library functions.

my $percentsymbol = '';


my %lex = ('Attendance Report' => 'Attendance Report',
	   'Main' => 'Main',
	   'Attendance' => 'Attendance',
	   'Periods Per Day' => 'Periods Per Day',
	   'View/Download' => 'View/Download',
	   'View Log File' => 'View Log File',
	   'Last Name' => 'Last Name',
	   'First' => 'First',
	   'Middle' => 'Middle',
	   'Birthdate' => 'Birthdate',
	   'Status' => 'Status',
	   'Reserve' => 'Reserve',
	   'Mother' => 'Mother',
	   'School Days' => 'School Days',
	   'Error' => 'Error',
	   'Date' => 'Date',
	   'Paper Size' => 'Paper Size',
	   'Letter' => 'Letter',
	   'Legal' => 'Legal',
	   'A4' => 'A4',
	   'Continue' => 'Continue',
	   'Records per Page' => 'Records per Page',
	   'Not Defined' => 'Not Defined',
	   'Grade' => 'Grade',
	   'Student' => 'Student',
	   'Not Found' => 'Not Found',
	   'Band' => 'Band',
	   'Font Size' => 'Font Size',
	   'Group' => 'Group',
	   'HTML' => 'HTML',
	   'Homeroom' => 'Homeroom',
	   'PDF' => 'PDF',
	   'Separate with Spaces' => 'Separate with Spaces',
	   'Report Type' => 'Report Type',
	   'Missing' => 'Missing',
	   'Days Open' => 'Days Open',
	   'Total' => 'Total',
	   'Month' => 'Month',
	   'Average' => 'Average',
	   'Division' => 'Division',
	   'Periods per Day' => 'Periods per Day',
	   'Select' => 'Select',
	   'School' => 'School',
	   'CSV' => 'CSV',
	   'Show' => 'Show',
	   'Gender' => 'Gender',
	   'Raw' => 'Raw',
	   'Results' => 'Results',
	   'Percentage' => 'Percentage',
	   'Remote' => 'Remote',
	   'Database' => 'Database',

	   );

my $self = 'rptatt3global.pl';

my $oabase = '/opt/openadmin';


use DBI;
use CGI;
use Date::Business;
use Number::Format qw(:all);

# Constants
# Grade Divisions: map Grade => Division.
my %grDiv = ('K' => '1', '1' => '1', '2' => '1', '3' => '1', '4' => '2', '5' => '2',
'6' => '2', '7' => '3', '8' => '3', '9' => '3', '10' => '4', '11' => '4', '12' => '4');

my %skipgrades = ('K' => '1', 'PK' => '1'); # grades to skip over.



my $defmaxrecords = 28; # Maximum records per PDF page.
my $group = 'grade';
my $pdf = '1';
my $html;

# Read config variables
eval require "admin.conf";
if ( $@ ) {
    print $lex{Error}. ": $@<br>\n";
    die $lex{Error}. ": $@\n";
}

eval require "../lib/libattend.pl";
if ( $@ ) {
    print $lex{Error}. ": $@<br>\n";
    die $lex{Error}. ": $@\n";
}

eval { require "$globdir/global.conf"; };
if ( $@ ) {
    print $lex{Error}. " $self: $@<br>\n";
    die $lex{Error}. "$self: $@\n";
}

@dbase = sort keys %alldbase; # global records.


my $q = new CGI;
my %arr = $q->Vars;
print $q->header( -charset, $charset );



my @tim = localtime(time);
my $year = $tim[5] + 1900;
my $month = $tim[4] + 1;
my $day = $tim[3];
if (length($month) == 1){ $month = "0".$month;}
if (length($day) == 1){ $day = "0".$day;}
my $currsdate = "$year-$month-$day";
my $currdate = "$month[$month] $day, $year";


# HTML Header
my $title = qq{$lex{'Attendance Report'} 3};
print "$doctype\n<html><head><title>$title</title>
<link rel=\"stylesheet\" href=\"/division.css\" type=\"text/css\">\n";

#print "<link rel=\"stylesheet\" type=\"text/css\" media=\"all\" href=\"/js/calendar-blue.css\" title=\"blue\">
#<script type=\"text/javascript\" src=\"/js/calendar.js\"></script>
#<script type=\"text/javascript\" src=\"/js/lang/calendar-en.js\"></script>
#<script type=\"text/javascript\" src=\"/js/calendar-setup.js\"></script>\n";


print "$chartype\n</head><body>\n";
print "[ <a href=\"$homepage\">$lex{Main}</a> ]\n";

print qq{<h1>$title };
if ( $schoolname ) { print " &ndash; $schoolname"; }
print qq{</h1>\n};

if ( not $arr{page} ) {
    showStartPage();

} elsif ( $arr{page} == 1 ) {
    delete $arr{page};

    if ( $arr{reporttype} eq 'html' ) {
	mkHTMLReport();
    } elsif ( $arr{reporttype} eq 'csv' ) {
	mkCSVReport();
    }

}





#---------------
sub mkHTMLReport {
#---------------

    # foreach my $key ( sort keys %arr ) { print qq{K:$key V:$arr{$key}<br>\n}; }


    # Load Remote database
    if ( not $arr{db} ) {
	print qq{<h3>$lex{Database} $lex{'Not Found'}</h3>\n};
	print qq{</body></html>\n};
	exit;
    }

    my $db = $arr{db};
    my $dsn = "DBI:mysql:database=$db;host=$remotehost";
    my $dbh = DBI->connect($dsn,$remoteuser,$remotepassword);


    # Load Configuration Variables;
    my $sth = $dbh->prepare("select id, datavalue from conf_system where filename = 'admin'");
    $sth->execute;
    if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
    while (	my ($id, $datavalue) = $sth->fetchrow ) {
	eval $datavalue;
	if ( $@ ) {
	    print "$lex{Error}: $@<br>\n";
	    die "$lex{Error}: $@\n";
	}
    }

    print "<h3>School Start:$schoolstart | \n";
    print "School End:$schoolend</h3>\n";


    my $enddate = $schoolend;
    my $startdate = $schoolstart;


    # Functions from LibAttend
    my @schoolmonths = mkSchoolMonths( $startdate, $enddate );

    #~~ print "School Months", @schoolmonths, "<br>\n";

    # Uses $schoolstartdate for starting date.
    # Returns array listing yyyy-mm months in order starting from 1 (zeroth is
    # blank) Extracting the mm component and using this as index on @month
    # or @shortmonth will give the month name.

    %schooldays = mkSchoolDays( $startdate, $enddate, $dbh );
    # returns hash of schooldays in month. key is yyyy-mm and value is schooldays in month
    #~~ print "School Days", %schooldays, "<br>\n";

=head1
if ( $pdf ) { # Open TEX file, and print Header
    $logfile = "pdflog$$.txt";
    $shortname = "inac$$";
    $filename = "$shortname.tex";
    open(TEX,">$filename") || die "Can't open tex file";
    print_tex_doc_start(); # Setup the start of the file.
}
=cut


    # Find the Students, first get current students
    my %students = ();
    my $sth = $dbh->prepare("select studid, studnum from student");
    $sth->execute;
    if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }
    while ( my ($id, $studnum) = $sth->fetchrow ) {
	$students{$studnum} = 1;
    }



    # select to see if they have any student record.
    my $sth1 = $dbh->prepare("select count(*) from studentall where studnum = ?");
    # check for attendance records
    my $sth2 = $dbh->prepare("select count(*) from attend where studentid = ?");


    # now find any withdrawn ones, that were here this year.
    my $sth = $dbh->prepare("select id, studnum from transfer 
     where type = 'withdraw' and to_days( date ) >= to_days( '$startdate' ) and
     to_days( date ) <= to_days( '$enddate' )");
    $sth->execute;
    if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }
    while ( my ($id, $studnum) = $sth->fetchrow ) {

	# Check to make sure they have a student record
	$sth1->execute;
	if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }
	my $count = $sth1->fetchrow;

	$sth2->execute;
	if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }
	my $attcount = $sth2->fetchrow;

	if ( not $count and $attcount ) { # have attendance recs but no student rec
	    print "$lex{Error}: Attendance records exist for missing student: $studnum<br>\n";
	}

	if ( $count ) {
	    $students{$studnum} = 1;
	}
    }


    my %studgrade = ();
    my %gender = ();
    $sth = $dbh->prepare("select grade, sex from studentall where studnum = ?");
    foreach my $studnum ( keys %students ) {

	$sth->execute( $studnum );
	if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }
	my ($grade, $gender) = $sth->fetchrow;

	if ( $skipgrades{$grade} ) { next; } # next student

	if ( not $grade ) { 
	    print "$lex{Missing} $lex{Grade}. $lex{Student}: $studnum<br>\n";
	    #delete $students{$studnum}; # No Grade = Remove.
	} else { # put in the grade value
	    if ( not $studgrade{$grade} ) { $studgrade{$grade} = []; }
	    push @{ $studgrade{$grade} }, $studnum;
	    $gender{$studnum} = $gender;
	}
    }

# use Data::Dumper;
# print Dumper %studgrade;


    my %lexi = ('Absent' => 'Absent',
		'Late' => 'Late');

    # Now Loop through all grades
    my %attend = ();
    my %genderatt = ();
    my %divatt = (); # Division Attendance
    my $failflag = (); # fail before printing.

    foreach my $grade ( sort {$a <=> $b} keys %studgrade ) {


	my $ppd = $g_ppd{ $grade };
	if ( not $ppd ) {
	    print "$lex{Missing} $lex{'Periods per Day'} - $lex{Grade}:$grade:<br>\n"; 
	    $failflag = 1; # fail before output.
	}
	my $division = $grDiv{$grade}; # The Grade Division 1-4


	# Students in this grade
	foreach my $studnum ( @{ $studgrade{$grade} } ) {
    
	    my $sex = lc( $gender{$studnum});

	    my $ref = calcMonthlyEnrollment( $studnum, $startdate, $enddate, $dbh );
	    my %enrolYM = %$ref;
	    # format:  $enrolYM{yearmonth}-> start, end, days.

	    foreach my $ym ( sort keys %enrolYM ) {

		my $result = calcMonthlyAttendance( $studnum, $ym, $ppd, '',\%lexi, $dbh ); # no end date req'd.
		my ($absent, $late) = split(/:/, $result);

		my $enrolled = $enrolYM{$ym}->{days};
		my $present = $enrolled - $absent;

		$attend{$grade}{$ym}->{attend} += $present;
		$attend{$grade}{$ym}->{enrol} += $enrolled;

		$genderatt{$grade}{$ym}{$sex}->{attend} += $present;
		$genderatt{$grade}{$ym}{$sex}->{enrol} += $enrolled;

		$divatt{$division}{$ym}->{attend} += $present;
		$divatt{$division}{$ym}->{enrol} += $enrolled;

	    }
	}
    }

#use Data::Dumper;
#print Dumper %attend;
#print "<p></p>\n";

    if ( $failflag ) {
	print "</body></html>\n";
	exit;
    }


    # Now print it out!
    # First Row: Days Open, Then each grade for each month; Year:Month along the top.
    print "<table cellspacing=\"0\" cellpadding=\"3\" border=\"1\">\n";
    print "<tr><th></th>";
    for my $idx ( 1 .. $#schoolmonths ) {
	print "<th>$schoolmonths[$idx]</th>";
    }
    print "<th>$lex{Total}</th></tr>\n";


    # Days Open
    print "<tr style=\"font-weight:bold;\"><td>$lex{'Days Open'}</td>";
    my $totaldays;
    for my $idx ( 1 .. $#schoolmonths ) {
	my $ym = $schoolmonths[$idx];
	print "<td>$schooldays{$ym}</td>";
	$totaldays += $schooldays{$ym};
    }
    print "<td>$totaldays</td></tr>\n";


    # Grades
    my (%monthAtt, %monthEn);
    my $highestgrade; # track largest grade value for division printing
    foreach my $grade ( sort {$a <=> $b} keys %studgrade ) {

	$highestgrade = $grade;
	if ( $grade == 4 or $grade == 7 or $grade == 10 ) { # print division line
	    my $div = $grDiv{ $grade - 1 };
	    if ( %{$divatt{$div}} and $arr{showdivision} ) { # we have values to print
		
		my ( $divAttend, $divEnrolled );
		print "<tr style=\"background-color:#CCC;\"><td><b>$lex{Division} $div</b></td>";
		for my $idx ( 1 .. $#schoolmonths ) {
		    my $ym = $schoolmonths[$idx];
		    
		    my $attend =  $divatt{$div}{$ym}->{attend};
		    my $enrolled = $divatt{$div}{$ym}->{enrol};

		    # for totals at far right of row
		    $divAttend += $attend;
		    $divEnrolled += $enrolled;

		    my $average;
		    if ( $enrolled ) {
			$average = format_number( $attend / $enrolled * 100, 1,1); 
		    }

		    print "<td>";
		    if ( $arr{showraw} ) { 
			print "$attend / $enrolled ";
		    }
		    if ( $arr{showpercent} ) {
			print " $average$percentsymbol ";
		    }
		    print "</td>";

		}

		# Division totals
		my $divAverage;
		if ( $divEnrolled ) { 
		    $divAverage = format_number( $divAttend / $divEnrolled * 100, 1,1); 
		}
		print "<td>";
		if ( $arr{showraw} ) {
		    print "$divAttend / $divEnrolled ";
		}
		if ( $arr{showpercent} ) {
		    print " $divAverage$percentsymbol ";
		}
		print "</td></tr>\n";

	    }
	} # end of Division printing Row


	my ($totalAttend, $totalEnrolled);
	print "<tr><td>$lex{Grade} $grade</td>";
	for my $idx ( 1 .. $#schoolmonths ) {
	    my $ym = $schoolmonths[$idx];
	    
	    my $attend =  $attend{$grade}{$ym}->{attend};
	    my $enrolled = $attend{$grade}{$ym}->{enrol};

	    my $boyattend =  $genderatt{$grade}{$ym}{'m'}->{attend};
	    my $boyenrolled = $genderatt{$grade}{$ym}{'m'}->{enrol};

	    my $girlattend =  $genderatt{$grade}{$ym}{'f'}->{attend};
	    my $girlenrolled = $genderatt{$grade}{$ym}{'f'}->{enrol};


	    $totalAttend += $attend;
	    $totalEnrolled += $enrolled;

	    $monthAtt{$ym} += $attend;
	    $monthEn{$ym} += $enrolled;

	    my ($boyaverage, $girlaverage);
	    if ( $boyenrolled ) { 
		$boyaverage = format_number( $boyattend / $boyenrolled * 100, 1,1); 
	    }
	    if ( $girlenrolled ) { 
		$girlaverage = format_number( $girlattend / $girlenrolled * 100, 1,1); 
	    }

	    my $average;
	    if ( $enrolled ) {
		$average = format_number( $attend / $enrolled * 100, 1,1); 
	    }


	    print '<td>';
	    if ( $arr{showraw} ) {
		print "$attend / $enrolled";
	    } 
	    if ( $arr{showpercent} ) {
		print " $average$percentsymbol ";
	    }
	    if ( $arr{showgender} ) {
		print "<br>B:$boyaverage$percentsymbol G:$girlaverage$percentsymbol";
	    }
	    print '</td>';


	}

	# Grade totals
	my $gradeAverage;
	if ( $totalEnrolled ) { 
	    $gradeAverage = format_number( $totalAttend / $totalEnrolled * 100, 1,1); 
	}

	print "<td>";
	if ( $arr{showraw} ) {
	    print "$totalAttend / $totalEnrolled ";
	}
	if ( $arr{showpercent} ) {
	    print " $gradeAverage$percentsymbol";
	}
	print "</td></tr>\n";

    }


    # Division Row 
    my $div = $grDiv{ $highestgrade };
    if ( %{$divatt{$div}} and $arr{showdivision} ) { # we have values to print

	my ( $divAttend, $divEnrolled );
	print "<tr style=\"background-color:#CCC;\"><td><b>$lex{Division} $div</b></td>";
	for my $idx ( 1 .. $#schoolmonths ) {
	    my $ym = $schoolmonths[$idx];
	    
	    my $attend =  $divatt{$div}{$ym}->{attend};
	    my $enrolled = $divatt{$div}{$ym}->{enrol};
	    
	    # for totals at far right of row
	    $divAttend += $attend;
	    $divEnrolled += $enrolled;

	    my $average;
	    if ( $enrolled ) {
		$average = format_number( $attend / $enrolled * 100, 1,1); 
	    }

	    print "<td>";
	    if ( $arr{showraw} ) {
		print "$attend / $enrolled ";
	    }
	    if ( $arr{showpercent} ) {
		print " $average$percentsymbol ";
	    }
	    print "</td>";

	}

	# Division totals
	my $divAverage;
	if ( $divEnrolled ) { 
	    $divAverage = format_number( $divAttend / $divEnrolled * 100, 1,1); 
	}

	print "<td>";
	if ( $arr{showraw} ) { 
	    print "$divAttend / $divEnrolled ";
	} 
	if ( $arr{showpercent} ) {
	    print " $divAverage$percentsymbol ";
	}
	print "</td></tr>\n";


    } # end of Division row



    # Monthly Stats:
    print "<tr style=\"background-color:#DDD;\"><td>$lex{Month} $lex{Average}</td>";
    for my $idx ( 1 .. $#schoolmonths ) {
	my $ym = $schoolmonths[$idx];

	if ( $monthEn{$ym} ) { 
	    $monthAverage = format_number( $monthAtt{$ym} / $monthEn{$ym} * 100, 1,1); 
	}

	print "<td>";
	if ( $arr{showraw} ) {
	    print "$monthAtt{$ym} / $monthEn{$ym} ";
	}
	if ( $arr{showpercent} ) {
	    print " $monthAverage$percentsymbol ";
	}
	print "</td>\n";


	$gtotalAtt += $monthAtt{$ym};
	$gtotalEn += $monthEn{$ym};
    }

    if ( $gtotalEn ) { 
	$gtotalAverage = format_number( $gtotalAtt / $gtotalEn * 100, 1,1); 
    }
    print "<td>";
    if ( $arr{showraw} ) {
	print "$gtotalAtt / $gtotalEn ";
    }
    if ( $arr{showpercent} ) {
	print " $gtotalAverage$percentsymbol ";
    }
    print "</td></tr>\n";


    print "</table>\n";
    print "<p>&nbsp;</p>\n";
    print "</body></html>\n";

}




#----------------
sub showStartPage {
#----------------

    # Get remote databases and years
    my $remotedbase = 'information_schema';
    my $dsnr = "DBI:mysql:database=$remotedbase;host=$remotehost";
    my $dbhr = DBI->connect($dsnr,$remoteuser,$remotepassword);
    #my $dbh = DBI->connect($dsn,'tsecremote','ts99ec4');

    my $sth = $dbhr->prepare("select distinct table_schema from TABLES order by table_schema");
    $sth->execute;
    if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }
    my %remotedb;

    while ( my $db = $sth->fetchrow ) {
#	print qq{DB:$db<br>\n};
	if ( $db eq 'mysql' or $db eq 'information_schema' ) { next; }
	$remotedb{$db} = 1;
    }


    print "<form action=\"$self\" method=\"post\">\n";
    print "<input type=\"hidden\" name=\"page\" value=\"1\">\n";

    print "<table cellpadding=\"3\" cellspacing=\"0\" border=\"0\" ";
    print "style=\"padding:0.5em;border:1px solid gray;\">\n";

    print qq{<tr><td class="bra">$lex{Select} $lex{Remote} $lex{Database}</td><td></td></tr>\n};
    foreach my $db ( sort keys %remotedb ) { 
	print qq{<tr><td class="la"><input type="radio" name="db" value="$db"> $db</td></tr>\n};
    }


    print "<tr><td class=\"bra\">$lex{'Report Type'}</td>\n";
    print "<td><select name=\"reporttype\"><option value=\"html\">$lex{HTML}</option>\n";
    print "<option value=\"csv\">$lex{CSV}</option></select></td></tr>\n";


    print "<tr><td class=\"bra\">$lex{Show} $lex{Gender} $lex{Results}</td>\n";
    print qq{<td><input type="checkbox" name="showgender" value="1" checked="checked"></td></tr>\n};

    print "<tr><td class=\"bra\">$lex{Show} $lex{Raw} $lex{Results}</td>\n";
    print qq{<td><input type="checkbox" name="showraw" value="1" checked="checked"></td></tr>\n};

    print "<tr><td class=\"bra\">$lex{Show} $lex{Percentage} $lex{Results}</td>\n";
    print qq{<td><input type="checkbox" name="showpercent" value="1" checked="checked"></td></tr>\n};

    print "<tr><td class=\"bra\">$lex{Show} $lex{Division}</td>\n";
    print qq{<td><input type="checkbox" name="showdivision" value="1" checked="checked"></td></tr>\n};



=head
    print "<tr><td class=\"ra\">". $lex{Group}. "</td>\n<td>";
    print "<select name=\"groupname\"><option>$lex{Band}</option>\n";
    print "<option>$lex{Grade}</option><option>$lex{Homeroom}</option>\n";;
    print "</select>\n <input type=\"text\" name=\"groupvalue\" size=\"12\"> ";
    print $lex{'Separate with Spaces'}. "</td></tr>\n";

    # Paper Size
    $defaultpapersize =~ s/paper//; # strip out word paper so lex works; from admin.conf
    my $defpaper = ucfirst( $defaultpapersize );

    print "<tr><td align=\"right\">". $lex{'Paper Size'}. "</td>\n<td><b>";
    print $lex{Legal}. "</b></td></tr>\n";
    print "<select name=\"papersize\">";
    if ( $lex{$defpaper} ) { print "<option>". $lex{$defpaper}. "</option>\n"; }
    my @sizes = qw(Letter A4 Legal);
    foreach my $size ( @sizes ) {
	if ( $size eq $defpaper ) { next; }
	print "<option>$lex{$size}</option>";
    }
    print "</select></td></tr>\n ";

    print "<tr><td align=\"right\">". $lex{'Font Size'}. "</td>\n<td>";
    print "<select name=\"fontsize\"><option>10pt</option><option>11pt</option>\n";
    print "<option>12pt</option></select></td></tr>\n ";

    print "<tr><td class=\"ra\">". $lex{'Records per Page'}. "</td>\n<td>";
    print "<input type=\"text\" name=\"maxrecords\" size=\"4\" value=\"$defmaxrecords\"></td></tr>\n";

=cut

    print "<tr><td></td><td><input type=\"submit\" value=\"";
    print $lex{Continue}. "\"></td></tr>\n";

    print "</table></form>\n";

=head1
    print "<script type=\"text/javascript\">\n";
    print "Calendar.setup({ \n";
    print "inputField  : \"monthatt_date\", \n";
    print "ifFormat    : \"%Y-%m-%d\", \n";
    print "button      : \"monthatt_trigger\", \n";
    print "singleClick : false, \n";
    print "step : 1 \n";
    print "});</script>\n";

=cut

    print "</body></html>\n";

    exit;

}

