#! /usr/bin/perl
#  Copyright 2001-2014 Leslie Richardson
#  This file is part of Open Admin for Schools.

# Query School Courses and compare to provincial enrollments.

# Passed Values: none.

my $self = 'qryschcourses.pl';

use DBI;
use CGI;
use XML::Writer;
use XML::Writer::String;
use Data::UUID;
use HTTP::Request::Common qw(POST); 
use HTTP::Headers; 
use LWP::UserAgent; 
use XML::LibXML;

my $self = 'qryschcourses_global.pl';
my $basedir = '/opt/openadmin';

my %lex = ( 'Main' => 'Main',
	    'Error' => 'Error',
	    'Continue' => 'Continue',
	    'Select' => 'Select',
	    'School' => 'School',

	    );


eval require "../admin.conf";
if ( $@ ) {
    print $lex{Error}. " $@<br>\n";
    die $lex{Error}. " $@\n";
}


eval require "slxmllibNew.pl";
if ( $@ ) {
    print $lex{Error}. " $@<br>\n";
    die $lex{Error}. " $@\n";
}


my $q = new CGI;
my %arr = $q->Vars;
print $q->header( -charset, $charset);


my ($dsn, $dbh, $schooldir);
if ( $arr{dbase} ) {
    $dbase = $arr{dbase};
    $dsn = "DBI:$dbtype:dbname=$dbase";
    $dbh = DBI->connect($dsn,$user,$password);
    $dbh->{mysql_enable_utf8} = 1;

    $schooldir = $basedir. q{/}. $alldirs{ $dbase };
    # print "DBASE: $dbase School Dir: $schooldir<br>\n";

}


# Print Page Header
my $title = "Query Course Enrollment";
print "$doctype\n<html><head><title>$title</title>\n";
print "<link rel=\"stylesheet\" href=\"$css\" type=\"text/css\">\n";
print "<style type=\"text/css\">body { padding: 1em 3em; }</style>\n";
print "$chartype\n</head><body>\n";
print "[ <a href=\"$homepage\">Main</a> |\n";

print qq{<h1>$title</h1>\n};



if ( not $arr{page} ) {
    showStartPage();

} else {
    delete $arr{page};
}



# Find all 10-12 subjects and add those with students enrolled into a hash.
my %enrolledsubjects;
my  $sth = $dbh->prepare("select distinct eval.subjcode from eval
 left outer join subject on eval.subjcode = subject.subjsec
 where subject.grade = 10 or subject.grade = 11 or subject.grade = 12");
$sth->execute;
if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr;}
while (my $subjsec = $sth->fetchrow) {
    $enrolledsubjects{$subjsec} = 1;
}


print qq{<h1>$alldbase{$dbase}</h1>\n};

print "<p>Query Sask Ed's records of student course enrollment.</p>\n";

@month = ('','January','February','March','April','May','June','July',
	  'August','September','October','November','December');
# $sec, $min, $hour, $mday, $mon, 
# $year, $wday, $yday, $isdst ) = localtime(time);

@tim = localtime(time);
$year = $tim[5] + 1900;
$tim[4]++;
for (0..4){if (length($tim[$_]) == 1){ $tim[$_] = '0'.$tim[$_];}}
$currdate = "$year-$tim[4]-$tim[3]";
$currtime = "$tim[2]:$tim[1]:$tim[0]";

# Load the sds userid and password
{
    my $sdsfile = $schooldir. q{/}. 'etc/admin.conf.root';
    eval require $sdsfile;
    if ( $@ ) {
	print $lex{Error}. " $@<br>\n";
	die $lex{Error}. " $@\n";
    }
}

# Load any sasked values
# get sasked info from configuration system
my $sth = $dbh->prepare("select datavalue from conf_system where sectionname = 'sasked'");
$sth->execute;
while ( my $datavalue = $sth->fetchrow ) {
    eval $datavalue;
    if ( $@ ) {
	print $lex{Error}. " $@<br>\n";
	die $lex{Error}. " $@\n";
    }
}


# Create a new user agent
my $ua = LWP::UserAgent->new();
$ua->agent("OpenAdmin");

$count=1;
mkQueryString($count);

$debug = 0;

# DEBUG Data Errors
if ($debug){
    print "<div style=\"color: blue;\">\n";
    print "<h1>DEBUG - Request sent to Sask Ed</h1>\n";
    my $temp = $output->value;
    $temp =~ s/</&lt;/g;
    $temp =~ s/>/&gt;/g;
    print "<pre>",$temp,"</pre></div><p></p>\n";
}

# Create the https post request 

my $req = POST $url, [ XML=>$output->value ];

$req->content_type('application/xml;charset="utf-8"');
$req->authorization_basic($sds_userid, $sds_password); 

# Issue the request and receive a response
my $res = $ua->request($req);

# Check the status of the response
if ($res->is_success) {

    # For Debugging Data Errors
    if ($debug){
	print "<h1>DEBUG - Sask Ed XML Response</h1>\n";
	print "<div style=\"color:green;\">\n";
	my $temp = $res->content;
	$temp =~ s/</&lt;/g;
	$temp =~ s/>/&gt;/g;
	print "<br>",$temp,"</div>\n";
    }


    # Parse the response.
    my $parser = XML::LibXML->new();
    
    eval {$doc = $parser->parse_string($res->content)};
    if ($@){
	print "Error: $@<br>\n";
	print "<pre>Sask Ed Error:\n",$res->content,"</pre><br>\n";
	print "</body></html>\n"; die;
    }

    $doc->setEncoding('UTF-8');
    $root = $doc->getDocumentElement;
    $root->setNamespace($xmlns,'sl',1);

    # find Message ID.
    $mastermsgid = $root->findvalue('//sl:SL_MsgId');
    $status = $root->findvalue('//sl:SL_Status/sl:SL_StatusCode');

    if ($status eq 'Errors' or $status eq 'Invalid'){ 

	$statusmsg = $root->findvalue('//sl:SL_Status/sl:SL_StatusMsg');
	$errormsg = $root->findvalue('//sl:SL_Error/sl:SL_ErrorMsg');
	# Print out error and exit
	print "<h1 style=\"color:red;\">Error: $statusmsg</h1>\n";
	print "<h1 style=\"color:red;\">Error: $errormsg</h1>\n";

	print "</body></html>\n";
	exit;

    } elsif ($status eq 'Successful'){ 
	# Done below instead.
	# parseStudentCourseEnrollments();

    } else { # print warnings... ($status eq 'Warnings')
	print "There were warnings...";
    }
    
}  else { # Transfer Error!
    my $err = $res->status_line;
    print "<h1>Transfer Error: $err</h1>";
}

my $course_ref = parseSCEbyClass();

print "<div style=\"width:80%;border:1px solid black;
padding:1em;text-align:left;\">"; print "<p>A <b
style=\"color:red;\">red value</b> in the <b>classid</b> above the
table shows that Sask Ed has stored a value that is actually
just a subject code, not a real subject-section. The script has looked
for a subject that matches this classid and used that instead.</p>\n";


print "<p>A <b style=\"color:red;\">red value</b> in the <b>total</b>
at the bottom of the table for a class indicates that there is
difference of opinion between Sask Ed and the local
enrollment records. This means that you should run the synchronization
script for that subject-section to update enrollments for that
class. Currently OA will not remove an entire missing class
enrollment.</p>\n"; 

print "</div><p></p>\n";

CLASS:
foreach my $cl (keys %{$course_ref}) {

    my ($count, $subjsec, $description);
    my ($subj,$startdate,$enddate) = split /:/,$cl;

    # Check if subjsec... normally should be...
    my $sth1 = $dbh->prepare("select id, description from subject
           where subjsec = ?");
    $sth1->execute( $subj );
    if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr;}
    my ($id, $description) = $sth1->fetchrow;
    if ($id) {
	$subjsec = $subj;
    } else { # no id found, look at subject only for match.
	my $sth1 = $dbh->prepare("select id, description, subjsec from subject
         where subjcode = ?");
	$sth1->execute( $subj );
	if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr;}

	my ($id, $description, $sub) = $sth1->fetchrow;
	if (not $id) {
	    print "<div style=\"color:red;font-weight:bold;\">";
	    print "No Matching local Subject found for $subj. Error!</div>\n";
	    next CLASS; # jump to next class;
	} else {
	    $subjsec = $sub;
	}
    }

    # remove from total count so ones left don't have kids enrolled.
    delete $enrolledsubjects{$subjsec};    


    my (%localstudents, %totalstudents, %slstudents);
    my $localenrollment;

    # Find local students for this course.
    my $sth1 = $dbh->prepare("select distinct studnum, count(id) from eval
     where subjcode = ? group by studnum");
    $sth1->execute($subjsec);
    if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr;}

    # loop through all local students for this course.
    while ( my ($sn, $count) = $sth1->fetchrow ) {

	$localstudents{$sn} = $count;

	my ($lastname, $firstname, $studnum, $provnum) = split /:/, 
	  getStudentInfo($sn,0);
	$totalstudents{"$lastname:$firstname:$studnum:$provnum"} = 1;

    }


    my ($mode,$marksource) = split /:/,$coursedata{$cl};
    my ($tmp, $termstart,$termend) = split /:/,$cl;
    if ($tmp ne $subjsec) { $tmp = "<font color=\"red\">$tmp</font>"; }

    print "<div style=\"font-size:130%; font-weight:bold;\">";
    print "$description ($subjsec)</div>\n";

    print "<b>ClassId</b> $tmp&nbsp;&nbsp;<b>Mode</b> $mode&nbsp;&nbsp;";
    if ($marksource eq 'Blended') { 
	$marksource .= " [ <span style=\"color:red;font-size:130%;font-weight:800;\">Dept Exams</span> ]"; 
    }
    print "<b>Mark Source</b> $marksource<br>";
    print "<b>Start</b> $termstart&nbsp;&nbsp;<b>End</b> $termend<br>\n";

    print "<table cellpadding=\"4\" cellspacing=\"0\" border=\"1\">\n";
    print "<tr><th>Name (Studnum)</th><th>Prov<br>Number</th>";
    print "<th>Trms</th><th>Local<br>Enrol</th><th>SkLrn<br>Enrol</th>";
    print "</tr>\n";
	
    foreach my $pn ( @{ $course_ref->{$cl} } ) {

	# Get student info for this provnum
	my ($lastname, $firstname, $studnum, $provnum) = split /:/, 
	  getStudentInfo(0,$pn);

	$slstudents{$pn} = 1;
	$totalstudents{"$lastname:$firstname:$studnum:$pn"} = 1;

    }


    my $localcount;
    my $provcount;
    my $count = 1; # simple student counter
    # counts the number of students that have values in local recs.

    foreach my $rec (sort keys %totalstudents) {
	my ($lastname,$firstname, $studnum, $pn) = split /:/,$rec;

	my $provenrol;
	if ($slstudents{$pn}) { 
	    $provenrol = 'Y';
	    $provcount++;
	} else { 
	    $provenrol = '<font color=red><b>N</b></font>';
	}

	my $localenrol;
	if ($localstudents{$studnum}) { 
	    $localenrol = 'Y';
	    $localcount++;
	} else { 
	    $localenrol = '<font color=red><b>N</b></font>';
	}


	print "<tr><td>$count.<b>$lastname</b>, $firstname ($studnum)</td>";
	print "<td>$pn</td><td align=\"center\">$localstudents{$studnum}</td>";
	print "<td align=\"center\">$localenrol</td>";
	print "<td align=\"center\">$provenrol</td></tr>\n";
	$count++;
    }

    my $newstyle;
    if ($provcount != $localcount) { 
	$newstyle = 'style="color:red;background-color:#DDD;"'; 
    }
    
    print "<tr><th colspan=\"5\" $newstyle>SaskEd Count: $provcount ";
    print "Local Count: $localcount</th></tr>\n";
    print "</table><p></p>\n";

}


# Subjects without registrations with Sask Ed.
print "<h1>Subjects <b>not</b> registered with Sask Ed</h1>\n";
print "<table cellpadding=\"3\" cellspacing=\"0\" border=\"1\">\n";
print "<tr><th>Subject</th><th>Teacher</th><th>Grade</th>";
print "<th>Enrollment</th></tr>\n";

foreach my $subjsec (keys %enrolledsubjects) {
    my $sth = $dbh->prepare("select description, grade, teacher from subject
           where subjsec = '$subjsec'");
    $sth->execute;
    if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr;}
    my ($description, $grade, $teacher) = $sth->fetchrow;

    # Now get enrollment.
    my $enrollment;
    # Find local enrollment in this subject
    { my $sth1 = $dbh->prepare("select count(distinct studnum) from eval
       where subjcode = '$subjsec'");
      $sth1->execute;
      if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr;}
      $enrollment = $sth1->fetchrow;
    }

    print "<tr><td><b>$description</b> ($subjsec)</td><td>$teacher</td>";
    print "<td align=\"center\">$grade</td><td align=\"center\">$enrollment</td></tr>\n";
}

print "</table></center>\n";

print "</body></html>\n";


# Functions =============================

#--------------
sub mkQueryString { # for Course Enrollments
#--------------
    my $grade = shift;
    my $provnum = shift;
    my $idcount = shift;
    # Date and schoolnumber are globals.

    # Create Writer Instance
    $output = new XML::Writer::String;
    my $datamode = 0;
    if ($debug){ $datamode = 1;} # pretty print xml output
    $wr = new XML::Writer(OUTPUT => $output, 
			  DATA_MODE => $datamode, 
			  DATA_INDENT => '2');

    # Set XML Header and write Root Element
    $wr->xmlDecl("UTF-8");
    $wr->startTag('SL_Message','xmlns' =>$xmlns, 
		  'xmlns:xsi' =>$xmlnsxsi, 
		  'xsi:schemaLocation' => $xsischemaLocation);

    $wr->startTag('SL_Request');
    mkSL_Header($currdate, $currtime, $schoolnumber,$idcount);

    $wr->startTag('SL_Query');

    $wr->startTag('QueryBySchool',
		  'RefId' => "$schoolnumber",
		  'ObjectName' => 'StudentCourseEnrollments',
		  'ScopeCode' => 'Current'
		  );

    $wr->dataElement('SchoolId',$schoolnumber); 

    my $sdate = $schoolstart;
    $sdate =~ s/-//g;
    my $edate = $schoolend;
    $edate =~ s/-//g;

    $wr->dataElement('FromDate', $sdate );
    $wr->dataElement('ToDate', $edate );
#    print "Start:$sdate End:$edate<br>\n";

    $wr->endTag('QueryBySchool');

    $wr->endTag('SL_Query');
    $wr->endTag('SL_Request');
    $wr->endTag('SL_Message');
    $wr->end();
    
}


#----------------------
sub parseStudentCourseEnrollments { # passed document root object.
#----------------------
    my @studinfo = $root->findnodes('//sl:StudentCourseEnrollments');

    my $count = 1;

    foreach my $student (@studinfo){
	my $provnum = $student->findvalue('sl:StudentIdentification/sl:DeptAssignedPersonId');
	my $birthdate = $student->findvalue('sl:StudentIdentification/sl:BirthDate');

	my $sth = $dbh->prepare("select lastname, firstname from student
           where provnum = '$provnum'");
	$sth->execute;
	if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr;}
	my ($lastname, $firstname) = $sth->fetchrow;

	print "<h1>$firstname $lastname ($provnum - $birthdate</h1>\n";
	print "<table cellpadding=\"3\" cellspacing=\"0\" border=\"1\">\n";
	print "<tr><th>Schoolyear</th><th>Termstart</th><th>Termend</th>";
	print "<th>ClassId</th><th>Mode</th><th>Mark Src</th></tr>\n";

	my @classes = $student->findnodes('sl:StudentSchoolClasses/sl:StudentTermClasses');
	foreach my $class (@classes) {
	    my $schoolyear = $class->findvalue('sl:TermInfo/@SchoolYear');
	    my $termstart = $class->findvalue('sl:TermInfo/sl:StartDate');
	    my $termend = $class->findvalue('sl:TermInfo/sl:EndDate');
	    my @sections = $class->findnodes('sl:StudentClass');
	    foreach my $section (@sections) {
		my $classid = $section->findvalue('sl:ClassId');
		my $mode = $section->find('sl:ModeOfInstruction/@Code');
		my $marksource = $section->findvalue('sl:MarkSource/@Code');
		print "<tr><td>$schoolyear</td><td>$termstart</td><td>$termend</td>\n";
		print "<td>$classid</td><td>$mode</td><td>$marksource</td></tr>\n";
	    }
	}
	print "</table>\n";
    }

} # End of Sub 


#---------------------- 
sub parseSCEbyClass { # passed document root object.  
#---------------------- 

# Parse SchoolCourseEnrollments, and return a reference to a courses
# hash containing refs to a list of provincial numbers.

    my @studinfo = $root->findnodes('//sl:StudentCourseEnrollments');

    my $count = 1;
    my %courses;

    foreach my $student (@studinfo){
	my $provnum = $student->findvalue('sl:StudentIdentification/sl:DeptAssignedPersonId');
	# my $birthdate = $student->findvalue('sl:StudentIdentification/sl:BirthDate');

	my @classes = $student->findnodes('sl:StudentSchoolClasses/sl:StudentTermClasses');
	foreach my $class (@classes) {
	    #my $schoolyear = $class->findvalue('sl:TermInfo/@SchoolYear');
	    my $termstart = $class->findvalue('sl:TermInfo/sl:StartDate');
	    my $termend = $class->findvalue('sl:TermInfo/sl:EndDate');
	    my @sections = $class->findnodes('sl:StudentClass');
	    foreach my $section (@sections) {
		my $classid = $section->findvalue('sl:ClassId');
		my $mode = $section->find('sl:ModeOfInstruction/@Code');
		my $marksource = $section->findvalue('sl:MarkSource/@Code');
		$coursedata{"$classid:$termstart:$termend"} = "$mode:$marksource";
		if (not defined $courses{"$classid:$termstart:$termend"}) {
		    $courses{"$classid:$termstart:$termend"} = [ $provnum ];
		} else {
		    push @{$courses{"$classid:$termstart:$termend"}},$provnum;
		}
	    }
	} # End of StudentSchoolClasses

    } # End of student loop

    return \%courses;

} # End of parseSCEbyCourse


#-----------------
sub getStudentInfo {
#-----------------

    my $sn = shift; # passed student number (studnum)
    my $pn = shift; # passed provincial number (provnum)

    my $sth;
    if ($pn) {
	$sth = $dbh->prepare("select lastname, firstname, studnum, provnum
         from student where provnum = '$pn'");
    } elsif ($sn) {
	$sth = $dbh->prepare("select lastname, firstname, studnum, provnum
         from student where studnum = '$sn'");
    } else {
	return; # nothing passed.
    }

    $sth->execute;
    if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr; }
    my ($lastname, $firstname,$studnum,$provnum) = $sth->fetchrow;

    if (not $studnum) {  # look them up in withdrawn student table.
	if ($pn) {
	    $sth = $dbh->prepare("select lastname, firstname,studnum,provnum
             from studentwd where provnum = '$pn'");
	} elsif ($sn) {
	    $sth = $dbh->prepare("select lastname, firstname,studnum,provnum
             from studentwd where studnum = '$sn'");
	}
	$sth->execute;
	if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr; }
	($lastname, $firstname,$studnum, $provnum) = $sth->fetchrow;
	$lastname = "<font color=\"red\">WD</font> ". $lastname;
	if (not $studnum) { 
	    $lastname = "<font color=\"red\">Not Found</font>"; 
	}
    }

    return "$lastname:$firstname:$studnum:$provnum";

} # end of getStudentInfo




#----------------
sub showStartPage {
#----------------

    print qq{<form action="$self" method="post">\n};
    print qq{<input type=\"hidden\" name=\"page\" value=\"1\">\n};

    print "<table cellpadding=\"3\" cellspacing=\"0\" border=\"0\" ";
    print "style=\"padding:0.5em;border:1px solid gray;\">\n";

    print "<tr><td class=\"bra\">$lex{Select} $lex{School}</td><td class=\"la\">";
    print "<select name=\"dbase\"><option></option>\n";
    foreach my $db (sort keys  %alldbase ) { print qq{<option value="$db">$alldbase{$db}</option>}; }
    print "</select></td></tr>\n";

=head
    print "<tr><td class=\"bra\">$lex{'Report Type'}</td>\n";
    print "<td><select name=\"reporttype\"><option value=\"html\">$lex{HTML}</option>\n";
    print "<option value=\"csv\">$lex{CSV}</option></select></td></tr>\n";


    print "<tr><td class=\"bra\">$lex{Show} $lex{Gender} $lex{Results}</td>\n";
    print qq{<td><input type="checkbox" name="showgender" value="1" checked="checked"></td></tr>\n};

    print "<tr><td class=\"bra\">$lex{Show} $lex{Raw} $lex{Results}</td>\n";
    print qq{<td><input type="checkbox" name="showraw" value="1" checked="checked"></td></tr>\n};

    print "<tr><td class=\"bra\">$lex{Show} $lex{Percentage} $lex{Results}</td>\n";
    print qq{<td><input type="checkbox" name="showpercent" value="1" checked="checked"></td></tr>\n};

    print "<tr><td class=\"bra\">$lex{Show} $lex{Division}</td>\n";
    print qq{<td><input type="checkbox" name="showdivision" value="1" checked="checked"></td></tr>\n};
=cut

    print qq{<tr><td></td><td><input type="submit" value="$lex{Continue}"></td></tr>\n};
    print qq{</table></form>\n};

    print "</body></html>\n";

    exit;

}
