#!/usr/bin/perl
#  Copyright 2001-2010 Leslie Richardson

#  This file is part of Open Admin for Schools.

#  Open Admin for Schools is free software; you can redistribute it 
#  and/or modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2 of 
#  the License, or (at your option) any later version.

my %lex = ('No Student(s) Found' => 'No Student(s) Found',
	   'Student Search Results' => 'Student Search Results',
	   'Main' => 'Main',
	   'Name' => 'Name',
	   'last' => 'last',
	   'first or last or initials' => 'first or last or initials',
	   'Student Number' => 'Student Number',
	   'or' => 'or',
	   'Search' => 'Search',
	   'Cannot open' => 'Cannot open',
	   'Error' => 'Error',

	   );


use DBI;
use CGI;

eval require "admin.conf";
if ( $@ ) {
    print $lex{Error}. ": $@<br>\n";
    die $lex{Error}. ": $@\n";
}

eval require "$globdir/global.conf";
if ( $@ ) {
    print $lex{Error}. ": $@<br>\n";
    die $lex{Error}. ": $@\n";
}



my $q = new CGI;
print $q->header( -charset, $charset ); 
my %arr = $q->Vars;


# passed value...
my $student = $arr{student};

# print HTML heading
my $title = $lex{'Student Search Results'};
print qq{$doctype\n<html><head><title>$title</title>};
print qq{<link rel="stylesheet" href="$css" type="text/css">\n};
print qq{$chartype\n</head><body>\n};
print qq{[ <a href="$homepage">$lex{Main}</a> ]\n};


my $first = 1;
my $foundflag = 0;
my %metaval;
my $text; # holds template layout, labels replaced.

foreach my $database (keys %alldbase) { # defined in global config file.

    my $dsn = "DBI:mysql:$database";
    my $dbh = DBI->connect($dsn,$guser,$gpassword);


    if ( $first ) {

	# Read in Meta Values
	my $sth = $dbh->prepare("select fieldid, fieldname from meta where tableid = 'student'");
	$sth->execute;
	if ($DBI::errstr){print $DBI::errstr; die $DBI::errstr;}
	while ( my ($fid, $fname ) = $sth->fetchrow ) {
	    $metaval{ $fid } = $fname;
	}

	# Read in Template
	unless (open (FH,"<../template/student.tpl")) {
	    print $lex{'Cannot open'}. " template file: $!\n";
	    die $lex{'Cannot open'}. " template file: $!\n";
	}

	# slurp template
	{ local $/; $text = <FH>; close FH;}


	# Replace meta values.
	$text =~ s{\<\*(.*?)\*\>}
	{ exists($metaval{$1}) 
	      ? $metaval{$1} 
	      : $1
	}gsex;


	$first = 0;
    }


    # Setup the Search
    if ($student =~ /\d+/) {  # we have a student number
	$studentnumber = $student;
	$sth = $dbh->prepare("select * from student where studnum = ?");
	$sth->execute( $student );

    } else { # we have words hopefully with a comma

	my ( $lastname,$firstname )  = split(/\,/, $student);
        $firstname =~ s/^\s*//;
	$lastname =~ s/^\s*//;
	if ( $lastname and $firstname ){ # both entered.
	$sth = $dbh->prepare("select * from student where lastname = ? and firstname = ?");
	$sth->execute( $lastname, $firstname );

	} elsif ($lastname and not $firstname){ # only lastname (no comma)

	    if ( length($lastname) == 2 ){ # search by initials: fi, li.
		my $fi = substr($lastname,0,1). '%'; 
		my $li = substr($lastname,1,1). '%';
		$sth = $dbh->prepare("select * from student where lastname like ? and firstname like ?");
		$sth->execute( $li, $fi );

	    } else {
		$sth = $dbh->prepare("select * from student where lastname = ? order by firstname");
		$sth->execute( $lastname );
	    }
	}

    } # Last Else

    if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr; }

    # Start the Table
    # print qq{<table cellpadding="3" cellspacing="0" border="1">\n};


    my $firststudent = 1;

    # Loop through and display all records.
    while ( my $ref = $sth->fetchrow_hashref ){

	my %r = %$ref;

	if ( $firststudent ) { # print school name
	    print qq{<h2 style="color:blue;">$alldbase{$database}</h2>\n};
	    $firststudent = 0;
	}


	#print qq{<tr><td colspan="4" style="font-size:130%;">$r{firstname} $r{lastname}</td></tr>\n};
	print qq{<h2>$r{firstname} $r{lastname}</h2>\n};

	if ($r{pic} eq 'Y') {
	    print "<tr><td colspan=\"5\" style=\"background-color:#DDD; text-align:left;\">";
	    print "<img src=\"/pic-sm/$studnum.jpg\" align=\"middle\">";
	    print " <span style=\"font-size:200%;font-weight:bold;\">$r{firstname} $r{lastname}</td></tr>\n";
	} 


	
	my $formtext = $text; # text defined before function call.

	# Now put replacement text back in.
	$formtext =~ s{\<\@(.*?)\@\>}
              { exists($r{$1}) 
		  ? $r{$1} 
	          : $1
	      }gsex;
	print $formtext,"\n";

    } # End of Student Loop

    print qq{</table><p></p>\n};


} # end of school database loop


print qq{<p>[ <a href="$homepage">$lex{Main}</a> ]</p>\n};
mkSearchError();
print "</body></html>\n";


#----------------
sub mkSearchError {
#----------------
    #print "<div style=\"position:absolute;right:1em;\">\n";

    print "<div>";
    print "<form action=\"studsearch.pl\" method=\"post\">";
    print "<b>". $lex{Name}. "</b>(". $lex{last}. ','. $lex{'first or last or initials'};
    print ") ". $lex{or}. " <b>". $lex{'Student Number'}. "</b><br>\n";
    print "<input type=\"text\" name=\"student\" size=\"30\">\n";
    print "<input type=\"submit\" value=\"". $lex{Search}. "\"></form></div>\n";
}


#-------------------
sub prStudentRecord {
#-------------------

    # print the student record (as a row in a table)
    # Passed in values: $studnum, (global: $picdirurl, $cgiurl )
    # Also read in template value.

    my ($studnum,$text, $dbh) = @_;



} # End of PrStudentRecord
