#!/usr/bin/perl
#  Copyright 2001-2011 Leslie Richardson

#  This file is part of Open Admin for Schools.

#  Open Admin for Schools is free software; you can redistribute it 
#  and/or modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2 of 
#  the License, or (at your option) any later version.

my %lex = ('Main' => 'Main',
	   'Report Card' => 'Report Card',
	   'Transcript' => 'Transcript',
	   'Post' => 'Post',
	   'Mark' => 'Mark',
	   'Student' => 'Student',
	   'Subject' => 'Subject',
	   'Subjects' => 'Subjects',
	   'Duplicate' => 'Duplicate',
	   'Error' => 'Error',
	   'Replace Existing' => 'Replace Existing',
	   'Record' => 'Record',
	   'Update Record' => 'Update Record',
	   'Skipping' => 'Skipping',
	   'Check' => 'Check',
	   'Select' => 'Select',
	   'Subject' => 'Subjects',
	   'Identity Mismatch' => 'Identity Mismatch',
	   'Edit' => 'Edit',
	   'Term' => 'Term',

	   );

# rounding precsion of mark.
my $precision = 1;

my $self = 'tscpost.pl';
my $idfield = 'provnum'; # change to suit the field for state/provincial id or SSN, etc.

use DBI;
use CGI;
use Number::Format qw(:subs);


eval require "../../etc/admin.conf";
if ( $@ ) {
    print $lex{Error}. " $@<br>\n";
    die $lex{Error}. " $@\n";
}

my $q = CGI->new;
print $q->header( -charset, $charset );
my %arr = $q->Vars;

my $fmt = new Number::Format(-decimal_fill => '1', -decimal_digits => '2');

my $dsn = "DBI:$dbtype:dbname=$dbase";
my $dbh = DBI->connect($dsn,$user,$password);
$dbh->{mysql_enable_utf8} = 1;


# Page Header
my $title = "$lex{Transcript} $lex{Post}";
print "$doctype\n<html><head><title>$title</title>
<link rel=\"stylesheet\" href=\"$css\" type=\"text/css\">
$chartype\n</head><body style=\"padding:1em 2em;\">\n";
print "[ <a href=\"$homepage\">$lex{Main}</a> |\n";
print "<a href=\"$reppage\">$lex{'Report Card'}</a> ]\n";

print "<h1>$title</h1>\n";

if ( not $arr{flag} ) { # We have a selected student.
    selectTerm();
} elsif ( $arr{flag} == 1 ) {
    delete $arr{flag};
    selectSubjects();
} elsif ( $arr{flag} == 2 ) {
    delete $arr{flag};
    postSubjects();
} 


#-------------
sub selectTerm {
#-------------

    # Term Selection
    my @terms = ();
    my $sth = $dbh->prepare("select distinct endrptperiod from subject 
     where endrptperiod != '' and endrptperiod is not null order by endrptperiod");
    $sth->execute;
    if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }
    while ( my $trm = $sth->fetchrow ) {
	push @terms, $trm;
    }

    # Print Start of Selection.
    print "<h3>$lex{Post}: $lex{Select} $lex{Term}</h3>\n";

    print "<form action=\"$self\" method=\"post\">\n";
    print "<input type=\"hidden\" name=\"flag\" value=\"1\">\n";
    print "<input type=\"submit\" value=\"$lex{Select} $lex{Term}\">\n";
    print "<select name=\"term\"><option></option>\n";

    foreach my $trm ( @terms ) {
	print "<option>$trm</option>";
    }
    print "</select><br>$lex{Check} $lex{Subjects}\n"; 
    print " <input type=\"checkbox\" name=\"checked\" value=\"checked\">\n";

    print "</form></body></html>\n";

    exit;

}


#-----------------
sub selectSubjects {
#-----------------

    #foreach my $key (keys %arr ) { print "K:$key V:$arr{$key}<br>\n"; }


    # Subject Selection - first subjects with this term
    my $sth = $dbh->prepare("select distinct subjcode from eval 
     where term = ? and subjcode != '' and subjcode is not null");
    $sth->execute( $arr{term} );
    if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }
    while ( my $subj = $sth->fetchrow ) {
	$subjects{$subj} = 1;
    }

    $sth1 = $dbh->prepare("select endrptperiod, description from subject where subjsec = ?");
    foreach my $subjsec ( keys %subjects ) {
	$sth1->execute( $subjsec );
	if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }
	my ($endrptperiod, $description) = $sth1->fetchrow;
	if ( $endrptperiod == $arr{term} ) { # the subject ends this term...
	    $subjects{$subjsec} = $description;
	} else { # no match; remove value;
	    delete $subjects{$subjsec};
	}
    }

    # At this point %subjects hash only contains subjects that have
    # endrptperiod in the selected term.


    # Print Start of Selection.
    print "<h3>$lex{Select} $lex{Subjects}</h3>\n";

    print "<form action=\"$self\" method=\"post\">\n";
    print "<input type=\"hidden\" name=\"flag\" value=\"2\">\n";
    print "<input type=\"hidden\" name=\"term\" value=\"$arr{term}\">\n";

    print "<div style=\"width:20em;border:1px solid gray;padding:0.5em;\">";
    print "$lex{'Replace Existing'} $lex{Record}? ";
    print "<input type=\"checkbox\" name=\"replacemode\" value=\"1\"></div>\n";

    print "<input type=\"submit\" value=\"$lex{Select} $lex{Subjects}\">\n";
    print "<table cellpadding=\"3\" border=\"1\" cellspacing=\"0\">\n";

    foreach my $subjsec ( keys %subjects ) {
	print "<tr><td align=\"right\">$subjects{$subjsec} ($subjsec)</td>\n";
	print "<td align=\"left\">";
	print "<input type=\"checkbox\" name=\"$subjsec\" value=\"1\" $arr{checked}>";
	print "</td></tr>\n";    
    }

    print "</table>\n";
    print "<input type=\"submit\" value=\"$lex{Select} $lex{Subjects}\">\n";
    print "</form></body></html>\n";

    exit;

}



#---------------
sub postSubjects {
#---------------

    # foreach my $key (keys %arr ) { print "K:$key V:$arr{$key}<br>\n"; }

    # Get the r_MarkField Record from configuration system
    my $sth = $dbh->prepare("select datavalue from conf_system where dataname = ?");
    $sth->execute( 'r_MarkField' );
    my $datavalue = $sth->fetchrow;
    eval $datavalue;
    if ( $@ ) {
	print $lex{Error}. " $@<br>\n";
	die $lex{Error}. " $@\n";
    }

    # Get values from configuration system
    my @fieldnames = qw( markToLetter term_desc term_key ); 
    my $sth = $dbh->prepare("select datavalue from conf_system where dataname = ?");
    foreach my $var ( @fieldnames ) {
	$sth->execute( $var );
	my $datavalue = $sth->fetchrow;
	eval $datavalue;
	if ( $@ ) {
	    print "$lex{Error}: $@<br>\n";
	    die "$lex{Error}: $@\n";
	}
    }


    # Load transcript conf; defines markToLetter;
    # eval require "../../etc/transcript.conf";
    # if ( $@ ) {
    # print $lex{Error}. " $@<br>\n";
    # die $lex{Error}. " $@\n";
    # }


    my $term = $arr{term};  # the term we are posting....
    delete $arr{term};

    my $replacemode;
    if ( $arr{replacemode} ) {
	$replacemode = $arr{replacemode};
    }
    delete $arr{replacemode};



    # SQL Selections
    my $sth1 = $dbh->prepare("select lastname, firstname, initial, birthdate, 
     $idfield, graddate from studentall where studnum = ?");
    my $sth2 = $dbh->prepare("select count(*) from tscriptident where studnum = ?");
    my $sth3 = $dbh->prepare("select lastname, firstname, birthdate,studentid from tscriptident
     where studnum = ?");
    # Get the subject fields
    my $sth4 = $dbh->prepare("select description, credit, difficulty, area, calcavg 
      from subject where subjsec = ?");
    my $sth5 = $dbh->prepare("select id, $r_MarkField from eval where subjcode = ? and studnum = ? 
      order by term");

    # Print heading Section
    print "<table cellspacing=\"0\" cellpadding=\"3\" border=\"1\">\n";
    print "<tr><th>$lex{Student}</th><th>$lex{Subject}</th><th>$lex{Mark}</th></tr>\n";

    foreach my $subjsec ( keys %arr ) {

	# Get the subject information
	$sth4->execute ( $subjsec );
	if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
	my ( $description, $credit, $difficulty, $area, $calcavg ) = $sth4->fetchrow;
	

	# select each student record in turn and post.
	my $sth = $dbh->prepare("select $r_MarkField , studnum from eval
          where term = ? and subjcode = ? order by studnum");
	$sth->execute( $term, $subjsec );
	if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }

STUDENT:
	while ( my ($mark, $studnum) = $sth->fetchrow ) {

	    # Skip NULL values in mark
	    if ( not defined $mark ) { # NULL in field value
		print "<tr style=\"color:red\"><td>$lex{Skipping} NULL - ";
		print "$firstname $lastname</td>\n";
		print "<td>$description ($subjsec)</td>\n";
		print "<td>NULL</td></tr>\n";
		next STUDENT;
	    }

	    # Check to see if mark contains letters 
	    my $markContainsLetters;
	    if ( $mark =~ m/[a-zA-Z]/ ) {
		$markContainsLetters = 1;
		print "Mark Contains Letters: $mark<br>\n";
	    }


	    # Get full name and birthdate from studentall
	    $sth1->execute( $studnum );
	    if ( $DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }
	    my ($lastname,$firstname,$middlename,$birthdate,$studentid,$graddate)=$sth1->fetchrow;
	    # $studentid field is configurable at top of script.

	    # Check for existing ident record, and match; fail on error; otherwise create.
	    $sth2->execute( $studnum );
	    if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
	    my $count = $sth2->fetchrow;
	    if ( $count > 1 ) { 
		print "Fatal Error: more than 1 student ident record with same studnum!<br>\n";
		print "for student number: $studnum. Contact Administrator!<br>\n";
		die "Student Number duplicate in tscriptident: $studnum\n";

	    } elsif ( $count < 1 ) { # add a new ident record.
		$lastname = $dbh->quote( $lastname );
		$firstname = $dbh->quote( $firstname );
		$middlename = $dbh->quote( $middlename );
		$birthdate= $dbh->quote( $birthdate );
		$studentid = $dbh->quote( $studentid );
		if ( not $graddate or $graddate eq '0000-00-00' ) {
		    $graddate = 'NULL';
		} else {
		    $graddate = $dbh->quote( $graddate );
		}

		my $sth6 = $dbh->prepare("insert into tscriptident values (
                  $sql{default}, '$studnum', $lastname , $firstname, $middlename,
                   $birthdate, $studentid, $graddate )");
		$sth6->execute;
		if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }

	    } else { # we have an existing record. Check for a full match;
		$sth3->execute( $studnum );
		if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }
		my ($testlastname, $testfirstname, $testbirthdate) = $sth3->fetchrow;

		if ( $lastname ne $testlastname or 
		     $firstname ne $testfirstname or 
		     $birthdate ne $testbirthdate) { # We have a problem
		    print "<tr><td colspan=\"3\">\n";

		    my ($extraurl, $table, $id);
		    foreach my $tbl qw( student studentwd ) {
			my $sth6 = $dbh->prepare("select studid from $tbl where studnum = ?");
			$sth6->execute( $studnum );
			if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }
			$id = $sth6->fetchrow;
			if ( $id ) {
			    if ( $tbl eq 'studentwd' ) { $extraurl = '&tbl=wd';}
			    last; 
			}
		    }

		    print $lex{'Identity Mismatch'};

		    print ":$firstname|$lastname|$birthdate|($studnum) ";
		    print "$lex{Transcript}:$testfirstname|$testlastname|($testbirthdate)|\n";
		    print " <a href=\"../studed.pl?id=$id$extraurl\">$lex{Edit}</a>\n";
		    print "</td></tr></table></body></html>\n";
		    die "$lex{Transcript} Identity Mismatch for $firstname $lastname ($studnum)\n";
		}
	    }
	    # Now done with updating tscriptident records, if required.


	    # If using average method for mark, calculate it now and replace previous mark value.
	    if ( $calcavg eq 'Y' and not $markContainsLetters ) {
		$sth5->execute( $subjsec, $studnum ); # Get all mark field values.
		if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
		my ( $markcount, $marktotal );
		while ( my ( $id, $tmpmark ) = $sth5->fetchrow ) {
		    $markcount++;
		    $marktotal += $tmpmark;
		}

		if ( $markcount ) {
		    $mark = $fmt->format_number( $marktotal / $markcount );
		} else {
		    $mark = 0;
		}
	    }

	    my $letter; # letter grade.
	    if ( $markContainsLetters ) { # We have a 'letter' mark, copy into $letter 
		$letter = $mark;
	    } else { # find the matching letter for this numeric value..
		foreach my $threshold (reverse sort keys %markToLetter ) { # from large to small...
		    if ( $mark >= $threshold ) {
			$letter = $markToLetter{$threshold};
			last;
		    }
		}
	    }


	    # Check for duplicate course records - same studnum, same
	    # subjsec, same year, same term.  Skip if duplicate
	    $sth6 = $dbh->prepare("select id from tscriptdata
	     where subjectcode = ? and studnum = ? and schoolyear = ? and term = ?");
	    $sth6->execute( $subjsec, $studnum, $schoolyear, $term );
	    if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
	    my $id = $sth6->fetchrow;

	    # Setup the Term Description info from the tscript.conf.
	    my $term_desc = $term_desc{ $term_key{$term} };


	    if ( $id > 0 ) {

		if ( not $replacemode ) { # skip this student/subject; already done.
		    print "<tr style=\"color:red;\"><td>". $lex{Duplicate}. "! ";
		    print $lex{'Skipping'};
		    print " $firstname $lastname</td><td>$description</td><td>$mark</td></tr>\n";
		    next STUDENT; # skip to next record!

		} else { # replace existing record
		    print "<tr><td><b>$lex{'Update Record'}</b>:";
		    print " $firstname $lastname</td><td>$description</td><td>$mark</td></tr>\n";
		    my $sth6 = $dbh->prepare("update tscriptdata set
                      studnum = '$studnum', subjectcode = '$subjsec', 
                      subjecttext = '$description', subjectarea = '$area', score_mark = '$mark', 
                      score_letter = '$letter', score_diff = '$difficulty', 
                      schoolyear = '$schoolyear', crdate = now(), 
                      credit = '$credit', term = '$term', term_desc = '$term_desc' 
                    where id = ?");
		    $sth6->execute( $id );
		    if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
		}


	    } else { # no record found; insert one.

		# add the data record
		$sth6 = $dbh->prepare("insert into tscriptdata values (
                 $sql{default}, '$studnum','$subjsec','$description','$area','$mark',
                 '$letter','$difficulty','$schoolyear', now(), '$credit','$term','$term_desc')");
		$sth6->execute;
		if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }

		print "<tr><td>$firstname $lastname</td><td>$description</td>\n";
		print "<td>$mark ($letter)</td></tr>\n";

	    }


	} # End of this student / subject posting.

    } # End of this subject

    print "</table>\n";

    print "<p>[ <a href=\"$self\">$lex{Post} $lex{Subjects}</a> |\n";
    print " <a href=\"$reppage\">". $lex{'Report Card'}. "</a> ]\n";
    print "</body></html>\n";


    exit;

}
