# Library function for attendance scripts.

my %lex = ( 'Error' => 'Error',
	    'Transfer' => 'Transfer',
	    'Student' => 'Student',
	    'Date' => 'Date',
	    'Missing' => 'Missing',
	    'Value' => 'Value',

    );



# Update: Jan 27, 2011 - update findEnrollmentBlocks
# Update: April 5, 2009 - enddate, lexref parameters for calcMonthlyAttendance
# update to mkSchoolDays to obey enddate, too.

#  Date::Business needed for several functions.

# - mkSchoolDays($startdate, $enddate, $dbh) - Return %schooldays hash of
#   days in month school is open.  key is 'yyyy-mm' format and value
#   is schooldays in the month.

# Deprecated: Use SchoolDays instead.
# - mkSchoolMonths($startdate, $enddate) - return an array for yyyy-mm values for each month
#   of school year. It is 1 based so first (zeroth) entry in index is
#   blank. 

# - calcMonthlyEnrollment($studnum, $yearmonth, $active) calculate
#   student attendance by month. Returns: $daysenrolled, $active

# - calcMonthlyAttendance($studnum, $yearmonth, $periodsperday, $enddate, $lexiref) - calculate
#   the attendance for the student for the month. Return: daysabsent:timeslate
# Update: add a new optional parameter for date in month to stop at.

# - calcDaysBetween($startdate,$enddate, $dbh) - calculate the number of
#   school days between 2 dates (inclusive). Return: $daysbetween

# - findEnrollmentBlocks($studnum, $startdate, $enddate, $dbh ) - return time blocks student is
#   enrolled.


#----------------------
sub chkHomeroomAttEntry { # check that attendance has been done for a particular homeroom.
#----------------------

    # Passed: chkdate - date to check on, homeroom to check (and matching teacher), and dbh (datbase handle)
    my ($chkdate,$homeroom, $dbh) = @_;

    # Get Teacher of the homeroom.
    my $sth = $dbh->prepare("select userid from staff_multi where field_name = 'homeroom' and field_value = ?");
    if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }
    $sth->execute( $homeroom );
    my $userid = $sth->fetchrow;
    if ( not $userid ) { return undef; }

    # Now Get the periods for this date, and return them
    my %periods;
    my $sth = $dbh->prepare("select * from tattend where userid = ? and attdate = ?");
    $sth->execute( $userid, $chkdate );
    if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }
    while ( my $ref = $sth->fetchrow_hashref ) {
	my $periods = $ref->{periods};
	my @periods = split(',', $periods);
	foreach my $per ( @periods ) {
	    $periods{$per} = 1;
	}
    }
    my @periods = sort keys %periods;

    return @periods;

}


#---------------------
sub chkSubjsecAttEntry { # check that attendance has been done for a particular subject-section.
#---------------------

    # Passed: chkdate - date to check on, homeroom to check (and matching teacher), and dbh (datbase handle)
    my ($chkdate,$subjsec, $dbh) = @_;

    my $sth = $dbh->prepare("select teacher from subject where subjsec = ?");
    if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }
    $sth->execute( $homeroom );
    my $userid = $sth->fetchrow;
    if ( not $userid ) { return undef; }

    # Now Get the subjects for this date for this teacher, return them
    my %periods;
    my $sth = $dbh->prepare("select * from tattend where userid = ? and attdate = ?");
    $sth->execute( $userid, $chkdate );
    if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }
    while ( my $ref = $sth->fetchrow_hashref ) {
	my $periods = $ref->{periods};
	my @periods = split(',', $periods);
	foreach my $per ( @periods ) {
	    $periods{$per} = 1;
	}
    }
    my @periods = sort keys %periods;

    return @periods;



}




#---------------
sub mkSchoolDays { # Make a %schooldays hash containing yyyy-mm:schoolday vals 
#---------------

    use Number::Format qw(:all);
    use Time::JulianDay;

    # Loop through all months and create a hash %schooldays
    #  with yyyy-mm key and value containing schooldays in that month.
    #  Stop at month (and Day!) with $enddate.

    # Passed: $startdate, $enddate in iso format yyyy-mm-dd
    my ($startdate,$enddate, $dbh) = @_;

    my %schooldays = (); # structure to return

    my ($syr, $smo, $sda ) = split(/-/, $startdate);
    my ($eyr, $emo, $eda ) = split(/-/, $enddate);

    my $startjd = julian_day( split(/-/, $startdate) );
    my $endjd = julian_day( split(/-/, $enddate) );

    my $indexjd = $endjd; # start from the end

    while ( $indexjd >= $startjd ) {

	    my $monthendjd = $indexjd;

	    my @edate = inverse_julian_day( $monthendjd );
	    if ( length $edate[1] == 1 ) { $edate[1] = '0'. $edate[1]; } # fix month length
	    my $yearmonth = "$edate[0]-$edate[1]";
	    my $enddate = join('-',@edate);

	    my $monthstartjd = $monthendjd - ($edate[2] - 1); # set to first of the month.
	    if ( $monthstartjd < $startjd ) { $monthstartjd = $startjd; } # start
	    my $startdate = join('-', inverse_julian_day( $monthstartjd ));

	    my $opendays = calcDaysBetween( $startdate, $enddate, $dbh);
	    if ( $opendays == 0 ) { $opendays = 0; }

	    # Only need these other things if we want the dates in the month...??
	    #my $ref = {};
	    #$ref->{start} = $monthstartjd;
	    #$ref->{end} = $monthendjd;
	    #$ref->{days} = $opendays;
	    # $schooldays{$yearmonth} = $ref; # replaced
	    $schooldays{$yearmonth} = $opendays;

	    $indexjd = $monthstartjd - 1; # now set to previous month end

    } # end of loop

    return %schooldays;

} # End of mkSchoolDays



# don't use any more...
#-----------------
sub mkSchoolMonths {  # return array of current school months
#-----------------

    my ( $startdate, $enddate ) = @_;

    if ( not $enddate) { 
	print "<br>No End Date for mkSchoolMonths() in libattend.pl<br>\n";
	return ();
    } # return if no end value;
    if ( not $startdate) { 
	print "<br>No Start Date for mkSchoolMonths() in libattend.pl<br>\n";
	return ();
    } # return if no end value;



    # First find the months to end of current month from year start.
    my ($styear, $stmonth,$stday) = split /-/,$startdate;
    my ($endyear, $endmonth,$endday) = split /-/,$enddate;
    if (length($endmonth) == 1){ $endmonth = '0'.$endmonth;}
    if (length($startmonth) == 1){ $startmonth = '0'.$startmonth;}

    my $tmpyear = $styear;
    my $tmpmonth = $stmonth;
    my @schoolmonths = (); # local array only.

    #loop to put in year-month format in array
    push @schoolmonths, ''; # Blank first zeroth record [0]

    until ( $tmpmonth eq $endmonth and $tmpyear eq $endyear ){
	if ( length($tmpmonth) == 1 ){ $tmpmonth = "0".$tmpmonth; }
	push @schoolmonths, "$tmpyear-$tmpmonth";
	$tmpmonth++;
	if ($tmpmonth > 12){ $tmpmonth = '01'; $tmpyear++;}
	elsif (length($tmpmonth) == 1){ $tmpmonth = "0".$tmpmonth; }
    }
    push @schoolmonths, "$tmpyear-$tmpmonth";  # push in current month.

    return @schoolmonths;

}

#------------------------
sub calcMonthlyEnrollment { # calculate enrollment for months for 1 child
#------------------------

    use Time::JulianDay;

    # Passed: student number, start and end dates for period of interest.
    my ( $studnum, $startdate, $enddate, $dbh ) = @_; # same as findEnrollmentBlocks

    # find enrollment blocks for this time period, split up into months.
    my @enrolblocks = findEnrollmentBlocks( $studnum, $startdate, $enddate, $dbh );


    if ( not $enrolblocks[0] ) { # print "No Enrollment for $studnum<br>\n"; 
	return undef; 
    }


    my %enrolYM = (); # holds results $enrolYM{$yearmonth} -> {start},{end},{days}

    # Loop through each enrollment block
    foreach my $ref ( @enrolblocks ) {
	my $startdate = $ref->{start};
	my $enddate = $ref->{end};

	# print "Studnum:$studnum Start:|$startdate| End:|$enddate|<br>\n";

	my ($syr, $smo, $sda ) = split(/-/, $startdate);
	my ($eyr, $emo, $eda ) = split(/-/, $enddate);
	my $blockstartjd = julian_day( split(/-/, $startdate) );
	my $blockendjd = julian_day( split(/-/, $enddate) );
	my $indexjd = $blockendjd; # start from the end of the block

	while ( $indexjd >= $blockstartjd ) {

	    my $monthendjd = $indexjd;

	    my @edate = inverse_julian_day( $monthendjd );
	    if ( length $edate[1] == 1 ) { $edate[1] = '0'. $edate[1]; } # fix month length
	    my $yearmonth = "$edate[0]-$edate[1]";
	    my $enddate = join('-',@edate);

	    my $monthstartjd = $monthendjd - ($edate[2] - 1); # this is the first of the month.
	    if ( $monthstartjd < $blockstartjd ) { $monthstartjd = $blockstartjd; } # start of block
	    my $startdate = join('-', inverse_julian_day( $monthstartjd ));

	    my $enroldays = calcDaysBetween( $startdate, $enddate, $dbh);
	    if ( $enroldays == 0 ) { $enroldays = 0; }


	    # Check if an entry already exists
	    if ( $enrolYM{$yearmonth} ) { # update total
		my $prevdays = $enrolYM{$yearmonth}->{days};
		my $newdays = $prevdays + $enroldays;
		$enrolYM{$yearmonth}->{days} = $newdays;
		if ( $enrolYM{$yearmonth}->{start} > $monthstartjd ) {
		    $enrolYM{$yearmonth}->{start} = $monthstartjd;
		}
		if ( $enrolYM{$yearmonth}->{end} < $monthendjd ) {
		    $enrolYM{$yearmonth}->{end} = $monthendjd;
		}

	    } else { # make a new entry
		my $ref = {};
		$ref->{start} = $monthstartjd;
		$ref->{end} = $monthendjd;
		$ref->{days} = $enroldays;
		$enrolYM{$yearmonth} = $ref;
	    }

	    $indexjd = $monthstartjd - 1; # now set to previous month end

	} # end of this enrollment block
    } # end of enrollment blocks


    return \%enrolYM;

} # End of NEW calcMonthlyEnrollment 



#------------------------
sub calcMonthlyAttendance { # calculate absences,lates for 1 month for 1 child
#------------------------

    my ( $studnum, $yearmonth, $periodsperday, $enddate, $lexref, $dbh ) = @_;

    # $enddate added for the final date in month to use, if present);
    # Previously it always did the entire month...
    my %lexi = %{ $lexref };

    # yearmonth will have yyyy-mm format
    $yearmonth =~ s/-//; # remove single dash

    my $sth = $dbh->prepare("select absdate, reason from attend 
     where studentid = ? and 
     extract(year_month from absdate) = ?
     order by absdate,period");
    if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }
    $sth->execute( $studnum, $yearmonth );
    if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }
    # my $rows = $sth->rows;
    # print "CalcMonthlyAttendance: SN: $studnum YM: $yearmonth Rows: $rows<br>\n";

    if ( not $lexi{Absent} ) { 
	print "Missing Lex for Absent"; die "Missing Lex for Absent"; 
    }


    my $absentcount = 0;
    my $latecount = 0;

    my ( $eyr, $emo, $eda ) = split /-/, $enddate;
    if ( length( $emo ) == 1 ) { $emo = '0'. $emo; }
    if ( length( $eda ) == 1 ) { $eda = '0'. $eda; }
    $enddate = $eyr . $emo. $eda;

    while ( my ( $absdate, $reason ) = $sth->fetchrow ){
	$absdate =~ s/-//g; # strip hyphens
	if ( $enddate and $absdate > $enddate ) {
	    #print "Abs:$absdate  End:$enddate<br>\n";
	    last; }
	if ( $reason =~ m/$lexi{Absent}/ ) { $absentcount++; }
	if ( $reason =~ /$lexi{Late}/ ) { $latecount++; }
    }

    my $absentdays;
    if ( $periodsperday ) { 
	$absentdays = round($absentcount / $periodsperday, 2); #convert periods into days
	#$absentdays = $absentcount / $periodsperday;
    } else {
	$absentdays = -1;  # Flag error condition ??
    }

    return "$absentdays:$latecount";
}



#------------------
sub calcDaysBetween {
#------------------
    # this calculates the number of schooldays in a date range
    #  taking school holidays into account.
    # Requires: $dbh, Date::Business

    use Date::Business;

    my ($startdate, $enddate, $dbh) = @_;

    # Normalize dates, and remove hyphens for Date::Business.
    if ( $startdate =~ /-/ ) { # it contains hyphens
	my @temp = split /-/, $startdate;
	if ( length( $temp[1] ) == 1 ) { $temp[1] = '0'. $temp[1]; }
	if ( length( $temp[2] ) == 1 ) { $temp[2] = '0'. $temp[2]; }
	$startdate = join('',@temp);
    }

    if ( $enddate =~ /-/ ) { # it contains hyphens, too.
	my @temp = split /-/, $enddate;
	if ( length( $temp[1] ) == 1 ) { $temp[1] = '0'. $temp[1]; }
	if ( length( $temp[2] ) == 1 ) { $temp[2] = '0'. $temp[2]; }
	$enddate = join('',@temp);
    }

    my $end = new Date::Business(DATE=>$enddate);
    my $start = new Date::Business(DATE=>$startdate);
    my $schooldays = $end->diffb($start,'prev','next');
    $schooldays++;
    # We now have the number of school days in current term
    # excluding holidays/pd days.

    # Now find holidays
    # Find number of days school not in session this period.
    my $sth = $dbh->prepare("select id, date, dayfraction from dates 
     where to_days(date) >= to_days('$startdate') and 
     to_days(date) <= to_days('$enddate')");
    $sth->execute;
    if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }
    my $holidays;
    while ( my ($id, $date, $dayfraction) = $sth->fetchrow ) {
	if ( not $dayfraction ) {
#	    print "<h3>$lex{Error}: $date $lex{Missing} $lex{Value}</h3>\n";
	    $dayfraction = 1;
	}
	$holidays += $dayfraction;
    }

    # don't round: my $tmpdays = format_number($schooldays - $holidays, 2, 2);
    my $tmpdays = $schooldays - $holidays;

    return $tmpdays;
}


#-----------------------
sub findEnrollmentBlocks {
#-----------------------

    # Return a list of hash pointers to enrollment blocks.

    my ( $studnum, $startdate, $enddate, $dbh ) = @_;

    my @enrollment = ();

    # Check for Transfer Records, this time period
    my $sth = $dbh->prepare("select count(*) from transfer where studnum = ? and 
      to_days(date) >= to_days('$startdate') and 
      to_days(date) <= to_days('$enddate')");
    $sth->execute( $studnum );
    if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr;}
    my $count = $sth->fetchrow;

    if ( not $count ) {

	# check to see if active, during this time period
	# Check for next following record.
	my $sth = $dbh->prepare("select type from transfer where studnum = ? and 
          to_days(date) > to_days('$enddate') order by date");
	$sth->execute( $studnum );
	if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr;}
	my $followingtype = $sth->fetchrow;

	if ( $followingtype ) {
	    if ( $followingtype ne 'withdraw' ) { # they weren't enrolled
		return undef;
	    } else {
		my $hashref = {};
		$hashref->{start} = $startdate;
		$hashref->{end} = $enddate;
		push @enrollment, $hashref;
		return @enrollment;
	    }
	}
	    
	# Check for any previous enrollment record.
	$sth = $dbh->prepare("select type from transfer where studnum = ? and 
         to_days(date) < to_days('$startdate') order by date desc");
	$sth->execute( $studnum );
	if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr;}
	my $leadingtype = $sth->fetchrow;

	if ( $leadingtype ) {
	    if ( $leadingtype eq 'withdraw' ) { # they aren't enrolled anymore
		return undef;
	    } else {
		my $hashref = {};
		$hashref->{start} = $startdate;
		$hashref->{end} = $enddate;
		push @enrollment, $hashref;
		return @enrollment;
	    }
	}



#~~
#	my $sth = $dbh->prepare("select lastname, firstname, grade from student where studnum = ?"); 
#	$sth->execute( $studnum );
#	my ( $lastname, $firstname, $grade ) = $sth->fetchrow;
#	print "No Transfer Records: $firstname $lastname ($studnum) Grade: $grade<br>\n";
#	return undef;

	$sth = $dbh->prepare("select count(*) from student where studnum = ?"); 
	$sth->execute($studnum);
	if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr;}
	my $count = $sth->fetchrow;
	if ( not $count ) { return undef; } # not active, return undef
    
	# we have an active student, return full block enrolled
	my $hashref = {};
	$hashref->{start} = $startdate;
	$hashref->{end} = $enddate;
	push @enrollment, $hashref;
	return @enrollment;

    }
    

    # We have enrollment records...
    # Read enrollment changes from the transfer table.
    $sth = $dbh->prepare("select id, date, type from transfer 
      where studnum = ? and to_days(date) >= to_days('$startdate')
      and to_days(date) <= to_days('$enddate')
      order by date, type");

    $sth->execute( $studnum );
    if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr;}

    # put records into enrolblock data structure.
    my @recids = ();
    my %enrolblock = ();
    my ($maxid, $minid); # record ids

    while ( my ($id, $date, $type) = $sth->fetchrow ) {

	# Transfer Records for this student.
	# print "ID:$id  Date:$date  Type:$type<br>\n"; # transfer records

	if ( $id > $maxid ) { $maxid = $id; }
	$enrolblock{$id} = { 'date' => $date, 'type' => $type };
	push @recids, $id;
    }

    # check the start entry
    my $startid = $recids[0];
    if ( $enrolblock{$startid}->{type} eq 'withdraw' ) { # add new start or remove entry.

	if ( $enrolblock{$startid}->{date} eq $startdate ) { # remove entry
	    delete $enrolblock{$startid};
	    shift @recids;
	    if ( not @recids ) {
		return undef;
	    }
	} else { # add a new entry
	    $maxid += 10; # get a new larger value.
	    $enrolblock{$maxid} = { 'date' => $startdate, 'type' => 'enrol' };
	    unshift @recids, $maxid;
	}
    }


    # check the end entry
    my $endid = $recids[-1];
    if ( $enrolblock{$endid}->{type} ne 'withdraw' ) { # add new end or remove entry.
	
	if ( $enrolblock{$endid}->{date} eq $enddate ) { # remove entry
	    delete $enrolblock{$endid};
	    my $end = pop @recids;
	    if ( not @recids ) {
		return undef;
	    }
	    
	} else {
	    $maxid += 10; # get a new larger value.
	    $enrolblock{$maxid} = { 'date' => $enddate, 'type' => 'withdraw' };
	    push @recids, $maxid;
	}
    }


    # Record ID's printout
    #	print "<p>Recs:";
    #	foreach my $r (@recids) { print "$r "; }
    #	print "</p>\n";

    # Check Enrollment block data structure
    # print "Studnum:$studnum<br>\n";
    # foreach my $id ( @recids ) { print "Id:$id  Date:$enrolblock{$id}->{date} Type:$enrolblock{$id}->{type} <b>||</b> \n"; }
    # print "<br><br>\n";

    # run through the structure checking for matching pairs
    my $first = 1;
    my ($prevtype, $currtype);


    foreach my $id ( @recids ) {

	if ( $first ) {
	    # Check that currtype is 'enrol'
	    if ( $enrolblock{$id}->{type} eq 'withdraw' ) { # error
		print "<p>$lex{Transfer} $lex{Error} - $lex{Student}:$studnum ID:$id ";
		print "$lex{Date}:$enrolblock{$id}->{date}</p>\n";
		return undef;
	    }
	    $first = 0;
	    $currtype = 'enrol';
	    
	} else {
	    $prevtype = $currtype; 
	    $currtype = $enrolblock{$id}->{type};
	    
	    if ( ( $currtype eq 'withdraw' and $prevtype eq 'withdraw' ) or 
		 ( $currtype ne 'withdraw' and $prevtype ne 'withdraw' ) ) { # both same values
		
		print "<p>$lex{Transfer} $lex{Error} - $lex{Student}:$studnum ID:$id ";
		print "$lex{Date}:$enrolblock{$id}->{date}</p>\n";
		return undef;
	    }
	}

    } # end of enrollment records loop.

    
    while ( @recids ) { # put values into array structure

	my $startid = shift @recids;
	my $endid = shift @recids;

#	print "Start:$startid - $enrolblock{$startid}->{type} $endid End: $enrolblock{$endid}->{type}<br>\n";

	# Check values again.
	if ( ( $enrolblock{$startid}->{type} eq 'withdraw' ) or 
	     ( $enrolblock{$endid}->{type} ne 'withdraw' ) ) { # error

	    print "<p>$lex{Transfer} $lex{Error} - $lex{Student}:$studnum ID:$id ";
	    print "$lex{Date}:$enrolblock{$id}->{date}</p>\n";
	    return undef;
	}

	my $hashref = {};
	$hashref->{start} = $enrolblock{$startid}->{date};
	$hashref->{end} = $enrolblock{$endid}->{date};
	push @enrollment, $hashref;
    }


    # Show Result:
    #    foreach my $bl ( @enrollment ){
    #	print "SN:$studnum  ST:$bl->{start} ";
    #   	print "ED:$bl->{end}<br>\n";
    # }
 
    return @enrollment;

} # End of findEnrollmentBlocks


# Return true.
1;
