#! /usr/bin/perl
#  Copyright 2001-2013 Leslie Richardson

#  This file is part of Open Admin for Schools.

#  Open Admin for Schools is free software; you can redistribute it 
#  and/or modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2 of 
#  the License, or (at your option) any later version.


my %lex = ('Main' => 'Main',
	   'Student Rubrics' => 'Student Rubrics',
	   'View/Download Student Rubrics' => 'View/Download Student Rubrics',
	   'Area of Development' => 'Area of Development',
	   'Target Skill' => 'Target Skill',
	   'View Log File' => 'View Log File',
	   'Name' => 'Name',
	   'Page' => 'Page',
	   'Comparison of objectives of the PPP and Regular Curriculum Objectives/Outcomes' =>
	     'Comparison of objectives of the PPP and Regular Curriculum Objectives/Outcomes',
	   'No Records Found' => 'No Records Found',
	   'Records' => 'Records',
	   'School' => 'School',
	   'Please Log In' => 'Please Log In',
	   'Student Outcome Rubric' => 'Student Outcome Rubric',
	   'Date' => 'Date',
	   'Gender' => 'Gender',
	   'Age' => 'Age',
	   'Student Team' => 'Student Team',
	   'Comparison of objectives of the PPP and' => 'Comparison of objectives of the PPP and',
	   'Regular Curriculum Objectives/Outcomes' => 'Regular Curriculum Objectives/Outcomes',
	   'Current Level of Performance' => 'Current Level of Performance',
	   
	   'Grade' => 'Grade',
	   'Continue' => 'Continue',

	   );

my @desc = ('1\\\\Much less\\\\than expected', '2\\\\Somewhat less\\\\than expected',
'3\\\\Expected', '4\\\\Somewhat more\\\\than expected', '5\\\\Much more\\\\than expected');

my @fieldlist = ( id, studnum, dev_area, target_skill,
  perf_lvl_current, currdate, perf_lvl_mless, perf_lvl_swless, perf_lvl_expect, 
  perf_lvl_swmore, perf_lvl_mmore,
  objective_var, evaluator, ye_date, attain_level);

my $self = 'rptrubric.pl';

use DBI;
use CGI;
use CGI::Session;

# Read config variables
eval require "ppp.conf";
if ( $@ ) {
    print $lex{Error}. " $@<br>\n";
    die $lex{Error}. " $@\n";
}

eval require "$globdir/global.conf";
if ( $@ ) {
    print $lex{Error}. " $@<br>\n";
    die $lex{Error}. " $@\n";
}

eval require "liblatex.pl";
if ( $@ ) {
    print $lex{Error}. " $@<br>\n";
    die $lex{Error}. " $@\n";
}


my ($sec, $min, $hour, $mday, $mon, $year, $wday, $yday, 
 $iddst) = localtime(time);
$year = $year + 1900;
$mon++; 
$wday++;
my $currdate = "$dow[$wday], $month[$mon] $mday, $year";
my $currsdate = "$year-$mon-$mday";

my $q = new CGI;
my %arr = $q->Vars;


# Basic Constants
my $logfile = "pdflog$$.txt";
my $shortname = "rubric$$";
my $filename = "$shortname.tex";

my $dsn = "DBI:$dbtype:dbname=$dbase";
my $dbh = DBI->connect($dsn,$user,$password);


# Get Session
my $session = new CGI::Session("driver:$dbtype;serializer:FreezeThaw",
 undef,{Handle => $dbh}) or die CGI::Session->errstr;

if ( not $session->param('logged_in') ){
    $userid = $session->param('userid');
    print $q->header;
    print "$doctype\n<html><head>\n";
    print "<link rel=\"stylesheet\" href=\"$iepcss\" type=\"text/css\"></head><body>\n";
    print "[ <a href=\"$iephome\">". $lex{Main}. "</a> ]\n";
    print "<form action=\"login.pl\" method=\"post\" style=\"display:inline;\">\n";
    print "<input type=\"hidden\" name=\"userid\" value=\"$userid\">\n";
    print "<input type=\"hidden\" name=\"script\" value=\"$self\">\n";
    print "<input type=\"submit\" value=\"". $lex{'Please Log In'}. "\">\n";
    print "</form></center></body></html>\n";
    die;
}

my $userid = $session->param('userid');
my $schooldbase = $session->param('schooldbase');

# Reset Login time
my $duration = $session->param('duration');
$session->expire('logged_in', $duration );

print $session->header;

# Get User Name
my $sth = $dbh->prepare("select lastname, firstname from educator where userid = ?");
$sth->execute( $userid );
if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
my ( $lastname, $firstname ) = $sth->fetchrow;

my $title = $lex{'Student Rubrics'};
print "$doctype\n<html><head><title>$title</title>
<link rel=\"stylesheet\" href=\"$iepcss\" type=\"text/css\">
$chartype\n</head><body>\n";
print "<p>[ <a href=\"$iephome\">$lex{Main}</a> ] $firstname $lastname</p>\n";

print "<h1>$title</h1>\n";


if ( not $arr{page} ) {
    showStartPage();
}
delete $arr{page};

# Get Sorted Student Liset
my $sth = $dbh->prepare("select lastname, firstname from special where studnum = ?");
my %students;
foreach my $studnum ( keys %arr ) {
    $sth->execute( $studnum );
    if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
    my ($lastname, $firstname) = $sth->fetchrow;
    $students{"$lastname$firstname$studnum"} = $studnum;
}

#my $sdb = $dbh->quote( $schooldbase );
#my $select = "and school = $sdb ";


if ( not %students ) {
    print "<h1>$lex{'No Records Found'}</h1>\n";
    print "</body></html>\n";
    die $lex{'No Records Found'}."\n";
}



unless ( open(TEX,">$filename") ) {
    print "Can't open tex file";
    die "Can't open tex file";
}

my $papersize;
if ( $defaultpapersize ) {
    $papersize = $defaultpapersize;
} else {
    $papersize = 'letterpaper';
}

print TEX "\\documentclass[12pt, $papersize ]{article}
\\usepackage{times,colortbl, array,inputenc}
\\inputencoding{latin1}
\\pagestyle{empty}
\\setlength{\\textwidth}{7.5in}
\\setlength{\\textheight}{10.3in}
\\setlength{\\hoffset}{-1in}
\\setlength{\\voffset}{-1.4in}
\\setlength{\\parindent}{0pt}
\\setlength{\\extrarowheight}{2pt}
\\setlength{\\tabcolsep}{3.5pt}\n";

print TEX "\\begin{document}\n";

my $page = 0;

my $sth1 = $dbh->prepare("select userid, jobtitle from team where studnum = ?");
my $sth2 = $dbh->prepare("select lastname, firstname, school, birthdate, sex, school 
 from special where studnum = ?");

# Student Loop
foreach my $key ( sort keys %students ) {

    my $studnum = $students{$key};

    # Get Student Name, etc.
    $sth2->execute($studnum);
    if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }
    my ($lastname, $firstname, $school, $birthdate, $sex, $school) = $sth2->fetchrow;

    my $age = calcAge($birthdate, $currsdate);
    my ($y,$m) = split(/:/,$age);
    $age = "$y\Y$m\M";

    # Rubrics Loop
    my $sth = $dbh->prepare("select * from eval_rubric where studnum = ?");
    $sth->execute( $studnum);
    if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }

    while ( my $ref = $sth->fetchrow_hashref ) {

	my %sr = %$ref;

	foreach my $key ( keys %sr ) {
	    ($sr{$key}) = latex_filter( $sr{$key} );
	}
   
	# Title
	print TEX "\\begin{tabular}{p{11cm}p{7cm}}";
	print TEX "\\parbox{8.9cm}{ \\rule[-6pt]{0pt}{14pt} $divisionname \\\\";
	print TEX "{\\LARGE ". $lex{'Student Outcome Rubric'}. "} }";

	print TEX "& \\parbox{8.9cm}{{\\bf $firstname $lastname} \\\\";
	print TEX $lex{Date}. ": $currdate\\\\". $lex{Gender}. ": $sex ". $lex{Age}. ": $age }\\\\";
	print TEX "\\end{tabular}\n\n";


	print TEX "\\begin{tabular}{|p{3cm}|p{7.7cm}|p{7.7cm}|}\\hline\n";

	# Area of Development / Target Skill
	print TEX "\\rowcolor[gray]{0.95} & \\parbox[t]{7.6cm}{{\\footnotesize ";
	print TEX $lex{'Area of Development'};
	print TEX "}\n\n{\\bf $sr{dev_area}}}";
	print TEX "\\rule[-10mm]{0mm}{13mm} & \\parbox{7.6cm}{\\footnotesize ";
	print TEX $lex{'Target Skill'};
	print TEX "}\n\n{\\bf $sr{target_skill}}\\\\\\hline\n";

	# Current Level of Performance
	print TEX "\n\\parbox[t]{2.8cm}{\\raggedright ". $lex{'Current Level of Performance'}. "} &";
	print TEX "\\multicolumn{2}{l|}{\\rule[-20mm]{0mm}{20mm}";
	print TEX "\\parbox[t]{15.3cm}{$sr{perf_lvl_current}} }\\\\ \\hline\n"; 

	# Much Less
	print TEX "\n\\parbox[t]{2.8cm}{\\raggedright 1\\\\Much less than expected} &";
	print TEX "\\multicolumn{2}{l|}{\\rule[-20mm]{0mm}{20mm}";
	print TEX "\\parbox[t]{15.3cm}{$sr{perf_lvl_mless}} }\\\\ \\hline\n"; 

	# SW Less
	print TEX "\n\\parbox[t]{2.8cm}{\\raggedright 2\\\\Somewhat less than expected} &";
	print TEX "\\multicolumn{2}{l|}{\\rule[-20mm]{0mm}{20mm}";
	print TEX "\\parbox[t]{15.3cm}{$sr{perf_lvl_swless}} }\\\\ \\hline\n"; 

	# Expected
	print TEX "\n\\parbox[t]{2.8cm}{\\raggedright 3\\\\Expected} &";
	print TEX "\\multicolumn{2}{l|}{\\rule[-20mm]{0mm}{20mm}";
	print TEX "\\parbox[t]{15.3cm}{$sr{perf_lvl_expect}} }\\\\ \\hline\n"; 

	# SW More
	print TEX "\n\\parbox[t]{2.8cm}{\\raggedright 4\\\\Somewhat more than expected} &";
	print TEX "\\multicolumn{2}{l|}{\\rule[-20mm]{0mm}{20mm}";
	print TEX "\\parbox[t]{15.3cm}{$sr{perf_lvl_swmore}} }\\\\ \\hline\n"; 

	# Much More
	print TEX "\n\\parbox[t]{2.8cm}{\\raggedright 5\\\\Much more than expected} &";
	print TEX "\\multicolumn{2}{l|}{\\rule[-20mm]{0mm}{20mm}";
	print TEX "\\parbox[t]{15.3cm}{$sr{perf_lvl_mmore}} }\\\\ \\hline\n"; 
	print TEX "\\end{tabular}\n\n\\bigskip\n\n";

	print TEX "\\begin{tabular}{p{7cm}p{11cm}}";
	print TEX "\\parbox[t]{6.9cm}{ {\\small ";
	print TEX "Comparison of objectives of the IEP and\\\\";
	print TEX "Regular Curriculum Objectives/Outcomes\\\\}  }";

	print TEX "&  \\parbox[t]{10.9cm}{ \\bf $sr{objective_var} } \\\\\n";
	print TEX "\\end{tabular}\n\n";

	print TEX "\\bigskip\n\n";

	# Team Info
	print TEX "\\begin{tabular}{|p{8cm}|p{8cm}|}\\hline\n";
	print TEX "\\multicolumn{2}{|c|}{\\bf ". $lex{'Student Team'}. "} \\\\ \\hline\n";

	#print TEX "\\rowcolor[gray]{0.95}Name & Position\\\\\\hline\n";

	$sth1->execute($studnum);
	if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
	my $tmcount = 1;
	while (my ($userid, $jobtitle) = $sth1->fetchrow) {
	    
	    if ($jobtitle ne 'Classroom Teacher' and $jobtitle ne 'Teacher Assistant') { next; }

	    if ($tmcount % 3 == 0){ 
		print TEX "\\end{tabular}\n\n";
		print TEX "\\begin{tabular}{|p{8cm}|p{8cm}|}\\hline\n";
	    }

	    print TEX "$userid & $jobtitle\\\\\\hline\n";
	    $tmcount++;
	
	}
	print TEX "\\end{tabular}\n\n\\bigskip\n";

	# Final Line - evaluator, date, attainment
	print TEX "\\bigskip\n\n";
	print TEX "\\fbox{{\\scriptsize Evaluator} {\\bf $sr{evaluator}}\\quad";
	print TEX "{\\scriptsize Year End Date} {\\bf $sr{ye_date}}\\quad";
	print TEX "{\\scriptsize Attainment Level} {\\bf $sr{attain_level}}}\n\n";
	
	print TEX "\\newpage\n\n";
	$page++;
    } # end of rubric loop
} # end of student loop

print TEX "\n\\end{document}";
close TEX;

system("$pdflatex $filename >$logfile");
system("mv $shortname.pdf $downloaddir");
system("mv $logfile $downloaddir");
system("rm $shortname.*");

# print rest of html page
print "<h2><a href=\"$webdownloaddir/$shortname.pdf\">";
print "$lex{'View/Download Student Rubrics'}</a></h2>\n";

print "[ <a href=\"$iephome\">$lex{Main}</a> |\n";
print "<a href=\"$webdownloaddir/$logfile\">$lex{'View Log File'}</a> ]\n";
print "</body></html>\n";


#----------
sub calcAge {
#----------
    # Passed (birthdate, $currdate)
    my $birthDate = shift;
    my $currentDate = shift;

    my ($byear,$bmonth,$bday) = split /-/,$birthDate;
    my ($cyear,$cmonth,$cday) = split /-/,$currentDate;
    my $age = $cyear - $byear;
    my $month = $cmonth - $bmonth;

    if ($cmonth < $bmonth){
	$month = $month + 12;
	if ($cday < $bday){ $month--;}
	$age--;
    }
    elsif ($cmonth == $bmonth and $cday < $bday){
	$age--; 
	$month = 11;
    } elsif ($cmonth > $bmonth and $cday < $bday) {
	$month--;
    }

    return "$age:$month";
}



#----------------
sub showStartPage {
#----------------

    my $sth = $dbh->prepare("select lastname, firstname, studnum, grade from special
     where school = ? order by lastname, firstname"); # $select is set by session code above
    $sth->execute( $schooldbase );
    if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }

    # Start the Form.
    print "<form action=\"$self\" method=\"post\">\n";
    print "<input type=\"hidden\" name=\"page\" value=\"1\">\n";

    print "<input type=\"submit\" value=\"$lex{Continue}\">\n";

    print "<table cellpadding=\"4\" cellspacing=\"0\" border=\"0\" style=\"text-align:left;\">\n";

    while ( my ($lastname, $firstname, $studnum, $grade ) = $sth->fetchrow ){
	print qq{<tr><td class="la"><input type="checkbox" name="$studnum" value="1"> <b>$lastname</b>, $firstname ($studnum)</td></tr>\n};
    }
	
    print "</table>\n";
    print "<input type=\"submit\" value=\"$lex{Continue}\">\n";

    print "</form></body></html>\n";

    exit;

} # End of showStartPage

