#! /usr/bin/perl
#  Copyright 2001-2018 Leslie Richardson

#  This file is part of Open Admin for Schools.

#  Open Admin for Schools is free software; you can redistribute it 
#  and/or modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2 of 
#  the License, or (at your option) any later version.

use DBI;
use CGI;

my %lex = ('Attendance' => 'Attendance',
	   'Perfect Attendance' => 'Perfect Attendance',
	   'Report' => 'Report',
	   'Not Found' => 'Not Found',
	   'Name' => 'Name',
	   'Student Number' => 'Student Number',
	   'Start Date' => 'Start Date',
	   'End Date' => 'End Date',
	   'Error' => 'Error',
	   'Grade' => 'Grade',
	   'Homeroom' => 'Homeroom',

	   );

my $self = 'rptattperf.pl';

# Get passed date for perfect attendance (inclusive)

# Read config variables
eval require "../../etc/admin.conf";
if ( $@ ) {
    print $lex{Error}. ": $@<br>\n";
    die $lex{Error}. ": $@\n";
}

# Get Date
my @tim = localtime(time);
my $year = @tim[5] + 1900;
my $month = @tim[4] + 1;
my $day = @tim[3];
my $currdate = "$year-$month-$day";
my $currdate1 = "@month[$month] $day, $year";

my $q = new CGI;
my %arr = $q->Vars;
print $q->header( -charset, $charset );

my $startdate = $arr{startdate};
my $enddate = $arr{enddate};

my $dsn = "DBI:$dbtype:dbname=$dbase";
my $dbh = DBI->connect($dsn,$user,$password);
$dbh->{mysql_enable_utf8} = 1;

# Print Page Header
my $title = qq{$lex{'Perfect Attendance'} $lex{Report}};
print qq{$doctype\n<html><head><title>$title</title>\n};
print qq{<link rel="stylesheet" href="$css" type="text/css">};
print qq{$chartype\n</head><body>\n};
print qq{[ <a href="$attpage">$lex{Attendance}</a> ]\n};

if ( not $startdate) { $startdate = $schoolstart; }
if ( not $enddate) { $enddate = $currdate; }

print qq{<h1>$title</h1>\n};

print qq{<h3>$lex{'Start Date'}:$startdate };
print qq{$lex{'End Date'}: $enddate</h3>\n};

# @ignore_reason should be defined in /etc/admin.conf to give reasons to ignore.
# It lists the attend reason indexes to ignore...
my $reason_string;
print qq{<h3>Reasons ignored - \n};
foreach my $index (@ignore_reason) {
    print $attend[$index],', ';
    $reason_string .= "and reason != '$attend[$index]' ";
}
print qq{</h3>\n};




print qq{<table cellpadding="3" cellspacing="0" border="1" style="margin:1em;">\n};
print qq{<tr><th>$lex{Name} ($lex{'Student Number'})</th><th>$lex{Grade}</th><th>$lex{Homeroom}</th></tr>\n};

my $sth = $dbh->prepare("select lastname, firstname, studnum, grade, homeroom from student 
  order by lastname, firstname");
$sth->execute;
if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }

my $noperfect = 1;  # flag in case no perfect attendance

while ( my ($lastname, $firstname, $studnum, $grade, $homeroom ) = $sth->fetchrow ) {

    # Now do the query to find student numbers.
    $sth1 = $dbh->prepare("select count(*) from attend where
       studentid = ? and
       to_days(attend.absdate) >= to_days('$startdate') and 
       to_days(attend.absdate) <= to_days('$enddate')
       $reason_string");
    $sth1->execute( $studnum );
    if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }
    my $count = $sth1->fetchrow;
    if ( $count < 1) {
	$noperfect = 0;
	print qq{<tr><td><b>$lastname</b>, $firstname ($studnum)</td><td>$grade</td><td>$homeroom</td></tr>\n};
    }
}

# In case of no perfect attendance...
if ( $noperfect ) {
    print qq{<tr><td><b>$lex{'Perfect Attendance'} $lex{'Not Found'}</b></td></tr>\n};
}


print qq{</table><p>[ <a href="$attpage">$lex{Attendance}</a> ]</p>\n};
print qq{</body></html>\n};
