#!/usr/bin/perl
#  Copyright 2001-2017 Leslie Richardson

#  This file is part of Open Admin for Schools.


my %lex = ('Main' => 'Main',
	   'Export' => 'Export',

	   'Records' => 'Records',
	   'Combine failed on input' => 'Combine failed on input',
	   'Download CSV File' => 'Download CSV File',
	   'CSV (Comma Separated Value) can be directly imported into Open Office' =>
	     'CSV (Comma Separated Value) can be directly imported into Open Office',
	   'Note' => 'Note',
	   'Error' => 'Error',

	   );


use DBI;
use CGI;
use Text::CSV_XS;


eval require "../admin.conf";
if ( $@ ) {
    print $lex{Error}. " $@<br>\n";
    die $lex{Error}. " $@\n";
}

my $q = CGI->new;
print $q->header( -charset, $charset );

my $title = "Export Students for Gmail";
print qq{$doctype\n<html><head><title>$title</title>\n};
print qq{<link rel="stylesheet" type="text/css" href="$css">\n};
print qq{$chartype\n</head><body>\n};
print qq{[ <a href="$homepage">$lex{Main}</a> |\n};
print qq{<a href="$exppage">$lex{Export}</a> ]\n};

print qq{<h1>$title</h1>\n};


my (%mastername, %data);
my $collisioncount = 1;

foreach my $dbase ( keys %alldbase ) {
    
    my $dsn = "DBI:$dbtype:dbname=$dbase";
    my $dbh = DBI->connect($dsn,$user,$password);
    $dbh->{mysql_enable_utf8} = 1;

    my $sth = $dbh->prepare("select firstname, lastname, password, initial, grade, studnum, birthdate from student");
    $sth->execute;
    while ( my $sref = $sth->fetchrow_hashref ) {
	%r = %$sref;

	my @fn = split(/\s+/, $r{firstname});
	$r{firstname} = $fn[0]; # only use the first name of first name.
	$r{firstname} =~ s/^\s+//;
	$r{firstname} =~ s/\s+$//;
	$r{firstname} =~ s/\W|-//g; # remove any non letters or dashes.

	$r{lastname} =~ s/^\s+//;
	$r{lastname} =~ s/\s+$//;
	my @ln = split(/\s+/, $r{lastname});
	$r{lastname} = $ln[0];
	$r{lastname} =~ s/\W|-//g;
	
	$r{initial} =~ s/^\s+//;
	$r{initial} =~ s/\s+$//;

	
	$sref->{lastname} = $r{lastname};
	$sref->{firstname} = $r{firstname};
	
	
	print "FN:$r{firstname}:LN:$r{lastname}:MN:$r{initial}:\n";
	
	my $name = "$r{lastname},$r{firstname}";

	if ( $mastername{$name} ) { # we have a collision.

	    # This is already added.
	    my ($db,$studnum) = split('-', $mastername{$name});
	    my $ref = $data{$db}{$studnum};
	    my %cr = %$ref;

	    if ( $cr{birthdate} eq $r{birthdate} and $r{birthdate} ) { # ok, same person
		next;
	    }
	    if ( $dbase eq 'chieftaylor' and $db eq 'pewasenakwan' ) { # remove pewa record; put in chieftaylor
		$mastername{$name} = "$dbase-$r{studnum}";
		$data{$dbase}{$r{studnum}} = $sref; # save everything;
		next;
	    }

	    # If here, then we have to create 2 mastername records, one for each student;
	    # First the original in %mastername;
	    my $originalname = $name;

	    my @mn = split(/\s+/, $r{initial}); # split on spaces.
	    my $firstmiddlename = $mn[0]; # only take the first one.

	    
	    # Now the second student, not yet in mastername hash
	    my @cmn = split(/\s+/, $cr{initial}); # split on spaces.
	    my $secondmiddlename = $cmn[0]; # only take the first one.

	    if ( $firstmiddlename eq $secondmiddlename and $firstmiddlename ) { # same student;
		print qq{\n Same Middle Name! $firstmiddlename - $r{firstname} $r{lastname}\n\n};
	    }

	    # reset first name
	    my $newname = $name. ",$firstmiddlename";
	    $mastername{$newname} = $mastername{$name};
	    delete $mastername{$name};

	    # Add Second name to mastername;
	    my $newname = $name. ",$secondmiddlename";
	    $mastername{$newname} = "$dbase-$r{studnum}";
	    
	    
#	    print qq{$collisioncount. Collision:$name ($r{birthdate}) Grade:$r{grade} - $dbase $r{initial} with\n};
#	    print qq{  $cr{lastname},$cr{firstname} ($cr{birthdate}) $cr{grade} - $db - $cr{initial}\n};
	    $collisioncount++;
	    
	} else { # add to mastername
	    $mastername{$name} = "$dbase-$r{studnum}";
	}

	$data{$dbase}{$r{studnum}} = $sref; # save everything;
    }

}



foreach my $dbase ( keys %data ) {


    my $csv = Text::CSV_XS->new( {binary => 1} );

    # Open output file
    my $fileName = "gmail$dbase$$.csv";
    open (EX,">$fileName") || die "Can't open Export file $fileName";


    foreach my $key ( keys %mastername ) {
	my $val = $mastername{$key};
	my ($db, $studnum) = split('-', $val);
	if ( $db ne $dbase ) { next; }
    
	my $ref = $data{$db}{$studnum};
	my %r = %$ref;
    
	my ($ln,$fn) =  split(',',$key);
	my $emailname = "$ln$fn";

	$email = $emailname. '@tsec.ca';
    
	my @r;
	push @r, $r{firstname};
	push @r, $r{lastname};
	push @r, $email;
	push @r, $r{password};
   
	if ($csv->combine(@r)) {
	    my $record = $csv->string;
	    print EX $record, "\r\n";
	} else {
	    my $err = $csv->error_input;
	    print $lex{'Combine failed on input'}. ": $err\n\n";
	}
    }

    close EX;

} # end of school loop
#system("mv $fileName $downloaddir");

print qq{<p><a href="$webdownloaddir/$fileName">\n};
print qq{$lex{'Download CSV File'}</a></p>\n};

print qq{</body></html>\n};

