#!/usr/bin/perl
#  Copyright 2001-2017 Leslie Richardson

#  This file is part of Open Admin for Schools.

#  Open Admin for Schools is free software; you can redistribute it 
#  and/or modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2 of 
#  the License, or (at your option) any later version.

#  Description: Grade based subject enrollment. List all the students
#   for all the subjects in that grade. Deselect those that you don't
#   have in those subjects.

my %lex = ('Main' => 'Main',
	   'Report Card' => 'Report Card',
	   'Student Group' => 'Student Group',
	   'Grade' => 'Grade',
	   'Homeroom' => 'Homeroom',
	   'Continue' => 'Continue',
	   'Show Withdrawn Students' => 'Show Withdrawn Students',
	   'Subject Enrollments' => 'Subject Enrollments',
	   'Add' => 'Add',
	   'Checked' => 'Checked',
	   'Blank=All' => 'Blank=All',
	   'Separate with Spaces' => 'Separate with Spaces',
	   'Starts with' => 'Starts with',
	   'Terms' => 'Terms',
	   'Start' => 'Start',
	   'End' => 'End',
	   'Student' => 'Student',
	   'Selection' => 'Selection',
	   'Subject' => 'Subject',
	   'Grade(s)' => 'Grade(s)',
	   'Starts with' => 'Starts with',
	   'Subjects' => 'Subjects',
	   'Students' => 'Students',
	   'Record(s) Stored' => 'Record(s) Stored',
	   'Record Exists' => 'Record Exists',
	   'Contact' => 'Contact',
	   'Error' => 'Error',
	   'Checked' => 'Checked',
	   'Term' => 'Term',
	   'Not Found' => 'Not Found',
	   'Skip' => 'Skip',
	   'and' => 'and',
	   'Or' => 'Or',
	   'Name' => 'Name',
	   'Student Number' => 'Student Number',
	   'Next Page' => 'Next Page',
	   'Sort by' => 'Sort by',
	   'Mdl' => 'Mdl',
	   'Save' => 'Save',

	   );

my $self = 'enroladdall.pl';

use DBI;
use CGI;

eval require "../../etc/admin.conf.root";
if ( $@ ) {
    print $lex{Error}. " $@<br>\n";
    die $lex{Error}. " $@\n";
}

my $dbtype = 'mysql';
my $dsn = "DBI:$dbtype:dbname=$dbase";
my $dbh = DBI->connect($dsn,$user,$password);
$dbh->{mysql_enable_utf8} = 1;

# Load Configuration Variables;
my $sth = $dbh->prepare("select id, datavalue from conf_system where filename = 'admin'");
$sth->execute;
if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
while (	my ($id, $datavalue) = $sth->fetchrow ) {
    eval $datavalue;
    if ( $@ ) {
	print "$lex{Error}: $@<br>\n";
	die "$lex{Error}: $@\n";
    }
}

# Load subjects to skip for moodle table.
my $sth = $dbh->prepare("select id, datavalue from conf_system 
  where dataname = ?");
foreach my $val ( 'r_SupressSubject','r_AdditionalComments' ) {
    $sth->execute( $val );
    if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
    while (	my ($id, $datavalue) = $sth->fetchrow ) {
	eval $datavalue;
	if ( $@ ) {
	    print "$lex{Error}: $@<br>\n";
	    die "$lex{Error}: $@\n";
	}
    }
}

my $q = new CGI;
print $q->header( -charset, $charset ); 
my %arr = $q->Vars;

my $grade = $arr{grade};
my $checked = $arr{checked};


# Print Page Header
my $title = "$lex{Add} $lex{'Subject Enrollments'}";
print qq{$doctype\n<html><head><title>$title</title>\n};
print qq{<link rel="stylesheet" href="$css" type="text/css">\n};
print qq{$chartype\n</head><body>\n};

print qq{[ <a href="$homepage">$lex{Main}</a> |\n};
print qq{<a href="$reppage">$lex{'Report Card'}</a> ]\n};

print qq{<h1>$title</h1>\n};

if ( not $arr{page} ) {
    showStartPage();

} elsif ( $arr{page} == 1 ) {
    delete $arr{page};
    selectGroups();

} elsif ( $arr{page} == 2 ) {
    delete $arr{page};
    selectSubjStud();

} elsif ( $arr{page} == 3 ) {
    delete $arr{page};
    writeRecords();
}


#----------------
sub showStartPage {
#----------------

    print "<form action=\"$self\" method=\"post\">\n";
    print "<input type=\"hidden\" name=\"page\" value=\"1\">\n";

    print "<center><div style=\"padding:1em;margin:0.5em 5em;border:1px solid gray;\">\n";
    print "<div style=\"color:white; background-color:blue; font-size:150%;\">\n";
    print $lex{'Student'}. q{ }. $lex{'Selection'}. "</div>\n";
    print "<table border=\"0\" cellpadding=\"3\" cellspacing=\"0\">\n";

    print "<tr><td align=\"right\">". $lex{'Student Group'}. "</td>\n<td align=\"left\">";
    print "<select name=\"group\"><option>". $lex{Grade}. "</option><option>";
    print $lex{Homeroom}. "</option><option>". $lex{'Student Number'}. "</option></select>\n";
    print "<input type=\"input\" name=\"groupid\" size=\"12\">\n";
    print $lex{'Separate with Spaces'}. q{, }. $lex{'Blank=All'}. "</td></tr>\n";

    print "<tr><td align=\"right\">";
    print "$lex{'Next Page'} $lex{Checked}?</td>\n<td>";
    print "<input type=\"checkbox\" name=\"studcheck\" value=\"CHECKED\" checked=\"checked\"></td></tr>\n";

    print "<tr><td align=\"right\">". $lex{'Show Withdrawn Students'}. "</td>\n";
    print "<td align=\"left\">";
    print "<input type=\"checkbox\" name=\"showwithdrawn\" value=\"1\">";
    print "</td></tr>\n";

    print "<tr><td align=\"right\">$lex{'Sort by'} $lex{Name}</td>\n";
    print "<td align=\"left\">";
    print "<input type=\"checkbox\" name=\"sortbyname\" value=\"1\">";
    print "</td></tr>\n";


    print "</table></div>\n";


    # Now Subject Selection.

    print "<center><div style=\"padding:1em;margin:0.5em 5em;border:1px solid gray;\">\n";
    print "<div style=\"color:white; background-color:blue; font-size:150%;\">\n";
    print $lex{'Subject'}. q{ }. $lex{'Selection'}. "</div>\n";
    print "<table border=\"0\" cellpadding=\"3\" cellspacing=\"0\">\n";

    # Search for Subjects by Grade
    print "<tr><td align=\"right\">";
    print $lex{'Grade(s)'}. "</td>\n<td>";
    print "<input type=\"input\" name=\"grade\" size=\"12\"> ";
    print $lex{'Separate with Spaces'}. "</td></tr>\n";

    print "<tr><td align=\"right\"></td><td align=\"left\"><b>";
    print $lex{Or}. "</b></td></tr>\n";


    # Search by Subject Name (Description)
    print "<tr><td align=\"right\">";
    print $lex{'Starts with'}. "</td>\n<td>";
    print "<input type=\"input\" name=\"subjname\" size=\"16\"></td></tr>\n";

    print "<tr><td align=\"right\"></td><td align=\"left\"><b>";
    print $lex{Or}. "</b></td></tr>\n";


#    Replace the two lines above with this once ready to do AND operations.
#    print "<tr><td></td><td align=\"left\"><select name=\"selecttype\">\n";
#    print "<option>". $lex{Or}. "</option><option>". $lex{and}. "</option></select>\n";
#    print "</td></tr>\n";

    # Search for Subjects by Term
    print "<tr><td align=\"right\">";
    print $lex{Terms}. q{ (}. $lex{Start}. '-', $lex{End}. ")</td>\n<td>";
    print "<input type=\"input\" name=\"term\" size=\"12\"> 3-4 6-8\n";
    print $lex{'Separate with Spaces'}. "</td></tr>\n";

    print qq{<tr><td colspan="2">&nbsp;</td></tr>\n};

    print "<tr><td align=\"right\">";
    print "$lex{'Next Page'} $lex{Checked}?</td>\n<td>";
    print "<input type=\"checkbox\" name=\"subjcheck\" value=\"CHECKED\" checked=\"checked\"></td></tr>\n";

    print "<tr><td></td><td><input type=\"submit\" value=\"$lex{Continue}\">\n";
    print "</td></tr>\n</form></div></body></html>\n";

    exit;

}



#-----------------
sub selectGroups { # select groups of students and subjects to enrol.
#-----------------

    # foreach my $key ( sort keys %arr) { print "K:$key V:$arr{$key}<br>\n"; }

    # If printing all students (no groupid), then turn on sorting by
    # name, and also turn off class color changes.
    if ( not $arr{groupid} ) { $arr{sortbyname} = 1; }


    my %sortedstudents;

    my $table = 'student'; # overridden to studentall if show withdrawn selected.
    my $group;
    if ( $arr{group} eq $lex{Grade} ) {
	$group = 'grade';
    } elsif ( $arr{group} eq $lex{Homeroom} ) {
	$group = 'homeroom';
    } elsif (  $arr{group} eq $lex{'Student Number'} ) {
	$group = 'studnum';
    } else { $group = 'grade'; } # default to grade.

    if ( $arr{showwithdrawn} ) { # change to both student tables.
	$table = 'studentall';
    }
    
    my $studcheck = $arr{studcheck};
    my $subjcheck = $arr{subjcheck};
    my @groups = split /\s/, $arr{groupid};

    my $select;
    my $orderby;
    if ( not @groups ) { # select all students 
	my $sth = $dbh->prepare("select lastname, firstname, studnum, $group from $table
          order by lastname, firstname");
	$sth->execute;
	if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr; }
	while ( my ( $lastname, $firstname, $studnum, $grp ) = $sth->fetchrow ) {
	    push @students, $studnum;
	    $students{$studnum} = "$lastname, $firstname ($grp)";
	}

	
    } elsif ( $group eq 'studnum' ) { # student numbers

	my $sth = $dbh->prepare("select lastname, firstname, grade from $table 
          where studnum = ?");
	foreach my $studnum ( @groups ) {

	    $sth->execute( $studnum );
	    if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr; }
	    my ( $lastname, $firstname, $grade ) = $sth->fetchrow;
	    push @students, $studnum;
	    $students{$studnum} = "$lastname, $firstname ($grade)";
	}


    } else { # we have groups of students 

	my $sth = $dbh->prepare("select lastname, firstname, studnum, $group from $table
          where $group = ? order by lastname, firstname");

	foreach my $grp ( @groups ) {
	    $sth->execute( $grp );
	    if ( $DBI::errstr ){ print $DBI::errstr; die $DBI::errstr; }
	    while ( my ( $lastname, $firstname, $studnum, $grp ) = $sth->fetchrow ) {
		push @students, $studnum;
		$students{$studnum} = "$lastname, $firstname ($grp)";
		$sortedstudents{"$lastname$firstname$studnum"} = $studnum;
	    }
	}

	# Secondary processing of @students to get sorting by name.
	if ( $arr{sortbyname} ) { 
	    @students = ();	    
	    foreach my $key ( sort keys %sortedstudents ) {
		push @students, $sortedstudents{$key};
	    }
	}

    }
    # All students now in @students and %students;


    # Now do Subject Selection.
    my ( @subjects_name, %subjects_name);

    # Search by description if any...
    if ( $arr{subjname} ) { 

	# Find all subjects with this name.
	my $sth = $dbh->prepare("select description, subjsec from subject 
             where description like ? order by description");
	$desc = $arr{subjname}. '%';
	$sth->execute( $desc );
	if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }

	while ( my ( $description, $subjsec ) = $sth->fetchrow ) {
	    $description =~ s/://g; # strip any colons
	    push @subjects_name, $subjsec;
	    $subjects_name{$subjsec} = $description;
	}
	
    }


    # Search by grades, if any.
    my ( @subjects_grade, %subjects_grade);

    if ( $arr{grade} ) { 
	my @grades = split /\s+/, $arr{grade};
	
	foreach my $gr (sort  @grades ) {
	    # Find all subjects in this grade...
	    my $sth = $dbh->prepare("select description, subjsec from subject 
                 where grade = ? order by description");
	    $sth->execute( $gr );
	    if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
	    
	    while ( my ( $description, $subjsec ) = $sth->fetchrow ) {
		$description =~ s/://g; # strip any colons
		push @subjects_grade, $subjsec;
		$subjects_grade{$subjsec} = $description;
	    }
	}
    }


    # Find all subjects for these terms
    my ( @subjects_term, %subjects_term);
    if ( $arr{term} ) {
	$sth = $dbh->prepare("select description, subjsec from subject 
         where startrptperiod = ? and endrptperiod = ? order by description");

	my @termgroups = split /\s+/, $arr{term};
	foreach my $val ( @termgroups ) {
	    my ( $start, $end ) = split /-/, $val;
	    $sth->execute( $start, $end );
	    if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
	    
	    while ( my ( $description, $subjsec ) = $sth->fetchrow ) {
		$description =~ s/://g; # strip any colons
		push @subjects_term, $subjsec;
		$subjects_term{$subjsec} = $description;
	    }
	}
    }

    # selecttype eq 'and' and we are doing double selection by both terms AND grades
    # Now do some fancy intersections of them...
    

    # Now should have @subjects and %subjects populated.
    #print "Subjects:",  @subjects, "<br>\n";

    # Print Form start
    print qq{<form action="$self" method="post">\n};
    print qq{<input type="hidden" name="page" value="2">\n};

    # Get Checkbox state (for next page for select particular subject-student enrol).
    print qq{<p class="la"><span style="border:1px solid gray;margin:1em 2em;padding:1em;">};
    print qq{$lex{'Next Page'} $lex{Checked}?\n};
    print qq{ <input type="checkbox" name="checked" value="CHECKED" checked="checked"></span></p>\n};


    # Print Student Information.
    print qq{<table cellpadding="3" cellspacing="0" border="1" };
    print qq{style="float:left;margin:1em;">\n};

    print qq{<tr><td class="cn">};
    print qq{<input type="submit" value="$lex{Continue}"></td></tr>\n};

    my $class = 'blue';
    
    foreach my $studnum ( @students ){  #loop through students

	# Manage Class value.
	if ( not $arr{sortbyname}  ) {
	    my ($dud, $grp) = split(/\(/, $students{$studnum});
	    chop $grp; # remove trailing parens.
	    $oldgrp = $currgrp;
	    $currgrp = $grp;
	    if ( $currgrp ne $oldgrp ) { # switch colors;
		if ( $class eq 'blue' ) { $class = 'gray'; } else { $class = 'blue'; }
	    }
	}

	print qq{<tr class="$class"><td>};
	print qq{<input type="checkbox" name="$students{$studnum}:ST:$studnum" };
	print qq{value="1" $studcheck>\n};
	print qq{ $students{$studnum}</td></tr>\n};

    }

    print qq{<tr><td class="cn">};
    print qq{<input type="submit" value="$lex{Continue}"></td></tr>\n};
    print qq{</table>\n};


    # Print Subject Information, now
    print qq{<table cellpadding="3" cellspacing="0" border="1" };
    print qq{style="float:left;margin:1em;">\n};

    print qq{<tr><td class="cn">};
    print qq{<input type="submit" value="$lex{Continue}"></td></tr>\n};


    $sth = $dbh->prepare("select startrptperiod, endrptperiod, teacher from subject 
      where subjsec = ?");

    if ( @subjects_name ) {
	print qq{<tr><th>$lex{Name} $arr{subjname}</th></tr>\n};
	foreach my $subjsec ( @subjects_name ){  # loop through subjects by Name

	    $sth->execute( $subjsec );
	    if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
	    my ( $startterm, $endterm, $teacher ) = $sth->fetchrow;
	
	    print qq{<tr class="gray"><td>};
	    print qq{<input type="checkbox" name="$subjects_name{$subjsec}:SU:$subjsec" };
	    print qq{value="1" $subjcheck>\n};
	    print qq{ $subjects_name{$subjsec} ($subjsec) $teacher - $lex{Terms} $startterm\-$endterm</td></tr>\n};
	}
    }

    if ( @subjects_grade ) {
	print qq{<tr><th>$lex{'Grade(s)'} $arr{grade}</th></tr>\n};
	foreach my $subjsec ( @subjects_grade ){  # loop through subjects by Name

	    $sth->execute( $subjsec );
	    if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
	    my ( $startterm, $endterm, $teacher ) = $sth->fetchrow;
	    my ($tname, $dud) = split /\(/, $teacher; # strip userid.	

	    print qq{<tr class="gray"><td>};
	    print qq{<input type="checkbox" name="$subjects_grade{$subjsec}:SU:$subjsec" };
	    print qq{value="1" $subjcheck>\n};
	    print qq{ $subjects_grade{$subjsec} ($subjsec) $tname $lex{Terms} };
	    print qq{ $startterm\-$endterm</td></tr>\n};
	}
    }


    if ( @subjects_term ) {
	print qq{<tr><th>$lex{Terms} $arr{term}</th></tr>\n};
	foreach my $subjsec ( @subjects_term ){  # loop through subjects by Name

	    $sth->execute( $subjsec );
	    if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
	    my ( $startterm, $endterm, $teacher ) = $sth->fetchrow;
	    my ($tname, $dud) = split /\(/, $teacher; # strip userid.	

	    print qq{<tr class="gray"><td>};
	    print qq{<input type="checkbox" name="$subjects_term{$subjsec}:SU:$subjsec" };
	    print qq{value="1" $subjcheck>\n};
	    print qq{ $subjects_term{$subjsec} ($subjsec) $tname &ndash; ". $lex{Terms} };
	    print qq{ $startterm\-$endterm</td></tr>\n};
	}
    }


    print qq{<tr><td class="cn">};
    print qq{<input type="submit" value="$lex{Continue}"></td></tr>\n};
    print qq{</table></form></body></html>\n};

    exit;

}




#-----------------
sub selectSubjStud {
#-----------------

    # foreach my $key ( sort keys %arr) { print "K:$key V:$arr{$key}<br>\n"; }

    # select Student-Subject enrollments - one student in one subject;
    my $checked = $arr{checked};
    delete $arr{checked};


    foreach my $key ( sort keys %arr) {
	my ($dud, $sigil, $value ) = split /:/, $key;
	if ( $sigil eq 'ST' ) {
	    push @students, $value; 
	} elsif ( $sigil eq 'SU' ) { 
	    push @subjects, $value; 
	} else { # problems!
	    print $lex{Error}. " $key<br>\n";
	}

    }

    # Find subject descriptions / student name
    my $sth = $dbh->prepare("select description from subject where subjsec = ?");
    my $sth1 = $dbh->prepare("select lastname, firstname from studentall where studnum = ?");


    print qq{<form action="$self" method="post">\n};
    print qq{<input type="hidden" name="page" value="3">\n};

    print qq{<div style="text-align:left;">\n};
    print qq{<input type="submit" value="$lex{Save}"></div>\n};

    foreach my $subjsec ( @subjects ) {

	# Get Subject Description
	$sth->execute( $subjsec );
	if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr; }
	my $description = $sth->fetchrow;
	
	# Print Table Heading
	print qq{<table cellpadding="3" cellspacing="0" border="1" };
	print qq{style="float:left;margin:1em;">\n};
	print qq{<tr><th>$description ($subjsec)</th></tr>\n};

	foreach my $studnum ( @students ) {

	    # Get Student Name
	    $sth1->execute( $studnum );
	    if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr; }
	    my ( $lastname, $firstname ) = $sth1->fetchrow;

	    # print a current table row
	    print qq{<tr><td>};
	    print qq{<input type="checkbox" name="$subjsec:$studnum" value="1" $checked>\n};
	    print qq{<b>$lastname,</b> $firstname</td>\n};

	}

	print qq{</table>\n};

    }

    print qq{<br clear="left">\n};
    print qq{<div style="text-align:left;"><input type="submit" value="$lex{Save}"></div>\n};

    print qq{</form></body></html>\n};

    exit;


} # end of selectSubjects



#---------------
sub writeRecords {
#---------------
    
    #foreach my $key (keys %arr) { print "K:$key V:$arr{$key}<br>\n"; }

    print "</h1>\n<table cellpadding=\"3\" cellspacing=\"0\" border=\"1\">\n";
    print '<tr><th>'. $lex{Students}. '</th><th>'. $lex{Subjects};
    print '</th><th>'. $lex{Terms}. "</th></tr>\n";

    my $sth = $dbh->prepare("select description, startrptperiod, endrptperiod 
     from subject where subjsec = ?");
    my $sth1 = $dbh->prepare("select lastname, firstname from studentall where studnum = ?");

    foreach my $key ( sort keys %arr ) {  # $key contains studnum:subjsec 

	my ( $subjsec, $studnum ) = split(/:/,$key);
       
	# Read the subject information
	$sth->execute( $subjsec );
	if ( $DBI::errstr ){ print $DBI::errstr; die $DBI::errstr; }
	my ( $description, $startterm, $endterm ) = $sth->fetchrow;

	# Get Student Name
	$sth1->execute( $studnum );
	if ( $DBI::errstr){ print $DBI::errstr; die $DBI::errstr; }
	my ( $lastname, $firstname ) = $sth1->fetchrow;
	if ( not $lastname ) { 
	    $lastname = qq{<span style="color:red;">$lex{'Not Found'}</span>}; 
	}

	# Check for the existence of the same record(s),
	# one for each term for each student in each subject.
	my ( $term, %terms );

	$sth2 = $dbh->prepare("select term from eval
          where subjcode = ? and studnum = ? order by term");
	$sth2->execute( $subjsec, $studnum );
	if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr;}


	# Put existing term records in the 'terms' hash; check for dups.
	while ( my $term = $sth2->fetchrow ) {
	    if ( $terms{$term} ) {
		print "$lex{'Record Exists'} $lex{Term}: $term ". 
		print " $description ($subjsec) - $lastname, $firstname ($studnum).<br>\n";
		# miss steak? next;
	    } else { # add to hash.
		$terms{$term} = 1;
	    }
	}

	# print line for this record.
	print "<tr><td><b>$lastname</b>, $firstname ($studnum)</td>";
	print "<td>$description ($subjsec)</td>\n<td> ";


	# Loop through and check where term is missing...
	for ( my $i = $startterm; $i <=$endterm; $i++ ) {
	    if ( not $terms{$i} ){ # fill in holes!
		$sth3 = $dbh->prepare("insert into eval ( subjcode, studnum, term )
                  values ( ?, ?, ? )");
		$sth3->execute( $subjsec, $studnum, $i ); # $i is the term
		if ( $DBI::errstr ){ print $DBI::errstr; die $DBI::errstr; }
		print " $i ";
	    } else { 
		print '('. $lex{Skip}. " $i) ";
	    }
	} # end of Terms Loop

	# Check and Insert into external table for Moodle (ext_moodle table)
	my $sth4 = $dbh->prepare("select count(*) from ext_moodle 
          where studnum = ? and subjsec = ?");
	$sth4->execute( $studnum, $subjsec );
	if ( $DBI::errstr ){ print $DBI::errstr; die $DBI::errstr; }
	my $count = $sth4->fetchrow;
	
	# skip if member of %r_SupressSubject or AdditionalComments
	my ($tsubjsec, $dud) = split('-', $subjsec );
	my $skipflag;
	if ( $count or $r_SupressSubject{$tsubjsec} or $r_SupressSubject{$subjsec} or
	     $r_AdditionalComments{$tsubjsec} or $r_AdditionalComments{$subjsec} ) {
	    $skipflag = 1;
	}

	if ( not $skipflag ) {

	    my ($subjcode, $section) = split('-',$subjsec);
	    my $sth4 = $dbh->prepare("insert into ext_moodle (studnum, subjsec, subjcode, role)
              values ( ?, ?, ?, 'student' )");
	    $sth4->execute( $studnum, $subjsec, $subjcode );
	    if ( $DBI::errstr ){ print $DBI::errstr; die $DBI::errstr; }
	    print qq{ $lex{Mdl} }; # short form for Moodle.

	} else {
	    print qq{ ($lex{Skip} $lex{Mdl})}; # short form for Moodle.
	}


	print qq{</td></tr>\n}; # end of printed row

	
    } # Next Student/Subject combination.

    if ( not $DBI::errstr ) {
	print qq{</table>\n};
	print qq{<h3>$lex{'Record(s) Stored'}</h3></p>\n};
    } else {
	print qq{</table><p>$lex{Error}: $DBI::errstr<br>\n};
	print qq{$lex{'Contact'}:$adminname <a href="mailto:$adminemail">$adminemail</a>\n};
    }

    print qq{<p>[ <a href="$reppage">$lex{'Report Card'}</a> ]\n};
    print qq{</p></body></html>\n};

    exit;

} # End of writeRecords
