#!/usr/bin/perl
#  Copyright 2001-2014 Leslie Richardson

#  This file is part of Open Admin for Schools.

#  Open Admin for Schools is free software; you can redistribute it 
#  and/or modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2 of 
#  the License, or (at your option) any later version.

my %lex = ('Courses' => 'Courses',
	   'Main' => 'Main',
	   'Report Card' => 'Report Card',
	   'Subject' => 'Subject',
	   'Teacher' => 'Teacher',
	   'Term' => 'Term',
	   'Grade' => 'Grade',
	   'Sort by' => 'Sort by',
	   'No Teacher' => 'No Teacher',
	   'Subj-Sec' => 'Subj-Sec',
	   'Terms' => 'Terms',
	   'Edit' => 'Edit',
	   'Delete' => 'Delete',
	   'Clone' => 'Clone',
	   'SmDesc' => 'SmDesc',
	   'Enrol' => 'Enrol',
	   'Error' => 'Error',
	   'Fix' => 'Fix',
	   'Field' => 'Field',
	   'Credit' => 'Credit',
	   'Difficulty' => 'Diff',

	   );

# Remove after version 7.00+
my $teacherFixFlag; # flag teacher field containing name (still).


use DBI;
use CGI;
use Number::Format qw(:all);

eval require "../../etc/admin.conf";
if ( $@ ) {
    print $lex{Error}. ": $@<br>\n";
    die $lex{Error}. ": $@\n";
}

my $q = CGI->new;
print $q->header( -charset, $charset );
my %arr = $q->Vars;


my $sortorder;
if ($arr{sortorder} eq $lex{Subject} ){
    $sortorder = 'description, grade, teacher';
} elsif ($arr{sortorder} eq $lex{Teacher} ){
    $sortorder = 'teacher, description, grade';
} elsif ($arr{sortorder} eq $lex{Grade} ){
    $sortorder = 'grade, description, teacher';
} elsif ($arr{sortorder} eq $lex{Term}){
    $sortorder = 'startrptperiod, endrptperiod, grade, description, teacher';
} else {
    $sortorder = 'description,grade,teacher';
    $arr{sortorder} = $lex{Subject};
}

my $dsn = "DBI:$dbtype:dbname=$dbase";
my $dbh = DBI->connect($dsn,$user,$password);
$dbh->{mysql_enable_utf8} = 1;


# Get the Subjects in sorted order.
my @subjects = ();
if ( $arr{sortorder} eq $lex{Grade} ){
    my @grades = ();
    my $sth = $dbh->prepare("select distinct grade from subject 
     where grade is not NULL and grade != ''");
    $sth->execute;
    if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }
    while ( my $gr = $sth->fetchrow ) {
	push @grades, $gr;
    }
    my @grades = sort {$a <=> $b} @grades; # get into correct order. 

    foreach my $grade ( @grades ) {

	my $sth = $dbh->prepare("select subjsec from subject where grade = ? order by description, teacher");
	$sth->execute( $grade );
	if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }
	while ( my $subjsec = $sth->fetchrow ) {
	    push @subjects, $subjsec;
	}
    }

} else { # normal
    my $sth = $dbh->prepare("select subjsec from subject order by $sortorder");
    $sth->execute;
    if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }
    while ( my $subjsec = $sth->fetchrow ) {
	push @subjects, $subjsec;
    }
}
# Now have subjects in @subjects.



my $title = "$lex{Edit}/$lex{Delete} $lex{Courses}";
print "$doctype\n<html><head><title>$title</title>
<link rel=\"stylesheet\" href=\"$css\" type=\"text/css\">
$chartype\n</head><body>\n";


print "[ <a href=\"$homepage\">$lex{Main}</a> |\n";
print "<a href=\"$reppage\">$lex{'Report Card'}</a> ]\n";


# Sorting Form
print qq{<form action="$self" method="post" style="display:inline;">\n};
print qq{<input type="submit" value="$lex{'Sort by'}">\n};
print qq{<select name="sortorder">\n};
if ( $arr{sortorder} ) {
    print "<option>$arr{sortorder}</option>\n";
}
print "<option>$lex{Subject}</option><option>$lex{Teacher}</option>\n";
print "><option>$lex{Grade}</option><option>$lex{Term}</option>\n";
print "</select></form>\n";


print "<h1>$title</h1>\n";

print "<table cellpadding=\"3\" border=\"1\" cellspacing=\"0\">";
print "<tr><th>$lex{Subject}</th><th>$lex{'Subj-Sec'}";
print "</th><th>$lex{SmDesc}</th><th>$lex{Grade}";
print "</th><th>$lex{Teacher}</th><th>$lex{Terms}</th>";
print "<th>$lex{Credit}<br>$lex{Difficulty}</th>\n";
print "<th>$lex{Enrol}</th><th>$lex{Edit}</th>\n";
print "<th>$lex{Delete}</th><th>$lex{Clone}</th></tr>\n";

my $sth1 = $dbh->prepare("select count(*) from eval where subjcode = ?");
my $sth2 = $dbh->prepare("select lastname, firstname from staff where userid = ?");

my $sth = $dbh->prepare("select id, description, smdesc, grade, 
 teacher, startrptperiod, endrptperiod, credit, difficulty 
 from subject where subjsec = ?");


foreach my $subjsec ( @subjects ) {

    # Get subject info
    $sth->execute( $subjsec );
    if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }
    my ( $id, $description, $smdesc, $grade, $teacher, 
	 $startterm, $endterm, $credit, $difficulty ) = $sth->fetchrow;
    
    # Get Eval record count
    $sth1->execute( $subjsec );
    if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }
    my $count = $sth1->fetchrow;

    my $terms = ($endterm - $startterm) + 1;
    if ($terms < 1) { $terms = 1; }
    my $enrollments = round( $count / $terms, 2);

    if ( $teacher =~ m/\(/ ) { # brackets/parenthesis found, strip/notify.
	my ($name, $userid ) = split(/\(/, $teacher );
	$teacherFixFlag = qq{<span style="color:red;">***</span>};
    } else { # pull in teacher name from staff table.
	$sth2->execute( $teacher );
	if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }
	my ($lastname, $firstname) = $sth2->fetchrow;
	$teacher = "$lastname, $firstname ($teacher)";
    }


    if ( not $teacher ){ 
	$teacher = "<font color=\"red\">$lex{'No Teacher'}</font>"; 
    }

    print "<tr><td>$description</td><td>$subjsec</td><td>$smdesc</td>\n";
    print "<td>$grade</td><td>$teacher $teacherFixFlag</td><td>$startterm - $endterm</td>";
    print "<td>$credit<br>$difficulty</td><td>$enrollments</td>\n";

    print "<td><form action=\"subjed.pl\" method=\"post\" style=\"display:inline;\">\n";
    print "<input type=\"hidden\" name=\"id\" value=\"$id\">\n";
    print "<input type=\"submit\" value=\"$lex{Edit}\"></form></td>\n";

    print "<td><form action=\"subjdel.pl\" method=\"post\" style=\"display:inline;\">\n";
    print "<input type=\"hidden\" name=\"id\" value=\"$id\">\n";
    print "<input type=\"submit\" value=\"$lex{Delete}\"></form></td>\n";

    print "<td><form action=\"subjadd.pl\" method=\"post\" style=\"display:inline;\">\n";
    print "<input type=\"hidden\" name=\"subjsec\" value=\"$subjsec\">\n";
    print "<input type=\"submit\" value=\"$lex{Clone}\"></form></td>\n";

}

print "<tr><td colspan=\"11\" align=\"center\"><span style=\"color:red;\">***</span> = ";
print "<form action=\"fixsubjectteacher.pl\" method=\"post\" style=\"display:inline;\">\n";
print "<input type=\"submit\" value=\"$lex{Fix} $lex{Teacher} $lex{Field}\"></form></td></tr>\n";

print "</table>\n";

print "</body></html>\n";
