#! /usr/bin/perl
#  Copyright 2001-2017 Leslie Richardson
#  This file is part of Open Admin for Schools.

# Get the Student Enrollment history to see if he/she is enrolled at
# any other Sask Schools, in the event that the student is not coming
# to school and has a long attendance absent list.


my $self = 'getStudentEnrollmentHistory.pl';

use DBI;
use CGI;
use XML::Writer;
use XML::Writer::String;
use Data::UUID;
use HTTP::Request::Common qw(POST); 
use HTTP::Headers; 
use LWP::UserAgent; 
use XML::LibXML;
use Time::JulianDay;

my %lex = ( 'Main' => 'Main',
    'Error' => 'Error',
    );



eval require "../../etc/admin.conf";
if ( $@ ) {
    print $lex{Error}. " $@<br>\n";
    die $lex{Error}. " $@\n";
}



eval require "slxmllibNew.pl";
if ( $@ ) {
    print $lex{Error}. " $@<br>\n";
    die $lex{Error}. " $@\n";
}


my $dsn = "DBI:$dbtype:dbname=$dbase";
$dbh = DBI->connect($dsn,$user,$password);
$dbh->{mysql_enable_utf8} = 1;


my $q = new CGI;
my %arr = $q->Vars;
print $q->header( -charset, $charset);

my $debug;
if ( $arr{debug} ) {
    $debug = 1;
    delete $arr{debug};
}


my $title = "Student Enrollment History";
print qq{$doctype\n<html><head><title>$title</title>\n};
print qq{<link rel="stylesheet" href="$css" type="text/css">\n};
print qq{$chartype\n</head><body>\n};
print qq{[ <a href="$homepage">Main</a> | <a href="$exppage">Export</a> ]\n};
print qq{<h1>$title</h1>\n};


my @tim = localtime(time);
my $year = $tim[5] + 1900;
$tim[4]++;
for (0..4){if (length($tim[$_]) == 1){ $tim[$_] = '0'.$tim[$_];}}
my $currdate = "$year-$tim[4]-$tim[3]";
my $currtime = "$tim[2]:$tim[1]:$tim[0]";
my $currjd = julian_day($year,$tim[4],$tim[3]);


# First run a duplicate provnum check
my $sth = $dbh->prepare("select studnum, lastname, firstname, provnum 
from student where provnum != '' order by provnum");
$sth->execute;
if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr;}

my $currstud = '-1';
my $currname = '';

while ( my ($studnum, $lastname, $firstname, $provnum) = $sth->fetchrow ) {
    $oldstud = $currstud;
    $currstud = $provnum;
    $oldname = $currname;
    $currname = "$firstname $lastname";

    if ($currstud == $oldstud) { # Ack! We have a duplicate!
	print qq{<div style="border:1px solid black;padding:1em;">};
	print qq{Duplicate for provincial student number: $currstud<br>\n};
	print qq{<b>$currname</b> and <b>$oldname</b> are duplicated!</div>\n};
    }
}




if ( not $arr{page} ) {
    showStartPage();

} elsif ( $arr{page} == 1 ) {
    delete $arr{page};
    selectStudents();
    
} elsif ( $arr{page} == 2 ) {
    delete $arr{page};
    showReport();
}


#----------------
sub showStartPage {
#----------------

    # Get the grades
    my @grades;
    my $sth = $dbh->prepare("select distinct grade from student 
      where grade is not NULL and grade != ''");
    $sth->execute;
    if ( DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
    while ( my $gr = $sth->fetchrow ) {
	if ( $g_AttendanceEntryMethod{$gr} eq 'subject' ) {
	    push @grades, $gr;
	}
    }


    # Get the homerooms.
    my @homerooms;

    my $sth1 = $dbh->prepare("select distinct grade from student where homeroom = ? 
      and grade != '' and grade is not NULL");

    my $sth = $dbh->prepare("select distinct homeroom from student where 
     homeroom is not NULL and homeroom != ''");
    $sth->execute;
    if ( DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
    while ( my $hr = $sth->fetchrow ) {
	push @homerooms, $hr; # removed checking below
    }


    print qq{<form action="$self" method="post">\n};
    print qq{<input type="hidden" name="page" value="1">\n};

    print qq{<table cellpadding="3" border="0" cellspacing="0">\n};

    
    
    # Homerooms
    print qq{<tr><td class="bra">Select Homeroom</td>};
    print qq{<td class="la"><select name="homeroom"><option></option>\n}; 

    foreach my $hr ( sort {$a <=> $b} @homerooms ) {
	print qq{<option>$hr</option>\n};
    }
    print qq{</select></td></tr>\n};



    # Grades
    print qq{<tr><td class="bra"><span style="font-weight:normal;">or</span>&nbsp;};
    print qq{Grade</td>};
    print qq{<td class="la"><select name="grade"><option></option>\n}; 

    foreach my $gr ( sort {$a <=> $b} @grades ) {
	print qq{<option>$gr</option>\n};
    }
    print qq{</select></td></tr>\n};

    
    print qq{<tr><td colspan="2"><hr></td></tr>\n};


    # Show Withdrawn Students
    print qq{<tr><td class="bra">Show Withdrawn</td>};
    print qq{<td class="la"><input type="checkbox" name="showwithdrawn" value="1"></td></tr>\n}; 

    # Spacer
    print qq{<tr><td colspan="2"><hr></td></tr>\n};
    
    
   # Continue
    print qq{<tr><td></td><td class="la"><input type="submit" value="Continue">\n};
    print qq{</td></tr>\n};


    print qq{</table></form>\n};
    print qq{</body></html>\n};

    exit;

}


#-----------------
sub selectStudents {
#-----------------

    # foreach my $key ( sort keys %arr ) { print "K:$key V:$arr{$key}<br>\n"; }
    
    my $studtable = 'student';
    my $showwithdrawn;
    if ( $arr{showwithdrawn} ) {
	$studtable = 'studentall';
	$showwithdrawn = 1;
	delete $arr{showwithdrawn};
    }


    # foreach my $key ( sort keys %arr ) { print "K:$key V:$arr{$key}<br>\n"; }
    
    
    # Start Form
    print qq{<form action="$self" method="post">\n};
    print qq{<input type="hidden" name="page" value="2">\n};
#    foreach my $key ( sort keys %arr ) {  # pass arguments through
#	print qq{<input type="hidden" name="$key" value="$arr{$key}">\n};
#    }


    my $sth1 = $dbh->prepare("select count(*) from studentwd where studnum = ?");
    
    my $sth;
    if ( $arr{homeroom} and $arr{grade} ) { # Error
	print qq{<h3>Error: Select Only Grade or Homeroom</h3>\n};
	print qq{</body></html>\n};
	exit;
	
    } elsif ( $arr{homeroom} ) {
	$sth = $dbh->prepare("select lastname, firstname, studnum from $studtable
          where homeroom = ? order by lastname, firstname");
	$sth->execute( $arr{homeroom} );
			     
    } elsif ( $arr{grade} ) {
	$sth = $dbh->prepare("select lastname, firstname, studnum from $studtable
          where grade = ? order by lastname, firstname");
	$sth->execute( $arr{grade} );


    } else { # all students selected
	$sth = $dbh->prepare("select lastname, firstname, studnum from $studtable
          order by lastname, firstname");
	$sth->execute;
    }

    my $first = 1;
    if ( DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
    while ( my ($lastname, $firstname, $studnum ) = $sth->fetchrow ) {

	my $wd;
	if ( $showwithdrawn ) {
	    $sth1->execute( $studnum );
	    if ( DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
	    
	    if ( $sth1->fetchrow ) { # we have a WD student
		$wd = qq{<span style="color:red;font-weight:bold;">WD</span>\n};
	    }
	}
	

	if ( $first ) {
	    if ( $arr{grade} ) { 
		print qq{<h3>Grade $arr{grade}</h3>\n};
	    } elsif ( $arr{homeroom} ) { 
		print qq{<h3>Homeroom $arr{homeroom}</h3>\n};
	    } else { # all students
		print qq{<h3>All $lex{Students}</h3>\n};
	    }
				 
	    print qq{<table cellpadding="3" border="1" cellspacing="0">\n};
	    print qq{<tr><th>Select Students</th></tr>\n};
	    print qq{<tr><td><input type="submit" value="Continue"></td></tr>\n};
	    $first = 0;
	}

	print qq{<tr><td><input type="checkbox" name="$studnum" value="1"> };
	print qq{$wd <b>$lastname</b>, $firstname</td></tr>\n};
    }

    print qq{<tr><td><input type="submit" value="Continue"></td></tr>\n};
    
    if ( not $first ) {
	print qq{</table>\n};
    }

    print qq{</form>\n};
    print qq{</body></html>\n};

    exit;

}


#-------------
sub showReport {
#-------------

    # passed a list of students in %arr
    # foreach my $key ( sort keys %arr ) { print "K:$key V:$arr{$key}<br>\n"; }

    eval require "$globdir/global.conf"; # entry/exit codes for display
    if ( $@ ) {
	print $lex{Error}. " $@<br>\n";
	die $lex{Error}. " $@\n";
    }

    
    my %students;

    my $sth = $dbh->prepare("select provnum, birthdate from studentall where studnum = ?");

    my ($provnum, $birthdate);
    foreach my $studnum ( keys %arr ) {
	
	$sth->execute($studnum);
	if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr; }
	($provnum, $birthdate) = $sth->fetchrow;
	$students{$studnum} = { 'provnum' => $provnum, 'birthdate' => $birthdate };
    }
    
    
    # Create a new user agent
    my $ua = LWP::UserAgent->new();
    $ua->agent("OpenAdmin");

    # Grade and Provincial Number passed
    my $studref = \%students;
    
    mkQueryString($studref); # Generate $output string


    $debug = 0;
    
    # DEBUG Data Errors
    if ($debug){
	print qq{<div style="color: blue;">\n};
	print qq{<h1>DEBUG - Request sent to Sask ED</h1>\n};
	my $temp = $output->value;
	$temp =~ s/</&lt;/g;
	$temp =~ s/>/&gt;/g;
	print qq{<pre>$temp</pre></div><p></p>\n};
    }


    # Create the https post request 
    my $req = POST $url, [ XML=>$output->value ];

    $req->content_type('application/xml;charset="utf-8"');
    $req->authorization_basic($sds_userid, $sds_password); 

    # Issue the request and receive a response
    my $res = $ua->request($req);

    # Check the status of the response
    if ($res->is_success) {

	# For Debugging Data Errors
	if ($debug){
	    print "<h1>DEBUG - Sask Ed XML Response</h1>\n";
	    print "<div style=\"color:green;\">\n";
	    my $temp = $res->content;
	    $temp =~ s/</&lt;/g;
	    $temp =~ s/>/&gt;/g;
	    print "<pre>",$temp,"</pre></div>\n";
	}

	my $response = $res->content;
	$response =~ s/\xE9/\x65/g;

	# Parse the response.
	my $parser = XML::LibXML->new();
    
	eval {$doc = $parser->parse_string($response)};
	if ($@){
	    print "Error: $@<br>\n";
	    $response =~ s/</&lt;/g;
	    $response =~ s/>/&gt;/g;
	    print "<pre>Sask Ed Error:\n",$response,"</pre><br>\n";
	    print "</body></html>\n"; die;
	}

	$doc->setEncoding('UTF-8');
	$root = $doc->getDocumentElement;
	$root->setNamespace($xmlns,'sl',1);

	
	# find Status
	my $status = $root->findvalue('//sl:SL_Status/sl:SL_StatusCode');
	if ($status eq 'Errors' or $status eq 'Invalid'){ 
	    # Print out error and add to errorlog.
	    prErr("$lastname, $firstname ($studnum)");
	} elsif ( $status eq 'Successful' ) { 
	    
	    my $enrolref = parseStudentEnrollment( $root );

	    my %enrol = %{ $enrolref}; # $enrol{provnum}{schoolname}{entrydate}{otherinfo};

	    my $sth = $dbh->prepare("select studnum, lastname, firstname from studentall where provnum = ?");
    
	    
	    foreach my $provnum ( keys %enrol ) {
		
		$sth->execute( $provnum );
		if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr;}
		my ($studnum, $lastname, $firstname ) = $sth->fetchrow;
		
		
		print qq{<h3>$firstname $lastname ($studnum) Prov# $provnum</h3>\n};
		print qq{<table cellpadding="3" cellspacing="0" border="1">\n};
		print qq{<caption>Hover on Codes to see text description</caption>\n};
		print qq{<tr><th>School</th><th>Entry Date</th><th>Entry Code</th>};
		print qq{<th>Exit Date</th><th>Exit Code</th></tr>\n};

		foreach my $entrydate ( sort keys %{ $enrol{$provnum} } ) {		
		    foreach my $schoolname ( sort keys %{ $enrol{$provnum}{$entrydate} } ) {

			my $entrytype = $enrol{$provnum}{$entrydate}{$schoolname}{'entrytype'};
			my $exitdate = $enrol{$provnum}{$entrydate}{$schoolname}{'exitdate'};
			my $exittype = $enrol{$provnum}{$entrydate}{$schoolname}{'exittype'};

			# my $exitdate = $enrol{$provnum}{$entrydate}{schoolname}{};
			
			print qq{<tr><td>$schoolname</td><td>$entrydate</td>};
			print qq{<td title="$g_enrol{$entrytype}">$entrytype</td>\n};
			print qq{<td>$exitdate</td><td title="$g_wdraw{$exittype}">$exittype</td></tr>\n};
		    }
		}
		print qq{</table>\n};
	    }


	} else { # print warnings... ($status eq 'Warnings')
	    print "There were warnings...\n";
	}
    
    }  else { # Transfer Error!
	my $err = $res->status_line;
	print "<h1>Transfer Error: $err</h1>\n";
    }

    print "</body></html>\n";

} # end of showReport.




# Functions =============================

#----------------
sub mkQueryString {  # StudentEnrollment
#----------------

    # Date and schoolnumber are globals.
    my $ref = shift;
    my %students = %$ref;
    my $idcount = keys %students;
    
    
    # Create Writer Instance
    $output = new XML::Writer::String;
    my $datamode = 0;
    if ($debug){ $datamode = 1;} # pretty print xml output
    $wr = new XML::Writer(OUTPUT => $output, 
			  DATA_MODE => $datamode, 
			  DATA_INDENT => '2');

    # Set XML Header and write Root Element
    $wr->xmlDecl("UTF-8");
    $wr->startTag('SL_Message','xmlns' =>$xmlns, 
		  'xmlns:xsi' =>$xmlnsxsi, 
		  'xsi:schemaLocation' => $xsischemaLocation);

    $wr->startTag('SL_Request');

    # library call
    mkSL_Header($currdate, $currtime, $schoolnumber, $idcount);

    $wr->startTag('SL_Query');


    foreach my $studnum ( keys %students ) {

	my $provnum = $students{$studnum}{'provnum'};
	my $birthdate = $students{$studnum}{'birthdate'};
    
	$wr->startTag('QueryByStudent',
		      'RefId' => "$provnum",
		      'ObjectName' => 'StudentEnrollment'
	    );
	
	$wr->dataElement('DeptAssignedPersonId',$provnum);
	$wr->dataElement('BirthDate',$birthdate);
	$wr->endTag('QueryByStudent');

    }

    $wr->endTag('SL_Query');
    $wr->endTag('SL_Request');
    $wr->endTag('SL_Message');
    $wr->end();
    
}



#---------
sub prErr {  # Print Errors to Screen
#---------

    my $name = shift;

    my @errors = $root->findnodes('//sl:SL_Error');
    foreach my $error (@errors){
	# Get Error Elements
	my $objectname = $error->getAttribute('ObjectName');
	my $errmsg = $error->findvalue('./sl:SL_ErrorMsg');
	$errmsg =~ s/[\r|\n]/ /g;
	my $errcode = $error->findvalue('./sl:SL_ErrorCode');
	
	print "<p style=\"color:red;\">$errmsg ($errcode)</p>\n";
    }
    
}



#-------------------------
sub parseStudentEnrollment { # passed document root object, now loaded from library.
#-------------------------

    my $root = shift;

    my @studinfo = $root->findnodes('//sl:StudentEnrollment');

    my %enrol;  # $enrol{provnum}{schoolname}{entrydate}{otherinfo};

    
    foreach my $student ( @studinfo ) {

	my $provnum = $student->findvalue('sl:StudentIdentification/sl:DeptAssignedPersonId');
	my $birthdate = $student->findvalue('sl:StudentIdentification/sl:BirthDate');

	my @enrol = $student->findnodes('./sl:SchoolEnrollment');

	foreach my $e ( @enrol ) { # loop through all enrollments.

	    my $schoolid = $e->findvalue('sl:SchoolId');
	    my $schoolname = $e->findvalue('sl:SchoolName');

	    # print "School Name:$schoolname<br>\n";
	    
	    my @enrolinfo = $e->findnodes('./sl:SchoolEnrollmentInfo');

#	    print "\n\n<br><br>Enrol Info:", @enrolinfo, "<br>\n";

	    

	    foreach my $ei ( @enrolinfo ) {

		my $entrydate = $ei->findvalue('sl:EntryDate');
		my $entrytype = $ei->findvalue('sl:EntryType/@Code');

		my $exitdate = $ei->findvalue('sl:ExitDate');
		my $exittype = $ei->findvalue('sl:ExitType/@Code');

		my $immersiontype = $ei->findvalue('sl:ImmersionType/@Code');
		my $homebound = $ei->findvalue('sl:Homebound/@Code');
		
		my $tuitionstatuscode = $ei->findvalue('sl:TuitionStatus/@Code');
		my $tuitionstatuscollected = $ei->findvalue('sl:TuitionStatus/@Collected');
		
#		print qq{PN:$provnum $schoolname - Entry:$entrydate / $entrytype  Exit:$exitdate / $exittype<br>\n};

		$enrol{$provnum}{$entrydate}{$schoolname} = 
		  {'entrytype' => $entrytype, 'exitdate' => $exitdate, 'exittype' => $exittype };

	    }
	}
    }

 
 #   use Data::Dumper;
 #   print Dumper %enrol;
    
    return \%enrol;
	    

} # End of Sub StudentEnrollment
