#! /usr/bin/perl 
#  Copyright 2001-2017 Leslie Richardson

#  This file is part of Open Admin for Schools.

#  Open Admin for Schools is free software; you can redistribute it 
#  and/or modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2 of 
#  the License, or (at your option) any later version.


use DBI;
use CGI;

my %lex = ('Teacher' => 'Teacher',
	   'Main' => 'Main',
	   'Timetables' => 'Timetables',
	   'Timetable' => 'Timetable',
	   'Cannot open tex file' => 'Cannot open tex file',
	   'View/Download' => 'View/Download',
	   'View Log File' => 'View Log File',
	   'Select' => 'Select',
	   'Lastname, Firstname' => 'Lastname, Firstname',
	   'Homeroom, Lastname, Firstname' => 'Homeroom, Lastname, Firstname',
	   'Grade, Lastname, Firstname' => 'Grade, Lastname, Firstname',
	   'Band, Lastname, Firstname' => 'Band, Lastname, Firstname',
	   'Sort by' => 'Sort by',
	   'Grade' => 'Grade',
	   'Homeroom' => 'Homeroom',
	   'Continue' => 'Continue',
	   'No Staff Found' => 'No Staff Found',
	   'Error' => 'Error',
	   'Blank=All' => 'Blank=All',
	   'Term' => 'Term',
	   'Period' => 'Period',
	   'Day' => 'Day',
	   'Separate with Spaces' => 'Separate with Spaces',
	   'Term(s)' => 'Term(s)',
	   'Font Size' => 'Font Size',
	   'Paper Size' => 'Paper Size',
	   'Letter' => 'Letter',
	   'Legal' => 'Legal',
	   'A4' => 'A4',
	   'Missing' => 'Missing',
	   
	);

my $grayshade = '0.80';
my $maxchars = 10; # maximum characters for subject description (if not small desc field)
my $descfield = 'smdesc'; # either smdesc or description value

my $logowidth = '20mm';
my $logofile = 'schoollogo.jpg';

my $self = "rptstafftimetable.pl";


eval require "../../etc/admin.conf";
if ( $@ ) {
    print $lex{Error}. " $@<br>\n";
    die $lex{Error}. " $@\n";
}

eval require "../../lib/liblatex.pl";
if ( $@ ) {
    print $lex{Error}. " $@<br>\n";
    die $lex{Error}. " $@\n";
}


# Add period times, remove soon!
eval require "../../etc/schedule.conf";
if ( $@ ) {
    print $lex{Error}. ": $@<br>\n";
    die $lex{Error}. ": $@\n";
}


my $maxlines = 28;
my $shortname = "teachertimetable$$";
my $filename = "$shortname.tex";

my $dsn = "DBI:$dbtype:dbname=$dbase";
my $dbh = DBI->connect($dsn,$user,$password);
$dbh->{mysql_enable_utf8} = 1;

my $q = new CGI;
print $q->header( -charset, $charset );
my %arr = $q->Vars;



# print page header
my $title = qq{$lex{Teacher} $lex{Timetables}};
print qq{$doctype\n<html><head><title>$title</title>\n};
print qq{<link rel="stylesheet" href="$css" type="text/css">\n};

print qq{$chartype\n</head><body>\n};
print qq{[ <a href="$homepage">$lex{Main}</a> | \n};
print qq{<a href="$schpage">$lex{Timetable}</a> ]\n};


if ( not $arr{page} ) {
    showStartPage();
}

my %staffnames; # global


my $sortorder = "lastname, firstname";
if ( $arr{sortorder} eq 'grade' ) {
    $sortorder = "grade, lastname, firstname";
} elsif ( $arr{sortorder} eq 'band' ) {
    $sortorder = "band, lastname, firstname";
} elsif ( $arr{sortorder} eq 'homeroom' ) {
    $sortorder = "homeroom, lastname, firstname";
}
delete $arr{sortorder};

my @terms = split(/\s+/, $arr{term});
delete $arr{term};
print "Terms:@terms<br>\n";

my $select;
if ( $arr{group} ) {
    if ( $arr{select} eq $lex{Grade} ) { # Find this grade;
	my $grp = $dbh->quote( $arr{group} );
	$select = "where grade = $grp";

    } elsif ( $arr{select} eq $lex{Homeroom} ) {
	my $grp = $dbh->quote( $arr{group} );
	$select = "where homeroom = $grp";

    } elsif ( $arr{select} eq $lex{Band} ) {
	my $grp = $dbh->quote( $arr{group} );
	$select = "where band = $grp";

    } elsif ( $arr{select} eq $lex{'Student Number'} ) {
	my $val = $dbh->quote( $arr{group} );
	$select = "where studnum = $val";
    }
}



# Now create an array, @staff,  to store userid's of
my (@staff, @skipped);
my $sth = $dbh->prepare("select userid from staff $select order by $sortorder");
$sth->execute;
if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }
while ( my $userid = $sth->fetchrow ) {
    push @staff, $userid;
}

if ( @staff ) { # We have staff to print; setup LaTeX file.
    open(TEX,">$filename") || die $lex{'Cannot open tex file'};

    my ( $papersize, $textwidth, $textheight );

    if ( $arr{papersize} eq $lex{Letter} ) {
	$papersize = 'letterpaper';
	$textwidth = $g_letterpaper_textwidth;
	$textheight = $g_letterpaper_textheight;
    } elsif ( $arr{papersize} eq $lex{Legal} ) {
	$papersize = 'legalpaper';
	$textwidth = $g_legalpaper_textwidth;
	$textheight = $g_legalpaper_textheight;
    } elsif ( $arr{papersize} eq $lex{A4} ) {
	$papersize = 'a4paper';
	$textwidth = $g_a4paper_textwidth;
	$textheight = $g_a4paper_textheight;
    } 


    print TEX "\\documentclass[$arr{fontsize},$papersize, landscape]{article}
\\usepackage{array,colortbl,multicol,inputenc,graphicx,helvet}
$a_latex_header
\\pagestyle{empty}
\\setlength{\\textwidth}{$textwidth}
\\setlength{\\textheight}{$textheight}
\\setlength{\\evensidemargin}{0mm}
\\setlength{\\oddsidemargin}{0mm}
\\setlength{\\hoffset}{-10mm}
\\setlength{\\voffset}{-25mm}
\\setlength{\\extrarowheight}{2pt}
\\setlength{\\parindent}{0pt}\n";

    print TEX "\\begin{document}\n";

} else {
    print qq{<h3>$lex{'No Staff Found'}/h3>\n};
    print qq{</body></html>\n};
}

# Now Do Staff Timetables
my %subtimes; # used for global subject storage;

foreach my $userid ( @staff ) {

    prTimetable( $userid, \@terms ); # do 1 staff member
    
} # End of Staff Loop

print TEX qq{\\end{document}};
close TEX;

system("$pdflatex $filename >pdflog$$.txt");
system("mv $shortname.pdf $downloaddir");
system("mv pdflog$$.txt $downloaddir");
system("rm $shortname.*");

print qq{<h1><a href="$webdownloaddir/$shortname.pdf">\n};
print qq{$lex{'View/Download'} $lex{Teacher} $lex{Timetables}</a></h1>\n};

print qq{[ <a href="$schpage">$lex{Timetable}</a> |\n};
print qq{<a href="$webdownloaddir/pdflog$$.txt">$lex{'View Log File'}\n};
print qq{</a> ]\n};

if ( @skipped ) {
    print qq{<h3>$lex{Missing} $lex{Timetable}</h3>\n};
    print @skipped, qq{<br>\n};
}

print qq{</body></html>\n};


#--------------
sub prTimetable {
#--------------

    my ($userid, $ref) = @_;
    my @terms = @$ref;

    
    # Get staff info
    my $sth = $dbh->prepare("select * from staff where userid = ?");
    $sth->execute( $userid );
    if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
    my $ref = $sth->fetchrow_hashref;
    my %r = %$ref;

    # Check if they have any timetable info.
    my $skipflag = 1; # skip unless some subjects, timetable found for this teacher

    foreach my $term ( @terms ) {
	my $sth = $dbh->prepare("select count(t.subjsec) from subject s, schedat t where s.teacher = ? and 
         startrptperiod <= '$term' and endrptperiod >= '$term' and s.subjsec = t.subjsec");

	$sth->execute( $userid );
	if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
	my $tcount = $sth->fetchrow;
	if ( $tcount ) { # timetable count means we have entries for this term, at least.
	    $skipflag = 0;
	}
    }

    if ( $skipflag ) {  # no timetable (and courses) for these staff members.
	push @skipped, "$r{firstname} $r{lastname}<br>";
	return;
    }
    
    
    # Setup for Get Subject Information
    my $sth2 = $dbh->prepare("select description, smdesc, location, teacher 
      from subject where subjsec = ?");

    my $sth3 = $dbh->prepare("select count(distinct studnum) from eval where subjcode = ?");

    # Now print the Timetable in TeX, heading first.
    print TEX "\\includegraphics[width=$logowidth]{../../etc/$logofile}\\quad\n";

    print TEX "\\parbox[b]{150mm}{ {\\Large $schoolname}\n\n\n\n";
    print TEX "{\\LARGE\\bf $r{firstname} $r{lastname} } ($r{userid})\n\n";
	
    print TEX " }\n\n"; # end of parbox.
    print TEX "\\vspace{8mm}\n";

   
    foreach my $term ( @terms ) {
	
	# Get their courses this term; 
	$sth = $dbh->prepare("select * from subject where teacher = ? and 
          startrptperiod <= '$term' and endrptperiod >= '$term'");
	$sth->execute( $userid );
	if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }

	# Clean up global on later interations.
	foreach my $key ( keys %timetable ) {
	    delete $timetable{$key};
	}

	my %timetable; # timetable{period}{day}{subjsec} = 1
	my $maxdays = 0;

	my $sth1 = $dbh->prepare("select day, period from schedat
           where term = $term and subjsec = ? order by day, period");

	while ( my $ref  = $sth->fetchrow_hashref ) { # loop through all courses this term for this teacher.
	    my %s = %$ref;
	    my $subjsec = $s{subjsec};

	    # load each timetable record for this subject.
	    $sth1->execute( $subjsec ) ;
	    if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
	    while ( my ( $day, $period ) = $sth1->fetchrow ) {
		if ( $day > $maxdays ) { $maxdays = $day; }
		push @{ $timetable{$period}{$day} }, $subjsec;
	    }
	}

=head	
	# Testing %timetable
	print qq{<h3>Timetable Hash</h3>\n};
	foreach my $period (keys %timetable ) {
	    foreach my $day ( keys %{ $timetable{$period} } ) {
		print qq{Day:$day Period:$period Courses:}, @{ $timetable{$period}{$day} }, qq{<br>\n};
	    }
	}
=cut


	
	if ( not %timetable ) { next; } # term

	# Tabular Start
	print TEX "\\begin{tabular}{|l|";
	for ( 1 .. $maxdays ) { print TEX "l|" }
	print TEX "}\\hline\n\n";

	# Print Top Row here.
	print TEX "\\rowcolor[gray]{0.92}$lex{Term} $term / ". $lex{Day};

	my $dayvalue;
	for my $day ( 1 .. $maxdays ) { 
	    if ( $g_DaysPerCycle == 5 or $g_DaysPerCycle eq 'W' or $g_DaysPerCycle eq 'w' ) {  # weekly
		$dayvalue = $dow[$day + 1];
	    } else {
		$dayvalue = $day;
	    }
	    print TEX " & $dayvalue"; 
	}
	print TEX "\\\\ \\hline\n";

	my @periods = sort { $a <=> $b } keys %timetable;
	my $maxperiods = $periods[-1];

#	print "Max Days:$maxdays Max Periods:$maxperiods<br>\n";

	
        my %periodrows; # periodrows{period} = number of rows to print (more than 2 if backings)
	for my $period ( 1 .. $maxperiods ) {
	    for my $day ( 1 .. $maxdays ) {
		my $count = @{ $timetable{$period}{$day} };
		if ( $count > $periodrows{$period} ) {
		    $periodrows{$period} = $count;
		}
	    }
	}

=head	
	foreach my $period (keys %periodrows ) {
	    print qq{Period:$period Value:$periodrows{$period}<br>\n};
	}
=cut
	
	# Print 2 or more rows for this
	foreach my $period ( 1 .. $maxperiods ) {

	    my $rows = $periodrows{$period};
	    
	    foreach my $row (1..$rows) {
		
		if ($row == 1 ) {
		    print TEX $lex{Period}. " $period";
		} elsif ( $row == 2 ) {
#		    print TEX "$ptime[$period]";
		}
	    
		foreach my $day ( 1 .. $maxdays ) {
		    
		    my $subjsec = pop @{ $timetable{$period}{$day} };

		    my (%r,$studcount);
		    if ( $subjsec ) {
			# Load subject information.
			$sth2->execute( $subjsec );
			if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
			my $ref = $sth2->fetchrow_hashref;
			foreach my $key ( keys %$ref ) {
			    ($ref->{$key}) = latex_filter($ref->{$key});
			}
			%r = %$ref;
			if ( not $r{smdesc} ) {
			    $r{smdesc} = substr( $r{description}, $maxchars ); # defined at top.
			}

			# Load course enrollment
			$sth3->execute( $subjsec );
			if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
			$studcount = $sth3->fetchrow;
		    }

		    print TEX "& $r{smdesc} $subjsec ($studcount) $r{location}";

		} # end of day loop.
		print TEX "\\\\ \n";
	    }

	    print TEX "\\hline\n"; # line break
		
	} # end of period loop

	print TEX qq{\\end{tabular}\n\\medskip\n\n};
	
    } # end of Term Loop
    
    print TEX qq{\n\\newpage\n}; # new page for next teacher.


} # end of prTimetable




#-----------------
sub showStartPage {
#-----------------


    # Get default papersize
    my $papersize;
    if ( $defaultpapersize ) {
	$papersize = $defaultpapersize;
	$papersize =~ s/paper//; # strip off the 'paper' ending;
	$papersize = ucfirst( $papersize );
    }

    # print sortorder and selection input form.
    print qq{<h1>$title</h1>\n};
    print qq{<div><form action="$self" method="post">\n};
    print qq{<input type="hidden" name="page" value="1">\n};

    print qq{<table cellpadding="3" cellspacing="0" border="0">\n};

=head    
    print qq{<tr><td class="bra">$lex{'Sort by'}</td>\n};
    print qq{<td><select name="sortorder"><option value="name">$lex{'Lastname, Firstname'}</option>\n};
    print qq{<option value="homeroom">$lex{'Homeroom, Lastname, Firstname'}</option>\n};
    print qq{<option value="grade">$lex{'Grade, Lastname, Firstname'}</option>\n};
    print qq{<option value="band">$lex{'Band, Lastname, Firstname'}</option>\n};
    print qq{</select></td></tr>\n};

    print qq{<tr><td class="bra">$lex{Select}</td>\n};
    print qq{<td><select name="select"><option>$lex{Grade}</option>\n};
    print qq{<option>$lex{Homeroom}</option>\n};
    print qq{<option>$lex{Band}</option>\n};
    print qq{<option>$lex{'Student Number'}</option>\n};
    print qq{</select>\n};
    print qq{<input type="text" name="group" size="14"> $lex{'Blank=All'}</td></tr>\n};
=cut

    print qq{<tr><td class="bra">$lex{'Paper Size'}</td>\n};
    print qq{<td align="left"><select name="papersize">};
    if ( $papersize ) { print qq{<option>$papersize</option>\n}; }
    print qq{<option>$lex{Letter}</option>};
    print qq{<option>$lex{A4}</option>};
    print qq{<option>$lex{Legal}</option>};
    print qq{</select></td></tr>\n};

    print qq{<tr><td class="bra">$lex{'Font Size'}</td>\n};
    print qq{<td><select name="fontsize"><option>10</option><option>11</option>\n};
    print qq{<option>12</option></select></td></tr>\n};

#    print qq{<tr><td class="bra">$lex{Orientation}</td>\n};
#    print qq{<td><select name="orientation"><option value="portrait">$lex{Portrait}</option>};
#    print qq{<option value="landscape">$lex{Landscape}</option></select></td></tr>\n};

    
    # Now do the term
    print qq{<tr><td class="bra">$lex{'Term(s)'}</td><td>};
    print qq{<input type="text" name="term" size="12">\n};
    print qq{$lex{'Separate with Spaces'}</td></tr>\n};

    print qq{<tr><td class="cn" colspan="2">};
    print qq{<input type="submit" value="$lex{Continue}"></td></tr>\n};

    print qq{</table></form></div>\n};
    print qq{</body></html>\n};

    exit;
}
