#! /usr/bin/perl
#  Copyright 2001-2012 Leslie Richardson

#  This file is part of Open Admin for Schools.

#  Open Admin for Schools is free software; you can redistribute it 
#  and/or modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2 of 
#  the License, or (at your option) any later version.

my $templatepath = '../../template/';

my %lex = ('Main' => 'Main',
	   'Transport' => 'Transport',
	   'Transportation' => 'Transportation',
	   'Report' => 'Report',
	   'Type' => 'Type',
	   'Schedule' => 'Schedule',
	   'View/Download' => 'View/Download',
	   'View Log File' => 'View Log File',
	   'Cannot open' => 'Cannot open',
	   'Error' => 'Error',
	   'Homeroom' => 'Homeroom',
	   'Sort by' => 'Sort by',
	   'Group by' => 'Group by',
	   'Continue' => 'Continue',
	   'Grade' => 'Grade',
	   'Select by' => 'Select by',
	   'Font Size' => 'Font Size',
	   'Records per Page' => 'Records per Page',
	   'Name' => 'Name',
	   'Checked' => 'Checked',
	   'Paper Size' => 'Paper Size',
	   'Letter' => 'Letter',
	   'Legal' => 'Legal',
	   'A4' => 'A4',
	   'Blank=All' => 'Blank=All',
	   'Separate with Spaces' => 'Separate with Spaces',
	   'Select' => 'Select',
	   'Mode' => 'Mode',
	   'Missing' => 'Missing',
	   'Template' => 'Template',
	   'Creator' => 'Creator',
	   'Home' => 'Home',
	   'Work' => 'Work',
	   'Cell' => 'Cell',
	   'Gr' => 'Gr',
	   'Hr' => 'Hr',
	   'Ph' => 'Ph',
	   'Emergency' => 'Emergency',
	   'Parent' => 'Parent',
	   'Address' => 'Address',
	   'Phone' => 'Phone',
	   'Group' => 'Group',
	   'No Student(s) Found' => 'No Student(s) Found',
	   
	   );


my $defaultRecordsPerPage = 10;

my $self = 'rpttransport.pl';

use DBI;
use CGI;
use Cwd;


# Read config variables
eval require "../../etc/admin.conf";
if ( $@ ) {
    print $lex{Error}. " $@<br>\n";
    die $lex{Error}. " $@\n";
}

eval require "../../lib/liblatex.pl";
if ( $@ ) {
    print $lex{Error}. " $@<br>\n";
    die $lex{Error}. " $@\n";
}

my $q = new CGI;
my %arr = $q->Vars;
print $q->header( -charset, $charset );

# setup for teacher site, if called from there.
if ( getcwd() =~ /tcgi/ ){ # we are in tcgi
    $css = $tchcss;
    $homepage = $tchpage;
    $downloaddir = $tchdownloaddir;
    $webdownloaddir = $tchwebdownloaddir;
}


my ($sec, $min, $hour, $mday, $mon, $year, $wday, 
    $yday, $iddst) = localtime(time);
$year = $year + 1900;
$mon++;
$wday++;
my $currdate = "$dow[$wday], $month[$mon] $mday, $year";


# Page Header
my $title = "$lex{Transportation} $lex{Report}";

print "$doctype\n<html><head><title>$title</title>
<link rel=\"stylesheet\" href=\"$css\" type=\"text/css\">
$chartype\n</head><body>\n";

print "<div>[ <a href=\"$homepage\">$lex{Main}</a> | \n";
print "<a href=\"$schpage\">$lex{Schedule}</a> ]</div>\n";

print "<h1>$title</h1>\n";


# Setup Database access
my $dsn = "DBI:$dbtype:dbname=$dbase";
my $dbh = DBI->connect($dsn,$user,$password);
$dbh->{mysql_enable_utf8} = 1;

if ( not $arr{page} ) {
    showStartPage();
} else {
    delete $arr{page};
}

# show passed vars
#foreach my $key ( sort keys %arr ) { print "K:$key V:$arr{$key}<br>\n"; }
#exit;


# Get the report template...
my $templatefile = $arr{template};

unless ( open ( FH,"<", $templatepath. $templatefile )) {
    print "$lex{Error} $lex{Template}: $!\n";
    die "$lex{Error} $lex{Template}: $!\n";
}

my $formtext;
my $desc = <FH>;

my $modeline = <FH>;
chomp $modeline; # remove trailing CR/LF.
$modeline =~ s/^\s*//; # strip any leading spaces.
my ($mode, $fmtstart,$fmtend) = split /::/, $modeline;

my $header = <FH>;  # header information for both pdf or html.
$header =~ s/^\s*//; # strip any leading spaces

my $recordsPerPage = <FH>;
my $line5 = <FH>; # for future expansion, if required.
# Now load rest of template for layout.
{ local $/; $formtext = <FH>; close FH;}


if ( $arr{recordsperpage} ) { # override values from template
    $recordsPerPage = $arr{recordsperpage}; 
} 
if ( not $recordsPerPage ) {
    $recordsPerPage = $defaultRecordsPerPage;  # from top of script
}



# Create hash for fieldnames from meta.
my $sth = $dbh->prepare("select fieldid, fieldname, required 
 from meta where tableid = ?");
$sth->execute( 'student' );
if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
my %fieldnames = ();
while ( my ( $fieldid, $fieldname, $required ) = $sth->fetchrow ) {
    if ( $required ) { # make it bold
	if ( $mode eq 'html' ) {
	    $fieldname = "<span style=\"font-weight:bold;\">$fieldname</span>";
	} else { # pdf format
	    $fieldname = "{\\bf $fieldname}";
	}
    }
    $fieldnames{$fieldid} = $fieldname;
}

# Now put replacement fieldnames back in.
$formtext =~ s{\<\*(.*?)\*\>}
  { exists( $fieldnames{$1} ) 
	? $fieldnames{$1} 
	: $1
  }gsex;


# Replace other non-meta values in formtext.
my %nonmeta = ();
while ( $formtext =~ m/\<\#(.*)\#\>/g){
    if ( $lex{ $1 } ) { # if we have a translation....
	$nonmeta{$1} =  $lex{$1};
    } else { # no translation.
	$nonmeta{$1} = 'NT:'. $1;
    }
}

# now put them back in...
$formtext =~ s{\<\#(.*?)\#\>}
  { exists( $nonmeta{$1} ) 
	? $nonmeta{$1} 
	: $1
  }gsex;

# Update translation in header.
%lex = ( %lex, %fieldnames ); # join %lex and %fieldnames

#foreach my $key ( keys %lex ) {
#    ( $lex{$key} ) = latex_filter( $lex{$key} );
#}

if ( $header ) {

    $header =~ s{\<\*(.*?)\*\>}
    { exists( $lex{$1} ) 
	? $lex{$1} 
	: $1
	}gsex;
}

# Find all fields , so we only wrap forms around them
# ( faster than doing all fields in the table )
my @fields = ();
while ( $formtext =~ m/\<\@(.*)\@\>/g){
    push @fields, $1;
}

# foreach my $key ( sort keys %arr ) { print "K:$key V:$arr{$key}<br>\n"; }



# Now select Students
my @students = ();

my $ttype = $dbh->quote( $arr{transtype} );
my $select = "where trans_type = $ttype";

# Now group students
my $group = $arr{grouping};


# Set sortorder
my $sortorder =  $arr{sortorder};
if ( $sortorder eq 'name' ) {
  $sortorder = 'lastname, firstname';
} else {
  $sortorder = "$sortorder , lastname, firstname";
}


my @groups = ();
my $sth = $dbh->prepare("select distinct $group, count(*) from student 
 where $group is not NULL and $group != '' group by $group order by $group");
$sth->execute;
if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
while ( my ($grp,$dud) = $sth->fetchrow ) {
   push @groups, $grp;
}

# print "Select:$select<br>Group:$group<br>Sortorder:$sortorder  Groups:", @groups, "<br>\n";

if ( not @groups ) {
    print "<h1>$lex{'No Student(s) Found'}</h1>\n";
    print "</body></html>\n";
    exit;
}

$sth = $dbh->prepare("select studnum from student $select and $group = ? 
 order by $group, $sortorder");
if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }

foreach my $grp ( @groups ) {

    $sth->execute( $grp );
    if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
    while ( my $studnum = $sth->fetchrow ) {
	push @students, $studnum;
    }
}



# Now print output based on mode (html or pdf)
if ( $mode eq 'pdf' ) {
    printPDF($group);

} elsif ( $mode eq 'html' ) {
    printHTML($group);

} else { # error;
    print "<h3>$lex{Error}:$lex{Mode} $lex{Missing}</h3>\n";
    print "</body></html>\n";
    exit;
}




#-----------
sub printPDF {
#-----------

    my $groupfield = shift;

    # Start TEX Section.
    my $shortname = "transport$$";
    my $filename = "$shortname.tex";

    open(TEX,">$filename") || die $lex{'Cannot open'}. " tex file\n";

    # Set Paper Size, text width and height, fontsize
    my ( $papersize, $textwidth, $textheight );

    if ( $arr{papersize} eq 'letter' ) {
	$papersize = 'letterpaper';
	$textwidth = $g_letterpaper_textwidth;
	$textheight = $g_letterpaper_textheight;
    } elsif ( $arr{papersize} eq 'legal' ) {
	$papersize = 'legalpaper';
	$textwidth = $g_legalpaper_textwidth;
	$textheight = $g_legalpaper_textheight;
    } elsif ( $arr{papersize} eq 'a4' ) {
	$papersize = 'a4paper';
	$textwidth = $g_a4paper_textwidth;
	$textheight = $g_a4paper_textheight;
    } 
    # print "Papersize: $papersize  TW: $textwidth  TH: $textheight<br>\n";
    # Not required here, perhaps
    # $textheight =~ s/mm//;# $textheight += 16;# $textheight .= 'mm';

    my $fontsize = $arr{fontsize}. 'pt';

    print TEX "\\documentclass[ $fontsize, $papersize ]{article}
\\usepackage{newcent,graphicx,array,colortbl,inputenc,multicol}
$a_latex_header
\\renewcommand{\\familydefault}{\\sfdefault}
\\usepackage[pdftex,bookmarks=false,pdfstartview=FitH]{hyperref}
\\setlength{\\textwidth}{ $textwidth }
\\setlength{\\textheight}{ $textheight }
\\setlength{\\hoffset}{-20mm}
\\setlength{\\voffset}{-25mm}
\\setlength{\\topmargin}{0mm}
\\setlength{\\evensidemargin}{0mm}
\\setlength{\\oddsidemargin}{0mm}
\\setlength{\\parindent}{0mm}
\\setlength{\\extrarowheight}{4pt}
\\begin{document}
\\pagestyle{empty}\n";


    print TEX "{\\LARGE $schoolname }$lex{Transportation} $lex{Report}";
    print TEX " - $lex{Type}:$arr{transtype}\\hfill $currdate\n\\bigskip\n\n";


    # Create hash for fieldnames from meta.
    my $sth1 = $dbh->prepare("select fieldid, fieldname
      from meta where tableid = ?");
    $sth1->execute( 'student' );
    if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
    my %fieldnames = ();
    while ( my ( $fieldid, $fieldname ) = $sth1->fetchrow ) {
	$fieldnames{$fieldid} = $fieldname;
    }


    my $sth = $dbh->prepare("select * from student where studnum = ?");



    my $first = 1;
    my $currgroup = -1;
    my $counter = 1;


    foreach my $studnum ( @students ) {

	$sth->execute( $studnum );
	if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
	my $ref = $sth->fetchrow_hashref;
    
	my %rec = %$ref;
	foreach my $key ( keys %rec ) {
	    ( $rec{$key} ) = latex_filter( $rec{$key});
	}

	$oldgroup = $currgroup;
	$currgroup = $rec{ $groupfield };

	# put in extra page break if we are starting a new room/grade
	if ( $currgroup ne $oldgroup and not $first ) {
	    if ( $fmtend ) { print TEX $fmtend, "\n"; }
	    print TEX "\\newpage\n";
            print TEX "{\\large\\bf $lex{Group} $fieldnames{$groupfield}: $currgroup} \n\n";
	    if ( $fmtstart ) { print TEX $fmtstart, "\n"; }
	    if ( $header ) { print TEX $header, "\n";}

	    $counter = 1;  # Set counter back to one to print x recs/page

	} 

        if ( $first ) {
           print TEX "{\\large\\bf $lex{Group} $fieldnames{$groupfield}: $currgroup} \n\n";
           if ( $fmtstart ) {
	      print TEX $fmtstart, "\n";
           }

           $header =~ s/\_//g; # filter underscores
           if ( $header ) {
	      print TEX $header, "\n";
           }  

           $first = 0; 
        }
	
	my $text = $formtext; # make a new blank record to plug in...

	# Now put replacement text back in.
	$text =~ s{\<\@(.*?)\@\>}
  	  { $rec{$1} }gsex;

	$text =~ s/\_/ /g; # remove underscores in field names.

	print TEX $text,"\n";


	if ( not ( $counter % $recordsPerPage)) { 
	    if ( $fmtend ) { print TEX $fmtend, "\n"; }
	    print TEX "\\newpage\n";
            print TEX "{\\large\\bf $lex{Group} $fieldnames{$groupfield}: $currgroup} \n\n";
	    if ( $fmtstart ) { print TEX $fmtstart, "\n"; }
	    if ( $header ) { print TEX $header, "\n";}
	} 


	$counter++;

    }  # End of For loop


    if ( $fmtend ) {
	print TEX $fmtend;
    }
    print TEX "\\end{document}";
    close TEX;

    system("$pdflatex $filename > pdflog$$.txt");
    system("mv $shortname.pdf $downloaddir");
    system("mv pdflog$$.txt $downloaddir");
    system("rm $shortname.*");

    print "<h1><a href=\"$webdownloaddir/$shortname.pdf\">";
    print "$lex{'View/Download'} $lex{Transportation} $lex{Report}</a></h1>\n";
    print "[ <a href=\"$homepage\">$lex{Main}</a> | \n";
    print "<a href=\"$webdownloaddir/pdflog$$.txt\">$lex{'View Log File'}</a> ]\n";
    print "</body></html>\n";


} # end of printPDF.



#------------
sub printHTML {
#------------

    my $groupfield = shift;

    print "<h1>". $lex{Transportation}. q{ }. $lex{Report}. " - $schoolname</h1>\n";
    print "<h3>$currdate</h3>\n";

    if ( $fmtstart ) {
	print $fmtstart, "\n";
    }
    if ( $header ) {
	print $header, "\n";
    }

    my $sth = $dbh->prepare("select * from student where studnum = ?");

    my $first = 1;
    my $currgroup = -1;
    my $counter = 1;

    foreach my $studnum ( @students ) {

	$sth->execute( $studnum );
	if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
	my $ref = $sth->fetchrow_hashref;
	my %rec = %$ref;

	$oldgroup = $currgroup;
	$currgroup = $rec{ $groupfield };

	# put in extra page break if we are starting a new room/grade
	if ( $currgroup ne $oldgroup and not $first ) {
	    print "</table><p></p><table cellspacing=\"0\" cellpadding=\"3\" border=\"1\">\n";
	} else { $first = 0; }
	
	my $text = $formtext; # make a new blank record to plug in...

	# Now put replacement text back in.
	$text =~ s{\<\@(.*?)\@\>}
  	  { $rec{$1} }gsex;

	print $text,"\n";

    }  # End of record printing loop

    print "</table>";
    print "[ <a href=\"$homepage\">$lex{Main}</a> ]\n";
    print "</body></html>\n";


}




#----------------
sub showStartPage {
#----------------

    # Load Meta Values, possibly translated
    my $sth = $dbh->prepare("select fieldid, fieldname, required from meta where tableid = ?");
    $sth->execute('student');
    if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
    my %fieldnames = ();
    while ( my ( $fieldid, $fieldname, $required ) = $sth->fetchrow ) {
	$fieldnames{$fieldid} = $fieldname;
    }


    # Start Form
    print "<form action=\"$self\" method=\"post\">\n";
    print "<input type=\"hidden\" name=\"page\" value=\"1\">\n";
    print "<table cellpadding=\"3\" cellspacing=\"0\" border=\"0\">\n";

    # Open the Report Templates
    my @files = glob($templatepath."rpttransport*.tpl");
    my %templates = ();

    for my $tplfile ( @files ) {

	# read each label file and get description
	unless ( open (FH,"<$tplfile")) {
	    print "$lex{Error} $lex{Template}: $!\n";
	    die "$lex{Error} $lex{Template}: $!\n";
	}

	# read first 2 lines of the template only.
	my $desc = <FH>;
	chomp $desc;
	$desc =~ s/\[//g;  # strip any opening square labels, just in case.

	my $modeline = <FH>;
	$modeline =~ s/^\s*//; # strip any leading spaces
	my ($mode, $fmtstart,$fmtend) = split(/::/, $modeline );
	# ignore rest of the file, not required.

	$tplfile =~ s/^.*\///; # strip leading path
	$templates{"$desc ($mode)"} = $tplfile;
    }


    print "<tr><td class=\"bra\">$lex{Select} $lex{Report}</td>\n";
    print "<td><select name=\"template\">\n";
    foreach my $desc ( sort keys %templates ) {
	print "<option value=\"$templates{$desc}\">$desc</option>\n";
    }
    print "</select></td></tr>\n";


    # Do the select (By Type) - ie. Bus, Walk, Ride.
    $sth = $dbh->prepare("select distinct trans_type from student 
      where trans_type is not NULL and trans_type != '' order by trans_type");
    $sth->execute;
    if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }

    
    # Transport Type
    print "<tr><td class=\"bra\">$lex{Select} $lex{Transport} $lex{Type}</td>\n";
    print "<td>";

    my $first = 1;
    my $checked = q{checked="checked"};
    while ( my $type = $sth->fetchrow ) {
	print "<input type=\"radio\" name=\"transtype\" value=\"$type\" $checked> $type \n";
	$checked = '';
	$first = 0;
    }
    if ( $first ) { # put in at least a bus type }
	print "<input type=\"radio\" name=\"transtype\" value=\"bus\" $checked> Bus\n";
    }
    print "</td></tr>\n";



    # Grouping Order
    print "<tr><td class=\"bra\">$lex{'Group by'}</td>\n";
    print "<td><select name=\"grouping\">";
    print "<option value=\"trans_name\">$fieldnames{'trans_name'}</option>\n";
    print "<option value=\"trans_am_route\">$fieldnames{'trans_am_route'}</option>\n";
    print "<option value=\"trans_pm_route\">$fieldnames{'trans_pm_route'}</option>\n";
    print "<option value=\"trans_bus_stop_location\">$fieldnames{'trans_bus_stop_location'}</option>\n";
    print "</select></td></tr>\n";

    # Sort Order
    print "<tr><td class=\"bra\">$lex{'Sort by'}\n";
    print "</td><td><select name=\"sortorder\">";
    print "<option value=\"name\">$lex{Name}</option>\n";
    print "<option value=\"grade\">$lex{Grade}</option>\n";
    print "<option value=\"homeroom\">$lex{Homeroom}</option>\n";
    print "<option value=\"trans_pickup_time\">$fieldnames{'trans_pickup_time'}</option>\n";
    print "<option value=\"trans_dropoff_time\">$fieldnames{'trans_dropoff_time'}</option>\n";
    print "<option value=\"trans_bus_stop_location\">$fieldnames{'trans_bus_stop_location'}</option>\n";
    print "</select></td></tr>\n";

    # Paper Size
    print "<tr><td class=\"bra\">$lex{'Paper Size'}</td><td>";
    print "<select name=\"papersize\">\n";
    print "<option value=\"letter\">$lex{Letter}</option>\n";
    print "<option value=\"legal\">$lex{Legal}</option>\n";
    print "<option value=\"a4\">$lex{A4}</option>\n";
    print "</select></td></tr>\n";

    # Font Size
    print "<tr><td class=\"bra\">$lex{'Font Size'}</td>\n";
    print "<td><select name=\"fontsize\"><option>10</option><option>11</option>\n";
    print "<option>12</option></select></td></tr>\n";

    # Records per page (max)
    print "<tr><td class=\"bra\">$lex{'Records per Page'}\n";
    print "</td><td><input type=\"text\" size=\"3\" name=\"recordsperpage\" ";
    print "value=\"$recordsPerPage\"></td></tr>\n";

    # Continue
    print "<tr><td class=\"cn\" colspan=\"2\">\n";
    print "<input type=\"submit\" value=\"$lex{Continue}\"></td></tr>\n";
    print "</table></form>\n";

    print "</body></html>\n";

    exit;

}

