#!/usr/bin/perl
#  Copyright 2001-2016 Leslie Richardson

#  This file is part of Open Admin for Schools.

#  Open Admin for Schools is free software; you can redistribute it 
#  and/or modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2 of 
#  the License, or (at your option) any later version.

my $self = 'tt_report1.pl';

my %lex = ('View' => 'View',
	   'Main' => 'Main',
	   'Timetable' => 'Timetable',
	   'No Record(s) Found' => 'No Record(s) Found',
	   'Grade' => 'Grade',
	   'Term' => 'Term',
	   'Delete Entries' => 'Delete Entries',
	   'No Value' => 'No Value',
	   'Period' => 'Period',
	   'Day' => 'Day',
	   'Term' => 'Term',
	   'Error' => 'Error',
	   'Course' => 'Course',
	   'Missing' => 'Missing',
	   'Enrollments' => 'Enrollments',

	   );

use DBI;
use CGI;
use Cwd;

# Set prepath for config file: (/tcgi or cgi/schedule)
my $prepath = '../..';
if (getcwd() =~ /tcgi/){ # we are in tcgi
    $prepath = '..';
}

eval require "$prepath/etc/admin.conf";
if ( $@ ) {
    print $lex{Error}. ": $@<br>\n";
    die $lex{Error}. ": $@\n";
}

my $q = new CGI;
print $q->header( -charset, $charset );
my %arr = $q->Vars;

# Get current dir so know what CSS to display;
if (getcwd() =~ /tcgi/){ # we are in tcgi
    $css = $tchcss;
    $homepage = $tchpage;
}

my $dsn = "DBI:$dbtype:dbname=$dbase";
my $dbh = DBI->connect($dsn,$user,$password);
$dbh->{mysql_enable_utf8} = 1;

my ($select, $term);
if ($arr{term} =~ m/\d/) { # must be a digit
    $term = $arr{term};
    my $trm = $dbh->quote($arr{term});
    $select = "where term = $trm";
}


# print page header.
my $title = "$lex{View} $lex{Timetable}";
print "$doctype\n<html><head><title>$title</title>
<link rel=\"stylesheet\" href=\"$css\" type=\"text/css\">
$chartype\n</head><body>\n";

print "[ <a href=\"$homepage\">$lex{Main}</a> \n";
unless (getcwd() =~ /tcgi/){ # unless we are in tcgi
    print "| <a href=\"$schpage\">$lex{Timetable}</a> \n";
}
print qq{]\n};
print qq{<div style="clear:both;margin-bottom:1em;"></div>\n};


my (%course, %timetable, %noenrol, %nottbl, %grades);

my $sth1 = $dbh->prepare("select distinct term, count(*) from schedat where subjsec = ?");
my $sth2 = $dbh->prepare("select count(distinct studnum) from eval where subjcode = ?");


my $sth = $dbh->prepare("select subjsec, description, grade, startrptperiod, endrptperiod from subject");
$sth->execute;
if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr; }
my $maxterm = 1; # max term value for looping through timetable entries.

while ( my ( $subjsec, $desc, $grade, $startterm, $endterm ) = $sth->fetchrow ) {

    if ( $endterm > $maxterm ) { $maxterm = $endterm; }

    $course{$subjsec}{desc} = $desc;
    $course{$subjsec}{start} = $startterm;
    $course{$subjsec}{end} = $endterm;
    $course{$subjsec}{grade} = $grade;
    $grades{$grade} = 1;

    # Timetable Term, Count
    $sth1->execute($subjsec);
    if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr; }
    my $tfirst = 1;
    while ( my ($term, $tcount) = $sth1->fetchrow ) {
	$timetable{$subjsec}{$term} = $tcount;
	$tfirst = 0;
    }
    if ( $tfirst ) { $nottbl{$subjsec} = 1; } # track if no timetable done.

    # Enrollment Count
    $sth2->execute($subjsec);
    if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr; }
    my $ecount = $sth2->fetchrow;
    if (not $ecount ) { 
	$noenrol{$grade}{$subjsec} = 1; # track if no student enrollments
    } else {
	$course{$subjsec}{enrol} = $ecount;
    }
}


# Loop Through Each Grade, and Display Courses and Timetable Values.
foreach my $grade ( sort {$a <=> $b} keys %grades ) {

    print qq{<table cellpadding="3" cellspacing="0" border="1" style="float:left;">\n};
    print qq{<caption style="font-weight:bold;">$lex{Grade} $grade</caption>\n};
    print qq{<tr><th>$lex{Course}</th><th colspan="$maxterm">Term / Timetable Rec</th></tr>\n};
    print qq{<tr><th></th>};
    foreach my $trm ( 1..$maxterm ) {
	print qq{<th>$trm</th>};
    }
    print qq{</tr>\n};

    # Find courses in this grade and alphabetize, skip any without enrollments.
    my %gc; # ie. grade courses
    foreach my $subjsec ( keys %course ) {
	if ( $noenrol{$subjsec} ) { next; } # skip no enrollments.
	# print "Subjsec:$subjsec - $course{$subjsec}{grade}<br>\n";
	if ( $course{$subjsec}{grade} eq $grade ) {
	    my $desc = $course{$subjsec}{desc};
	    $gc{"$desc$subjsec"} = $subjsec;
	}
    }

   

    # Now use this %gc hash to go through this grades courses
    foreach my $key ( sort keys %gc ) {
	my $subjsec = $gc{$key};

	print qq{<tr><td>$course{$subjsec}{desc} ($subjsec)</td>};
	foreach my $trm ( 1..$maxterm ) {
	    print qq{<td>$timetable{$subjsec}{$trm}</td>};
	}
	print qq{</tr>\n};
    }
    print qq{</table>\n};

    # Display No Enrollments for this grade
    if ( $noenrol{$grade} ) {
	print qq{<table cellpadding="3" cellspacing="0" border="1" style="float:left;">\n};
	print qq{<caption style="font-weight:bold;">$lex{Missing} $lex{Enrollments}</caption>\n};
	print qq{<tr><th>$lex{Course}</th></tr>\n};
	foreach my $subjsec ( keys %{ $noenrol{$grade} } ) {
	    print qq{<tr><td>$course{$subjsec}{desc} ($subjsec)</td></tr>\n};
	}
	print qq{<table>\n};
    }

    print qq{<div style="clear:both;margin-bottom:1.5em;"></div>\n};

}

print "</body></html>\n";



# NOT USED IN THIS VERSION
#-----------------
sub printTimetable { # print timetable for 1 grade for 1 term.
#-----------------

    my ($grade, $term) = @_;  # passed grade and term.
    print "<p style=\"font-size: 150%;\">$lex{Grade}: $grade  $lex{Term}: $term\n";
    print "[ <a href=\"schedel.pl?gr=$grade&trm=$term\">$lex{'Delete Entries'}</a> ]</p>\n";

    my $sth = $dbh->prepare("select day, period, subjsec from schedat 
      where term = ? and grade = ? order by period, day");
    $sth->execute($term, $grade);

    # Create 2D subject array (of [period][day] indexes)
    my @subject = ();
    while ( my ($day, $period, $subjsec) = $sth->fetchrow ) {
	$subject[$period][$day] .= "$subjsec ";
    }

    my $tblrows = $#subject;
    my $tblcols;
    for my $i ( 1 .. $tblrows ) {
	if ($#{$subject[$i]} > $tblcols){ $tblcols = $#{$subject[$i]};}
    }
    $sth = $dbh->prepare("select description from subject where subjsec = ?");

    print qq{<table cellpadding="3" cellspacing="0" border="1">\n};

    # heading
    print "<tr><th></th>";
    for my $day ( 1 .. $tblcols ){ print "<th>$lex{Day} $day</th>"; }
    print "</tr>\n";
    
    # Main body of table.
    for my $i ( 1 .. $tblrows ) {
	print "<tr><td>$lex{Period} $i</td>";
	for ( 1 .. $tblcols ) {
	    if ( $subject[$i][$_] ){ # if we have a subjsec value;
		my $firstflag = 1;
		@subjval = split /\s/,$subject[$i][$_];

		print "<td>";
		foreach my $sv ( @subjval ) {
		    if (not $firstflag){ print "<br>"; } else { $firstflag = 0;}

		    $sth->execute($sv);
		    if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr;}	    
		    my $desc = $sth->fetchrow;
		    print "$desc ($sv)";

		}
		print "</td>";

	    } else {
		print "<td style=\"font-color: red;\">". $lex{'No Value'}. "</td>";
	    }
	}
	print "</tr>\n";
    }
    print "</table>\n";

} # End of printTimetable
