#!/usr/bin/perl
#  Copyright 2001-2017 Leslie Richardson

#  This file is part of Open Admin for Schools.

#  Open Admin for Schools is free software; you can redistribute it 
#  and/or modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2 of 
#  the License, or (at your option) any later version.

# Passed Values: Subject/Section value (passed as subjsec). 

# change right side values for alternate languages, let me know of any mistakes
%lex = ('All' => 'All',
	'All Groups' => 'All Groups',
	'All Terms' => 'All Trm',
	'Please Log In' => 'Please Log In',
	'No User Id' => 'No User Id',
	'No Password' => 'No Password',
	'Select Subject' => 'Select Subject',
	'Delete' => 'Delete',
	'Edit' => 'Edit',
	'Add' => 'Add',
	'Assessment Item' => 'Assessment Item',
	'Del/Ed Item' => 'Del/Ed Item',
	'Reports' => 'Reports',
	'Rpt' => 'Rpt',
	'Adjust the weights of individual assessment items' => 
 	  'Adjust the weights of individual assessment items',
	'Weight Item' => 'Weight Item',
	'Adjust the weightings of groups of assessment items' =>
	  'Adjust the weightings of groups of assessment items',
	'Weight Grp' => 'Weight Grp',
	'Post (copy) student averages to the report card system' =>
	  'Post (copy) student averages to the report card system',
	'Post' => 'Post',
	'Export these grades to a CSV file' => 
	  'Export these grades to a CSV file',
	'Exp' => 'Exp',
	'Maintenance: Sortorder, Withdraw, etc' => 
	'Maintenance: Sortorder, Withdraw, etc',
	'Mtce' => 'Mtce',
	'Edit multiple scores at the same time' =>
	  'Edit multiple scores at the same time',
	'MultiEdit' => 'MultiEdit&nbsp;',
	'No Maximum Score for Test' => 'No Maximum Score for Test',
	'Enter Gradebook' => 'Enter Gradebook',
	'Main' => 'Main',
	'Search' => 'Search',
	'Error' => 'Error',
	'Search: Scores, Averages' => 'Search: Scores, Averages',
	'Cookie Duration' => 'Cookie Duration',
	'User Id' => 'User Id',
	'Password' => 'Password',
	'Log Out' => 'Log Out',
	'min' => 'min',
	'Assessments' => 'Assessments',
	'Test Weight' => 'Test Weight',
	'Test Average' => 'Test Average',
	'Avg' => 'Avg',
	'Version' => 'Version',
	'Gradebook' => 'Gradebook',

	);

my $self = 'gbmain.pl';
my $version = '2011-01-29';
my $specchar = '*';  # causes this test to be skipped in calculating avg.

my $minimumtime = 1;  # Minimum time for cookie duration.
my $maximumtime = 40;  # Max cookie time.
my $defaulttime = 20;  # Default cookie time.

my $altcolor = '#3B7'; #1B6 Complement to the 063 green of teacher site.
my $maincolor = '#063'; 

use DBI;
use CGI::Session;
use CGI qw/:escape unescape/;
use Number::Format qw(round);

my $q = CGI->new;
my %arr = $q->Vars; # Get passed values

eval require "../../etc/admin.conf";
if ( $@ ) {
    print $lex{Error}. " $@<br>\n";
    die $lex{Error}. " $@<br>\n";
}

eval require "../../etc/gbook.conf";
if ( $@ ) {
    print $lex{Error}. " $@<br>\n";
    die $lex{Error}. " $@<br>\n";
}

# needed to $additionalcomments subject so that it can be skipped.
eval require "../../etc/repcard.conf";
if ( $@ ) {
    print $lex{Error}. " $@<br>\n";
    die $lex{Error}. " $@<br>\n";
}

my $dbtype = 'mysql';
my $dsn = "DBI:$dbtype:dbname=$dbase";
my $dbh = DBI->connect($dsn,$user,$password);
$dbh->{mysql_enable_utf8} = 1;

my ($sec, $min, $hour, $mday, $mon, $year, $wday, 
 $yday, $iddst) = localtime(time);
$year = $year + 1900;
$mon++;
$wday++;
$currdate = "$dow[$wday], $month[$mon] $mday, $year";


# Define Userid
my $userid;
if ($arr{userid}) {
    $userid = $arr{userid};
}

# define term. This will pass through session setup and be defined in
# session if it exists, and otherwise will be read from session. If it
# doesn't exist in the session, it will be set to the $lex{'All Terms'}
# value.
my $term;  # current term
if ( $arr{term} ) {
    $term = $arr{term};
}


# Define View and Subview
my ($view, $subview);

if ($arr{vw} and $arr{subview}){
    $view = $arr{vw}; 
    $subview = $arr{subview};
} elsif ($arr{vw}) { # only is defined, split it.
    my ($g,$sg) = split(/:/,$arr{vw});
    if ($sg){ # vw contains a subgroup...
	$subview = $sg;
	$view = $g;
    } else { # no subgroup
	$view = $arr{vw};
	$subview = $lex{All};
    }
}

# Define $subjsec
my $subjsec;
if ($arr{subjsec}){
    $subjsec = $arr{subjsec};
}


$session = new CGI::Session("driver:mysql;serializer:FreezeThaw",
 undef,{Handle => $dbh}) or die CGI::Session->errstr;

# Check for Logout
if ($arr{logout}){
    $session->delete;
    $session->flush;
    print $q->header( -charset, $charset );
    login( $lex{'Please Log In'} );
    print "</body></html>\n";
    exit;
}


# Get/Set  Session Values (a defined userid means it was passed)
if ($userid){ # we want to login, passed userid/password pair.

    # Check password/userid against database (-1 no user, -2 wrong password;
    my $error = &checkPassword($arr{userid}, $arr{password});
    if ($error == -1){ print $q->header( -charset, $charset ); &login($lex{'No User Id'}); die;}
    if ($error == -2){ print $q->header( -charset, $charset ); &login($lex{'No Password'}); die; }

    $cookietime = checkCookieTime( $arr{duration} );

    # Set values for userid and logged_in in session
    $session->param('logged_in','1');
    $session->expire('logged_in',$cookietime);
    $session->param('userid',$arr{userid});
    $session->param('duration', $cookietime );

} else { # check logged_in value in session
    my $logged_in = $session->param(logged_in);
    if (not $logged_in){
        print $q->header( -charset, $charset );
        login( $lex{'Please Log In'} ); 
        exit;
    }
    # Ok, we have a login. Values below we have in environment.

    $userid = $session->param('userid');
    $duration = $session->param('duration');

    if ( not ( $duration =~ m/\+/)) { # add + and m if not present
	$duration = checkCookieTime( $duration );
    }
    $session->expire('logged_in',$duration);

} # End of check for logged_in value


# Read or Write Session for views
if ($view){ # could be reset, if so update session.
    $session->param('view',$view);
} else { $view = $session->param('view');}

if ($subview){ 
    $session->param('subview',$subview);
} else { $subview = $session->param('subview');}


# Read/Write session for term
if ($term){ # if already defined we use that to set session value
    $session->param('term',$term); 
} else { # term not defined. try to check from session.  
    my $tempterm = $session->param('term'); 
    if (not $tempterm) { # not in session either...  
	# Set session and term to all terms.
	$session->param('term',$lex{'All Terms'}); 
	$term = $lex{'All Terms'}; 
    } else { # we have a session value... use it;
	$term = $tempterm; 
    } 
} # $term is now defined from passed value, session, or to default allterms.


# Read/Write Session for subjsec
if ($subjsec){ 
    $session->param('subjsec',$subjsec);
} else { 
    $subjsec = $session->param('subjsec');
}

$session->flush;
print $session->header( -charset, $charset );


# Get the subject description, grade information
$sth = $dbh->prepare("select description,smdesc, grade from subject where subjsec = ?");
$sth->execute( $subjsec );
if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr;}
my ($description,$smdesc, $grade) = $sth->fetchrow;

# Set the grade and track.
my $track = $g_MTrackTermType{$grade};

# Set the Dates for test selection by term.
my ($dstart,$dend); # Start and End dates
if ($term ne $lex{'All Terms'}) { # set selection dates
    my $trm = $term;
    $trm =~ s/\D//g; # strip non-digits cruft.
    $dstart = $g_MTrack{$track}{$trm}{'start'};
    $dend = $g_MTrack{$track}{$trm}{'end'};
}



# Get the other subjects desc and subjsec; stuff into 'subjects' array.
$sth = $dbh->prepare("select description, smdesc, subjsec, subjcode from subject 
  where teacher = ? order by description, section");
$sth->execute( $userid );
if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr;}
while ( my ($description, $smdesc, $subjsec, $subjcode ) = $sth->fetchrow){

    # skip Additional Comments / Supress Subjects  from repcard configuration
    if ( $r_AdditionalComments{$subjsec}  or $r_AdditionalComments{$subjcode} ) { next; }
    if ( $r_SupressSubject{$subjsec} or $r_SupressSubject{$subjcode} ) { next; }


    if ( not $smdesc ) { $smdesc = substr($description,0,15); }
    $smdesc =~ s/\(//g; # strip opening brackets.
    $description =~ s/\(//g;

    push @subjects,$subjsec;
    $smdesc{$subjsec} = $smdesc;
    $desc{$subjsec} = $description;

}

# Creates array controlling student sort order.
my @studnum = mkStudNum($subjsec);


# Now lookup tests for this class.
$sth = $dbh->prepare("select name, id, score, weight, description, grp 
  from gbtest where subjsec = ? order by tdate");
$sth->execute( $subjsec );
if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr;}
while (my ($name,$id,$score,$weight,$desc,$grp) = $sth->fetchrow){
    # skip the 'sortorder' test
    if ($name eq 'sortorder'){ next; }
    my $sg;
    ($grp,$sg) = split /:/,$grp;
    $name =~ s/://g; # strip colons, if any.
    push @test, "$name:$id:$score:$desc:$grp:$sg";
    $weight{$id} = $weight; # used for calculating averages.
    $maxscore{$id} = $score;
}
# Now done putting in the tests.

# Load the group weights from the current subject;
$sth = $dbh->prepare("select markscheme from subject where subjsec = ?");
$sth->execute( $subjsec );
if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr; }
my $markscheme = $sth->fetchrow;

# build a %groupweight hash from values in markscheme.
my @fields = split (/[\n|\r]/, $markscheme);
my %groupweight;
foreach my $fld (@fields) { 
    if ($fld) {
	#print "F:$fld<br>\n";
	my ($grp, $percent) = split(/=/, $fld);
	$groupweight{$grp} = $percent;
    }
}

# Find the groups for this subject, and push into a group array
#  Also create a $groupref to point to subgroups of a top group.
#  ie. $groupref{term1} -> test,homework,labs as $groupref{term1}[0] = test
$sth = $dbh->prepare("select distinct grp 
  from gbtest where subjsec = ? and grp != '' order by grp");
$sth->execute( $subjsec );
if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr;}

my $currgroup = '-1';
my $prevgroup;
my @group;

while (my $grp = $sth->fetchrow){
    if (not $grp){ next;}
    my ($topgroup,$subgroup) = split(/:/,$grp);

    $prevgroup = $currgroup;
    $currgroup = $topgroup;

    if ($subgroup){ # push into hash of array elements
	push @{$groupref{$topgroup}},$subgroup;
    }
    if ($prevgroup ne $currgroup){
	push @group, $topgroup; 
    }
}


# Heading of Document
print "$doctype\n<html><head><title>$lex{Gradebook} $lex{Main}</title>
 <link rel=\"stylesheet\" href=\"$tchcss\" type=\"text/css\" media=\"screen\">
 <style type=\"text/css\" media=\"print\">
 body,html { font-family: Arial, Helvetica, sans-serif; font-size: 10pt;}
 </style>
<style type=\"text/css\">input { padding:0;margin:0; } </style>
</head><body>\n";



# Subject Select (if not already done)
if ( not $subjsec ){
    print "[ <a href=\"$tchpage\">$lex{Main}</a> ]<br>\n";
    print "<div style=\"padding:0 4em;\">\n";
    print "<h3>$lex{'Select Subject'}</h3>";

    foreach my $subjsec ( @subjects ){ # subject with full descriptions
	print "<form action=\"$self\" method=\"post\" style=\"padding:0.2em;\">\n";
	print "<input type=\"hidden\" name=\"subjsec\" value=\"$subjsec\">\n";
	print "<input type=\"submit\" value=\"$desc{$subjsec} ($subjsec)\">\n";
	print "</form>\n";
    }


    print "</div></body></html>\n";
    exit;
}

#foreach my $key (keys %arr) { print "K: $key V:$arr{$key}<br>\n"; }


# Selection Form for New Subject or Group/Subgroup
print qq{<form action="$tchpage" method="get" style="margin:0;padding:0;display:inline;">\n};
print qq{<table cellpadding="2" cellspacing="0" border="0" style="display:inline;">\n};
print qq{<tr><td>};
print qq{<input type="submit" value="$lex{Main}" style="padding:0;margin:0;display:inline;">};
print qq{</td></tr></table></form>\n};


print qq{<form style="margin:0;padding:0;display:inline;" action="$self" method="post">\n};
print qq{<table cellpadding="2" cellspacing="0" border="0" style="display:inline;">\n};

print qq{<tr><td>};
print qq{<input type="submit" value="$lex{'Select Subject'}"></td>\n};

print qq{<td><select name="subjsec">\n};
print qq{<option value="$subjsec">$smdesc{$subjsec} ($subjsec)</option>\n};

foreach my $subj ( @subjects ){
    if ( $subj eq $subjsec ) { next; }
    print qq{<option value="$subj">$smdesc{$subj} ($subj)</option>\n};
}
print qq{</select></td>\n};

# Print Group Options
print qq{<td><select name="vw">};
if ( defined $view ){
    print qq{<option>$view</option><option>$lex{'All Groups'}</option>\n};
} else { # just put up all groups view...
    print qq{<option>$lex{'All Groups'}</option>\n};
}
foreach my $gp (@group){ 
    if ($gp eq $view) { next; }
    print qq{<option>$gp</option>};
}
print qq{</select></td>\n};


# Print Subgroup Options
if ($groupref{$view}) {
    print qq{<td><select name="subview">\n};
    if ($subview ne $lex{All}){ print qq{<option>$subview</option>\n}; }
    print qq{<option>$lex{All}</option>\n};
    foreach my $sg ( @{$groupref{$view}} ){ 
	print qq{<option>$sg</option>};
    }
    print qq{</select></td>\n};
}


# Print Term Options
print qq{<td><select name="term">\n};
if ($term eq $lex{'All Terms'}) {
    print qq{<option>$lex{'All Terms'}</option>\n};
    
} else {
    print qq{<option value="$term">$g_TermDisplay{$track}{$term}</option>\n};
    print qq{<option>$lex{'All Terms'}</option>\n};
}


#  Get the terms.
$sth = $dbh->prepare("select distinct term from eval where subjcode = ? order by term");
$sth->execute( $subjsec );
if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr;}
while ( my $trm = $sth->fetchrow ) {
    print qq{<option value="$trm">$g_TermDisplay{$track}{$trm}</option>};
}
print qq{</select></td>\n};


# Display Subject Name and end Form
print qq{<td style="color:$maincolor;">};
print qq{ $description</td></tr></table></form>\n};



## Print Methods Line ##
print "<div id=\"methods\" style=\"padding:4px;\">\n";

print "<form action=\"testadd.pl\" method=\"post\" style=\"display:inline;padding:0;margin:0;\" 
 title=\"$lex{Add} $lex{'Assessment Item'}\">\n";
print "<input type=\"submit\" value=\"$lex{Add} $lex{'Assessment Item'}\" style=\"padding:0;margin:0;\">\n";
print "</form>\n";

print "<form action=\"testdeled.pl\" method=\"post\" style=\"display:inline;\"
 title=\"$lex{Delete}/$lex{Edit} $lex{'Assessment Item'}\">\n";
print "<input type=\"submit\" value=\"$lex{'Del/Ed Item'}\"></form>\n";

print "<form action=\"rptmenu.pl\" method=\"post\" style=\"display:inline;\"
 title=\"$lex{Gradebook} $lex{Reports}\">\n";
print "<input type=\"submit\" value=\"$lex{Rpt}\"></form>\n";

print "<form action=\"weightadj.pl\" method=\"post\" style=\"display:inline;\"
 title=\"$lex{'Adjust the weights of individual assessment items'}\">\n";
print "<input type=\"submit\" value=\"$lex{'Weight Item'}\"></form>\n";

print "<form action=\"weightgrp.pl\" method=\"post\" style=\"display:inline;\" 
 title=\"$lex{'Adjust the weightings of groups of assessment items'}\">\n";
print "<input type=\"submit\" value=\"$lex{'Weight Grp'}\"></form>\n";

print "<form action=\"post.pl\" method=\"post\" style=\"display:inline;\"
 title=\"$lex{'Post (copy) student averages to the report card system'}\">\n";
print "<input type=\"submit\" value=\"$lex{Post}\"></form>\n";

print "<form action=\"export.pl\" method=\"post\" style=\"display:inline;\"
 title=\"$lex{'Export these grades to a CSV file'}\">\n";
print "<input type=\"submit\" value=\"$lex{Exp}\"></form>\n";

print "<form action=\"mtcemenu.pl\" method=\"post\" style=\"display:inline;\"
 title=\"$lex{'Maintenance: Sortorder, Withdraw, etc'}\">\n";
print "<input type=\"submit\" value=\"$lex{Mtce}\"></form>\n";

print "<form action=\"search.pl\"  method=\"post\" style=\"display:inline;\"
 title=\"$lex{'Search: Scores, Averages'}\">\n";
print "<input type=\"submit\" value=\"$lex{Search}\"></form>\n";

print "</div>\n";
#### End of Methods Line ###


# Outer Table...################
print "<table cellpadding=\"0\" cellspacing=\"0\" border=\"0\">\n";
print "<tr><td valign=\"top\">\n";


# Print the table header
print "<table cellpadding=\"3\" cellspacing=\"0\" border=\"1\"  id=\"table_main\">\n";
print "<tr style=\"background-color:$maincolor;\">
<td align=\"center\"><span style=\"color:white;\">". $lex{Assessments}. "</span></td>
<td align=\"center\"><span style=\"color:white;\">". $lex{Avg}. "</span></td></tr>\n";


# Print the Edit Rows for the tests.
print "<tr valign=\"middle\" style=\"background-color: $altcolor;\">\n";
print "<td align=\"center\">
<a href=\"multiedit.pl\" ";
print "title=\"$lex{'Edit multiple scores at the same time'}\" \n"; 
print "style=\"color:white;\">$lex{MultiEdit}</a>";
print "</td><td>&nbsp;</td></tr>\n";
# End of Edit Rows


# Populate tests array for use by 'calcaverage' sub.
$sth = $dbh->prepare("select id, score, weight from gbtest where subjsec = ? and name != 'sortorder'");
$sth->execute( $subjsec );
if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr;}
    # Loop Through all tests...skipping sortorder.
while ( my ($id,$maxscore,$weight) = $sth->fetchrow ) {
    if (not $maxscore or $maxscore == 0){ # Get name, flag error and skip.
	my $sth1 = $dbh->prepare("select name from gbtest where id = ?");
	$sth1->execute( $id );
	if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr;}
	my $testname = $sth1->fetchrow;
	
	print $lex{'No Maximum Score for Test'}. ": $testname ($id); skipping...<br>\n"; 
	next; 
    }

    push @tests,"$id:$maxscore:$weight";
}



# Print the students and their marks;
my $studcount;
foreach my $studnum (@studnum) {
    $sth = $dbh->prepare("select lastname, firstname from studentall where studnum = ?");
    $sth->execute( $studnum );
    if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr;}
    my ($lastname, $firstname) = $sth->fetchrow;
    my $avg = calcAverage($studnum,$subjsec,\@tests);

    $lastname =~ s/\s+/&nbsp;/g;  # replace spaces with non-breaking space.
    $firstname =~ s/\s+/&nbsp;/g;

    print "<tr><td><b>$lastname</b>,&nbsp;$firstname</td>\n";
    print "<td>$avg</td></tr>\n";
    $studcount++;
}
# End Student Loop


# Print the Test Average Row;
print "<tr style=\"background-color: #CCC;\">
 <td colspan=\"2\" align=\"center\">". $lex{'Test Average'}. "</td></tr>\n";


# Print the Test Weighting Row
print "<tr style=\"background-color: $altcolor;\">
 <td colspan=\"2\" align=\"center\">". $lex{'Test Weight'}. "</td></tr>\n";


# Print the Assessment Row (again)
print "<tr style=\"background-color:$maincolor;\"><td colspan=\"2\" style=\"text-align:center;\">\n";
print "<span style=\"color:white;\">". $lex{Assessments}. "</span></td></tr>\n";
print "</table>\n";


# Some Javascript here for resizing...
print "<script type=\"text/javascript\">
 <!-- 
 function resize_iframe() {
  document.getElementById(\"iframe_main\").height=document.getElementById(\"table_main\").offsetHeight + 24;
  document.getElementById(\"iframe_main\").width=document.body.offsetWidth-document.getElementById(\"table_main\").offsetWidth;
 }
 -->
 </script>\n";

# Bottom Section Below tables.
print "<div style=\"padding:4px; line-height: 140%;\">";
print "[ $specchar = NA | \n";
print "<form action=\"$self\" method=\"post\" style=\"display:inline;padding:0;margin:0;\">\n";
print "<input type=\"hidden\" name=\"logout\" value=\"1\">\n";
print "<input type=\"submit\" value=\"$lex{'Log Out'}\"></form><br>\n";

print "<span style=\"margin: 0.2em; border:1px solid black; font-size:80%;\n";
print "padding: 2px 6px; background-color:$altcolor;\">";
print "$lex{Version}: $version</span></div>\n";

my $group;
if ($view ne $lex{'All Groups'}){
    $group = $view;
}

# Figure out the iframe height
my $frameheight = $studcount * 27 + 130; # 138 for the other 4 rows...

my $newgroup = escape($group);

# Do the IFrame
print "</td><td valign=\"top\">\n";
print "<iframe src=\"gbframe.pl?subjsec=$subjsec&cellvw=p&grp=$newgroup&dend=$dend&dstart=$dstart\"";
print " width=\"600\" \n height=\"$frameheight\" id=\"iframe_main\" ";
print "frameborder=\"0\" style=\"padding:0; margin:0;\"  onload=\"resize_iframe()\"></iframe>\n";
# Below Iframe content here.

print "</td></tr></table>\n"; 
print "</body></html>\n";



#--------
sub login { # print error, login screen and die;
#--------
    my $error = shift;
    print "$doctype\n<html><head><title>$error</title>\n";
    print "<link rel=\"stylesheet\" href=\"$tchcss\" type=\"text/css\">\n";
    print "$chartype\n</head><body onload=\"document.forms[0].elements[0].focus()\">\n";
    print "<center><h1>$error</h1>\n";
    print "<p><form action=\"$self\" method=\"post\">\n";
    print $lex{'User Id'}. ": <input type=\"text\" name=\"userid\" size=\"8\">\n";
    print $lex{Password}. ": <input type=\"password\" name=\"password\" size=\"8\"><br>\n";
    print $lex{'Cookie Duration'}. ": (". $lex{min}. ") ";
    print "<input type=\"text\" name=\"duration\" size=\"4\" value=\"20\"><br>\n";
    print "<input type=\"hidden\" name=\"subjsec\" value=\"$arr{subjsec}\">\n";
    print "<input type=\"submit\" value=\"". $lex{'Enter Gradebook'}. "\"></form></p>\n";

    print "[ <b><a href=\"$tchpage\">". $lex{Main}. "</a></b> ]</center></body></html>\n";
    
    exit;

}


#------------
sub mkStudNum {
#------------

    my $subjsec = shift;
    my (@eval, %eval, @studnum, %studnum);

    # Create the studnum array that controls the order that students are
    # displayed in. Load the eval records first into an array, sorted by
    # lastname and firstname. (@eval) Load the sort order records in order
    # into an array and also into a hash (%studnum)
    # Any remaining array elements in eval are added to the end of
    # the studnum array once done. If there _are_ no sortorder recs, then
    # the final array becomes the eval records only. This has the desired
    # behavior.

    # Find the enrollments for this class and read them into @eval array
    $sth = $dbh->prepare("select distinct eval.studnum from eval
     left outer join studentall on studentall.studnum = eval.studnum 
     where eval.subjcode = ? 
     order by studentall.lastname, studentall.firstname");

    $sth->execute( $subjsec );
    if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }
    while (my $studnum = $sth->fetchrow){
	push @eval,$studnum;
	$eval{$studnum} = 1;
    }
    
    # Load 'sortorder' test if it exists... 
    $sth = $dbh->prepare("select id from gbtest where
     subjsec = ? and name = 'sortorder'");
    $sth->execute($subjsec);
    if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }
    my $sortid = $sth->fetchrow;

    if ($sortid){ # We have a sortorder 'test'.
	my $sth = $dbh->prepare("select id, studnum from gbscore where
          testid = ? order by score");
	$sth->execute($sortid);
	if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }

	my $sth1 = $dbh->prepare("delete from gbscore where id = ?");

	while (my ($id, $sn) = $sth->fetchrow){
	    # Check that student is still enrolled...
	    if (not $eval{$sn}) { # student is withdrawn...
		$sth1->execute($id); # remove his/her record from gbscore; # sortorder
		next; 
	    } 

	    push @studnum, $sn;
	    $studnum{$sn} = 1;
	}
	

	# Now add on @eval elements who are not in @studnum
	foreach my $sn (@eval){
	    if ( not $studnum{$sn} ){ push @studnum, $sn; }
	}

	return @studnum;

    } else { # no sortorder
	return @eval;
    }
}


#----------------
sub checkPassword {
#----------------
    my ($userid, $password) = @_;
    
    if (not $userid){ return -1;}
    if (not $password){ return -2;}

    #check for presence of userid
    my $sth = $dbh->prepare("select count(userid) from staff where userid = ?");
    $sth->execute( $userid );
    if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr;}
    my $count = $sth->fetchrow;
    if ($count < 1){ return -1;} # no userid

    #check for presence of correct password and userid
    my $sth = $dbh->prepare("select count(userid) from staff where userid = ? and password = ?");
    $sth->execute( $userid, $password );
    if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr;}
    my $count = $sth->fetchrow;
    if ($count < 1){ return -2;} # not correct password

    return 0; # if all ok...

}


#------------------
sub checkCookieTime {
#------------------
    my ($duration) = @_;

    if ($duration) { 
	$cookietime = $duration;
    } else { 
	$cookietime = $defaulttime;
    }  

    $cookietime = $minimumtime if $cookietime < $minimumtime; 
    $cookietime = $maximumtime if $cookietime > $maximumtime; 
    $cookietime = "+".$cookietime."m"; # set format
    
    return $cookietime;

}


#-----------------
sub calcAverage { # This is the new one...
#-----------------

    require strict;
    # needed: $maxscore{$id}, $weight{$id}, $groupweight{$grp}

    my ($studnum, $subjsec) = @_; # get student number and subject/section.

    my $totalweight;
    my $totalscore;

    # Get test info and test score ( of that item ) at same time.
    my $sth = $dbh->prepare("select s.id, s.testid, s.score, t.grp from gbscore as s, gbtest as t 
      where s.testid = t.id and t.name != 'sortorder' and 
      s.studnum = ? and t.subjsec = ? order by t.grp");
    $sth->execute( $studnum, $subjsec );
    if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }

    my $currgrp = -1;
    my $oldgrp;
    my $first = 1;
    my $totalscore; # running total for score; zeroed for each new group.
    my $totalweight; # running total for weight; zeroed for each new group.
    my %groupscore;

    while ( my ( $id, $testid, $score, $grp) = $sth->fetchrow ) {
	$score =~ s/\s//g; # strip any spaces in score;
	if ($score eq $specchar or $score eq '' or not defined $score ) { next; } # skip

	$oldgrp = $currgrp;
	$currgrp = $grp;
	if ($currgrp ne $oldgrp and not $first) {
	    # setup group values for the current group
	    if ($totalweight) {
		$groupscore{$oldgrp} = $totalscore / $totalweight;
	    } else {
		$groupscore{$oldgrp} = 0;
	    }
	    $totalscore = 0;
	    $totalweight = 0;
	}
	if ( $first ) { $first = 0; }

	
	if ($score =~ /\d/) { # if score is a digit... do it.
	    $totalscore += $score / $maxscore{$testid} * $weight{$testid};
	    $totalweight += $weight{$testid};
	} else {
	    # some sort of text score...count as zero... update total weight.
	    $totalweight += $weight{$testid};
	}	    

	#print "TS: $totalscore TW:$totalweight GP:$grp<br>\n";

    } # End of Scores Loop


    # Get Last group, if any;
    if ($totalweight) { 
	$groupscore{$currgrp} = $totalscore / $totalweight;
    }
    $totalscore = 0; # necessary?
    $totalweight = 0; # necessary?

    #foreach my $key (keys %groupscore) { print "K: $key GS: $groupscore{$key}<br>\n"; }

    # Add up the group scores and weight them for overall average
    foreach my $grp (keys %groupscore) {
	$totalscore += $groupscore{$grp} * $groupweight{$grp};
	#print "GRP: $grp GS: $groupscore{$grp} GW:$groupweight{$grp}<br>\n";
	$totalweight += $groupweight{$grp};
    }

    #print "TS:$totalscore TW:$totalweight<br><br>\n";
    if ($totalweight) { 
	return round( 100 * $totalscore / $totalweight, 2). '%';
    } else {
	return 0;
    }


} # End of CalcAverage (new version)
