#!/usr/bin/perl
#  Copyright 2001-2013 Leslie Richardson

#  This file is part of Open Admin for Schools.

#  Open Admin for Schools is free software; you can redistribute it 
#  and/or modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2 of 
#  the License, or (at your option) any later version.

my %lex = ('Main' => 'Main',
	   'Error' => 'Error',
	   'Reading Tests' => 'Reading Tests',
	   'Edit' => 'Edit',
	   'Reading Level' => 'Reading Level',
	   'Category' => 'Category',	   
	   'Name' => 'Name',
	   'Save' => 'Save',
	   'Score' => 'Score',
	   'Test Date' => 'Test Date',
	   'No Blanks Allowed' => 'No Blanks Allowed',
	   'Values must be from 1-4' => 'Values must be from 1-4',
	   'No User Id' => 'No User Id',
	   'No Password' => 'No Password',
	   'Please Log In' => 'Please Log In',
	   'Record(s) Updated' => 'Record(s) Updated',
	   'Edit not allowed' => 'Edit not allowed',

	   );

use DBI;
use CGI;
use CGI::Session;

my $self = 'fpEdit.pl';

# Configuration Settings -----------------------
my $allow_author = 1; # allow author to edit
my $allow_dra = 1; # allow user with dra access code to edit
#-----------------------------------------------

my $q = new CGI;
my %arr = $q->Vars;

my @time = localtime(time);
my $year = $time[5] + 1900;
my $month = $time[4] + 1;
my $currdate = "$year-$month-$time[3]";


eval require "../../etc/admin.conf";
if ( $@ ) {
    print $lex{Error}. " $self: $@<br>\n";
    die $lex{Error}. "$self: $@\n";
}

my $dsn = "DBI:$dbtype:dbname=$dbase";
my $dbh = DBI->connect($dsn,$user,$password);

# Session Setup Here
my $session = new CGI::Session("driver:mysql;serializer:FreezeThaw",
			       undef,{Handle => $dbh}) or die CGI::Session->errstr;

# Get/Set  Session Values (a defined userid means it was passed)
if ( $arr{userid} ){ # we want to login, passed userid/password pair.

    # Check password/userid against database (-1 no user, -2 wrong password);
    my $error = checkPassword($arr{userid}, $arr{password});
    if ($error == -1){ print $q->header( -charset, $charset ); login($lex{'No User Id'}); }
	if ($error == -2){ print $q->header( -charset, $charset ); login($lex{'No Password'}); }
    
    $cookietime = checkCookieTime( $arr{duration} );
    
    # Set values for userid and logged_in in session
    $session->param( 'logged_in','1');
    $session->expire( 'logged_in', $cookietime );
    $session->param( 'userid',$arr{userid} );
    $session->param( 'duration',$cookietime );
    
    $userid = $arr{userid};


} else { # check logged_in value in session
	
    if ( not $session->param('logged_in') ){
	$userid = $session->param('userid');
	print $q->header( -charset, $charset );
	print $lex{'Please Log In'}. "</body></html>\n"; 
	exit;
    }
    # Ok, we have a login. Values below we have in environment.

    $userid = $session->param('userid');
    $duration = $session->param('duration');

    if ( not ($duration =~ /\+/)) { # if not in +20m format, do so.
	$duration = checkCookieTime( $duration );
    }

    $session->expire('logged_in', $duration );


} # End of check for logged_in value

print $session->header( -charset, $charset );
# Session Setup End


# Check for dra access code
my $sth = $dbh->prepare("select field_value from staff_multi 
  where field_name = 'access' and userid = ?");
$sth->execute( $userid );
if ( $DBI::errstr ){ print $DBI::errstr; die $DBI::errstr; }

my $accessdra;
while ( my $val = $sth->fetchrow ) {
    if ( uc $val eq 'DRA' ) { $accessdra = 1; }
    # print "VAL:$val ACCESS:$accessdra<br>\n";
}




# Page Header
print "$doctype\n<html><head><title>". $lex{Edit}. q{ }. $lex{'Reading Tests'};
print "</title><link rel=\"stylesheet\" href=\"$tchcss\" type=\"text/css\">\n";

print "<link rel=\"stylesheet\" type=\"text/css\" media=\"all\" ";
print "href=\"/js/calendar-blue.css\" title=\"blue\">\n";
print "<script type=\"text/javascript\" src=\"/js/calendar.js\"></script>\n";
print "<script type=\"text/javascript\" src=\"/js/lang/calendar-en.js\"></script>\n";
print "<script type=\"text/javascript\" src=\"/js/calendar-setup.js\"></script>\n";


print "$chartype\n</head><body style=\"padding:1em 2em;\">\n";
print "[ <a href=\"$tchpage\">$lex{Main}</a> ]\n";
print "<h1>$lex{Edit} $lex{'Reading Tests'}</h1>\n";


if ( not $arr{page} ) {
    editTest( $arr{testid} );

} elsif ( $arr{page} == 1 ) {
    delete $arr{page};
    writeTest();
}


#-------------
sub editTest {
#-------------

    # foreach my $key ( sort keys %arr ) { print "K:$key V:$arr{$key}<br>\n"; }

    my $testid = shift;

    # Get Reading Test fields.
    my $sth = $dbh->prepare("select * from read_test_fp where id = ?");
    $sth->execute( $testid );
    if ( $DBI::errstr ){ print $DBI::errstr; die $DBI::errstr; }
    my $ref = $sth->fetchrow_hashref;
    my %t = %$ref;

    # Show record values, if necessary.
    # foreach my $key ( sort keys %t ) { print "K:$key V:$t{$key}<br>\n"; }


    # Access Controls
    my $failflag;
    if ( not $allow_author and not $allow_dra ) { $failflag = 1; } # nobody edits
    elsif ( $allow_author and $t{'tauthor'} ne $userid ) { $failflag = 1; } # not author

    if ( $allow_dra and $accessdra ) { $failflag = 0; } # allow override of author failure
    if ( $failflag ) {
	print "<h3>$lex{'Edit not allowed'}</h3>\n";
	print "</body></html>\n";
	exit;
    }

    # Get Student Name
    my $sth1 = $dbh->prepare("select lastname, firstname from student where studnum = ?");
    $sth1->execute( $studnum );
    if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
    my ( $lastname, $firstname ) = $sth1->fetchrow;


    # Print Form Heading
    print "<form action=\"$self\" method=\"post\"> \n";
    print "<input type=\"hidden\" name=\"page\" value=\"1\">\n";
    print "<input type=\"hidden\" name=\"id\" value=\"$testid\">\n";

    # Save Button
    print "<div style=\"text-align:left;\">";
    print "<input type=\"submit\" value=\"$lex{Save}\"></div>\n";


    # Print Table Header and Heading values
    print "<table cellpadding=\"3\" cellspacing=\"0\" border=\"0\" ";
    print "style=\"float:left;margin:0.4em;padding:0.4em;border:1px solid gray;\">\n";
    print "<tr><td colspan=\"3\" class=\"bcn\">$firstname $lastname</td></tr>\n";

    # Test Date
    my $trigger = $studnum. '_trigger';
    my $idfield = $studnum. '_date';
    print "<tr><td class=\"bra\">$lex{'Test Date'}</td>\n";
    print "<td><input type=\"text\" name=\"tdate\" id=\"$idfield\" size=\"10\" value=\"$t{tdate}\">\n";
    print "<button type=\"reset\" id=\"$trigger\">...</button>\n";
    print "</td></tr>\n";

=head
    # Type
    print "<tr><td class=\"bra\">Type</td>";
    print "<td class=\"la\"><select name=\"ttype\"><option>$t{'ttype'}</option>";
    print "<option value=\"independent\">Independent</option>\n";
    print "<option value=\"instructional\">Instructional</option></select>\n";
    print "</td></tr>\n";
=cut

    # Level
    print "<tr><td class=\"bra\">Level</td>";
    print "<td class=\"la\"><select name=\"tlevel\"><option>$t{'tlevel'}</option>";
    foreach my $letter ('A'..'Z'){ print "<option>$letter</option>"; }
    print "</select></td></tr>\n";

    print "<tr><td class=\"bra\">Season</td>";
    print "<td class=\"la\"><select  name=\"season\"><option>$t{'season'}</option>";
    print "<option value=\"2012-spring\">2012 Spring</option>";
    print "<option value=\"2012-fall\">2012 Fall</option>";
    print "<option value=\"2011-fall\">2011 Fall</option>";
    print "<option value=\"2011-spring\">2011 Spring</option>";
    print "</td></tr>\n";

=head
    print "<tr><td class=\"bra\">Accuracy</td>";
    print "<td class=\"la\"><input type=\"text\" size=\"12\" value=\"$t{accuracy}\" name=\"accuracy\">";
    print "</td></tr>\n";

    print "<tr><td class=\"bra\">Comprehension</td>";
    print "<td class=\"la\"><input type=\"text\" size=\"12\" value=\"$t{comprehension}\" name=\"comprehension\">";
    print "</td></tr>\n";

    print "<tr><td class=\"bra\">Self-Correction</td>";
    print "<td class=\"la\"><input type=\"text\" size=\"12\" value=\"$t{selfcorrect}\"  name=\"selfcorrect\">";
    print "</td></tr>\n";

    print "<tr><td class=\"bra\">Fluency</td><td class=\"la\">";
    print "<select name=\"fluency\"><option>$t{fluency}</option>";
    for my $i (0..3) { print "<option>$i</option>"; }
    print "</select></td></tr>\n";

    print "<tr><td class=\"bra\">Rate (words per minute)</td>";
    print "<td class=\"la\"><input type=\"text\" value=\"$t{rate}\" size=\"12\" name=\"rate\">";
    print "</td></tr>\n";


    print "<tr><td class=\"bra\">Writing About Reading</td>\n<td class=\"la\">";
    print "<select name=\"writeread\"><option>$t{writeread}</option>";
    for my $i (0..3) { print "<option>$i</option>"; }
    print "</select></td></tr>\n";

    print "<tr><td class=\"bra\">Non-Fiction / Fiction</td>";
    print "<td class=\"la\"><select name=\"fictnonfict\"><option>$t{fictnonfict}</option>";
    print "<option value=\"nonfiction\">Non-Fiction</option>\n";
    print "<option value=\"fiction\">Fiction</option></select></td></tr>\n";
=cut

    print "<tr><td class=\"bra\">Behaviors/Comments</td><td></td></tr>\n";
    print "<tr><td class=\"cn\" colspan=\"2\">";
    print "<textarea name=\"tcomment\" rows=\"4\" cols=\"50\">\n";
    print "$t{tcomment}</textarea></td></tr>\n";
    
    print "</table>\n";

    print "<script type=\"text/javascript\">
       Calendar.setup({
       inputField     :    \"$idfield\", // id of the input field
       ifFormat       :    \"%Y-%m-%d\", // format of the input field
       button         :    \"$trigger\", // trigger for the calendar (button ID)
       singleClick    :    false,        // double-click mode
       step           :    1             // show all years in drop-down boxes 
     });
     </script>\n";


    print "<br clear=\"left\">\n";
    print "<div style=\"text-align:left;\">";
    print "<input type=\"submit\" value=\"$lex{Save}\"></div>\n";
	
    print "</form></center></body></html>\n";

    exit;

}


#--------------
sub writeTest {
#--------------


    # foreach my $key ( sort keys %arr ) { print "K:$key V:$arr{$key}<br>\n"; }


    my $id = $arr{id};
    delete $arr{id};

    foreach my $key ( sort keys %arr ) {
	# print "K:$key V:$arr{$key}<br>\n";

	my $field = $dbh->quote( $key );
	my $sth = $dbh->prepare("update read_test_fp set $key = ? where id = ?");

	$sth->execute( $arr{$key}, $id );
	if ( $DBI::errstr ){ print $DBI::errstr; die $DBI::errstr; }

    }


    print "<h1>$lex{'Record(s) Updated'}</h1>\n";

    print "<p>[ <a href=\"$tchpage\">$lex{Main}</a> ]</p>\n";
    print "</center></body></html>\n";

    exit;

}
