#!/usr/bin/perl
#  Copyright 2001-2013 Leslie Richardson

#  This file is part of Open Admin for Schools.

#  Open Admin for Schools is free software; you can redistribute it 
#  and/or modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2 of 
#  the License, or (at your option) any later version.

my %lex = ('Main' => 'Main',
	   'Error' => 'Error',
	   'View' => 'View',
	   'Reading Scores' => 'Reading Scores',
	   'Name' => 'Name',
	   'Date' => 'Date',
	   'Author' => 'Author',
	   'Score' => 'Score',
	   'No Records Found' => 'No Records Found',
	   'Continue' => 'Continue',
	   'Select' => 'Select',
	   'Student' => 'Student',
	   'Chk' => 'Chk',
	   'Homeroom' => 'Homeroom',
	   'Grade' => 'Grade',
	   'Blank=All' => 'Blank=All',
	   'Edit' => 'Edit',
	   'Tests' => 'Tests',
	   'Delete' => 'Delete',
	   'Start Date' => 'Start Date',
	   'End Date' => 'End Date',
	   'Equivalent' => 'Equivalent',
	   'Next Page' => 'Next Page',
	   'No Blanks Allowed' => 'No Blanks Allowed',

	   );

use DBI;
use CGI;
use Cwd;

my $self = 'fpView.pl';
my $editscript = 'fpEdit.pl';
my $deletescript = 'fpDelete.pl';


eval require "../../etc/admin.conf";
if ( $@ ) {
    print "$lex{Error}: $self: $@<br>\n";
    die "$lex{Error}: $self: $@\n";
}


# Get current dir so know what CSS to display and shift settings.
if ( getcwd() !~ /tcgi/ ) { # we are in cgi
    $tchcss = $css;
    $tchpage = $homepage;
    $tchdownloaddir = $downloaddir;
    $tchwebdownloaddir = $webdownloaddir;
}


my $q = new CGI;
print $q->header;
my %arr = $q->Vars;

my $dsn = "DBI:$dbtype:dbname=$dbase";
my $dbh = DBI->connect($dsn,$user,$password);


# Page Header
my $title = "$lex{View} $lex{'Reading Scores'}";
print "$doctype\n<html><head><title>$title</title>\n";
print "</title><link rel=\"stylesheet\" href=\"$tchcss\" type=\"text/css\">\n";

if ( not $arr{page} ) { # calendar popup.
    print "<link rel=\"stylesheet\" type=\"text/css\" media=\"all\" ";
    print "href=\"/js/calendar-blue.css\" title=\"blue\">\n";
    print "<script type=\"text/javascript\" src=\"/js/calendar.js\"></script>\n";
    print "<script type=\"text/javascript\" src=\"/js/lang/calendar-en.js\"></script>\n";
    print "<script type=\"text/javascript\" src=\"/js/calendar-setup.js\"></script>\n";
}

print "$chartype\n</head><body style=\"padding:1em 2em;\">\n";

print "[ <a href=\"$homepage\">$lex{Main}</a> ]\n";
print "<h1>$title</h1>\n";


if ( not $arr{page} ) {
    showStartPage();

} elsif ( $arr{page} == 1 ) {
    delete $arr{page};
    selectStudents();

} elsif ( $arr{page} == 2 ) {
    delete $arr{page};
    showTests();
} 


#----------
sub fmtDate {
#----------

    my ( $year, $mon, $day ) = split /-/, shift;
    return "$year-$s_month[$mon]-$day";
}


#----------------
sub showStartPage {
#----------------

    my @tim = localtime(time);
    my $year = $tim[5] + 1900;
    my $month = $tim[4] + 1;
    my $currdate = "$year-$month-$tim[3]";


    # Find by Student Group
    print "<form action=\"$self\" method=\"post\"> \n";
    print "<input type=\"hidden\" name=\"page\" value=\"1\">\n";
    print "<table cellpadding=\"3\" cellspacing=\"0\" border=\"0\" ";
    print "style=\"padding:0.5em;border:1px solid gray;\">\n";

    print "<tr><td class=\"ra\">$lex{Select} $lex{Student}\n";
    print "</td><td><select name=\"groupname\">";
    print "<option value=\"grade\">$lex{Grade}</option>\n";
    print "<option value=\"homeroom\">$lex{Homeroom}</option>\n";
    print "</select>\n";
    print "<input type=\"text\" name=\"groupvalue\" size=\"10\"> ";
    print "$lex{'Blank=All'}</td></tr>\n";

    print "<tr><td class=\"ra\">$lex{Chk} $lex{'Next Page'}</td>\n";
    print "<td><input type=\"checkbox\" name=\"chk\" value=\"1\">\n";
    print "</td></tr>\n";

    print "<tr><td align=\"right\">$lex{'Start Date'}</td><td><input type=\"text\" ";
    print "name=\"startdate\" id=\"sdate2\" size=\"10\" value=\"$schoolstart\">";
    print "<button type=\"reset\" id=\"start_trigger2\">...</button>\n";
    print "</td></tr>\n";

    print "<tr><td align=\"right\">$lex{'End Date'}</td><td><input type=\"text\" ";
    print "name=\"enddate\" id=\"edate2\" size=\"10\" value=\"$currdate\">";
    print "<button type=\"reset\" id=\"end_trigger2\">...</button></td></tr>\n";

    print "<tr><td></td><td>\n<input type=\"submit\" value=\"$lex{Continue}\">";
    print "</td></tr>\n";

    print "</table></form></div>\n";

    print "<script type=\"text/javascript\">
     Calendar.setup({
        inputField     :    \"sdate2\",
        ifFormat       :    \"%Y-%m-%d\",
        button         :    \"start_trigger2\",
        singleClick    :    false,
        step           :    1
    });

    Calendar.setup({
        inputField     :    \"edate2\",
        ifFormat       :    \"%Y-%m-%d\",
        button         :    \"end_trigger2\",
        singleClick    :    false,
        step           :    1
    });
   </script>\n";


    print "</body></html>\n";

    exit;

} # end of showStartPage



#----------------
sub selectStudents {
#----------------

    # foreach my $key ( sort keys %arr ) { print "K:$key V:$arr{$key}<br>\n"; }    

    my $groupvalue = $arr{groupvalue};
    delete $arr{groupvalue};

    my $chk;
    if ( $arr{chk} ) {
	$chk = 'CHECKED';
    }
    delete $arr{chk};


    my $select;
    if ( $groupvalue ) { # we're picking a group
	if ( $arr{groupname} eq 'grade' ) {
	    $select = 'where grade = ?';
	} else {
	    $select = 'where homeroom = ?';
	}
    }

    my $sth = $dbh->prepare("select lastname, firstname, studnum from student
      $select order by lastname, firstname");
    if ( $select ) {
	$sth->execute( $groupvalue );
    } else {
	$sth->execute;
    }
    if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr; }

    # Form Header
    print "<form action=\"$self\" method=\"post\"> \n";
    print "<input type=\"hidden\" name=\"page\" value=\"2\">\n";
    print "<input type=\"hidden\" name=\"startdate\" value=\"$arr{startdate}\">\n";
    print "<input type=\"hidden\" name=\"enddate\" value=\"$arr{enddate}\">\n";

    print "<table cellpadding=\"3\" cellspacing=\"0\" border=\"0\">\n";

    print "<tr><td class=\"ra\"><input type=\"submit\" value=\"$lex{Continue}\">";
    print "</td><td></td></tr>\n";

    my $sth1 = $dbh->prepare("select count(*) from read_test_fp where studnum = ? and 
      to_days( tdate ) >= to_days( '$arr{startdate}' ) 
      and to_days( tdate ) <= to_days( '$arr{enddate}')");
    
    my $sth2 = $dbh->prepare("select count(*) from read_test_fp where studnum = ?");

    print "<tr><th>$lex{Student}</th><th>$lex{Tests}</th><th></th></tr>\n";

    while ( my ( $lastname, $firstname, $studnum ) = $sth->fetchrow ) {
	
	# Count - Date Limited
	$sth1->execute( $studnum );
	if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr; }
	my $count = $sth1->fetchrow;

	# Count of All Tests
	$sth2->execute( $studnum );
	if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr; }
	my $overallcount = $sth2->fetchrow;

	print "<tr><td class=\"ra\">$lastname, $firstname ($studnum)</td>\n";
	print "<td style=\"text-align:center;\">$count / $overallcount</td><td>";
	print "<input type=\"checkbox\" name=\"$studnum\" value=\"1\" $chk></td></tr>\n";
    }

    print "<tr><td class=\"ra\"><input type=\"submit\" value=\"$lex{Continue}\">";
    print "</td><td></td></tr>\n";

    print "</table></form></body></html>\n";

    exit;

} # end of selectStudents



#------------
sub showTests {
#------------

    #foreach my $key ( sort keys %arr ) { print "K:$key V:$arr{$key}<br>\n"; }

    my $startdate = $arr{startdate};
    my $enddate = $arr{enddate};
    delete $arr{startdate};
    delete $arr{enddate};


    my $sth1 = $dbh->prepare("select lastname, firstname from student where studnum = ?");
    my $first = 1;

    foreach my $studnum ( sort keys %arr ) {

	# Get Student Name
	$sth1->execute( $studnum );
	if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
	my ( $lastname, $firstname ) = $sth1->fetchrow;

	# Display Existing Tests (if any)
	my $sth = $dbh->prepare("select * from read_test_fp 
          where studnum = ? and to_days( tdate ) >= to_days( '$startdate' ) 
          and to_days( tdate ) <= to_days( '$enddate') order by tdate, id");

	$sth->execute( $studnum );
	if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }

	while ( my $ref = $sth->fetchrow_hashref ) {

	    my %sr = %$ref;

	    $first = 0;

	    # Print Table Header and Heading values
	    print "<table cellpadding=\"3\" cellspacing=\"0\" border=\"0\" ";
	    print "style=\"float:left;margin:0.4em;padding:0.4em;border:1px solid gray;\">\n";

	    print "<tr><td colspan=\"3\" class=\"la\"><b>$firstname $lastname</b>  $sr{tdate} $sr{tauthor}";

	    # Edit Button
	    print "<form action=\"$editscript\" method=\"post\" target=\"_blank\" ";
	    print "style=\"display:inline;\">\n";
	    print "<input type=\"hidden\" name=\"testid\" value=\"$ref->{id}\">\n";
	    print "<input type=\"submit\" value=\"$lex{Edit}\">\n";
	    print "</form>\n";

	    # Delete Button
	    print "<form action=\"$deletescript\" method=\"post\" target=\"_blank\" ";
	    print "style=\"display:inline;\">\n";
	    print "<input type=\"hidden\" name=\"testid\" value=\"$ref->{id}\">\n";
	    print "<input type=\"submit\" value=\"$lex{Delete}\">\n";
	    print "</form></div>\n";

	    print "</td></tr>\n";

	    print "<tr><td class=\"bra\">Level</td>";
	    print "<td class=\"la\">$sr{'tlevel'}";
	    print "</td></tr>\n";

=head
	    print "<tr><td class=\"bra\">Type</td>";
	    print "<td class=\"la\">$sr{'ttype'}";
	    print "</td></tr>\n";

	    print "<tr><td class=\"bra\">Accuracy</td>";
	    print "<td class=\"la\">$sr{'accuracy'}";
	    print "</td></tr>\n";
	    
	    print "<tr><td class=\"bra\">Comprehension</td>";
	    print "<td class=\"la\">$sr{'comprehension'}";
	    print "</td></tr>\n";

	    print "<tr><td class=\"bra\">Self-Correction</td>";
	    print "<td class=\"la\">$sr{'selfcorrect'}";
	    print "</td></tr>\n";

	    print "<tr><td class=\"bra\">Fluency</td><td class=\"la\">";
	    print "$sr{'fluency'}</td></tr>\n";


	    print "<tr><td class=\"bra\">Rate</td>";
	    print "<td class=\"la\">$sr{'rate'}</td></tr>\n";


	    print "<tr><td class=\"bra\">Writing About Reading</td>\n<td class=\"la\">";
	    print "$sr{'writeread'}</td></tr>\n";


	    print "<tr><td class=\"bra\">Non-Fiction / Fiction</td>";
	    print "<td class=\"la\">$sr{'fictnonfict'}</td></tr>\n";
=cut

	    print "<tr><td class=\"bra\">Behaviors/Comments</td><td></td></tr>\n";
	    print "<tr><td class=\"cn\" colspan=\"2\">$sr{'tcomment'}</td></tr>\n";
	
	    print "</table><p></p>\n";

	
	} # end of student loop.

	if ( $first ) {
	    print "<div style=\"font-weight:bold;\">";
	    print "$lex{'No Records Found'}</div>\n";
	}

    } # end of student number loop

    print "</body></html>\n";
    
    exit;

}
