#!/usr/bin/perl
#  Copyright 2001-2018 Leslie Richardson

#  This file is part of Open Admin for Schools.

#  Open Admin for Schools is free software; you can redistribute it 
#  and/or modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2 of 
#  the License, or (at your option) any later version.

# Now called "Report Discipline" (rptdisc.pl) - was rptstud.pl
#  April 4: Modified to not show records with a Y in private field.

my %lex = ('Student Infractions by Date' => 'Student Infractions by Date',
	   'Main' => 'Main',
	   'Password not found or User not teacher' => 'Password not found or User not teacher',
	   'Student Discipline Infractions' => 'Student Discipline Infractions',
	   'No Record(s) Found' => 'No Record(s) Found',
	   'If Repeated' => 'If Repeated',
	   'Parent' => 'Parent',
	   'Please Log In' => 'Please Log In',
	   'Not Found' => 'Not Found',
	   'Error' => 'Error',
	   'Withdrawn' => 'Withdrawn',
	   'Select by' => 'Select by',
	   'Grade' => 'Grade',
	   'Homeroom' => 'Homeroom',
	   'OR' => 'OR',
	   'Show Withdrawn' => 'Show Withdrawn',
	   'Continue' => 'Continue',
	   'Student' => 'Student',
	   'Event' => 'Event',
	   'Count' => 'Count',
	   'Select' => 'Select',

	   'Infract/Par' => 'Infract/Par',
	   'Date' => 'Date',
	   'Description' => 'Description',
	   'Action' => 'Action',
	   'Parent' => 'Parent',
	   'Repeat' => 'Repeat',
	   'Demerit' => 'Demerit',
	   'Staff' => 'Staff',
	   
	   );

use DBI;
use CGI;
use CGI::Session;

my $q = new CGI;
my %arr = $q->Vars;

eval require "../etc/admin.conf";
if ( $@ ) {
    print $lex{Error}. ": $@<br>\n";
    die $lex{Error}. ": $@\n";
}

my $dsn = "DBI:$dbtype:dbname=$dbase";
my $dbh = DBI->connect($dsn,$user,$password);
$dbh->{mysql_enable_utf8} = 1;

my $title = $lex{'Student Discipline Infractions'};

# Get Session
my $session = new CGI::Session("driver:mysql;serializer:FreezeThaw",
 undef,{Handle => $dbh}) or die CGI::Session->errstr;


# Get Session Values (a defined userid means it was passed)
if ( not $session->param('logged_in') ){

    $userid = $session->param('userid');
    print $q->header( -charset, $charset );
    print qq{$doctype\n<html><head><title>$title</title>\n};
    print qq{<link rel="stylesheet" href="$tchcss" type="text/css">\n};
    print qq{$chartype\n</head><body>[ <a href="$tchpage">$lex{Main}</a> ]\n};

    print qq{<form action="tlogin.pl" method="post" style="margin:1em;">\n};
    print qq{<input type="hidden" name="script" value="rptdisc.pl">\n};
    print qq{<input type="submit" value="$lex{'Please Log In'}">\n};
    print qq{</form></body></html>\n};
    exit;
}
# Ok, we have a login. Values below we have in environment.

$userid = $session->param('userid');
$duration = $session->param('duration');
if (not ($duration =~ m/\+/ )) {
    $duration = '+'. $duration. 'm';
}
$session->expire('logged_in', $duration );

print $session->header( -charset, $charset );


# Print HTML Header
print qq{$doctype\n<html><head><title>$title</title>\n};
print qq{<link rel="stylesheet" href="$tchcss" type="text/css">\n};
print qq{$chartype\n</head><body>\n};
print qq{[ <a href="$tchpage">$lex{Main}</a> ]\n};

print qq{<h1>$title</h1>\n};

if ( not $arr{page} ) {
    showStartPage();

} elsif ( $arr{page} == 1) {
    delete $arr{page};
    selectStudents();

} elsif ( $arr{page} == 2) {
    delete $arr{page};
    showRecords();
}



#-----------------
sub selectStudents {
#-----------------

    my $studenttable = 'student';
    if ( $arr{showwithdrawn} ) {
	$studenttable = 'studentall';
    }


    my ($select, $selectval, $groupname);
    if ( $arr{grade} ) {
	$select = "where grade = ?";
	$selectval = $arr{grade};
	$groupname = $lex{Grade};

    } elsif ( $arr{homeroom} ) {
	$select = "where homeroom = ?";
	$selectval = $arr{homeroom};
	$groupname = $lex{Homeroom};

    } else {
	print qq{<h3>$lex{'No Selection'}</h3>\n};
	print qq{</body></html>\n};
	exit;
    }


    print qq{<h3>$groupname $selectval</h3>\n};


    # Start Form
    print qq{<form action="$self" method="post">\n};
    print qq{<input type="hidden" name="page" value="2">\n};

    print qq{<div><input type="submit" value="$lex{Continue}"></div>\n};

    # Start Table
    print qq{<table cellpadding="3" cellspacing="0" border="1">\n};
    print qq{<caption style="color:red;">Red = Withdrawn</caption>\n};
    print qq{<tr><th>$lex{Select}</th><th>$lex{Student}</th><th>$lex{Event}<br>$lex{Count}</th></tr>\n};

    # Loop through all students in this grade/homeroom
    my $sth = $dbh->prepare("select lastname, firstname, studnum from $studenttable
       $select order by lastname, firstname");
    $sth->execute( $selectval );
    if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }


    # Check for discipline events
    my $sth1 = $dbh->prepare("select count(*) from discipline where userid = ?");

    # Check for Withdrawn
    my $sth2 = $dbh->prepare("select count(*) from studentwd where studnum = ?");


    while ( my $ref = $sth->fetchrow_hashref ) {
	my %r = %$ref;

	# Check for discipline records
	$sth1->execute( $r{studnum} );
	if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }
	my $disccount = $sth1->fetchrow;

	my $wd;
	if ( $arr{showwithdrawn} ) { # check if withdrawn
	    $sth2->execute( $r{studnum} );
	    if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }
	    $wd = $sth2->fetchrow;
	}


	print qq{<tr>};
	if ( $disccount ) { 
	    print qq{<td><input type="checkbox" name="$r{studnum}" value="1"></td>\n}; 
	} else { 
	    print qq{<td></td>\n}; 
	}

	my $name;
	if ( $wd ) {
	    $name = qq{<span style="color:red;"><b>$r{lastname}</b>, $r{firstname}</span>};
	} else {
	    $name = qq{<b>$r{lastname}</b>, $r{firstname}};
	}


	print qq{<td>$name ($r{studnum})</td><td class="cn">$disccount</td></tr>\n};
			   
    }


    print qq{</table>\n};
    print qq{<div><input type="submit" value="$lex{Continue}"></div>\n};

    print qq{</form></body></html>\n};

    exit;

}


#----------------
sub showStartPage { 
#----------------

    my (@homerooms, @grades, @schoolyears );
    # Get Homerooms
    my $sth = $dbh->prepare("select distinct homeroom from student 
      where homeroom is not NULL and homeroom != ''");
    $sth->execute;
    if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }
    while ( my $hr = $sth->fetchrow ) {
	push @homerooms, $hr;
    }
    @homerooms = sort {$a <=> $b} @homerooms;

    # Get Grades
    $sth = $dbh->prepare("select distinct grade from student 
      where grade is not NULL and grade != ''");
    $sth->execute;
    if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }
    while ( my $gr = $sth->fetchrow ) {
	push @grades, $gr;
    }
    @grades = sort {$a <=> $b} @grades;


    print qq{<form action="$self" method="post">\n};
    print qq{<input type="hidden" name="page" value="1">\n};


    print qq{<table cellpadding="3" cellspacing="0" border="0">\n};


    # Select Grade
    print qq{<tr><td class="bra">$lex{'Select by'} $lex{Grade}</td>};
    print qq{<td><select name="grade"><option></option>\n};
    foreach my $gr ( @grades ) {
	print qq{<option>$gr</option>\n};
    }
    print qq{</select></td></tr>\n};

    # OR
    print qq{<tr><td colspan="2" class="cn">$lex{OR}</td></tr>\n};


    # Select Homeroom
    print qq{<tr><td class="bra">$lex{'Select by'} $lex{Homeroom}</td>};
    print qq{<td><select name="homeroom"><option></option>\n};
    foreach my $hr ( @homerooms ) {
	print qq{<option>$hr</option>\n};
    }
    print qq{</select></td></tr>\n};


    # Withdrawn
    print qq{<tr><td class="bra">$lex{'Show Withdrawn'}</td>};
    print qq{<td><input type="checkbox" name="showwithdrawn" value="1"></td></tr>\n};


    print qq{<tr><td class="ra"><input type="submit" value="$lex{Continue}"></td></tr>\n};
    print qq{</table>\n};
    print qq{</form></body></html>\n};

    exit;

}


#--------------
sub showRecords {
#--------------

    # foreach my $key ( sort keys %arr ) { print qq{K:$key V:$arr{$key}<br>\n}; }

    my $sth = $dbh->prepare("select * from discipline where userid = ? order by date desc");
    my $sth1 = $dbh->prepare("select lastname, firstname from studentall where studnum = ?");


    foreach my $studnum ( sort keys %arr ) {

	# Get the Name
	$sth1->execute($studnum);
	if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }
	my ( $lastname, $firstname ) = $sth1->fetchrow;

	print qq{<h3>$firstname $lastname ($studnum)</h3>\n};

	print qq{<table border="1" cellpadding="3" cellspacing="0">\n};
	print qq{<tr><th>$lex{Date}</th><th>$lex{'Infract/Par'}</th>\n};
	print qq{<th>$lex{Description}</th><th>$lex{Action}</th></tr>\n};

	# Get the Disc Records for this student
	$sth->execute($studnum);
	if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }

	while ( my $ref = $sth->fetchrow_hashref ) {
	    my %rec = %$ref;
	    if ( $rec{private} ) { print next; }
	    
	    print qq{<tr><td style="vertical-align:top;">$rec{date}<br><b>$lex{Staff}</b>:$rec{author}</td>\n};
	    print qq{<td>$rec{infraction}<br><b>$lex{Parent}:</b> $rec{parent}<br>\n};
	    print qq{<td>$rec{description}</td><td>$rec{action}<br>\n};
	    print qq{<b>$lex{Repeat}:</b> $rec{ifrepeat}<br><b>$lex{Demerit}</b>:$rec{demerit}</td></tr>\n};
	}

	print qq{</table><p></p>\n};

    } # end of student loop

    print qq{</body></html>\n};

    exit;

}
