#!/usr/bin/perl
#  Copyright 2001-2016 Leslie Richardson

#  This file is part of Open Admin for Schools.

#  Open Admin for Schools is free software; you can redistribute it 
#  and/or modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2 of 
#  the License, or (at your option) any later version.

# display teacher timetables. If passed teacher userid,
# display only that teacher's timetable.

my %lex = ('Day' => 'Day',
	   'Term' => 'Term',
	   'No Value' => '',
	   'View' => 'View',
	   'Teacher Timetable' => 'Teacher Timetable',
	   'Timetable' => 'Timetable',
	   'Per' => 'Per',
	   'Main' => 'Main',
	   'Teacher' => 'Teacher',
	   'View' => 'View',
	   'No Records Found' => 'No Records Found',
	   'Error' => 'Error',
	   'Period' => 'Period',

	   );

my $self = 'tchschedview.pl';

use DBI;
use CGI;
use CGI::Session;
use Cwd;


# Set prepath for config file: (/tcgi or cgi/schedule)
my $prepath = '../..';
if ( getcwd() =~ /tcgi/){ # we are in tcgi
    $prepath = '..';

}

eval require "$prepath/etc/admin.conf";
if ( $@ ) {
    print $lex{Error}. ": $@<br>\n";
    die $lex{Error}. ": $@\n";
}


my $dsn = "DBI:$dbtype:dbname=$dbase";
my $dbh = DBI->connect($dsn,$user,$password);
$dbh->{mysql_enable_utf8} = 1;


# Get Session and Userid if in teacher mode.
my $userid;
if ( getcwd() =~ /tcgi/){ # we are in tcgi
    my $session = new CGI::Session("driver:mysql;serializer:FreezeThaw",
				   undef,{Handle => $dbh}) or die CGI::Session->errstr;
    $userid = $session->param('userid');
}


my $q = new CGI;
print $q->header( -charset, $charset );
my %arr = $q->Vars;


if ( not $arr{userid} and $userid ) { $arr{userid} = $userid; }
# print "Userid:$userid   Arr Userid: $arr{userid}<br>\n";

my $select; 
if ( $arr{userid} ) {
    my $uid = $dbh->quote( $arr{userid} );
    $select = " where subject.teacher = $uid ";
}

if ( $arr{term} ){
    if ($arr{userid}){
	$select .= " and schedat.term = '$arr{term}' ";
    } else {
	$select = " where schedat.term = '$arr{term}' ";
    }
}


# Get current dir so know what CSS to display;
if (getcwd() =~ /tcgi/){ # we are in tcgi
    $css = $tchcss;
    $homepage = $tchpage;
}


# print page header.
print "$doctype\n<html><head><title>$lex{View} $lex{'Teacher Timetable'}</title>
<link rel=\"stylesheet\" href=\"$css\" type=\"text/css\">
$chartype\n</head><body>\n";

print "[ <a href=\"$homepage\">$lex{Main}</a> ";
unless (getcwd() =~ /tcgi/){ # unless we are in tcgi
    print "| <a href=\"$schpage\">$lex{Timetable}</a> ";
}

# Get Teacher values for the Form
my $sth = $dbh->prepare("select lastname, firstname, userid from staff order by lastname, firstname");
$sth->execute;
if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr; }
my (@staff, %staffname);
while ( my ( $lastname, $firstname, $userid)  = $sth->fetchrow ) {
    push @staff, $userid;
    $staffname{$userid} = qq{$lastname, $firstname};
}

$sth = $dbh->prepare("select distinct term from schedat order by term");
$sth->execute;
if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr; }
my @terms;
while ( my $t  = $sth->fetchrow ) {
    push @terms, $t;
}


print qq{]\n<form action="$self" method="post" style="display:inline;">\n};
print qq{$lex{Term} <select name="term"><option></option>\n};
foreach my $term ( @terms ) { print qq{<option>$term</option>}; }
print qq{</select>&nbsp;&nbsp;};
print qq{$lex{Teacher} <select name="userid"><option></option>\n};
foreach my $userid ( @staff ) {
    print qq{<option value="$userid">$staffname{$userid}</option>};
}
print qq{</select>};
print qq{<input type="submit" value="$lex{View}"></form>\n};
print qq{<div style="clear:both;"></div>\n};


# Find Numbers of Rows (periods) and Columns(Days) in Cycle
my $sth = $dbh->prepare("select max(period) from schedat");
$sth->execute;
if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr; }
my $tblrows = $sth->fetchrow;

$sth = $dbh->prepare("select max(day) from schedat");
$sth->execute;
if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr; }
$tblcols = $sth->fetchrow;

if ( $tblrows < 1 ) {
    print "<h3>$lex{'No Records Found'}</h3>\n";
    print "</body></html>\n";
    exit;
}


$sth = $dbh->prepare("select distinct schedat.subjsec, subject.description, 
  subject.teacher, schedat.term from schedat
  left outer join subject on schedat.subjsec = subject.subjsec
  $select order by subject.teacher, schedat.term");
$sth->execute;
if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr;}

my $oldterm = -1; 
my $oldtch = -1;
my $firstflag = 1;

while ( my ($subjsec,$desc,$teacher,$term) = $sth->fetchrow ) {
    #print "Subj: $subjsec Tch: $teacher $term<br>\n";
    $oldterm = $currterm;
    $currterm = $term;
    $oldtch = $currtch;
    $currtch = $teacher;

    if (($currterm ne $oldterm or $currtch ne $oldtch) and not $firstflag){ 
	# change in teacher or term
	# ship to printTimetable with $teacher, $term, and @subjects
	printTimetable($oldtch, $oldterm, \@subjects);
	if ( $currtch ne $oldtch ) { print qq{<div style="clear:both;"></div><hr>\n}; }
	@subjects = (); # empty array.
	push @subjects,$subjsec; # start new array with existing value.
    } else {
	push @subjects, $subjsec;
	$firstflag = 0;
    }

}

# print out the last table.
printTimetable($currtch, $currterm, \@subjects);

print "</body></html>\n";


#-----------------
sub printTimetable { # print timetable for one teacher for 1 term.
#-----------------

    my ($teacher, $term, $subjref) = @_; 

    if ( not $teacher or not $term ) {
	print qq{<h3>$lex{'No Records Found'}</h3>\n};
	return;
    }

    # Get Teacher Name
    my $sth = $dbh->prepare("select lastname, firstname from staff where userid = ?");
    $sth->execute( $teacher );
    my ($lastname, $firstname) = $sth->fetchrow;

    my @subject;
    my ($tblcols, $tblrows); # not globals anymore
    foreach my $subjsec ( @{ $subjref } ) {

	my $sth = $dbh->prepare("select day, period from schedat 
         where term = ? and subjsec = ? order by period, day");
	$sth->execute($term, $subjsec);
	while (my ($day, $period) = $sth->fetchrow ) {
	    # setup values for $tblrows and $tblcols
	    if ( $day > $tblcols ) { $tblcols = $day; }
	    if ( $period > $tblrows ) { $tblrows = $period; }

	    if ($subject[$period][$day] and $subject[$period][$day] ne $subjsec){ 
		# if a value exists and is not identical to subjsec then add to it.
		$subject[$period][$day] .= " $subjsec";
	    } else { # just slap it in...
		$subject[$period][$day] = $subjsec;
	    }
	}
    }
    # Done filling 2D array.

    my $sth = $dbh->prepare("select description, location  from subject where subjsec = ?");
    my $sth1 = $dbh->prepare("select count(distinct studnum) from eval where subjcode = ?");
    
    # Main body of table.
    my $first = 1;
    for $i (1..$tblrows){

	if ( $first ) {
	    # print heading
	    print qq{<table cellpadding="3" cellspacing="0" border="1" style="float:left;margin:0.4em;">\n};
	    print qq{<caption style="font-size:120%;font-weight:bold;">$firstname $lastname - };
	    print qq{$lex{Term} $term</caption>\n};
	    

	    print qq{<tr><th>$lex{Period}</th>};
	    my $dayvalue;
	    for my $day (1..$tblcols){ 
		if ( $g_DaysPerCycle == 5 or $g_DaysPerCycle eq 'W' or $g_DaysPerCycle eq 'w' ) {  # weekly
		    $dayvalue = $dow[$day + 1];
		} else {
		    $dayvalue = qq{$lex{Day} $day};
		}
		print qq{<th>$dayvalue</th>};
	    }
	    print qq{</tr>\n};
	    $first = 0;
	}

	print qq{<tr><td class="cn">$lex{Per} $i</td>};
	for (1..$tblcols){
	    if ($subject[$i][$_]){ # if we have a subjsec value;
		my $firstflag = 1;
		@subjval = split(/\s+/,$subject[$i][$_]);
		print "<td>";
		foreach my $sv (@subjval){
		    if (not $firstflag){ print "<br>";} else { $firstflag = 0;}

		    # Get Course Description and Location
		    $sth->execute($sv);
		    if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr; }
		    my ($desc,$location) = $sth->fetchrow;

		    # Get Enrollment count
		    $sth1->execute($sv);
		    if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr; }
		    my $enrolcount = $sth1->fetchrow;

		    
		    print "$desc<br>$sv ($enrolcount)<br>$location";
		}
		print "</td>";
	    } else {
		print qq{<td style="color:green;">$lex{'No Value'}</td>};
	    }
	}
	print "</tr>\n";
    }
    print "</table>\n";

    if ( $first ) { 
	print qq{<h3>$lex{'No Records Found'} - $firstname $lastname - $lex{Term} $term</h3>\n};
    }

    return;

}
