#!/usr/bin/perl
#  Copyright 2001-2013 Leslie Richardson

#  This file is part of Open Admin for Schools.

#  Open Admin for Schools is free software; you can redistribute it 
#  and/or modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2 of 
#  the License, or (at your option) any later version.

my %lex = ('School Enrollment' => 'School Enrollment',
	   'Main' => 'Main',
	   'Enrollment' => 'Enrollment',
	   'Current Date' => 'Current Date',
	   'Aging Date' => 'Aging Date',
	   'Current Enrollment' => 'Current Enrollment',
	   'Student' => 'Student',
	   'Grade' => 'Grade',
	   'Date' => 'Date',
	   'Type' => 'Type',
	   'Description' => 'Description',
	   'Enrolled' => 'Enrolled',
	   'Not Found' => 'Not Found',
	   'Error' => 'Error',
	   'Continue' => 'Continue',

	   );

use DBI;
use CGI;

my $self = 'rptenroldate_global.pl';

eval require "admin.conf";
if ( $@ ) {
    print $lex{Error}. ": $@<br>\n";
    die $lex{Error}. ": $@\n";
}

my ($sec, $min, $hour, $mday, $mon, $year, $wday, $yday, 
    $iddst) = localtime(time);
$year = $year + 1900;
$mon++;
$wday++;
#my $currsdate = "$year-$mon-$mday";
my $currdate = "$dow[$wday], $month[$mon] $mday, $year";

my $q = new CGI;
print $q->header( -charset, $charset );
my %arr = $q->Vars;

my $title = "$lex{Enrollment} at $lex{Date}";
print "$doctype\n<html><head><title>$title</title>\n";
print "<link rel=\"stylesheet\" href=\"$css\" type=\"text/css\">\n";
print "$chartype\n</head><body style=\"padding:1em 2em;\">\n";

print "[ <a href=\"$homepage\">$lex{Main}</a> ]\n";

print "<h1>$title</h1>\n";


if ( not $arr{page} ) {
    showStartPage();
}

#foreach my $key ( sort keys %arr ) { print "K:$key V:$arr{$key}<br>\n"; }

my $date = $arr{date};

foreach my $dbase ( sort keys %alldbase ) {

    my $schoolname = $alldbase{$dbase};

    print qq{<h2>$schoolname</h2>\n};

    my $dsn = "DBI:$dbtype:dbname=$dbase";
    my $dbh = DBI->connect($dsn,$user,$password);
    $dbh->{mysql_enable_utf8} = 1;

    # First count how many currently enrolled (in student table)
    my %enrollments;
    my $sth = $dbh->prepare("select distinct grade, count(grade) from student 
     where grade is not NULL and grade != '' group by grade");
    if ($DBI::errstr) {print $DBI::errstr; die $DBI::errstr; }
    $sth->execute;
    if ($DBI::errstr) {print $DBI::errstr; die $DBI::errstr; }
    while ( my ($grade, $gcount) = $sth->fetchrow ) {
	$enrollments{$grade} = $gcount;
    }

    print qq{<table cellpadding="4" cellspacing="0" border="1" style="float:left;margin:0.5em;">\n};
    print qq{<caption>Current $lex{Enrollment}</caption>\n};
    print qq{<tr><th>$lex{Grade}</th><th>$lex{Enrollment}</th></tr>\n};
    my $enroltotal;
    foreach my $grade ( sort {$a <=> $b} keys %enrollments ) {
	print qq{<tr><td>$grade</td><td>$enrollments{$grade}</td></tr>\n};
	$enroltotal += $enrollments{$grade};
    }
    print qq{<tr style="background-color:#DDD;"><td>Total</td><td>$enroltotal</td></tr>\n};
    print qq{</table>\n};


    # Now count backwards through entry records until we reach date of interest
    my $sth1 = $dbh->prepare("select * from transfer where to_days(date) >= to_days('$date') order by date desc");
    $sth1->execute;
    if ($DBI::errstr) {print $DBI::errstr; die $DBI::errstr; }

    # Setup query to get grade
    my $sth2 = $dbh->prepare("select grade from studentall where studnum = ?");

    while ( my $ref = $sth1->fetchrow_hashref ) {

	my %tr = %$ref;

	# Find student grade
	$sth2->execute( $tr{studnum} );
	if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }
	my $grade  = $sth2->fetchrow;
	if ( not $grade ) {
	    print "<div>$lex{Error}: $tr{studnum} $lex{'Not Found'}: $tr{date} - $tr{type}</div>\n";
	    next;
	}

#    print "<div>$tr{date} - $tr{type} - $tr{studnum} - $grade</div>\n";

	# Change Enrollment Numbers
	if ( $tr{type} ne 'withdraw' ) { # then was an enrol
	    $enrollments{$grade}--;
	} else {
	    $enrollments{$grade}++;
	}
    }

    # Now display the enrollment by grade
    print qq{<table cellpadding="4" cellspacing="0" border="1" style="float:left;margin:0.5em;">\n};
    print qq{<caption>$lex{Date} $date</caption>\n};
    print qq{<tr><th>$lex{Grade}</th><th>$lex{Enrollment}</th></tr>\n};
    my $enroltotal;
    foreach my $grade ( sort {$a <=> $b} keys %enrollments ) {
	print qq{<tr><td>$grade</td><td>$enrollments{$grade}</td></tr>\n};
	$enroltotal += $enrollments{$grade};
    }
    print qq{<tr style="background-color:#DDD;"><td>Total</td><td>$enroltotal</td></tr>\n};
    print qq{</table><br clear="left">\n};

} # next school


print "</body></html>\n";




#----------------
sub showStartPage {
#----------------

    print "<link rel=\"stylesheet\" type=\"text/css\" media=\"all\" ";
    print "href=\"/js/calendar-blue.css\" title=\"blue\">\n";
    print "<script type=\"text/javascript\" src=\"/js/calendar.js\"></script>\n";
    print "<script type=\"text/javascript\" src=\"/js/lang/calendar-en.js\"></script>\n";
    print "<script type=\"text/javascript\" src=\"/js/calendar-setup.js\"></script>\n";

    print "<form action=\"$self\" method=\"post\">\n";
    print "<input type=\"hidden\" name=\"page\" value=\"1\">\n";

    print "<table cellspacing=\"0\" border=\"0\" cellpadding=\"3\">\n";

    print "<tr><td class=\"bra\">$lex{'School Enrollment'} $lex{Date}</td><td class=\"la\">\n";
    print "<input type=\"text\" name=\"date\" size=\"12\" id=\"date\">\n";
    print "<button type=\"reset\" id=\"start_trigger\">...</button></td></tr>\n";

    print "<tr><td></td><td class=\"la\"><input type=\"submit\" value=\"$lex{Continue}\">\n";
    print "</td></tr></table></form>\n";

    print "<script type=\"text/javascript\">
     Calendar.setup({
        inputField     :    \"date\", // id of the input field
        ifFormat       :    \"%Y-%m-%d\", // format of the input field
        button         :    \"start_trigger\", // trigger for the calendar (button ID)
        singleClick    :    false,        // double-click mode
        step           :    1             // show all years in drop-down boxes 
    })";

    print "</script>\n";
    print "</body></html>\n";

    exit;

}
